@@ -98,6 +98,7 @@ from dagster._config.source import (
     IntSource as IntSource,
     StringSource as StringSource,
 )
+
 from dagster._core.definitions.asset_in import (
     AssetIn as AssetIn,
 )
@@ -107,6 +108,9 @@ from dagster._core.definitions.asset_out import (
 from dagster._core.definitions.asset_selection import (
     AssetSelection as AssetSelection,
 )
+from dagster._core.definitions.asset_sensor_definition import (
+    AssetSensorDefinition as AssetSensorDefinition,
+)
 from dagster._core.definitions.assets import (
     AssetsDefinition as AssetsDefinition,
 )
@@ -150,6 +154,9 @@ from dagster._core.definitions.decorators.sensor_decorator import (
     sensor as sensor,
     multi_asset_sensor as multi_asset_sensor,
 )
+from dagster._core.definitions.decorators.source_asset_decorator import (
+    observable_source_asset as observable_source_asset,
+)
 from dagster._core.definitions.dependency import (
     DependencyDefinition as DependencyDefinition,
     MultiDependencyDefinition as MultiDependencyDefinition,
@@ -175,6 +182,9 @@ from dagster._core.definitions.executor_definition import (
     multiple_process_executor_requirements as multiple_process_executor_requirements,
     multiprocess_executor as multiprocess_executor,
 )
+from dagster._core.definitions.freshness_policy import (
+    FreshnessPolicy as FreshnessPolicy,
+)
 from dagster._core.definitions.graph_definition import (
     GraphDefinition as GraphDefinition,
 )
@@ -200,6 +210,9 @@ from dagster._core.definitions.logger_definition import (
     build_init_logger_context as build_init_logger_context,
     logger as logger,
 )
+from dagster._core.definitions.logical_version import (
+    LogicalVersion as LogicalVersion,
+)
 from dagster._core.definitions.materialize import (
     materialize as materialize,
     materialize_to_memory as materialize_to_memory,
@@ -214,6 +227,7 @@ from dagster._core.definitions.metadata import (
     MarkdownMetadataValue as MarkdownMetadataValue,
     MetadataEntry as MetadataEntry,
     MetadataValue as MetadataValue,
+    NotebookMetadataValue as NotebookMetadataValue,
     PathMetadataValue as PathMetadataValue,
     PythonArtifactMetadataValue as PythonArtifactMetadataValue,
     TableMetadataValue as TableMetadataValue,
@@ -228,6 +242,11 @@ from dagster._core.definitions.metadata.table import (
     TableRecord as TableRecord,
     TableSchema as TableSchema,
 )
+from dagster._core.definitions.multi_asset_sensor_definition import (
+    MultiAssetSensorDefinition as MultiAssetSensorDefinition,
+    MultiAssetSensorEvaluationContext as MultiAssetSensorEvaluationContext,
+    build_multi_asset_sensor_context as build_multi_asset_sensor_context,
+)
 from dagster._core.definitions.op_definition import (
     OpDefinition as OpDefinition,
 )
@@ -296,20 +315,17 @@ from dagster._core.definitions.schedule_definition import (
     build_schedule_context as build_schedule_context,
 )
 from dagster._core.definitions.sensor_definition import (
-    AssetSensorDefinition as AssetSensorDefinition,
     DefaultSensorStatus as DefaultSensorStatus,
-    MultiAssetSensorDefinition as MultiAssetSensorDefinition,
-    MultiAssetSensorEvaluationContext as MultiAssetSensorEvaluationContext,
     SensorDefinition as SensorDefinition,
     SensorEvaluationContext as SensorEvaluationContext,
     build_sensor_context as build_sensor_context,
-    build_multi_asset_sensor_context as build_multi_asset_sensor_context,
 )
 from dagster._core.definitions.source_asset import (
     SourceAsset as SourceAsset,
 )
 from dagster._core.definitions.step_launcher import (
     StepLauncher as StepLauncher,
+    StepRunRef as StepRunRef,
 )
 from dagster._core.definitions.time_window_partition_mapping import (
     TimeWindowPartitionMapping as TimeWindowPartitionMapping,
@@ -326,9 +342,17 @@ from dagster._core.definitions.time_window_partitions import (
     monthly_partitioned_config as monthly_partitioned_config,
     weekly_partitioned_config as weekly_partitioned_config,
 )
+from dagster._core.definitions.multi_dimensional_partitions import (
+    MultiPartitionsDefinition as MultiPartitionsDefinition,
+    MultiPartitionKey as MultiPartitionKey,
+)
 from dagster._core.definitions.unresolved_asset_job_definition import (
     define_asset_job as define_asset_job,
 )
+
+from dagster._core.definitions.asset_reconciliation_sensor import (
+    build_asset_reconciliation_sensor as build_asset_reconciliation_sensor,
+)
 from dagster._core.definitions.utils import (
     config_from_files as config_from_files,
     config_from_pkg_resources as config_from_pkg_resources,
@@ -451,6 +475,9 @@ from dagster._core.event_api import (
     EventRecordsFilter as EventRecordsFilter,
     RunShardedEventsCursor as RunShardedEventsCursor,
 )
+from dagster._core.storage.asset_value_loader import (
+    AssetValueLoader as AssetValueLoader,
+)
 from dagster._core.storage.file_manager import (
     FileHandle as FileHandle,
     LocalFileHandle as LocalFileHandle,
@@ -470,6 +497,7 @@ from dagster._core.storage.io_manager import (
     io_manager as io_manager,
 )
 from dagster._core.storage.mem_io_manager import (
+    InMemoryIOManager as InMemoryIOManager,
     mem_io_manager as mem_io_manager,
 )
 from dagster._core.storage.memoizable_io_manager import (
@@ -478,6 +506,7 @@ from dagster._core.storage.memoizable_io_manager import (
 from dagster._core.storage.pipeline_run import (
     DagsterRun as DagsterRun,
     DagsterRunStatus as DagsterRunStatus,
+    RunsFilter as RunsFilter,
 )
 from dagster._core.storage.root_input_manager import (
     RootInputManager as RootInputManager,
@@ -518,11 +547,13 @@ from dagster._loggers import (
 )
 from dagster._core.execution.context.system import (
     DagsterTypeLoaderContext as DagsterTypeLoaderContext,
+    StepExecutionContext as StepExecutionContext,
 )
 from dagster._serdes.serdes import (
     deserialize_value as deserialize_value,
     serialize_value as serialize_value,
 )
+from dagster._core.storage.upath_io_manager import UPathIOManager as UPathIOManager
 from dagster._utils import (
     file_relative_path as file_relative_path,
 )
@@ -552,7 +583,7 @@ from dagster.version import __version__
 import importlib
 from typing import TYPE_CHECKING
 from typing import Any as TypingAny
-from typing import Callable, Mapping
+from typing import Callable, Mapping, Sequence
 from typing import Tuple as TypingTuple
 
 from typing_extensions import Final
@@ -610,5 +641,5 @@ def __getattr__(name: str) -> TypingAny:
         raise AttributeError("module '{}' has no attribute '{}'".format(__name__, name))
 
 
-def __dir__(_self):
+def __dir__(_self) -> Sequence:
     return [*globals(), *_DEPRECATED.keys(), *_DEPRECATED_RENAMED.keys()]
