@@ -1,7 +1,10 @@
+import base64
 import json
 import logging
 import os
 import sys
+import zlib
+from contextlib import ExitStack
 from typing import Any, Callable, Optional, cast
 
 import click
@@ -18,7 +21,7 @@ from dagster._core.events import DagsterEvent, DagsterEventType, EngineEventData
 from dagster._core.execution.api import create_execution_plan, execute_plan_iterator
 from dagster._core.execution.context_creation_pipeline import create_context_free_log_manager
 from dagster._core.execution.run_cancellation_thread import start_run_cancellation_thread
-from dagster._core.instance import DagsterInstance
+from dagster._core.instance import DagsterInstance, InstanceRef
 from dagster._core.origin import (
     DEFAULT_DAGSTER_ENTRY_POINT,
     PipelinePythonOrigin,
@@ -121,6 +124,7 @@ def _execute_run_command_body(
             recon_pipeline,
             pipeline_run,
             instance,
+            inject_env_vars=True,
         ):
             write_stream_fn(event)
             if event.event_type == DagsterEventType.PIPELINE_FAILURE:
@@ -193,10 +197,8 @@ def _resume_run_command_body(
         cancellation_thread, cancellation_thread_shutdown_event = start_run_cancellation_thread(
             instance, pipeline_run_id
         )
-    pipeline_run = instance.get_run_by_id(pipeline_run_id)
-    check.inst(
-        pipeline_run,
-        PipelineRun,
+    pipeline_run = check.not_none(
+        instance.get_run_by_id(pipeline_run_id),  # type: ignore
         "Pipeline run with id '{}' not found for run execution.".format(pipeline_run_id),
     )
     check.inst(
@@ -224,6 +226,7 @@ def _resume_run_command_body(
             pipeline_run,
             instance,
             resume_from_failure=True,
+            inject_env_vars=True,
         ):
             write_stream_fn(event)
             if event.event_type == DagsterEventType.PIPELINE_FAILURE:
@@ -315,10 +318,24 @@ def verify_step(instance, pipeline_run, retry_state, step_keys_to_execute):
         "interactively."
     ),
 )
-@click.argument("input_json", type=click.STRING)
-def execute_step_command(input_json):
+@click.argument("input_json", type=click.STRING, envvar="DAGSTER_EXECUTE_STEP_ARGS", required=False)
+@click.option(
+    "compressed_input_json",
+    "--compressed-input-json",
+    type=click.STRING,
+    envvar="DAGSTER_COMPRESSED_EXECUTE_STEP_ARGS",
+)
+def execute_step_command(input_json, compressed_input_json):
     with capture_interrupts():
 
+        check.invariant(
+            bool(input_json) != bool(compressed_input_json),
+            "Must provide one of input_json or compressed_input_json",
+        )
+
+        if compressed_input_json:
+            input_json = zlib.decompress(base64.b64decode(compressed_input_json.encode())).decode()
+
         args = deserialize_as(input_json, ExecuteStepArgs)
 
         with (
@@ -376,6 +393,14 @@ def _execute_step_command_body(
             step_key=single_step_key,
         )
 
+        location_name = (
+            pipeline_run.external_pipeline_origin.location_name
+            if pipeline_run.external_pipeline_origin
+            else None
+        )
+
+        instance.inject_env_vars(location_name)
+
         if args.should_verify_step:
             success = verify_step(
                 instance,
@@ -386,6 +411,13 @@ def _execute_step_command_body(
             if not success:
                 return
 
+        if pipeline_run.has_repository_load_data:
+            repository_load_data = instance.get_execution_plan_snapshot(
+                check.not_none(pipeline_run.execution_plan_snapshot_id)
+            ).repository_load_data
+        else:
+            repository_load_data = None
+
         recon_pipeline = recon_pipeline_from_origin(
             cast(PipelinePythonOrigin, pipeline_run.pipeline_code_origin)
         ).subset_for_execution_from_existing_pipeline(
@@ -398,6 +430,7 @@ def _execute_step_command_body(
             step_keys_to_execute=args.step_keys_to_execute,
             mode=pipeline_run.mode,
             known_state=args.known_state,
+            repository_load_data=repository_load_data,
         )
 
         yield from execute_plan_iterator(
@@ -549,6 +582,28 @@ def _execute_step_command_body(
     "code from this server.",
     envvar="DAGSTER_CONTAINER_CONTEXT",
 )
+@click.option(
+    "--inject-env-vars-from-instance",
+    is_flag=True,
+    required=False,
+    default=False,
+    help="Whether to load env vars from the instance and inject them into the environment.",
+    envvar="DAGSTER_INJECT_ENV_VARS_FROM_INSTANCE",
+)
+@click.option(
+    "--location-name",
+    type=click.STRING,
+    required=False,
+    help="Name of the code location this server corresponds to.",
+    envvar="DAGSTER_LOCATION_NAME",
+)
+@click.option(
+    "--instance-ref",
+    type=click.STRING,
+    required=False,
+    help="[INTERNAL] Serialized InstanceRef to use for accessing the instance",
+    envvar="DAGSTER_INSTANCE_REF",
+)
 def grpc_command(
     port=None,
     socket=None,
@@ -564,6 +619,9 @@ def grpc_command(
     use_python_environment_entry_point=False,
     container_image=None,
     container_context=None,
+    location_name=None,
+    instance_ref=None,
+    inject_env_vars_from_instance=False,
     **kwargs,
 ):
     from dagster._core.test_utils import mock_system_timezone
@@ -576,7 +634,7 @@ def grpc_command(
         raise click.UsageError("You must pass one and only one of --port/-p or --socket/-s.")
 
     configure_loggers(log_level=coerce_valid_log_level(log_level))
-    logger = logging.getLogger("dagster.code_server")
+    logger = logging.getLogger("dagster")
 
     container_image = container_image or os.getenv("DAGSTER_CURRENT_IMAGE")
 
@@ -605,11 +663,11 @@ def grpc_command(
             package_name=kwargs["package_name"],
         )
 
-    with (
-        mock_system_timezone(override_system_timezone)
-        if override_system_timezone
-        else seven.nullcontext()
-    ):
+    with ExitStack() as exit_stack:
+
+        if override_system_timezone:
+            exit_stack.enter_context(mock_system_timezone(override_system_timezone))
+
         server = DagsterGrpcServer(
             port=port,
             socket=socket,
@@ -628,6 +686,9 @@ def grpc_command(
             ),
             container_image=container_image,
             container_context=json.loads(container_context) if container_context != None else None,
+            inject_env_vars_from_instance=inject_env_vars_from_instance,
+            instance_ref=deserialize_as(instance_ref, InstanceRef) if instance_ref else None,
+            location_name=location_name,
         )
 
         code_desc = " "
