@@ -2,7 +2,7 @@ import os
 import re
 import sys
 import textwrap
-from typing import Any, Callable, Dict, List, Mapping, Optional, Tuple, cast
+from typing import Any, Callable, Mapping, Optional, Sequence, Tuple, cast
 
 import click
 import pendulum
@@ -12,17 +12,17 @@ import dagster._check as check
 from dagster import __version__ as dagster_version
 from dagster._cli.workspace.cli_target import (
     WORKSPACE_TARGET_WARNING,
-    get_external_pipeline_or_job_from_external_repo,
-    get_external_pipeline_or_job_from_kwargs,
+    get_external_job_from_external_repo,
+    get_external_job_from_kwargs,
     get_external_repository_from_kwargs,
     get_external_repository_from_repo_location,
-    get_pipeline_or_job_python_origin_from_kwargs,
+    get_job_python_origin_from_kwargs,
     get_repository_location_from_workspace,
     get_workspace_from_kwargs,
     job_repository_target_argument,
     job_target_argument,
+    python_job_config_argument,
     python_job_target_argument,
-    python_pipeline_or_job_config_argument,
 )
 from dagster._core.definitions.pipeline_base import IPipeline
 from dagster._core.errors import DagsterBackfillFailedError, DagsterInvariantViolationError
@@ -77,13 +77,11 @@ def apply_click_params(command, *click_params):
 )
 @job_repository_target_argument
 def job_list_command(**kwargs):
-    return execute_list_command(kwargs, click.echo, True)
+    return execute_list_command(kwargs, click.echo)
 
 
-def execute_list_command(cli_args, print_fn, using_job_op_graph_apis=False):
-    with get_instance_for_service(
-        "``dagster job list``" if using_job_op_graph_apis else "``dagster pipeline list``"
-    ) as instance:
+def execute_list_command(cli_args, print_fn):
+    with get_instance_for_service("``dagster job list``") as instance:
         with get_external_repository_from_kwargs(
             instance, version=dagster_version, kwargs=cli_args
         ) as external_repository:
@@ -91,30 +89,19 @@ def execute_list_command(cli_args, print_fn, using_job_op_graph_apis=False):
             print_fn(title)
             print_fn("*" * len(title))
             first = True
-            for pipeline in (
-                external_repository.get_external_jobs()
-                if using_job_op_graph_apis
-                else external_repository.get_all_external_pipelines()
-            ):
-                pipeline_title = "{pipeline_or_job}: {name}".format(
-                    pipeline_or_job="Job" if using_job_op_graph_apis else "Pipeline",
-                    name=pipeline.name,
-                )
+            for job in external_repository.get_all_external_jobs():
+                job_title = f"Job: {job.name}"
 
                 if not first:
-                    print_fn("*" * len(pipeline_title))
+                    print_fn("*" * len(job_title))
                 first = False
 
-                print_fn(pipeline_title)
-                if pipeline.description:
+                print_fn(job_title)
+                if job.description:
                     print_fn("Description:")
-                    print_fn(format_description(pipeline.description, indent=" " * 4))
-                print_fn(
-                    "{solid_or_op}: (Execution Order)".format(
-                        solid_or_op="Ops" if using_job_op_graph_apis else "Solids"
-                    )
-                )
-                for solid_name in pipeline.pipeline_snapshot.solid_names_in_topological_order:
+                    print_fn(format_description(job.description, indent=" " * 4))
+                print_fn("Ops: (Execution Order)")
+                for solid_name in job.pipeline_snapshot.solid_names_in_topological_order:
                     print_fn("    " + solid_name)
 
 
@@ -148,66 +135,59 @@ def get_job_instructions(command_name):
 @job_target_argument
 def job_print_command(verbose, **cli_args):
     with get_instance_for_service("``dagster job print``") as instance:
-        return execute_print_command(
-            instance, verbose, cli_args, click.echo, using_job_op_graph_apis=True
-        )
+        return execute_print_command(instance, verbose, cli_args, click.echo)
 
 
-def execute_print_command(instance, verbose, cli_args, print_fn, using_job_op_graph_apis=False):
-    with get_external_pipeline_or_job_from_kwargs(
+def execute_print_command(instance, verbose, cli_args, print_fn):
+    with get_external_job_from_kwargs(
         instance,
         version=dagster_version,
         kwargs=cli_args,
-        using_job_op_graph_apis=using_job_op_graph_apis,
     ) as external_pipeline:
         pipeline_snapshot = external_pipeline.pipeline_snapshot
 
         if verbose:
-            print_pipeline_or_job(
+            print_job(
                 pipeline_snapshot,
                 print_fn=print_fn,
-                using_job_op_graph_apis=using_job_op_graph_apis,
             )
         else:
-            print_solids_or_ops(
+            print_ops(
                 pipeline_snapshot,
                 print_fn=print_fn,
-                using_job_op_graph_apis=using_job_op_graph_apis,
             )
 
 
-def print_solids_or_ops(
+def print_ops(
     pipeline_snapshot: PipelineSnapshot,
     print_fn: Callable[..., Any],
-    using_job_op_graph_apis: bool = False,
 ):
     check.inst_param(pipeline_snapshot, "pipeline", PipelineSnapshot)
     check.callable_param(print_fn, "print_fn")
 
     printer = IndentingPrinter(indent_level=2, printer=print_fn)
-    printer.line(f"{'Job' if using_job_op_graph_apis else 'Pipeline'}: {pipeline_snapshot.name}")
+    printer.line(f"Job: {pipeline_snapshot.name}")
 
-    printer.line(f"{'Ops' if using_job_op_graph_apis else 'Solids'}")
+    printer.line("Ops")
     for solid in pipeline_snapshot.dep_structure_snapshot.solid_invocation_snaps:
         with printer.with_indent():
-            printer.line(f"{'Op' if using_job_op_graph_apis else 'Solid'}: {solid.solid_name}")
+            printer.line(f"Op: {solid.solid_name}")
 
 
-def print_pipeline_or_job(
+def print_job(
     pipeline_snapshot: PipelineSnapshot,
     print_fn: Callable[..., Any],
-    using_job_op_graph_apis: bool = False,
 ):
     check.inst_param(pipeline_snapshot, "pipeline", PipelineSnapshot)
     check.callable_param(print_fn, "print_fn")
     printer = IndentingPrinter(indent_level=2, printer=print_fn)
-    printer.line(f"{'Job' if using_job_op_graph_apis else 'Pipeline'}: {pipeline_snapshot.name}")
+    printer.line(f"Job: {pipeline_snapshot.name}")
     print_description(printer, pipeline_snapshot.description)
 
-    printer.line(f"{'Ops' if using_job_op_graph_apis else 'Solids'}")
+    printer.line("Ops")
     for solid in pipeline_snapshot.dep_structure_snapshot.solid_invocation_snaps:
         with printer.with_indent():
-            print_solid_or_op(printer, pipeline_snapshot, solid, using_job_op_graph_apis)
+            print_op(printer, pipeline_snapshot, solid)
 
 
 def print_description(printer, desc):
@@ -228,17 +208,14 @@ def format_description(desc: str, indent: str):
     return filled
 
 
-def print_solid_or_op(
+def print_op(
     printer: IndentingPrinter,
     pipeline_snapshot: PipelineSnapshot,
     solid_invocation_snap: SolidInvocationSnap,
-    using_job_op_graph_apis: bool,
 ) -> None:
     check.inst_param(pipeline_snapshot, "pipeline_snapshot", PipelineSnapshot)
     check.inst_param(solid_invocation_snap, "solid_invocation_snap", SolidInvocationSnap)
-    printer.line(
-        f"{'Op' if using_job_op_graph_apis else 'Solid'}: {solid_invocation_snap.solid_name}"
-    )
+    printer.line(f"Op: {solid_invocation_snap.solid_name}")
     with printer.with_indent():
         printer.line("Inputs:")
         for input_dep_snap in solid_invocation_snap.input_dep_snaps:
@@ -259,18 +236,18 @@ def print_solid_or_op(
     ),
 )
 @python_job_target_argument
-@python_pipeline_or_job_config_argument("list_versions", using_job_op_graph_apis=True)
+@python_job_config_argument("list_versions")
 def job_list_versions_command(**kwargs):
     with DagsterInstance.get() as instance:
         execute_list_versions_command(instance, kwargs)
 
 
-def execute_list_versions_command(instance: DagsterInstance, kwargs: Dict[str, object]):
+def execute_list_versions_command(instance: DagsterInstance, kwargs: Mapping[str, object]):
     check.inst_param(instance, "instance", DagsterInstance)
 
     config = list(check.opt_tuple_param(kwargs.get("config"), "config", default=(), of_type=str))
 
-    job_origin = get_pipeline_or_job_python_origin_from_kwargs(kwargs, True)
+    job_origin = get_job_python_origin_from_kwargs(kwargs)
     job = recon_pipeline_from_origin(job_origin)
     run_config = get_run_config_from_file_list(config)
 
@@ -285,7 +262,7 @@ def execute_list_versions_command(instance: DagsterInstance, kwargs: Dict[str, o
     add_step_to_table(memoized_plan)
 
 
-def get_run_config_from_file_list(file_list: Optional[List[str]]):
+def get_run_config_from_file_list(file_list: Optional[Sequence[str]]):
     check.opt_list_param(file_list, "file_list", of_type=str)
     return load_yaml_from_glob_list(file_list) if file_list else {}
 
@@ -320,19 +297,18 @@ def add_step_to_table(memoized_plan):
     ),
 )
 @python_job_target_argument
-@python_pipeline_or_job_config_argument("execute", using_job_op_graph_apis=True)
+@python_job_config_argument("execute")
 @click.option("--tags", type=click.STRING, help="JSON string of tags to use for this job run")
 def job_execute_command(**kwargs):
     with capture_interrupts():
         with get_instance_for_service("``dagster job execute``") as instance:
-            execute_execute_command(instance, kwargs, True)
+            execute_execute_command(instance, kwargs)
 
 
 @telemetry_wrapper
 def execute_execute_command(
     instance: DagsterInstance,
-    kwargs: Dict[str, object],
-    using_job_op_graph_apis: bool = False,
+    kwargs: Mapping[str, object],
 ):
     check.inst_param(instance, "instance", DagsterInstance)
 
@@ -345,7 +321,7 @@ def execute_execute_command(
 
     tags = get_tags_from_args(kwargs)
 
-    pipeline_origin = get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_apis)
+    pipeline_origin = get_job_python_origin_from_kwargs(kwargs)
     pipeline = recon_pipeline_from_origin(pipeline_origin)
     solid_selection = get_solid_selection_from_args(kwargs)
     result = do_execute_command(pipeline, instance, config, mode, tags, solid_selection, preset)
@@ -370,7 +346,7 @@ def get_tags_from_args(kwargs):
         ) from e
 
 
-def get_config_from_args(kwargs: Dict[str, str]) -> Dict[str, object]:
+def get_config_from_args(kwargs: Mapping[str, str]) -> Mapping[str, object]:
 
     config = kwargs.get("config")  # files
     config_json = kwargs.get("config_json")
@@ -414,10 +390,10 @@ def get_solid_selection_from_args(kwargs):
 def do_execute_command(
     pipeline: IPipeline,
     instance: DagsterInstance,
-    config: Optional[List[str]],
+    config: Optional[Sequence[str]],
     mode: Optional[str] = None,
-    tags: Optional[Dict[str, object]] = None,
-    solid_selection: Optional[List[str]] = None,
+    tags: Optional[Mapping[str, object]] = None,
+    solid_selection: Optional[Sequence[str]] = None,
     preset: Optional[str] = None,
 ):
     check.inst_param(pipeline, "pipeline", IPipeline)
@@ -443,7 +419,7 @@ def do_execute_command(
     ),
 )
 @job_target_argument
-@python_pipeline_or_job_config_argument("launch", True)
+@python_job_config_argument("launch")
 @click.option(
     "--config-json",
     type=click.STRING,
@@ -459,8 +435,7 @@ def job_launch_command(**kwargs):
 @telemetry_wrapper
 def execute_launch_command(
     instance: DagsterInstance,
-    kwargs: Dict[str, str],
-    using_job_op_graph_apis: bool = False,
+    kwargs: Mapping[str, str],
 ):
     preset = cast(Optional[str], kwargs.get("preset"))
     mode = cast(Optional[str], kwargs.get("mode"))
@@ -472,10 +447,9 @@ def execute_launch_command(
         external_repo = get_external_repository_from_repo_location(
             repo_location, cast(Optional[str], kwargs.get("repository"))
         )
-        external_pipeline = get_external_pipeline_or_job_from_external_repo(
+        external_pipeline = get_external_job_from_external_repo(
             external_repo,
-            cast(Optional[str], kwargs.get("pipeline_or_job")),
-            using_job_op_graph_apis,
+            cast(Optional[str], kwargs.get("job_name")),
         )
 
         log_external_repo_stats(
@@ -517,7 +491,7 @@ def _create_external_pipeline_run(
     mode: Optional[str],
     preset: Optional[str],
     tags: Optional[Mapping[str, object]],
-    solid_selection: Optional[List[str]],
+    solid_selection: Optional[Sequence[str]],
     run_id: Optional[str],
 ):
     check.inst_param(instance, "instance", DagsterInstance)
@@ -589,10 +563,10 @@ def _check_execute_external_pipeline_args(
     mode: Optional[str],
     preset: Optional[str],
     tags: Optional[Mapping[str, object]],
-    solid_selection: Optional[List[str]],
-) -> Tuple[Dict[str, object], str, Mapping[str, object], Optional[List[str]]]:
+    solid_selection: Optional[Sequence[str]],
+) -> Tuple[Mapping[str, object], str, Mapping[str, object], Optional[Sequence[str]]]:
     check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-    run_config = check.opt_dict_param(run_config, "run_config")
+    run_config = check.opt_mapping_param(run_config, "run_config")
     check.opt_str_param(mode, "mode")
     check.opt_str_param(preset, "preset")
     check.invariant(
@@ -603,7 +577,7 @@ def _check_execute_external_pipeline_args(
     )
 
     tags = check.opt_dict_param(tags, "tags", key_type=str)
-    check.opt_list_param(solid_selection, "solid_selection", of_type=str)
+    check.opt_sequence_param(solid_selection, "solid_selection", of_type=str)
 
     if preset is not None:
         pipeline_preset = external_pipeline.get_preset(preset)
@@ -682,13 +656,11 @@ def _check_execute_external_pipeline_args(
 @python_job_target_argument
 @click.option("--print-only-required", default=False, is_flag=True)
 def job_scaffold_command(**kwargs):
-    execute_scaffold_command(kwargs, click.echo, using_job_op_graph_apis=True)
+    execute_scaffold_command(kwargs, click.echo)
 
 
-def execute_scaffold_command(cli_args, print_fn, using_job_op_graph_apis=False):
-    pipeline_origin = get_pipeline_or_job_python_origin_from_kwargs(
-        cli_args, using_job_op_graph_apis
-    )
+def execute_scaffold_command(cli_args, print_fn):
+    pipeline_origin = get_job_python_origin_from_kwargs(cli_args)
     pipeline = recon_pipeline_from_origin(pipeline_origin)
     skip_non_required = cli_args["print_only_required"]
     do_scaffold_command(pipeline.get_definition(), print_fn, skip_non_required)
@@ -747,10 +719,10 @@ def do_scaffold_command(
 @click.option("--noprompt", is_flag=True)
 def job_backfill_command(**kwargs):
     with DagsterInstance.get() as instance:
-        execute_backfill_command(kwargs, click.echo, instance, True)
+        execute_backfill_command(kwargs, click.echo, instance)
 
 
-def execute_backfill_command(cli_args, print_fn, instance, using_graph_job_op_apis=False):
+def execute_backfill_command(cli_args, print_fn, instance):
     with get_workspace_from_kwargs(instance, version=dagster_version, kwargs=cli_args) as workspace:
         repo_location = get_repository_location_from_workspace(workspace, cli_args.get("location"))
         _execute_backfill_command_at_location(
@@ -759,7 +731,6 @@ def execute_backfill_command(cli_args, print_fn, instance, using_graph_job_op_ap
             instance,
             workspace,
             repo_location,
-            using_graph_job_op_apis,
         )
 
 
@@ -769,15 +740,12 @@ def _execute_backfill_command_at_location(
     instance,
     workspace,
     repo_location,
-    using_graph_job_op_apis=False,
 ):
     external_repo = get_external_repository_from_repo_location(
         repo_location, cli_args.get("repository")
     )
 
-    external_pipeline = get_external_pipeline_or_job_from_external_repo(
-        external_repo, cli_args.get("pipeline_or_job")
-    )
+    external_pipeline = get_external_job_from_external_repo(external_repo, cli_args.get("job_name"))
 
     noprompt = cli_args.get("noprompt")
 
@@ -788,9 +756,7 @@ def _execute_backfill_command_at_location(
     }
 
     if not pipeline_partition_set_names:
-        raise click.UsageError(
-            "No partition sets found for pipeline/job `{}`".format(external_pipeline.name)
-        )
+        raise click.UsageError(f"No partition sets found for job `{external_pipeline.name}`")
     partition_set_name = cli_args.get("partition_set")
     if not partition_set_name:
         if len(pipeline_partition_set_names) == 1:
@@ -817,10 +783,7 @@ def _execute_backfill_command_at_location(
     )
 
     try:
-        partition_names_or_error = repo_location.get_external_partition_names(
-            repo_handle,
-            partition_set_name,
-        )
+        partition_names_or_error = repo_location.get_external_partition_names(partition_set)
     except Exception as e:
         error_info = serializable_error_info_from_exc_info(sys.exc_info())
         raise DagsterBackfillFailedError(
@@ -835,9 +798,7 @@ def _execute_backfill_command_at_location(
     )
 
     # Print backfill info
-    print_fn("\n Pipeline/Job: {}".format(external_pipeline.name))
-    if not using_graph_job_op_apis:
-        print_fn("Partition set: {}".format(partition_set_name))
+    print_fn("\n Job: {}".format(external_pipeline.name))
     print_fn("   Partitions: {}\n".format(print_partition_format(partition_names, indent_level=15)))
 
     # Confirm and launch
