@@ -1,7 +1,7 @@
 import os
 import sys
 from contextlib import contextmanager
-from typing import TYPE_CHECKING, Dict, Generator, Iterable, List, Optional, Tuple, Union, cast
+from typing import TYPE_CHECKING, Generator, Iterable, List, Mapping, Optional, Tuple, Union, cast
 
 import click
 from click import UsageError
@@ -47,12 +47,12 @@ def _cli_load_invariant(condition: object, msg=None) -> None:
         raise UsageError(msg)
 
 
-def _check_cli_arguments_none(kwargs: Dict[str, str], *keys: str) -> None:
+def _check_cli_arguments_none(kwargs: Mapping[str, str], *keys: str) -> None:
     for key in keys:
         _cli_load_invariant(not kwargs.get(key))
 
 
-def are_all_keys_empty(kwargs: Dict[str, str], keys: Iterable[str]) -> bool:
+def are_all_keys_empty(kwargs: Mapping[str, str], keys: Iterable[str]) -> bool:
     for key in keys:
         if kwargs.get(key):
             return False
@@ -74,8 +74,8 @@ WORKSPACE_CLI_ARGS = (
 )
 
 
-def get_workspace_load_target(kwargs: Dict[str, str]):
-    check.dict_param(kwargs, "kwargs")
+def get_workspace_load_target(kwargs: Mapping[str, str]):
+    check.mapping_param(kwargs, "kwargs")
     if are_all_keys_empty(kwargs, WORKSPACE_CLI_ARGS):
         if kwargs.get("empty_workspace"):
             return EmptyWorkspaceTarget()
@@ -173,7 +173,7 @@ def get_workspace_load_target(kwargs: Dict[str, str]):
 
 
 def get_workspace_process_context_from_kwargs(
-    instance: DagsterInstance, version: str, read_only: bool, kwargs: Dict[str, str]
+    instance: DagsterInstance, version: str, read_only: bool, kwargs: Mapping[str, str]
 ) -> "WorkspaceProcessContext":
     from dagster._core.workspace.context import WorkspaceProcessContext
 
@@ -184,7 +184,7 @@ def get_workspace_process_context_from_kwargs(
 
 @contextmanager
 def get_workspace_from_kwargs(
-    instance: DagsterInstance, version: str, kwargs: Dict[str, str]
+    instance: DagsterInstance, version: str, kwargs: Mapping[str, str]
 ) -> Generator[WorkspaceRequestContext, None, None]:
     with get_workspace_process_context_from_kwargs(
         instance, version, read_only=False, kwargs=kwargs
@@ -192,12 +192,12 @@ def get_workspace_from_kwargs(
         yield workspace_process_context.create_request_context()
 
 
-def python_target_click_options(is_using_job_op_graph_apis: bool = False):
+def python_target_click_options():
     return [
         click.option(
             "--working-directory",
             "-d",
-            help=f"Specify working directory to use when loading the repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'}.",
+            help="Specify working directory to use when loading the repository or job",
             envvar="DAGSTER_WORKING_DIRECTORY",
         ),
         click.option(
@@ -206,26 +206,26 @@ def python_target_click_options(is_using_job_op_graph_apis: bool = False):
             # Checks that the path actually exists lower in the stack, where we
             # are better equipped to surface errors
             type=click.Path(exists=False),
-            help=f"Specify python file where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
+            help="Specify python file where repository or job function lives",
             envvar="DAGSTER_PYTHON_FILE",
         ),
         click.option(
             "--package-name",
-            help=f"Specify Python package where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
+            help="Specify Python package where repository or job function lives",
             envvar="DAGSTER_PACKAGE_NAME",
         ),
         click.option(
             "--module-name",
             "-m",
-            help=f"Specify module where repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} function lives",
+            help="Specify module where repository or job function lives",
             envvar="DAGSTER_MODULE_NAME",
         ),
         click.option(
             "--attribute",
             "-a",
             help=(
-                f"Attribute that is either a 1) repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'} or "
-                f"2) a function that returns a repository or {'job' if is_using_job_op_graph_apis else 'pipeline/job'}"
+                "Attribute that is either a 1) repository or job or "
+                "2) a function that returns a repository or job"
             ),
             envvar="DAGSTER_ATTRIBUTE",
         ),
@@ -261,7 +261,7 @@ def grpc_server_target_click_options():
     ]
 
 
-def workspace_target_click_options(using_job_op_graph_apis: bool = False):
+def workspace_target_click_options():
     return (
         [
             click.option("--empty-workspace", is_flag=True, help="Allow an empty workspace"),
@@ -273,28 +273,14 @@ def workspace_target_click_options(using_job_op_graph_apis: bool = False):
                 help=("Path to workspace file. Argument can be provided multiple times."),
             ),
         ]
-        + python_target_click_options(using_job_op_graph_apis)
+        + python_target_click_options()
         + grpc_server_target_click_options()
     )
 
 
-def python_pipeline_target_click_options():
-    return (
-        python_target_click_options()
-        + [
-            click.option(
-                "--repository",
-                "-r",
-                help=("Repository name, necessary if more than one repository is present."),
-            )
-        ]
-        + [pipeline_option()]
-    )
-
-
 def python_job_target_click_options():
     return (
-        python_target_click_options(is_using_job_op_graph_apis=True)
+        python_target_click_options()
         + [
             click.option(
                 "--repository",
@@ -306,7 +292,7 @@ def python_job_target_click_options():
     )
 
 
-def target_with_config_option(command_name, using_job_op_graph_apis):
+def target_with_config_option(command_name):
     return click.option(
         "-c",
         "--config",
@@ -319,23 +305,19 @@ def target_with_config_option(command_name, using_job_op_graph_apis):
             "files at the key-level granularity. If the file is a pattern then you must "
             "enclose it in double quotes"
             "\n\nExample: "
-            "dagster {pipeline_or_job} {name} -f hello_world.py {pipeline_or_job_flag} pandas_hello_world "
+            f"dagster job {command_name} -f hello_world.py -j pandas_hello_world "
             '-c "pandas_hello_world/*.yaml"'
             "\n\nYou can also specify multiple files:"
             "\n\nExample: "
-            "dagster {pipeline_or_job} {name} -f hello_world.py {pipeline_or_job_flag} pandas_hello_world "
-            "-c pandas_hello_world/solids.yaml -c pandas_hello_world/env.yaml"
-        ).format(
-            name=command_name,
-            pipeline_or_job="job" if using_job_op_graph_apis else "pipeline",
-            pipeline_or_job_flag="-j" if using_job_op_graph_apis else "-p",
+            f"dagster job {command_name} -f hello_world.py -j pandas_hello_world "
+            "-c pandas_hello_world/ops.yaml -c pandas_hello_world/env.yaml"
         ),
     )
 
 
-def python_pipeline_or_job_config_argument(command_name, using_job_op_graph_apis=False):
+def python_job_config_argument(command_name):
     def wrap(f):
-        return target_with_config_option(command_name, using_job_op_graph_apis)(f)
+        return target_with_config_option(command_name)(f)
 
     return wrap
 
@@ -355,7 +337,7 @@ def workspace_target_argument(f):
 def job_workspace_target_argument(f):
     from dagster._cli.job import apply_click_params
 
-    return apply_click_params(f, *workspace_target_click_options(using_job_op_graph_apis=True))
+    return apply_click_params(f, *workspace_target_click_options())
 
 
 def grpc_server_origin_target_argument(f):
@@ -403,28 +385,11 @@ def job_repository_target_argument(f):
     return apply_click_params(job_workspace_target_argument(f), *repository_click_options())
 
 
-def pipeline_option():
-    return click.option(
-        "--pipeline",
-        "-p",
-        "pipeline_or_job",
-        help=(
-            "Pipeline/Job within the repository, necessary if more than one pipeline/job is present."
-        ),
-    )
-
-
-def pipeline_target_argument(f):
-    from dagster._cli.job import apply_click_params
-
-    return apply_click_params(repository_target_argument(f), pipeline_option())
-
-
 def job_option():
     return click.option(
         "--job",
         "-j",
-        "pipeline_or_job",
+        "job_name",
         help=("Job within the repository, necessary if more than one job is present."),
     )
 
@@ -435,48 +400,38 @@ def job_target_argument(f):
     return apply_click_params(job_repository_target_argument(f), job_option())
 
 
-def get_pipeline_or_job_python_origin_from_kwargs(kwargs, using_job_op_graph_apis=False):
+def get_job_python_origin_from_kwargs(kwargs):
     repository_origin = get_repository_python_origin_from_kwargs(kwargs)
-    provided_pipeline_name = kwargs.get("pipeline_or_job")
+    provided_name = kwargs.get("job_name")
 
     recon_repo = recon_repository_from_origin(repository_origin)
     repo_definition = recon_repo.get_definition()
 
-    pipeline_or_job_names = set(
-        repo_definition.job_names if using_job_op_graph_apis else repo_definition.pipeline_names
-    )
+    job_names = set(repo_definition.pipeline_names)  # pipeline (all) vs job (non legacy)
 
-    if provided_pipeline_name is None and len(pipeline_or_job_names) == 1:
-        pipeline_name = next(iter(pipeline_or_job_names))
-    elif provided_pipeline_name is None:
+    if provided_name is None and len(job_names) == 1:
+        pipeline_name = next(iter(job_names))
+    elif provided_name is None:
         raise click.UsageError(
             (
-                "Must provide {flag} as there is more than one pipeline/job "
-                "in {repository}. Options are: {pipelines}."
-            ).format(
-                flag="--job" if using_job_op_graph_apis else "--pipeline",
-                repository=repo_definition.name,
-                pipelines=_sorted_quoted(pipeline_or_job_names),
+                "Must provide --job as there is more than one job "
+                f"in {repo_definition.name}. Options are: {_sorted_quoted(job_names)}."
             )
         )
-    elif not provided_pipeline_name in pipeline_or_job_names:
+    elif not provided_name in job_names:
         raise click.UsageError(
             (
-                'Pipeline/Job "{provided_pipeline_name}" not found in repository "{repository_name}". '
-                "Found {found_names} instead."
-            ).format(
-                provided_pipeline_name=provided_pipeline_name,
-                repository_name=repo_definition.name,
-                found_names=_sorted_quoted(pipeline_or_job_names),
+                f'Job "{provided_name}" not found in repository "{repo_definition.name}" '
+                f"Found {_sorted_quoted(job_names)} instead."
             )
         )
     else:
-        pipeline_name = provided_pipeline_name
+        pipeline_name = provided_name
 
     return PipelinePythonOrigin(pipeline_name, repository_origin=repository_origin)
 
 
-def _get_code_pointer_dict_from_kwargs(kwargs: Dict[str, str]) -> Dict[str, CodePointer]:
+def _get_code_pointer_dict_from_kwargs(kwargs: Mapping[str, str]) -> Mapping[str, CodePointer]:
     python_file = kwargs.get("python_file")
     module_name = kwargs.get("module_name")
     package_name = kwargs.get("package_name")
@@ -525,11 +480,11 @@ def _get_code_pointer_dict_from_kwargs(kwargs: Dict[str, str]) -> Dict[str, Code
         check.failed("Must specify a Python file or module name")
 
 
-def get_working_directory_from_kwargs(kwargs: Dict[str, str]) -> Optional[str]:
+def get_working_directory_from_kwargs(kwargs: Mapping[str, str]) -> Optional[str]:
     return check.opt_str_elem(kwargs, "working_directory") or os.getcwd()
 
 
-def get_repository_python_origin_from_kwargs(kwargs: Dict[str, str]) -> RepositoryPythonOrigin:
+def get_repository_python_origin_from_kwargs(kwargs: Mapping[str, str]) -> RepositoryPythonOrigin:
     provided_repo_name = cast(str, kwargs.get("repository"))
 
     if not (kwargs.get("python_file") or kwargs.get("module_name") or kwargs.get("package_name")):
@@ -685,67 +640,46 @@ def get_external_repository_from_kwargs(instance, version, kwargs):
         yield get_external_repository_from_repo_location(repo_location, provided_repo_name)
 
 
-def get_external_pipeline_or_job_from_external_repo(
+def get_external_job_from_external_repo(
     external_repo: ExternalRepository,
-    provided_pipeline_or_job_name: Optional[str],
-    using_job_op_graph_apis: bool = False,
+    provided_name: Optional[str],
 ) -> ExternalPipeline:
     check.inst_param(external_repo, "external_repo", ExternalRepository)
-    check.opt_str_param(provided_pipeline_or_job_name, "provided_pipeline_or_job_name")
+    check.opt_str_param(provided_name, "provided_name")
 
-    external_pipelines = {
-        ep.name: ep
-        for ep in (
-            external_repo.get_external_jobs()
-            if using_job_op_graph_apis
-            else external_repo.get_all_external_pipelines()
-        )
-    }
+    external_pipelines = {ep.name: ep for ep in (external_repo.get_all_external_jobs())}
 
     check.invariant(external_pipelines)
 
-    if provided_pipeline_or_job_name is None and len(external_pipelines) == 1:
+    if provided_name is None and len(external_pipelines) == 1:
         return next(iter(external_pipelines.values()))
 
-    if provided_pipeline_or_job_name is None:
+    if provided_name is None:
         raise click.UsageError(
             (
-                "Must provide {flag} as there is more than one pipeline/job "
-                "in {repository}. Options are: {pipelines}."
-            ).format(
-                flag="--job" if using_job_op_graph_apis else "--pipeline",
-                repository=external_repo.name,
-                pipelines=_sorted_quoted(external_pipelines.keys()),
+                "Must provide --job as there is more than one job "
+                f"in {external_repo.name}. Options are: {_sorted_quoted(external_pipelines.keys())}."
             )
         )
 
-    if not provided_pipeline_or_job_name in external_pipelines:
+    if not provided_name in external_pipelines:
         raise click.UsageError(
             (
-                '{pipeline_or_job} "{provided_pipeline_name}" not found in repository "{repository_name}". '
-                "Found {found_names} instead."
-            ).format(
-                pipeline_or_job="Job" if using_job_op_graph_apis else "Pipeline",
-                provided_pipeline_name=provided_pipeline_or_job_name,
-                repository_name=external_repo.name,
-                found_names=_sorted_quoted(external_pipelines.keys()),
+                f'Job "{provided_name}" not found in repository "{external_repo.name}". '
+                f"Found {_sorted_quoted(external_pipelines.keys())} instead."
             )
         )
 
-    return external_pipelines[provided_pipeline_or_job_name]
+    return external_pipelines[provided_name]
 
 
 @contextmanager
-def get_external_pipeline_or_job_from_kwargs(
-    instance, version, kwargs, using_job_op_graph_apis=False
-):
+def get_external_job_from_kwargs(instance, version, kwargs):
     # Instance isn't strictly required to load an ExternalPipeline, but is included
     # to satisfy the WorkspaceProcessContext / WorkspaceRequestContext requirements
     with get_external_repository_from_kwargs(instance, version, kwargs) as external_repo:
-        provided_pipeline_or_job_name = kwargs.get("pipeline_or_job")
-        yield get_external_pipeline_or_job_from_external_repo(
-            external_repo, provided_pipeline_or_job_name, using_job_op_graph_apis
-        )
+        provided_name = kwargs.get("job_name")
+        yield get_external_job_from_external_repo(external_repo, provided_name)
 
 
 def _sorted_quoted(strings):
