@@ -1,6 +1,6 @@
 import typing
 from enum import Enum as PythonEnum
-from typing import Dict, List, Optional, cast
+from typing import TYPE_CHECKING, Dict, Iterator, Optional, Sequence, cast
 
 import dagster._check as check
 from dagster._annotations import public
@@ -8,6 +8,9 @@ from dagster._builtins import BuiltinEnum
 from dagster._config import UserConfigSchema
 from dagster._serdes import whitelist_for_serdes
 
+if TYPE_CHECKING:
+    from .snap import ConfigSchemaSnapshot, ConfigTypeSnap
+
 
 @whitelist_for_serdes
 class ConfigTypeKind(PythonEnum):
@@ -63,19 +66,22 @@ class ConfigType:
         kind: ConfigTypeKind,
         given_name: Optional[str] = None,
         description: Optional[str] = None,
-        type_params: Optional[List["ConfigType"]] = None,
+        type_params: Optional[Sequence["ConfigType"]] = None,
     ):
 
         self.key: str = check.str_param(key, "key")
         self.kind: ConfigTypeKind = check.inst_param(kind, "kind", ConfigTypeKind)
         self.given_name: Optional[str] = check.opt_str_param(given_name, "given_name")
         self._description: Optional[str] = check.opt_str_param(description, "description")
-        self.type_params: Optional[List[ConfigType]] = (
-            check.list_param(type_params, "type_params", of_type=ConfigType)
+        self.type_params: Optional[Sequence[ConfigType]] = (
+            check.sequence_param(type_params, "type_params", of_type=ConfigType)
             if type_params
             else None
         )
 
+        # memoized snap representation
+        self._snap: Optional["ConfigTypeSnap"] = None
+
     @property
     def description(self) -> Optional[str]:
         return self._description
@@ -95,6 +101,22 @@ class ConfigType:
         """
         return value
 
+    def get_snapshot(self) -> "ConfigTypeSnap":
+        from .snap import snap_from_config_type
+
+        if self._snap is None:
+            self._snap = snap_from_config_type(self)
+
+        return self._snap
+
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        yield self
+
+    def get_schema_snapshot(self) -> "ConfigSchemaSnapshot":
+        from .snap import ConfigSchemaSnapshot
+
+        return ConfigSchemaSnapshot({ct.key: ct.get_snapshot() for ct in self.type_iterator()})
+
 
 @whitelist_for_serdes
 class ConfigScalarKind(PythonEnum):
@@ -187,6 +209,10 @@ class Noneable(ConfigType):
             type_params=[self.inner_type],
         )
 
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        yield from self.inner_type.type_iterator()
+        yield from super().type_iterator()
+
 
 class Array(ConfigType):
     """Defines an array (list) configuration type that contains values of type ``inner_type``.
@@ -196,10 +222,10 @@ class Array(ConfigType):
             The type of the values that this configuration type can contain.
     """
 
-    def __init__(self, inner_type):
+    def __init__(self, inner_type: object):
         from .field import resolve_to_config_type
 
-        self.inner_type = resolve_to_config_type(inner_type)
+        self.inner_type = cast(ConfigType, resolve_to_config_type(inner_type))
         super(Array, self).__init__(
             key="Array.{inner_type}".format(inner_type=self.inner_type.key),
             type_params=[self.inner_type],
@@ -211,6 +237,10 @@ class Array(ConfigType):
     def description(self):
         return "List of {inner_type}".format(inner_type=self.key)
 
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        yield from self.inner_type.type_iterator()
+        yield from super().type_iterator()
+
 
 class EnumValue:
     """Define an entry in a :py:class:`Enum`.
@@ -265,10 +295,10 @@ class Enum(ConfigType):
             # ...
     """
 
-    def __init__(self, name: str, enum_values: List[EnumValue]):
+    def __init__(self, name: str, enum_values: Sequence[EnumValue]):
         check.str_param(name, "name")
         super(Enum, self).__init__(key=name, given_name=name, kind=ConfigTypeKind.ENUM)
-        self.enum_values = check.list_param(enum_values, "enum_values", of_type=EnumValue)
+        self.enum_values = check.sequence_param(enum_values, "enum_values", of_type=EnumValue)
         self._valid_python_values = {ev.python_value for ev in enum_values}
         check.invariant(len(self._valid_python_values) == len(enum_values))
         self._valid_config_values = {ev.config_value for ev in enum_values}
@@ -398,6 +428,11 @@ class ScalarUnion(ConfigType):
             type_params=[self.scalar_type, self.non_scalar_type],
         )
 
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        yield from self.scalar_type.type_iterator()
+        yield from self.non_scalar_type.type_iterator()
+        yield from super().type_iterator()
+
 
 ConfigAnyInstance = Any()
 ConfigBoolInstance = Bool()
