@@ -1,6 +1,6 @@
 # encoding: utf-8
 import hashlib
-from typing import TYPE_CHECKING, Any, Dict, List
+from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Mapping, Sequence
 
 import dagster._check as check
 from dagster._annotations import public
@@ -11,6 +11,8 @@ from .config_type import Array, ConfigType, ConfigTypeKind
 if TYPE_CHECKING:
     from dagster._config import Field
 
+    from .snap import ConfigTypeSnap
+
 
 def all_optional_type(config_type: ConfigType) -> bool:
     check.inst_param(config_type, "config_type", ConfigType)
@@ -49,6 +51,11 @@ class _ConfigHasFields(ConfigType):
         self.fields = expand_fields_dict(fields)
         super(_ConfigHasFields, self).__init__(**kwargs)
 
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        for field in self.fields.values():
+            yield from field.config_type.type_iterator()
+        yield from super().type_iterator()
+
 
 FIELD_HASH_CACHE: Dict[str, Any] = {}
 
@@ -58,7 +65,7 @@ def _memoize_inst_in_field_cache(passed_cls, defined_cls, key):
         return FIELD_HASH_CACHE[key]
 
     defined_cls_inst = super(defined_cls, passed_cls).__new__(defined_cls)
-
+    defined_cls_inst._initialized = False  # pylint: disable=protected-access
     FIELD_HASH_CACHE[key] = defined_cls_inst
     return defined_cls_inst
 
@@ -131,6 +138,10 @@ class Shape(_ConfigHasFields):
         description=None,
         field_aliases=None,
     ):
+        # if we hit in the field cache - skip double init
+        if self._initialized:  # pylint: disable=access-member-before-definition
+            return
+
         fields = expand_fields_dict(fields)
         super(Shape, self).__init__(
             kind=ConfigTypeKind.STRICT_SHAPE,
@@ -141,6 +152,7 @@ class Shape(_ConfigHasFields):
         self.field_aliases = check.opt_dict_param(
             field_aliases, "field_aliases", key_type=str, value_type=str
         )
+        self._initialized = True
 
 
 class Map(ConfigType):
@@ -199,6 +211,11 @@ class Map(ConfigType):
     def key_label_name(self):
         return self.given_name
 
+    def type_iterator(self) -> Iterator["ConfigType"]:
+        yield from self.key_type.type_iterator()
+        yield from self.inner_type.type_iterator()
+        yield from super().type_iterator()
+
 
 def _define_permissive_dict_key(fields, description):
     return (
@@ -237,6 +254,10 @@ class Permissive(_ConfigHasFields):
         )
 
     def __init__(self, fields=None, description=None):
+        # if we hit in field cache avoid double init
+        if self._initialized:  # pylint: disable=access-member-before-definition
+            return
+
         fields = expand_fields_dict(fields) if fields else None
         super(Permissive, self).__init__(
             key=_define_permissive_dict_key(fields, description),
@@ -244,6 +265,7 @@ class Permissive(_ConfigHasFields):
             fields=fields or dict(),
             description=description,
         )
+        self._initialized = True
 
 
 def _define_selector_key(fields, description):
@@ -299,6 +321,10 @@ class Selector(_ConfigHasFields):
         )
 
     def __init__(self, fields, description=None):
+        # if we hit in field cache avoid double init
+        if self._initialized:  # pylint: disable=access-member-before-definition
+            return
+
         fields = expand_fields_dict(fields)
         super(Selector, self).__init__(
             key=_define_selector_key(fields, description),
@@ -306,6 +332,7 @@ class Selector(_ConfigHasFields):
             fields=fields,
             description=description,
         )
+        self._initialized = True
 
 
 # Config syntax expansion code below
@@ -319,31 +346,31 @@ def is_potential_field(potential_field: object) -> bool:
     )
 
 
-def convert_fields_to_dict_type(fields: Dict[str, object]):
+def convert_fields_to_dict_type(fields: Mapping[str, object]):
     return _convert_fields_to_dict_type(fields, fields, [])
 
 
 def _convert_fields_to_dict_type(
-    original_root: object, fields: Dict[str, object], stack: List[str]
+    original_root: object, fields: Mapping[str, object], stack: List[str]
 ) -> Shape:
     return Shape(_expand_fields_dict(original_root, fields, stack))
 
 
-def expand_fields_dict(fields: Dict[str, object]) -> Dict[str, "Field"]:
+def expand_fields_dict(fields: Mapping[str, object]) -> Mapping[str, "Field"]:
     return _expand_fields_dict(fields, fields, [])
 
 
 def _expand_fields_dict(
-    original_root: object, fields: Dict[str, object], stack: List[str]
-) -> Dict[str, "Field"]:
-    check.dict_param(fields, "fields")
+    original_root: object, fields: Mapping[str, object], stack: List[str]
+) -> Mapping[str, "Field"]:
+    check.mapping_param(fields, "fields")
     return {
         name: _convert_potential_field(original_root, value, stack + [name])
         for name, value in fields.items()
     }
 
 
-def expand_list(original_root: object, the_list: List[object], stack: List[str]) -> Array:
+def expand_list(original_root: object, the_list: Sequence[object], stack: List[str]) -> Array:
 
     if len(the_list) != 1:
         raise DagsterInvalidConfigDefinitionError(
@@ -364,7 +391,7 @@ def expand_list(original_root: object, the_list: List[object], stack: List[str])
     return Array(inner_type)
 
 
-def expand_map(original_root: object, the_dict: Dict[object, object], stack: List[str]) -> Map:
+def expand_map(original_root: object, the_dict: Mapping[object, object], stack: List[str]) -> Map:
 
     if len(the_dict) != 1:
         raise DagsterInvalidConfigDefinitionError(
@@ -402,7 +429,7 @@ def convert_potential_field(potential_field: object) -> "Field":
 def _convert_potential_type(original_root: object, potential_type, stack: List[str]):
     from .field import resolve_to_config_type
 
-    if isinstance(potential_type, dict):
+    if isinstance(potential_type, Mapping):
         # A dictionary, containing a single key which is a type (int, str, etc) and not a string is interpreted as a Map
         if len(potential_type) == 1:
             key = list(potential_type.keys())[0]
