@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional, cast
+from typing import Any, Dict, Mapping, Optional, cast
 
 from dagster import Field, Permissive
 from dagster._config.validate import process_config
@@ -19,8 +19,8 @@ SHARED_CONTAINER_CONTEXT_SCHEMA = Permissive(
 
 # Process fields shared by container contexts in all environments (docker / k8s / ecs etc.)
 def process_shared_container_context_config(
-    container_context_config: Optional[Dict[str, Any]]
-) -> Dict[str, Any]:
+    container_context_config: Optional[Mapping[str, Any]]
+) -> Mapping[str, Any]:
     shared_container_context = process_config(
         SHARED_CONTAINER_CONTEXT_SCHEMA, container_context_config or {}
     )
