@@ -1,11 +1,11 @@
 import warnings
+from collections import defaultdict
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
     Any,
     Callable,
     Dict,
-    FrozenSet,
     Iterable,
     List,
     Mapping,
@@ -21,6 +21,7 @@ from typing import (
 import dagster._check as check
 from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.metadata import MetadataUserInput, RawMetadataValue
+from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.selector.subset_selector import AssetSelectionData
 from dagster._utils.backcompat import ExperimentalWarning
 
@@ -91,7 +92,7 @@ def _resolve_input_to_destinations(
         return [NodeInputHandle(node_handle=handle, input_name=name)]
     all_destinations: List[NodeInputHandle] = []
     for mapping in node_def.input_mappings:
-        if mapping.definition.name != name:
+        if mapping.graph_input_name != name:
             continue
         # recurse into graph structure
         all_destinations += _resolve_input_to_destinations(
@@ -110,7 +111,7 @@ def _resolve_output_to_destinations(output_name, node_def, handle) -> Sequence[N
         return node_input_handles
 
     for mapping in node_def.output_mappings:
-        if mapping.definition.name != output_name:
+        if mapping.graph_output_name != output_name:
             continue
         output_pointer = mapping.maps_from
         output_node = node_def.solid_named(output_pointer.solid_name)
@@ -158,6 +159,7 @@ def _build_graph_dependencies(
         that are not assets. Each key is a node output handle.
     """
     dep_struct = graph_def.dependency_structure
+
     for sub_node_name, sub_node in graph_def.node_dict.items():
         curr_node_handle = NodeHandle(sub_node_name, parent=parent_handle)
         if isinstance(sub_node.definition, GraphDefinition):
@@ -169,7 +171,7 @@ def _build_graph_dependencies(
                 assets_defs_by_node_handle,
             )
             outputs_by_graph_handle[curr_node_handle] = {
-                mapping.definition.name: NodeOutputHandle(
+                mapping.graph_output_name: NodeOutputHandle(
                     NodeHandle(mapping.maps_from.solid_name, parent=curr_node_handle),
                     mapping.maps_from.output_name,
                 )
@@ -186,15 +188,17 @@ def _build_graph_dependencies(
         ]
 
 
-def _get_dependency_node_handles(
+def _get_dependency_node_output_handles(
     non_asset_inputs_by_node_handle: Mapping[NodeHandle, Sequence[NodeOutputHandle]],
     outputs_by_graph_handle: Mapping[NodeHandle, Mapping[str, NodeOutputHandle]],
-    dep_node_handles_by_node: Dict[NodeHandle, List[NodeHandle]],
+    dep_node_output_handles_by_node_output_handle: Dict[
+        NodeOutputHandle, Sequence[NodeOutputHandle]
+    ],
     node_output_handle: NodeOutputHandle,
-) -> Sequence[NodeHandle]:
+) -> Sequence[NodeOutputHandle]:
     """
-    Given a node handle and an optional output name of the node (if the node is a graph), return
-    all upstream op node handles (leaf nodes).
+    Given a node output handle, return all upstream op node output handles. All node output handles
+    belong in the same graph-backed asset node.
 
     Arguments:
 
@@ -202,56 +206,93 @@ def _get_dependency_node_handles(
         name as a key and a NodeOutputHandle containing the op output name and op node handle
     non_asset_inputs_by_node_handle: A mapping of all node handles to all upstream node handles
         that are not assets. Each key is a node output handle.
-    dep_node_handles_by_node: A mapping of each non-graph node to all non-graph node dependencies.
-        Used for memoization to avoid scanning already visited nodes.
+    dep_node_output_handles_by_node_output_handle: A mapping of each non-graph node output handle
+        to all non-graph node output handle dependencies. Used for memoization to avoid scanning
+        already visited nodes.
     curr_node_handle: The current node handle being traversed.
     graph_output_name: Name of the node output being traversed. Only used if the current node is a
         graph to trace the op that generates this output.
     """
     curr_node_handle = node_output_handle.node_handle
 
-    if curr_node_handle in dep_node_handles_by_node:
-        return dep_node_handles_by_node[curr_node_handle]
+    if node_output_handle in dep_node_output_handles_by_node_output_handle:
+        return dep_node_output_handles_by_node_output_handle[node_output_handle]
+
+    dependency_node_output_handles: List[
+        NodeOutputHandle
+    ] = []  # first node in list is node output handle that outputs the asset
 
-    dependency_node_handles: List[
-        NodeHandle
-    ] = []  # first node in list is node that outputs the asset
     if curr_node_handle not in outputs_by_graph_handle:
-        dependency_node_handles.append(curr_node_handle)
+        dependency_node_output_handles.append(node_output_handle)
     else:  # is graph
-        node_output_handle = outputs_by_graph_handle[curr_node_handle][
+        dep_node_output_handle = outputs_by_graph_handle[curr_node_handle][
             node_output_handle.output_name
         ]
-        dependency_node_handles.extend(
-            _get_dependency_node_handles(
+        dependency_node_output_handles.extend(
+            _get_dependency_node_output_handles(
                 non_asset_inputs_by_node_handle,
                 outputs_by_graph_handle,
-                dep_node_handles_by_node,
-                node_output_handle,
+                dep_node_output_handles_by_node_output_handle,
+                dep_node_output_handle,
             )
         )
-    for node_output_handle in non_asset_inputs_by_node_handle[curr_node_handle]:
-        dependency_node_handles.extend(
-            _get_dependency_node_handles(
+    for dep_node_output_handle in non_asset_inputs_by_node_handle[curr_node_handle]:
+        dependency_node_output_handles.extend(
+            _get_dependency_node_output_handles(
                 non_asset_inputs_by_node_handle,
                 outputs_by_graph_handle,
-                dep_node_handles_by_node,
-                node_output_handle,
+                dep_node_output_handles_by_node_output_handle,
+                dep_node_output_handle,
             )
         )
 
     if curr_node_handle not in outputs_by_graph_handle:
-        dep_node_handles_by_node[curr_node_handle] = dependency_node_handles
+        dep_node_output_handles_by_node_output_handle[
+            node_output_handle
+        ] = dependency_node_output_handles
 
-    return dependency_node_handles
+    return dependency_node_output_handles
 
 
-def _asset_key_to_dep_node_handles(
-    graph_def: GraphDefinition, assets_defs_by_node_handle: Mapping[NodeHandle, "AssetsDefinition"]
-) -> Mapping[AssetKey, Set[NodeHandle]]:
+def get_dep_node_handles_of_graph_backed_asset(
+    graph_def: GraphDefinition, assets_def: "AssetsDefinition"
+):
     """
-    For each asset in assets_defs_by_node_handle, returns all the op handles within the asset's node
-    that are upstream dependencies of the asset.
+    Given a graph-backed asset with graph_def, return a mapping of asset keys outputted by the graph
+    to a list of node handles within graph_def that are the dependencies of the asset.
+
+    Arguments:
+    graph_def: The graph definition of the graph-backed asset.
+    assets_def: The assets definition of the graph-backed asset.
+
+    """
+    # asset_key_to_dep_node_handles takes in a graph_def that represents the entire job, where each
+    # node is a top-level asset node. Create a dummy graph that wraps around graph_def and pass
+    # the dummy graph to asset_key_to_dep_node_handles
+    dummy_parent_graph = GraphDefinition("dummy_parent_graph", node_defs=[graph_def])
+    dep_node_handles_by_asset_key, _ = asset_key_to_dep_node_handles(
+        dummy_parent_graph,
+        {NodeHandle(name=graph_def.name, parent=None): assets_def},
+    )
+    return dep_node_handles_by_asset_key
+
+
+def asset_key_to_dep_node_handles(
+    graph_def: GraphDefinition,
+    assets_defs_by_node_handle: Mapping[NodeHandle, "AssetsDefinition"],
+) -> Tuple[Mapping[AssetKey, Set[NodeHandle]], Mapping[AssetKey, Sequence[NodeOutputHandle]]]:
+    """
+    For each asset in assets_defs_by_node_handle, determines all the op handles and output handles
+    within the asset's node that are upstream dependencies of the asset.
+
+    Returns a tuple with two objects:
+    1. A mapping of each asset key to a set of node handles that are upstream dependencies of the asset.
+    2. A mapping of each asset key to a list of node output handles that are upstream dependencies of the asset.
+
+    Arguments:
+
+    graph_def: The graph definition of the job, where each top level node is an asset.
+    assets_defs_by_node_handle: A mapping of each node handle to the asset definition for that node.
     """
     # A mapping of all node handles to all upstream node handles
     # that are not assets. Each key is a node handle with node output handle value
@@ -269,11 +310,14 @@ def _asset_key_to_dep_node_handles(
     )
 
     dep_nodes_by_asset_key: Dict[AssetKey, List[NodeHandle]] = {}
+    dep_node_outputs_by_asset_key: Dict[AssetKey, List[NodeOutputHandle]] = {}
 
     for node_handle, assets_defs in assets_defs_by_node_handle.items():
-        dep_node_handles_by_node: Dict[
-            NodeHandle, List[NodeHandle]
-        ] = {}  # memoized map of nodehandle to all node handle dependencies that are ops
+        dep_node_output_handles_by_node: Dict[
+            NodeOutputHandle, Sequence[NodeOutputHandle]
+        ] = (
+            {}
+        )  # memoized map of node output handles to all node output handle dependencies that are from ops
         for output_name, asset_key in assets_defs.keys_by_output_name.items():
             output_def = assets_defs.node_def.output_def_named(output_name)
             output_name = output_def.name
@@ -281,41 +325,59 @@ def _asset_key_to_dep_node_handles(
             dep_nodes_by_asset_key[
                 asset_key
             ] = []  # first element in list is node that outputs asset
+
+            dep_node_outputs_by_asset_key[asset_key] = []
+
             if node_handle not in outputs_by_graph_handle:
                 dep_nodes_by_asset_key[asset_key].extend([node_handle])
             else:  # is graph
+                # node output handle for the given asset key
                 node_output_handle = outputs_by_graph_handle[node_handle][output_name]
-                dep_nodes_by_asset_key[asset_key].extend(
-                    _get_dependency_node_handles(
-                        non_asset_inputs_by_node_handle,
-                        outputs_by_graph_handle,
-                        dep_node_handles_by_node,
-                        node_output_handle,
-                    )
+
+                dep_node_output_handles = _get_dependency_node_output_handles(
+                    non_asset_inputs_by_node_handle,
+                    outputs_by_graph_handle,
+                    dep_node_output_handles_by_node,
+                    node_output_handle,
                 )
 
-    # handle internal_asset_deps
-    for node_handle, assets_def in assets_defs_by_node_handle.items():
+                dep_node_outputs_by_asset_key[asset_key].extend(dep_node_output_handles)
+
+    # handle internal_asset_deps within graph-backed assets
+    for _, assets_def in assets_defs_by_node_handle.items():
         for asset_key, dep_asset_keys in assets_def.asset_deps.items():
             if asset_key not in assets_def.keys:
                 continue
             for dep_asset_key in [key for key in dep_asset_keys if key in assets_def.keys]:
-                output_node = dep_nodes_by_asset_key[asset_key][
+                if len(dep_node_outputs_by_asset_key[asset_key]) == 0:
+                    # This case occurs when the asset is not yielded from a graph-backed asset
+                    continue
+                node_output_handle = dep_node_outputs_by_asset_key[asset_key][
                     0
-                ]  # first item in list is the original node that output the asset
-                dep_asset_key_node_handles = [
-                    node for node in dep_nodes_by_asset_key[dep_asset_key] if node != output_node
+                ]  # first item in list is the original node output handle that outputs the asset
+                dep_asset_key_node_output_handles = [
+                    output_handle
+                    for output_handle in dep_node_outputs_by_asset_key[dep_asset_key]
+                    if output_handle != node_output_handle
                 ]
-                dep_nodes_by_asset_key[asset_key] = [
-                    node
-                    for node in dep_nodes_by_asset_key[asset_key]
-                    if node not in dep_asset_key_node_handles
+                dep_node_outputs_by_asset_key[asset_key] = [
+                    node_output
+                    for node_output in dep_node_outputs_by_asset_key[asset_key]
+                    if node_output not in dep_asset_key_node_output_handles
                 ]
 
+    # For graph-backed assets, we've resolved the upstream node output handles dependencies for each
+    # node output handle in dep_node_outputs_by_asset_key. We use this to find the upstream
+    # node handle dependencies.
+    for asset_key, dep_node_outputs in dep_node_outputs_by_asset_key.items():
+        dep_nodes_by_asset_key[asset_key].extend(
+            [node_output.node_handle for node_output in dep_node_outputs]
+        )
+
     dep_node_set_by_asset_key: Dict[AssetKey, Set[NodeHandle]] = {}
     for asset_key, dep_node_handles in dep_nodes_by_asset_key.items():
         dep_node_set_by_asset_key[asset_key] = set(dep_node_handles)
-    return dep_node_set_by_asset_key
+    return dep_node_set_by_asset_key, dep_node_outputs_by_asset_key
 
 
 def _asset_mappings_for_node(
@@ -419,10 +481,13 @@ class AssetLayer:
         assets_defs: Optional[Sequence["AssetsDefinition"]] = None,
         source_asset_defs: Optional[Sequence["SourceAsset"]] = None,
         io_manager_keys_by_asset_key: Optional[Mapping[AssetKey, str]] = None,
+        node_output_handles_to_dep_asset_keys: Optional[
+            Mapping[NodeOutputHandle, Set[AssetKey]]
+        ] = None,
     ):
         from dagster._core.definitions import SourceAsset
 
-        self._asset_keys_by_node_input_handle = check.opt_dict_param(
+        self._asset_keys_by_node_input_handle = check.opt_mapping_param(
             asset_keys_by_node_input_handle,
             "asset_keys_by_node_input_handle",
             key_type=NodeInputHandle,
@@ -434,10 +499,10 @@ class AssetLayer:
             key_type=NodeOutputHandle,
             value_type=AssetOutputInfo,
         )
-        self._asset_deps = check.opt_dict_param(
+        self._asset_deps = check.opt_mapping_param(
             asset_deps, "asset_deps", key_type=AssetKey, value_type=set
         )
-        self._dependency_node_handles_by_asset_key = check.opt_dict_param(
+        self._dependency_node_handles_by_asset_key = check.opt_mapping_param(
             dependency_node_handles_by_asset_key,
             "dependency_node_handles_by_asset_key",
             key_type=AssetKey,
@@ -445,13 +510,13 @@ class AssetLayer:
         )
         self._source_assets_by_key = {
             source_asset.key: source_asset
-            for source_asset in check.opt_list_param(
+            for source_asset in check.opt_sequence_param(
                 source_asset_defs, "source_assets_defs", of_type=SourceAsset
             )
         }
         self._assets_defs_by_key = {
             key: assets_def
-            for assets_def in check.opt_list_param(assets_defs, "assets_defs")
+            for assets_def in check.opt_sequence_param(assets_defs, "assets_defs")
             for key in assets_def.keys
         }
 
@@ -461,13 +526,22 @@ class AssetLayer:
             for node_handle in node_handles:
                 self._assets_defs_by_node_handle[node_handle] = self._assets_defs_by_key[asset_key]
 
-        self._io_manager_keys_by_asset_key = check.opt_dict_param(
+        self._io_manager_keys_by_asset_key = check.opt_mapping_param(
             io_manager_keys_by_asset_key,
             "io_manager_keys_by_asset_key",
             key_type=AssetKey,
             value_type=str,
         )
 
+        # Used to store the asset key dependencies of op node handles within graph backed assets
+        # See AssetLayer.downstream_dep_assets for more information
+        self._node_output_handle_to_dep_asset_keys = check.opt_dict_param(
+            node_output_handles_to_dep_asset_keys,
+            "node_output_handles_to_dep_asset_keys",
+            key_type=NodeOutputHandle,
+            value_type=set,
+        )
+
     @staticmethod
     def from_graph(graph_def: GraphDefinition) -> "AssetLayer":
         """Scrape asset info off of InputDefinition/OutputDefinition instances"""
@@ -500,16 +574,21 @@ class AssetLayer:
                 a NodeHandle pointing to the node in the graph where the AssetsDefinition ended up.
         """
         check.inst_param(graph_def, "graph_def", GraphDefinition)
-        check.dict_param(
+        check.mapping_param(
             assets_defs_by_node_handle, "assets_defs_by_node_handle", key_type=NodeHandle
         )
-
         asset_key_by_input: Dict[NodeInputHandle, AssetKey] = {}
         asset_info_by_output: Dict[NodeOutputHandle, AssetOutputInfo] = {}
         asset_deps: Dict[AssetKey, AbstractSet[AssetKey]] = {}
         io_manager_by_asset: Dict[AssetKey, str] = {
             source_asset.key: source_asset.get_io_manager_key() for source_asset in source_assets
         }
+
+        (
+            dep_node_handles_by_asset_key,
+            dep_node_output_handles_by_asset_key,
+        ) = asset_key_to_dep_node_handles(graph_def, assets_defs_by_node_handle)
+
         for node_handle, assets_def in assets_defs_by_node_handle.items():
             for key in assets_def.keys:
                 asset_deps[key] = resolved_asset_deps.get_resolved_upstream_asset_keys(
@@ -536,10 +615,22 @@ class AssetLayer:
                 node_output_handle = NodeOutputHandle(
                     check.not_none(inner_node_handle), inner_output_def.name
                 )
-                partition_fn = lambda context: {context.partition_key}
+
+                def partitions_fn(context: "OutputContext") -> AbstractSet[str]:
+                    from dagster._core.definitions.partition import PartitionsDefinition
+
+                    if context.has_partition_key:
+                        return {context.partition_key}
+
+                    return set(
+                        cast(
+                            PartitionsDefinition, context.asset_partitions_def
+                        ).get_partition_keys_in_range(context.asset_partition_key_range)
+                    )
+
                 asset_info_by_output[node_output_handle] = AssetOutputInfo(
                     asset_key,
-                    partitions_fn=partition_fn if assets_def.partitions_def else None,
+                    partitions_fn=partitions_fn if assets_def.partitions_def else None,
                     partitions_def=assets_def.partitions_def,
                     is_required=asset_key in assets_def.keys,
                 )
@@ -554,16 +645,20 @@ class AssetLayer:
                     }
                 )
 
+        node_output_handles_to_dep_asset_keys = defaultdict(set)
+        for asset_key, node_output_handles in dep_node_output_handles_by_asset_key.items():
+            for node_output_handle in node_output_handles:
+                node_output_handles_to_dep_asset_keys[node_output_handle].add(asset_key)
+
         return AssetLayer(
             asset_keys_by_node_input_handle=asset_key_by_input,
             asset_info_by_node_output_handle=asset_info_by_output,
             asset_deps=asset_deps,
-            dependency_node_handles_by_asset_key=_asset_key_to_dep_node_handles(
-                graph_def, assets_defs_by_node_handle
-            ),
+            dependency_node_handles_by_asset_key=dep_node_handles_by_asset_key,
             assets_defs=[assets_def for assets_def in assets_defs_by_node_handle.values()],
             source_asset_defs=source_assets,
             io_manager_keys_by_asset_key=io_manager_by_asset,
+            node_output_handles_to_dep_asset_keys=node_output_handles_to_dep_asset_keys,
         )
 
     @property
@@ -613,6 +708,26 @@ class AssetLayer:
     def io_manager_key_for_asset(self, asset_key: AssetKey) -> str:
         return self._io_manager_keys_by_asset_key.get(asset_key, "io_manager")
 
+    def is_source_for_asset(self, asset_key: AssetKey) -> bool:
+        return asset_key in self.source_assets_by_key
+
+    def is_observable_for_asset(self, asset_key: AssetKey) -> bool:
+        return (
+            asset_key in self.source_assets_by_key
+            and self.source_assets_by_key[asset_key].is_observable
+        )
+
+    def is_graph_backed_asset(self, asset_key: AssetKey) -> bool:
+        assets_def = self.assets_defs_by_key.get(asset_key)
+        return False if assets_def is None else isinstance(assets_def.node_def, GraphDefinition)
+
+    def op_version_for_asset(self, asset_key: AssetKey) -> Optional[str]:
+        assets_def = self.assets_defs_by_key.get(asset_key)
+        if assets_def is not None and isinstance(assets_def.node_def, OpDefinition):
+            return assets_def.node_def.version
+        else:
+            return None
+
     def metadata_for_asset(self, asset_key: AssetKey) -> Optional[MetadataUserInput]:
         if asset_key in self._source_assets_by_key:
             metadata = self._source_assets_by_key[asset_key].metadata
@@ -661,6 +776,41 @@ class AssetLayer:
 
         return None
 
+    def downstream_dep_assets(self, node_handle: NodeHandle, output_name: str) -> Set[AssetKey]:
+        """
+        Given the node handle of an op within a graph-backed asset and an output name,
+        returns the asset keys dependent on that output.
+
+        For example, for the following asset:
+
+        @op(out={"out_1": Out(is_required=False), "out_2": Out(is_required=False)})
+        def two_outputs_op(context):
+            return 1, 1
+
+
+        @op
+        def add_one(x):
+            return x + 1
+
+
+        @graph(out={"asset_one": GraphOut(), "asset_two": GraphOut()})
+        def my_graph():
+            out_1, out_2 = two_outputs_op()
+            return {"asset_one": out_1, "asset_two": add_one(out_2)}
+
+        two_assets = AssetsDefinition.from_graph(my_graph)
+
+        Calling downstream_dep_assets with the node handle of two_outputs_op will return:
+        - {AssetKey("asset_one")} if output_name="out_1"
+        - {AssetKey("asset_two")} if output_name="out_2"
+
+        Calling downstream_dep_assets with node handle add_one will return:
+        - {AssetKey("asset_two")} if output_name="result"
+        """
+        return self._node_output_handle_to_dep_asset_keys.get(
+            NodeOutputHandle(node_handle, output_name), set()
+        )
+
 
 def build_asset_selection_job(
     name: str,
@@ -672,18 +822,22 @@ def build_asset_selection_job(
     resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
     description: Optional[str] = None,
     tags: Optional[Mapping[str, Any]] = None,
-    asset_selection: Optional[FrozenSet[AssetKey]] = None,
+    asset_selection: Optional[AbstractSet[AssetKey]] = None,
     asset_selection_data: Optional[AssetSelectionData] = None,
 ) -> "JobDefinition":
-    from dagster._core.definitions import build_assets_job
+    from dagster._core.definitions.assets_job import (
+        build_assets_job,
+        build_source_asset_observation_job,
+    )
 
     if asset_selection:
-        included_assets, excluded_assets = _subset_assets_defs(
-            assets, source_assets, asset_selection
-        )
+        (included_assets, excluded_assets) = _subset_assets_defs(assets, asset_selection)
+        included_source_assets = _subset_source_assets(source_assets, asset_selection)
+
     else:
-        included_assets = cast(Iterable["AssetsDefinition"], assets)
-        excluded_assets = list(source_assets)
+        included_assets = list(assets)
+        excluded_assets = []
+        included_source_assets = []
 
     if partitions_def:
         for asset in included_assets:
@@ -694,29 +848,44 @@ def build_asset_selection_job(
                 f"{partitions_def}.",
             )
 
+    # We should disallow simultaneous selection of assets and source assets (but for
+    # backcompat we will simply ignore the source asset selection if any regular assets are
+    # selected).
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", category=ExperimentalWarning)
-        asset_job = build_assets_job(
-            name=name,
-            assets=included_assets,
-            config=config,
-            source_assets=excluded_assets,
-            resource_defs=resource_defs,
-            executor_def=executor_def,
-            partitions_def=partitions_def,
-            description=description,
-            tags=tags,
-            _asset_selection_data=asset_selection_data,
-        )
+        if len(included_assets) > 0:
+            asset_job = build_assets_job(
+                name=name,
+                assets=included_assets,
+                config=config,
+                source_assets=[*source_assets, *excluded_assets],
+                resource_defs=resource_defs,
+                executor_def=executor_def,
+                partitions_def=partitions_def,
+                description=description,
+                tags=tags,
+                _asset_selection_data=asset_selection_data,
+            )
+        else:
+            asset_job = build_source_asset_observation_job(
+                name=name,
+                source_assets=included_source_assets,
+                config=config,
+                resource_defs=resource_defs,
+                executor_def=executor_def,
+                partitions_def=partitions_def,
+                description=description,
+                tags=tags,
+                _asset_selection_data=asset_selection_data,
+            )
 
     return asset_job
 
 
 def _subset_assets_defs(
     assets: Iterable["AssetsDefinition"],
-    source_assets: Iterable["SourceAsset"],
     selected_asset_keys: AbstractSet[AssetKey],
-) -> Tuple[Iterable["AssetsDefinition"], Sequence[Union["AssetsDefinition", "SourceAsset"]]]:
+) -> Tuple[Sequence["AssetsDefinition"], Sequence["AssetsDefinition"],]:
     """Given a list of asset key selection queries, generate a set of AssetsDefinition objects
     representing the included/excluded definitions.
     """
@@ -725,12 +894,9 @@ def _subset_assets_defs(
     included_assets: Set[AssetsDefinition] = set()
     excluded_assets: Set[AssetsDefinition] = set()
 
-    included_keys: Set[AssetKey] = set()
-
     for asset in set(assets):
         # intersection
         selected_subset = selected_asset_keys & asset.keys
-        included_keys.update(selected_subset)
         # all assets in this def are selected
         if selected_subset == asset.keys:
             included_assets.add(asset)
@@ -752,9 +918,16 @@ def _subset_assets_defs(
                 "asset keys produced by this asset."
             )
 
-    all_excluded_assets: Sequence[Union["AssetsDefinition", "SourceAsset"]] = [
-        *excluded_assets,
-        *source_assets,
-    ]
+    return (
+        list(included_assets),
+        list(excluded_assets),
+    )
 
-    return list(included_assets), all_excluded_assets
+
+def _subset_source_assets(
+    source_assets: Iterable["SourceAsset"],
+    selected_asset_keys: AbstractSet[AssetKey],
+) -> Sequence["SourceAsset"]:
+    return [
+        source_asset for source_asset in source_assets if source_asset.key in selected_asset_keys
+    ]
