@@ -16,7 +16,9 @@ from typing import (
 import dagster._check as check
 from dagster._annotations import public
 from dagster._core.decorator_utils import get_function_params
+from dagster._core.definitions.asset_layer import get_dep_node_handles_of_graph_backed_asset
 from dagster._core.definitions.events import AssetKey
+from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.definitions.metadata import MetadataUserInput
 from dagster._core.definitions.partition import PartitionsDefinition
 from dagster._core.definitions.utils import DEFAULT_GROUP_NAME, validate_group_name
@@ -33,7 +35,7 @@ from .events import AssetKey, CoercibleToAssetKeyPrefix
 from .node_definition import NodeDefinition
 from .op_definition import OpDefinition
 from .partition import PartitionsDefinition
-from .partition_mapping import AllPartitionMapping, PartitionMapping
+from .partition_mapping import PartitionMapping, infer_partition_mapping
 from .resource_definition import ResourceDefinition
 from .resource_requirement import (
     ResourceAddable,
@@ -90,6 +92,7 @@ class AssetsDefinition(ResourceAddable):
         resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         group_names_by_key: Optional[Mapping[AssetKey, str]] = None,
         metadata_by_key: Optional[Mapping[AssetKey, MetadataUserInput]] = None,
+        freshness_policies_by_key: Optional[Mapping[AssetKey, FreshnessPolicy]] = None,
         # if adding new fields, make sure to handle them in the with_prefix_or_group
         # and from_graph methods
     ):
@@ -99,13 +102,13 @@ class AssetsDefinition(ResourceAddable):
             _validate_graph_def(node_def)
 
         self._node_def = node_def
-        self._keys_by_input_name = check.dict_param(
+        self._keys_by_input_name = check.mapping_param(
             keys_by_input_name,
             "keys_by_input_name",
             key_type=str,
             value_type=AssetKey,
         )
-        self._keys_by_output_name = check.dict_param(
+        self._keys_by_output_name = check.mapping_param(
             keys_by_output_name,
             "keys_by_output_name",
             key_type=str,
@@ -146,14 +149,29 @@ class AssetsDefinition(ResourceAddable):
             self._selected_asset_keys = all_asset_keys
         self._can_subset = can_subset
 
-        self._metadata_by_key = check.opt_dict_param(
-            metadata_by_key, "metadata_by_key", key_type=AssetKey, value_type=dict
+        self._metadata_by_key = dict(
+            check.opt_mapping_param(
+                metadata_by_key, "metadata_by_key", key_type=AssetKey, value_type=dict
+            )
         )
         for output_name, asset_key in keys_by_output_name.items():
             self._metadata_by_key[asset_key] = merge_dicts(
                 node_def.resolve_output_to_origin(output_name, None)[0].metadata,
                 self._metadata_by_key.get(asset_key, {}),
             )
+        for key, freshness_policy in (freshness_policies_by_key or {}).items():
+            check.param_invariant(
+                not (freshness_policy and self._partitions_def),
+                "freshness_policies_by_key",
+                "FreshnessPolicies are currently unsupported for partitioned assets.",
+            )
+
+        self._freshness_policies_by_key = check.opt_dict_param(
+            freshness_policies_by_key,
+            "freshness_policies_by_key",
+            key_type=AssetKey,
+            value_type=FreshnessPolicy,
+        )
 
     def __call__(self, *args, **kwargs):
         from dagster._core.definitions.decorators.solid_decorator import DecoratedSolidFunction
@@ -198,6 +216,7 @@ class AssetsDefinition(ResourceAddable):
         resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
         partition_mappings: Optional[Mapping[str, PartitionMapping]] = None,
         metadata_by_output_name: Optional[Mapping[str, MetadataUserInput]] = None,
+        can_subset: bool = False,
     ) -> "AssetsDefinition":
         """
         Constructs an AssetsDefinition from a GraphDefinition.
@@ -251,6 +270,7 @@ class AssetsDefinition(ResourceAddable):
             partition_mappings=partition_mappings,
             metadata_by_output_name=metadata_by_output_name,
             key_prefix=key_prefix,
+            can_subset=can_subset,
         )
 
     @public
@@ -327,21 +347,22 @@ class AssetsDefinition(ResourceAddable):
         partition_mappings: Optional[Mapping[str, PartitionMapping]] = None,
         metadata_by_output_name: Optional[Mapping[str, MetadataUserInput]] = None,
         key_prefix: Optional[CoercibleToAssetKeyPrefix] = None,
+        can_subset: bool = False,
     ) -> "AssetsDefinition":
         node_def = check.inst_param(node_def, "node_def", NodeDefinition)
         keys_by_input_name = _infer_keys_by_input_names(
             node_def,
-            check.opt_dict_param(
+            check.opt_mapping_param(
                 keys_by_input_name, "keys_by_input_name", key_type=str, value_type=AssetKey
             ),
         )
-        keys_by_output_name = check.opt_dict_param(
+        keys_by_output_name = check.opt_mapping_param(
             keys_by_output_name,
             "keys_by_output_name",
             key_type=str,
             value_type=AssetKey,
         )
-        internal_asset_deps = check.opt_dict_param(
+        internal_asset_deps = check.opt_mapping_param(
             internal_asset_deps, "internal_asset_deps", key_type=str, value_type=set
         )
         resource_defs = check.opt_mapping_param(
@@ -400,6 +421,7 @@ class AssetsDefinition(ResourceAddable):
             }
             if metadata_by_output_name
             else None,
+            can_subset=can_subset,
         )
 
     @public  # type: ignore
@@ -455,7 +477,7 @@ class AssetsDefinition(ResourceAddable):
 
     @public  # type: ignore
     @property
-    def resource_defs(self) -> Dict[str, ResourceDefinition]:
+    def resource_defs(self) -> Mapping[str, ResourceDefinition]:
         return dict(self._resource_defs)
 
     @public  # type: ignore
@@ -501,11 +523,19 @@ class AssetsDefinition(ResourceAddable):
             name: key for name, key in self.node_keys_by_input_name.items() if key in upstream_keys
         }
 
+    @property
+    def freshness_policies_by_key(self) -> Mapping[AssetKey, FreshnessPolicy]:
+        return self._freshness_policies_by_key
+
     @public  # type: ignore
     @property
     def partitions_def(self) -> Optional[PartitionsDefinition]:
         return self._partitions_def
 
+    @property
+    def is_versioned(self) -> bool:
+        return self.op.version is not None
+
     @property
     def metadata_by_key(self):
         return self._metadata_by_key
@@ -515,12 +545,23 @@ class AssetsDefinition(ResourceAddable):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore", category=ExperimentalWarning)
 
-            return self._partition_mappings.get(
-                in_asset_key,
-                self._partitions_def.get_default_partition_mapping()
-                if self._partitions_def
-                else AllPartitionMapping(),
-            )
+            partition_mapping = self._partition_mappings.get(in_asset_key)
+            return infer_partition_mapping(partition_mapping, self._partitions_def)
+
+    def get_output_name_for_asset_key(self, key: AssetKey) -> str:
+        for output_name, asset_key in self.keys_by_output_name.items():
+            if key == asset_key:
+                return output_name
+
+        check.failed(f"Asset key {key.to_user_string()} not found in AssetsDefinition")
+
+    def get_op_def_for_asset_key(self, key: AssetKey) -> OpDefinition:
+        """
+        If this is an op-backed asset, returns the op def. If it's a graph-backed asset,
+        returns the op def within the graph that produces the given asset key.
+        """
+        output_name = self.get_output_name_for_asset_key(key)
+        return cast(OpDefinition, self.node_def.resolve_output_to_origin_op_def(output_name))
 
     def with_prefix_or_group(
         self,
@@ -530,19 +571,19 @@ class AssetsDefinition(ResourceAddable):
     ) -> "AssetsDefinition":
         from dagster import DagsterInvalidDefinitionError
 
-        output_asset_key_replacements = check.opt_dict_param(
+        output_asset_key_replacements = check.opt_mapping_param(
             output_asset_key_replacements,
             "output_asset_key_replacements",
             key_type=AssetKey,
             value_type=AssetKey,
         )
-        input_asset_key_replacements = check.opt_dict_param(
+        input_asset_key_replacements = check.opt_mapping_param(
             input_asset_key_replacements,
             "input_asset_key_replacements",
             key_type=AssetKey,
             value_type=AssetKey,
         )
-        group_names_by_key = check.opt_dict_param(
+        group_names_by_key = check.opt_mapping_param(
             group_names_by_key, "group_names_by_key", key_type=AssetKey, value_type=str
         )
 
@@ -562,6 +603,11 @@ class AssetsDefinition(ResourceAddable):
             for key, group_name in self.group_names_by_key.items()
         }
 
+        replaced_freshness_policies_by_key = {
+            output_asset_key_replacements.get(key, key): policy
+            for key, policy in self._freshness_policies_by_key.items()
+        }
+
         return self.__class__(
             keys_by_input_name={
                 input_name: input_asset_key_replacements.get(key, key)
@@ -573,7 +619,10 @@ class AssetsDefinition(ResourceAddable):
             },
             node_def=self.node_def,
             partitions_def=self.partitions_def,
-            partition_mappings=self._partition_mappings,
+            partition_mappings={
+                input_asset_key_replacements.get(key, key): partition_mapping
+                for key, partition_mapping in self._partition_mappings.items()
+            },
             asset_deps={
                 # replace both the keys and the values in this mapping
                 output_asset_key_replacements.get(key, key): {
@@ -594,9 +643,48 @@ class AssetsDefinition(ResourceAddable):
                 **replaced_group_names_by_key,
                 **group_names_by_key,
             },
+            freshness_policies_by_key=replaced_freshness_policies_by_key,
         )
 
-    def subset_for(self, selected_asset_keys: AbstractSet[AssetKey]) -> "AssetsDefinition":
+    def _subset_graph_backed_asset(
+        self,
+        selected_asset_keys: AbstractSet[AssetKey],
+    ):
+        from dagster._core.definitions.graph_definition import GraphDefinition
+        from dagster._core.selector.subset_selector import convert_dot_seperated_string_to_dict
+
+        from .job_definition import get_subselected_graph_definition
+
+        if not isinstance(self.node_def, GraphDefinition):
+            raise DagsterInvalidInvocationError(
+                "Method _subset_graph_backed_asset cannot subset an asset that is not a graph"
+            )
+
+        # All asset keys in selected_asset_keys are outputted from the same top-level graph backed asset
+        dep_node_handles_by_asset_key = get_dep_node_handles_of_graph_backed_asset(
+            self.node_def, self
+        )
+        op_selection = []
+        for asset_key in selected_asset_keys:
+            dep_node_handles = dep_node_handles_by_asset_key[asset_key]
+            for dep_node_handle in dep_node_handles:
+                str_op_path = ".".join(dep_node_handle.path[1:])
+                op_selection.append(str_op_path)
+
+        # Pass an op selection into the original job containing only the ops necessary to
+        # generate the selected assets. The ops should all be nested within a top-level graph
+        # node in the original job.
+
+        resolved_op_selection_dict: Dict = {}
+        for item in op_selection:
+            convert_dot_seperated_string_to_dict(resolved_op_selection_dict, splits=item.split("."))
+
+        return get_subselected_graph_definition(self.node_def, resolved_op_selection_dict)
+
+    def subset_for(
+        self,
+        selected_asset_keys: AbstractSet[AssetKey],
+    ) -> "AssetsDefinition":
         """
         Create a subset of this AssetsDefinition that will only materialize the assets in the
         selected set.
@@ -604,45 +692,111 @@ class AssetsDefinition(ResourceAddable):
         Args:
             selected_asset_keys (AbstractSet[AssetKey]): The total set of asset keys
         """
+        from dagster._core.definitions.graph_definition import GraphDefinition
+
         check.invariant(
             self.can_subset,
             f"Attempted to subset AssetsDefinition for {self.node_def.name}, but can_subset=False.",
         )
-        return AssetsDefinition(
-            # keep track of the original mapping
-            keys_by_input_name=self._keys_by_input_name,
-            keys_by_output_name=self._keys_by_output_name,
-            # TODO: subset this properly for graph-backed-assets
-            node_def=self.node_def,
-            partitions_def=self.partitions_def,
-            partition_mappings=self._partition_mappings,
-            asset_deps=self._asset_deps,
-            can_subset=self.can_subset,
-            selected_asset_keys=selected_asset_keys & self.keys,
-            resource_defs=self.resource_defs,
-            group_names_by_key=self.group_names_by_key,
-        )
+
+        # Set of assets within selected_asset_keys which are outputted by this AssetDefinition
+        asset_subselection = selected_asset_keys & self.keys
+        # Early escape if all assets in AssetsDefinition are selected
+        if asset_subselection == self.keys:
+            return self
+        elif isinstance(self.node_def, GraphDefinition):  # Node is graph-backed asset
+            subsetted_node = self._subset_graph_backed_asset(
+                asset_subselection,
+            )
+
+            # The subsetted node should only include asset inputs that are dependencies of the
+            # selected set of assets.
+            subsetted_input_names = [input_def.name for input_def in subsetted_node.input_defs]
+            subsetted_keys_by_input_name = {
+                key: value
+                for key, value in self.node_keys_by_input_name.items()
+                if key in subsetted_input_names
+            }
+
+            subsetted_output_names = [output_def.name for output_def in subsetted_node.output_defs]
+            subsetted_keys_by_output_name = {
+                key: value
+                for key, value in self.node_keys_by_output_name.items()
+                if key in subsetted_output_names
+            }
+
+            # An op within the graph-backed asset that yields multiple assets will be run
+            # any time any of its output assets are selected. Thus, if an op yields multiple assets
+            # and only one of them is selected, the op will still run and potentially unexpectedly
+            # materialize the unselected asset.
+            #
+            # Thus, we include unselected assets that may be accidentally materialized in
+            # keys_by_output_name and asset_deps so that Dagit can populate an warning when this
+            # occurs. This is the same behavior as multi-asset subsetting.
+
+            subsetted_asset_deps = {
+                out_asset_key: set(self._keys_by_input_name.values())
+                for out_asset_key in subsetted_keys_by_output_name.values()
+            }
+
+            return AssetsDefinition(
+                keys_by_input_name=subsetted_keys_by_input_name,
+                keys_by_output_name=subsetted_keys_by_output_name,
+                node_def=subsetted_node,
+                partitions_def=self.partitions_def,
+                partition_mappings=self._partition_mappings,
+                asset_deps=subsetted_asset_deps,
+                can_subset=self.can_subset,
+                selected_asset_keys=selected_asset_keys & self.keys,
+                resource_defs=self.resource_defs,
+                group_names_by_key=self.group_names_by_key,
+                freshness_policies_by_key=self.freshness_policies_by_key,
+            )
+        else:
+            # multi_asset subsetting
+            return AssetsDefinition(
+                # keep track of the original mapping
+                keys_by_input_name=self._keys_by_input_name,
+                keys_by_output_name=self._keys_by_output_name,
+                node_def=self.node_def,
+                partitions_def=self.partitions_def,
+                partition_mappings=self._partition_mappings,
+                asset_deps=self._asset_deps,
+                can_subset=self.can_subset,
+                selected_asset_keys=asset_subselection,
+                resource_defs=self.resource_defs,
+                group_names_by_key=self.group_names_by_key,
+            )
 
     def to_source_assets(self) -> Sequence[SourceAsset]:
         result = []
-        for output_name, asset_key in self.keys_by_output_name.items():
-            # This could maybe be sped up by batching
-            output_def = self.node_def.resolve_output_to_origin(
-                output_name, NodeHandle(self.node_def.name, parent=None)
-            )[0]
-            result.append(
-                SourceAsset(
-                    key=asset_key,
-                    metadata=output_def.metadata,
-                    io_manager_key=output_def.io_manager_key,
-                    description=output_def.description,
-                    resource_defs=self.resource_defs,
-                    partitions_def=self.partitions_def,
-                    group_name=self.group_names_by_key[asset_key],
-                )
-            )
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=ExperimentalWarning)
 
-        return result
+            for output_name, asset_key in self.keys_by_output_name.items():
+                # This could maybe be sped up by batching
+                output_def = self.node_def.resolve_output_to_origin(
+                    output_name, NodeHandle(self.node_def.name, parent=None)
+                )[0]
+                result.append(
+                    SourceAsset(
+                        key=asset_key,
+                        metadata=output_def.metadata,
+                        io_manager_key=output_def.io_manager_key,
+                        description=output_def.description,
+                        resource_defs=self.resource_defs,
+                        partitions_def=self.partitions_def,
+                        group_name=self.group_names_by_key[asset_key],
+                    )
+                )
+
+            return result
+
+    def get_io_manager_key_for_asset_key(self, key: AssetKey) -> str:
+        output_name = self.get_output_name_for_asset_key(key)
+        return self.node_def.resolve_output_to_origin(
+            output_name, NodeHandle(self.node_def.name, parent=None)
+        )[0].io_manager_key
 
     def get_resource_requirements(self) -> Iterator[ResourceRequirement]:
         yield from self.node_def.get_resource_requirements()  # type: ignore[attr-defined]
@@ -704,6 +858,7 @@ class AssetsDefinition(ResourceAddable):
             can_subset=self._can_subset,
             resource_defs=relevant_resource_defs,
             group_names_by_key=self.group_names_by_key,
+            freshness_policies_by_key=self.freshness_policies_by_key,
         )
 
 
@@ -804,12 +959,12 @@ def _validate_graph_def(graph_def: "GraphDefinition", prefix: Optional[Sequence[
     """Ensure that all leaf nodes are mapped to graph outputs."""
     from dagster._core.definitions.graph_definition import GraphDefinition, _create_adjacency_lists
 
-    prefix = check.opt_list_param(prefix, "prefix")
+    prefix = check.opt_sequence_param(prefix, "prefix")
 
     # recursively validate any sub-graphs
     for inner_node_def in graph_def.node_defs:
         if isinstance(inner_node_def, GraphDefinition):
-            _validate_graph_def(inner_node_def, prefix=prefix + [graph_def.name])
+            _validate_graph_def(inner_node_def, prefix=[*prefix, graph_def.name])
 
     # leaf nodes have no downstream nodes
     forward_edges, _ = _create_adjacency_lists(graph_def.solids, graph_def.dependency_structure)
@@ -823,7 +978,7 @@ def _validate_graph_def(graph_def: "GraphDefinition", prefix: Optional[Sequence[
     }
 
     # leaf nodes which do not have an associated mapped output
-    unmapped_leaf_nodes = {".".join(prefix + [node]) for node in leaf_nodes - mapped_output_nodes}
+    unmapped_leaf_nodes = {".".join([*prefix, node]) for node in leaf_nodes - mapped_output_nodes}
 
     check.invariant(
         not unmapped_leaf_nodes,
