@@ -1,3 +1,4 @@
+import warnings
 from collections import defaultdict
 from typing import Any, Dict, Iterable, List, Mapping, Optional, Sequence, Set, Tuple, Union
 
@@ -5,10 +6,10 @@ from toposort import CircularDependencyError, toposort
 
 import dagster._check as check
 from dagster._annotations import experimental
-from dagster._core.definitions.utils import DEFAULT_IO_MANAGER_KEY
 from dagster._core.errors import DagsterInvalidDefinitionError
 from dagster._core.selector.subset_selector import AssetSelectionData
 from dagster._utils import merge_dicts
+from dagster._utils.backcompat import ExperimentalWarning
 
 from .asset_layer import AssetLayer
 from .assets import AssetsDefinition
@@ -25,11 +26,67 @@ from .resource_requirement import ensure_requirements_satisfied
 from .source_asset import SourceAsset
 from .utils import DEFAULT_IO_MANAGER_KEY
 
+# Prefix for auto created jobs that are used to materialize assets
+ASSET_BASE_JOB_PREFIX = "__ASSET_JOB"
+
+
+def is_base_asset_job_name(name) -> bool:
+    return name.startswith(ASSET_BASE_JOB_PREFIX)
+
+
+def get_base_asset_jobs(
+    assets: Sequence[AssetsDefinition],
+    source_assets: Sequence[SourceAsset],
+    resource_defs: Optional[Mapping[str, ResourceDefinition]],
+    executor_def: Optional[ExecutorDefinition],
+) -> Sequence[JobDefinition]:
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=ExperimentalWarning)
+
+        assets_by_partitions_def: Dict[
+            Optional[PartitionsDefinition], List[AssetsDefinition]
+        ] = defaultdict(list)
+        for assets_def in assets:
+            assets_by_partitions_def[assets_def.partitions_def].append(assets_def)
+
+        if len(assets_by_partitions_def.keys()) == 0 or assets_by_partitions_def.keys() == {None}:
+            return [
+                build_assets_job(
+                    name=ASSET_BASE_JOB_PREFIX,
+                    assets=assets,
+                    source_assets=source_assets,
+                    executor_def=executor_def,
+                    resource_defs=resource_defs,
+                )
+            ]
+        else:
+            unpartitioned_assets = assets_by_partitions_def.get(None, [])
+            partitioned_assets_by_partitions_def = {
+                k: v for k, v in assets_by_partitions_def.items() if k is not None
+            }
+            jobs = []
+
+            # sort to ensure some stability in the ordering
+            for i, (_, assets_with_partitions) in enumerate(
+                sorted(partitioned_assets_by_partitions_def.items(), key=lambda item: repr(item[0]))
+            ):
+                jobs.append(
+                    build_assets_job(
+                        f"{ASSET_BASE_JOB_PREFIX}_{i}",
+                        assets=assets_with_partitions + unpartitioned_assets,
+                        source_assets=[*source_assets, *assets],
+                        resource_defs=resource_defs,
+                        executor_def=executor_def,
+                    )
+                )
+
+            return jobs
+
 
 @experimental
 def build_assets_job(
     name: str,
-    assets: Iterable[AssetsDefinition],
+    assets: Sequence[AssetsDefinition],
     source_assets: Optional[Sequence[Union[SourceAsset, AssetsDefinition]]] = None,
     resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
     description: Optional[str] = None,
@@ -72,9 +129,8 @@ def build_assets_job(
     Returns:
         JobDefinition: A job that materializes the given assets.
     """
-
     check.str_param(name, "name")
-    check.iterable_param(assets, "assets", of_type=AssetsDefinition)
+    check.iterable_param(assets, "assets", of_type=(AssetsDefinition, SourceAsset))
     source_assets = check.opt_sequence_param(
         source_assets, "source_assets", of_type=(SourceAsset, AssetsDefinition)
     )
@@ -105,9 +161,14 @@ def build_assets_job(
 
         deps, assets_defs_by_node_handle = build_node_deps(assets, resolved_asset_deps)
 
+    if len(assets) > 0:
+        node_defs = [asset.node_def for asset in assets]
+    else:
+        node_defs = list(filter(None, [asset.node_def for asset in [*source_assets]]))
+
     graph = GraphDefinition(
         name=name,
-        node_defs=[asset.node_def for asset in assets],
+        node_defs=node_defs,
         dependencies=deps,
         description=description,
         input_mappings=None,
@@ -149,33 +210,143 @@ def build_assets_job(
     )
 
 
+@experimental
+def build_source_asset_observation_job(
+    name: str,
+    source_assets: Sequence[SourceAsset],
+    resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
+    description: Optional[str] = None,
+    config: Optional[Union[ConfigMapping, Mapping[str, object], PartitionedConfig]] = None,
+    tags: Optional[Mapping[str, object]] = None,
+    executor_def: Optional[ExecutorDefinition] = None,
+    partitions_def: Optional[PartitionsDefinition] = None,
+    _asset_selection_data: Optional[AssetSelectionData] = None,
+) -> JobDefinition:
+    """Builds a job that observes the given source assets.
+
+    There are never any dependencies between the ops in the job.
+
+    Args:
+        name (str): The name of the job.
+        source_assets (Sequence[SourceAsset]): A list of source assets to observe.
+        resource_defs (Optional[Mapping[str, ResourceDefinition]]): Resource defs to be included in
+            this job.
+        description (Optional[str]): A description of the job.
+
+    Examples:
+
+        .. code-block:: python
+
+            @asset
+            def asset1():
+                return 5
+
+            @asset
+            def asset2(asset1):
+                return my_upstream_asset + 1
+
+            my_assets_job = build_assets_job("my_assets_job", assets=[asset1, asset2])
+
+    Returns:
+        JobDefinition: A job that materializes the given assets.
+    """
+    check.str_param(name, "name")
+    check.iterable_param(source_assets, "source_assets", of_type=SourceAsset)
+    check.opt_str_param(description, "description")
+    check.opt_inst_param(_asset_selection_data, "_asset_selection_data", AssetSelectionData)
+
+    resource_defs = check.opt_mapping_param(resource_defs, "resource_defs")
+    resource_defs = merge_dicts({DEFAULT_IO_MANAGER_KEY: default_job_io_manager}, resource_defs)
+
+    # figure out what partitions (if any) exist for this job
+    partitions_def = partitions_def or build_job_partitions_from_assets(source_assets)
+
+    # Keeping this for now (copied from `build_assets_job` though it's probably not needed
+    assets: Sequence[AssetsDefinition] = []
+    resolved_asset_deps = ResolvedAssetDependencies(assets, source_assets)
+    deps, assets_defs_by_node_handle = build_node_deps(assets, resolved_asset_deps)
+    node_defs = list(filter(None, [asset.node_def for asset in [*source_assets]]))
+
+    graph = GraphDefinition(
+        name=name,
+        node_defs=node_defs,
+        dependencies=deps,
+        description=description,
+        input_mappings=None,
+        output_mappings=None,
+        config=None,
+    )
+
+    asset_layer = AssetLayer.from_graph_and_assets_node_mapping(
+        graph, assets_defs_by_node_handle, source_assets, resolved_asset_deps
+    )
+
+    all_resource_defs = get_all_resource_defs(assets, source_assets, resource_defs)
+
+    if _asset_selection_data:
+        original_job = _asset_selection_data.parent_job_def
+        return graph.to_job(
+            resource_defs=all_resource_defs,
+            config=config,
+            tags=tags,
+            executor_def=executor_def,
+            partitions_def=partitions_def,
+            asset_layer=asset_layer,
+            _asset_selection_data=_asset_selection_data,
+            _metadata_entries=original_job._metadata_entries,  # pylint: disable=protected-access
+            logger_defs=original_job.get_mode_definition().loggers,
+            hooks=original_job.hook_defs,
+            op_retry_policy=original_job._solid_retry_policy,  # pylint: disable=protected-access
+            version_strategy=original_job.version_strategy,
+        )
+
+    return graph.to_job(
+        resource_defs=all_resource_defs,
+        config=config,
+        tags=tags,
+        executor_def=executor_def,
+        partitions_def=partitions_def,
+        asset_layer=asset_layer,
+        _asset_selection_data=_asset_selection_data,
+    )
+
+
 def build_job_partitions_from_assets(
-    assets: Iterable[AssetsDefinition],
+    assets: Iterable[Union[AssetsDefinition, SourceAsset]],
 ) -> Optional[PartitionsDefinition]:
     assets_with_partitions_defs = [assets_def for assets_def in assets if assets_def.partitions_def]
 
     if len(assets_with_partitions_defs) == 0:
         return None
 
-    first_assets_with_partitions_def: AssetsDefinition = assets_with_partitions_defs[0]
-    for assets_def in assets_with_partitions_defs:
-        if assets_def.partitions_def != first_assets_with_partitions_def.partitions_def:
-            first_asset_key = next(iter(assets_def.keys)).to_string()
-            second_asset_key = next(iter(first_assets_with_partitions_def.keys)).to_string()
+    first_asset_with_partitions_def: Union[
+        AssetsDefinition, SourceAsset
+    ] = assets_with_partitions_defs[0]
+    for asset in assets_with_partitions_defs:
+        if asset.partitions_def != first_asset_with_partitions_def.partitions_def:
+            first_asset_key = _key_for_asset(asset).to_string()
+            second_asset_key = _key_for_asset(first_asset_with_partitions_def).to_string()
             raise DagsterInvalidDefinitionError(
                 "When an assets job contains multiple partitions assets, they must have the "
                 f"same partitions definitions, but asset '{first_asset_key}' and asset "
                 f"'{second_asset_key}' have different partitions definitions. "
             )
 
-    return first_assets_with_partitions_def.partitions_def
+    return first_asset_with_partitions_def.partitions_def
+
+
+def _key_for_asset(asset: Union[AssetsDefinition, SourceAsset]) -> AssetKey:
+    if isinstance(asset, AssetsDefinition):
+        return next(iter(asset.keys))
+    else:
+        return asset.key
 
 
 def build_node_deps(
     assets_defs: Iterable[AssetsDefinition],
     resolved_asset_deps: ResolvedAssetDependencies,
 ) -> Tuple[
-    Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]],
+    Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]],
     Mapping[NodeHandle, AssetsDefinition],
 ]:
     # sort so that nodes get a consistent name
@@ -228,7 +399,9 @@ def build_node_deps(
     return deps, assets_defs_by_node_handle
 
 
-def _has_cycles(deps: Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefinition]]) -> bool:
+def _has_cycles(
+    deps: Mapping[Union[str, NodeInvocation], Mapping[str, IDependencyDefinition]]
+) -> bool:
     """Detect if there are cycles in a dependency dictionary."""
     try:
         node_deps: Dict[str, Set[str]] = {}
@@ -254,7 +427,8 @@ def _has_cycles(deps: Dict[Union[str, NodeInvocation], Dict[str, IDependencyDefi
 
 
 def _attempt_resolve_cycles(
-    assets_defs: Iterable["AssetsDefinition"], source_assets: Iterable["SourceAsset"]
+    assets_defs: Iterable["AssetsDefinition"],
+    source_assets: Iterable["SourceAsset"],
 ) -> Sequence["AssetsDefinition"]:
     """
     DFS starting at root nodes to color the asset dependency graph. Each time you leave your
@@ -276,33 +450,33 @@ def _attempt_resolve_cycles(
     asset_deps = generate_asset_dep_graph(assets_defs, source_assets)
 
     # index AssetsDefinitions by their asset names
-    assets_defs_by_asset_name = {}
+    assets_defs_by_asset_key: Dict[AssetKey, AssetsDefinition] = {}
     for assets_def in assets_defs:
         for asset_key in assets_def.keys:
-            assets_defs_by_asset_name[asset_key.to_user_string()] = assets_def
+            assets_defs_by_asset_key[asset_key] = assets_def
 
     # color for each asset
     colors = {}
 
     # recursively color an asset and all of its downstream assets
-    def _dfs(name, cur_color):
-        colors[name] = cur_color
-        if name in assets_defs_by_asset_name:
-            cur_node_asset_keys = assets_defs_by_asset_name[name].keys
+    def _dfs(key, cur_color):
+        colors[key] = cur_color
+        if key in assets_defs_by_asset_key:
+            cur_node_asset_keys = assets_defs_by_asset_key[key].keys
         else:
             # in a SourceAsset, treat all downstream as if they're in the same node
-            cur_node_asset_keys = asset_deps["downstream"][name]
+            cur_node_asset_keys = asset_deps["downstream"][key]
 
-        for downstream_name in asset_deps["downstream"][name]:
+        for downstream_key in asset_deps["downstream"][key]:
             # if the downstream asset is in the current node,keep the same color
-            if AssetKey.from_user_string(downstream_name) in cur_node_asset_keys:
+            if downstream_key in cur_node_asset_keys:
                 new_color = cur_color
             else:
                 new_color = cur_color + 1
 
             # if current color of the downstream asset is less than the new color, re-do dfs
-            if colors.get(downstream_name, -1) < new_color:
-                _dfs(downstream_name, new_color)
+            if colors.get(downstream_key, -1) < new_color:
+                _dfs(downstream_key, new_color)
 
     # validate that there are no cycles in the overall asset graph
     toposorted = list(toposort(asset_deps["upstream"]))
@@ -314,14 +488,11 @@ def _attempt_resolve_cycles(
     color_mapping_by_assets_defs: Dict[AssetsDefinition, Any] = defaultdict(
         lambda: defaultdict(set)
     )
-    for name, color in colors.items():
-        asset_key = AssetKey.from_user_string(name)
+    for key, color in colors.items():
         # ignore source assets
-        if name not in assets_defs_by_asset_name:
+        if key not in assets_defs_by_asset_key:
             continue
-        color_mapping_by_assets_defs[assets_defs_by_asset_name[name]][color].add(
-            AssetKey.from_user_string(name)
-        )
+        color_mapping_by_assets_defs[assets_defs_by_asset_key[key]][color].add(key)
 
     ret = []
     for assets_def, color_mapping in color_mapping_by_assets_defs.items():
@@ -390,7 +561,7 @@ def get_all_resource_defs(
     assets: Iterable[AssetsDefinition],
     source_assets: Sequence[SourceAsset],
     resource_defs: Mapping[str, ResourceDefinition],
-) -> Dict[str, ResourceDefinition]:
+) -> Mapping[str, ResourceDefinition]:
 
     # Ensures that no resource keys conflict, and each asset has its resource requirements satisfied.
     check_resources_satisfy_requirements(assets, source_assets, resource_defs)
