@@ -564,7 +564,7 @@ class PendingNodeInvocation:
         )
 
     @public
-    def tag(self, tags: Optional[Dict[str, str]]) -> "PendingNodeInvocation":
+    def tag(self, tags: Optional[Mapping[str, str]]) -> "PendingNodeInvocation":
         tags = validate_tags(tags)
         return PendingNodeInvocation(
             node_def=self.node_def,
@@ -601,8 +601,8 @@ class PendingNodeInvocation:
         name: Optional[str] = None,
         description: Optional[str] = None,
         resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
-        config: Optional[Union[ConfigMapping, Dict[str, Any], "PartitionedConfig"]] = None,
-        tags: Optional[Dict[str, Any]] = None,
+        config: Optional[Union[ConfigMapping, Mapping[str, Any], "PartitionedConfig"]] = None,
+        tags: Optional[Mapping[str, Any]] = None,
         logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
         executor_def: Optional["ExecutorDefinition"] = None,
         hooks: Optional[AbstractSet[HookDefinition]] = None,
@@ -617,7 +617,7 @@ class PendingNodeInvocation:
                 "constructed using the `@graph` decorator support this method."
             )
 
-        tags = check.opt_dict_param(tags, "tags", key_type=str)
+        tags = check.opt_mapping_param(tags, "tags", key_type=str)
         hooks = check.opt_set_param(hooks, "hooks", HookDefinition)
         input_values = check.opt_mapping_param(input_values, "input_values")
         op_retry_policy = check.opt_inst_param(op_retry_policy, "op_retry_policy", RetryPolicy)
@@ -644,7 +644,7 @@ class PendingNodeInvocation:
         self,
         run_config: Optional[Any] = None,
         instance: Optional["DagsterInstance"] = None,
-        resources: Optional[Dict[str, Any]] = None,
+        resources: Optional[Mapping[str, Any]] = None,
         raise_on_error: bool = True,
         run_id: Optional[str] = None,
         input_values: Optional[Mapping[str, object]] = None,
@@ -900,6 +900,27 @@ def composite_mapping_from_output(
                 )
             )
 
+    elif isinstance(output, InvokedSolidDynamicOutputWrapper):
+        if len(output_defs) == 1:
+            defn = output_defs[0]
+            return {
+                defn.name: defn.mapping_from(
+                    output.solid_name, output.output_name, from_dynamic_mapping=True
+                )
+            }
+        else:
+            raise DagsterInvalidDefinitionError(
+                "Returned a single output ({solid_name}.{output_name}) in "
+                "{decorator_name} '{name}' but {num} outputs are defined. "
+                "Return a dict to map defined outputs.".format(
+                    solid_name=output.solid_name,
+                    output_name=output.output_name,
+                    decorator_name=decorator_name,
+                    name=solid_name,
+                    num=len(output_defs),
+                )
+            )
+
     output_mapping_dict = {}
     output_def_dict = {output_def.name: output_def for output_def in output_defs}
 
@@ -952,9 +973,8 @@ def composite_mapping_from_output(
                     handle.solid_name, handle.output_name
                 )
             elif isinstance(handle, InvokedSolidDynamicOutputWrapper):
-                unwrapped = handle.unwrap_for_composite_mapping()
                 output_mapping_dict[name] = output_def_dict[name].mapping_from(
-                    unwrapped.solid_name, unwrapped.output_name
+                    handle.solid_name, handle.output_name, from_dynamic_mapping=True
                 )
             else:
                 raise DagsterInvalidDefinitionError(
@@ -967,11 +987,6 @@ def composite_mapping_from_output(
 
         return output_mapping_dict
 
-    elif isinstance(output, InvokedSolidDynamicOutputWrapper):
-        return composite_mapping_from_output(
-            output.unwrap_for_composite_mapping(), output_defs, solid_name, decorator_name
-        )
-
     # error
     if output is not None:
         raise DagsterInvalidDefinitionError(
@@ -1084,7 +1099,7 @@ def do_composition(
     input_mappings = []
     for defn in actual_input_defs:
         mappings = [
-            mapping for mapping in context.input_mappings if mapping.definition.name == defn.name
+            mapping for mapping in context.input_mappings if mapping.graph_input_name == defn.name
         ]
 
         if len(mappings) == 0:
