@@ -17,6 +17,7 @@ import dagster._check as check
 from dagster._builtins import Nothing
 from dagster._config import UserConfigSchema
 from dagster._core.decorator_utils import get_function_params, get_valid_name_permutations
+from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.errors import DagsterInvalidDefinitionError
 from dagster._core.storage.io_manager import IOManagerDefinition
 from dagster._core.types.dagster_type import DagsterType
@@ -35,6 +36,7 @@ from ..events import AssetKey, CoercibleToAssetKeyPrefix
 from ..input import In
 from ..output import Out
 from ..partition import PartitionsDefinition
+from ..policy import RetryPolicy
 from ..resource_definition import ResourceDefinition
 from ..utils import DEFAULT_IO_MANAGER_KEY, NoValueSentinel
 
@@ -63,9 +65,12 @@ def asset(
     compute_kind: Optional[str] = ...,
     dagster_type: Optional[DagsterType] = ...,
     partitions_def: Optional[PartitionsDefinition] = ...,
-    op_tags: Optional[Dict[str, Any]] = ...,
+    op_tags: Optional[Mapping[str, Any]] = ...,
     group_name: Optional[str] = ...,
     output_required: bool = ...,
+    freshness_policy: Optional[FreshnessPolicy] = ...,
+    retry_policy: Optional[RetryPolicy] = ...,
+    op_version: Optional[str] = ...,
 ) -> Callable[[Callable[..., Any]], AssetsDefinition]:
     ...
 
@@ -87,9 +92,12 @@ def asset(
     compute_kind: Optional[str] = None,
     dagster_type: Optional[DagsterType] = None,
     partitions_def: Optional[PartitionsDefinition] = None,
-    op_tags: Optional[Dict[str, Any]] = None,
+    op_tags: Optional[Mapping[str, Any]] = None,
     group_name: Optional[str] = None,
     output_required: bool = True,
+    freshness_policy: Optional[FreshnessPolicy] = None,
+    retry_policy: Optional[RetryPolicy] = None,
+    op_version: Optional[str] = None,
 ) -> Union[AssetsDefinition, Callable[[Callable[..., Any]], AssetsDefinition]]:
     """Create a definition for how to compute an asset.
 
@@ -144,6 +152,11 @@ def asset(
         output_required (bool): Whether the decorated function will always materialize an asset.
             Defaults to True. If False, the function can return None, which will not be materialized to
             storage and will halt execution of downstream assets.
+        freshness_policy (FreshnessPolicy): A constraint telling Dagster how often this asset is intended to be updated
+            with respect to its root data.
+        retry_policy (Optional[RetryPolicy]): The retry policy for the op that computes the asset.
+        op_version (Optional[str]): (Experimental) Version string passed to the op underlying the
+            asset.
 
     Examples:
 
@@ -184,6 +197,9 @@ def asset(
             op_tags=op_tags,
             group_name=group_name,
             output_required=output_required,
+            freshness_policy=freshness_policy,
+            retry_policy=retry_policy,
+            op_version=op_version,
         )(fn)
 
     return inner
@@ -205,9 +221,12 @@ class _Asset:
         compute_kind: Optional[str] = None,
         dagster_type: Optional[DagsterType] = None,
         partitions_def: Optional[PartitionsDefinition] = None,
-        op_tags: Optional[Dict[str, Any]] = None,
+        op_tags: Optional[Mapping[str, Any]] = None,
         group_name: Optional[str] = None,
         output_required: bool = True,
+        freshness_policy: Optional[FreshnessPolicy] = None,
+        retry_policy: Optional[RetryPolicy] = None,
+        op_version: Optional[str] = None,
     ):
         self.name = name
 
@@ -230,6 +249,9 @@ class _Asset:
         self.resource_defs = dict(check.opt_mapping_param(resource_defs, "resource_defs"))
         self.group_name = group_name
         self.output_required = output_required
+        self.freshness_policy = freshness_policy
+        self.retry_policy = retry_policy
+        self.op_version = op_version
 
     def __call__(self, fn: Callable) -> AssetsDefinition:
         asset_name = self.name or fn.__name__
@@ -250,8 +272,7 @@ class _Asset:
                 io_manager_def = check.inst_param(
                     self.io_manager, "io_manager", IOManagerDefinition
                 )
-                out_asset_resource_key = "__".join(out_asset_key.path)
-                io_manager_key = f"{out_asset_resource_key}__io_manager"
+                io_manager_key = out_asset_key.to_python_identifier("io_manager")
                 self.resource_defs[io_manager_key] = cast(ResourceDefinition, io_manager_def)
             else:
                 io_manager_key = DEFAULT_IO_MANAGER_KEY
@@ -265,7 +286,7 @@ class _Asset:
             )
 
             op = _Op(
-                name="__".join(out_asset_key.path).replace("-", "_"),
+                name=out_asset_key.to_python_identifier(),
                 description=self.description,
                 ins=dict(asset_ins.values()),
                 out=out,
@@ -275,6 +296,8 @@ class _Asset:
                     **(self.op_tags or {}),
                 },
                 config_schema=self.config_schema,
+                retry_policy=self.retry_policy,
+                version=self.op_version,
             )(fn)
 
         keys_by_input_name = {
@@ -294,6 +317,9 @@ class _Asset:
             partition_mappings=partition_mappings if partition_mappings else None,
             resource_defs=self.resource_defs,
             group_names_by_key={out_asset_key: self.group_name} if self.group_name else None,
+            freshness_policies_by_key={out_asset_key: self.freshness_policy}
+            if self.freshness_policy
+            else None,
         )
 
 
@@ -309,10 +335,11 @@ def multi_asset(
     compute_kind: Optional[str] = None,
     internal_asset_deps: Optional[Mapping[str, Set[AssetKey]]] = None,
     partitions_def: Optional[PartitionsDefinition] = None,
-    op_tags: Optional[Dict[str, Any]] = None,
+    op_tags: Optional[Mapping[str, Any]] = None,
     can_subset: bool = False,
     resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
     group_name: Optional[str] = None,
+    retry_policy: Optional[RetryPolicy] = None,
 ) -> Callable[[Callable[..., Any]], AssetsDefinition]:
     """Create a combined definition of multiple assets that are computed using the same op and same
     upstream assets.
@@ -352,11 +379,12 @@ def multi_asset(
             context within the body of the function.
         group_name (Optional[str]): A string name used to organize multiple assets into groups. This
             group name will be applied to all assets produced by this multi_asset.
+        retry_policy (Optional[RetryPolicy]): The retry policy for the op that computes the asset.
     """
     if resource_defs is not None:
         experimental_arg_warning("resource_defs", "multi_asset")
 
-    asset_deps = check.opt_dict_param(
+    asset_deps = check.opt_mapping_param(
         internal_asset_deps, "internal_asset_deps", key_type=str, value_type=set
     )
     required_resource_keys = check.opt_set_param(
@@ -417,6 +445,8 @@ def multi_asset(
                     **({"kind": compute_kind} if compute_kind else {}),
                     **(op_tags or {}),
                 },
+                config_schema=config_schema,
+                retry_policy=retry_policy,
             )(fn)
 
         keys_by_input_name = {
