@@ -1,4 +1,4 @@
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from typing import (
     TYPE_CHECKING,
     AbstractSet,
@@ -6,7 +6,6 @@ from typing import (
     Dict,
     Iterable,
     Iterator,
-    List,
     Mapping,
     Optional,
     Sequence,
@@ -91,9 +90,9 @@ def _check_node_defs_arg(
 
 
 def _create_adjacency_lists(
-    solids: List[Node],
+    solids: Sequence[Node],
     dep_structure: DependencyStructure,
-) -> Tuple[Dict[str, Set[str]], Dict[str, Set[str]]]:
+) -> Tuple[Mapping[str, Set[str]], Mapping[str, Set[str]]]:
     visit_dict = {s.name: False for s in solids}
     forward_edges: Dict[str, Set[str]] = {s.name: set() for s in solids}
     backward_edges: Dict[str, Set[str]] = {s.name: set() for s in solids}
@@ -204,18 +203,19 @@ class GraphDefinition(NodeDefinition):
         )
 
         # Sequence[InputMapping]
-        self._input_mappings, input_defs = _validate_in_mappings(
-            check.opt_list_param(input_mappings, "input_mappings"),
+        self._input_mappings = check.opt_sequence_param(input_mappings, "input_mappings")
+        input_defs = _validate_in_mappings(
+            self._input_mappings,
             self._node_dict,
             self._dependency_structure,
             name,
             class_name=type(self).__name__,
         )
+
         # Sequence[OutputMapping]
-        self._output_mappings = _validate_out_mappings(
-            check.opt_list_param(output_mappings, "output_mappings"),
+        self._output_mappings, output_defs = _validate_out_mappings(
+            check.opt_sequence_param(output_mappings, "output_mappings"),
             self._node_dict,
-            self._dependency_structure,
             name,
             class_name=type(self).__name__,
         )
@@ -226,7 +226,7 @@ class GraphDefinition(NodeDefinition):
             name=name,
             description=description,
             input_defs=input_defs,
-            output_defs=[output_mapping.definition for output_mapping in self._output_mappings],
+            output_defs=output_defs,
             tags=tags,
             **kwargs,
         )
@@ -236,7 +236,7 @@ class GraphDefinition(NodeDefinition):
         self._solids_in_topological_order = self._get_solids_in_topological_order()
         self._dagster_type_dict = construct_dagster_type_dictionary([self])
 
-    def _get_solids_in_topological_order(self) -> List[Node]:
+    def _get_solids_in_topological_order(self) -> Sequence[Node]:
 
         _forward_edges, backward_edges = _create_adjacency_lists(
             self.solids, self.dependency_structure
@@ -251,7 +251,7 @@ class GraphDefinition(NodeDefinition):
 
     def get_inputs_must_be_resolved_top_level(
         self, asset_layer: "AssetLayer", handle: Optional[NodeHandle] = None
-    ) -> List[InputDefinition]:
+    ) -> Sequence[InputDefinition]:
         unresolveable_input_defs = []
         for node in self.node_dict.values():
             cur_handle = NodeHandle(node.name, handle)
@@ -271,7 +271,7 @@ class GraphDefinition(NodeDefinition):
                     )
                 else:
                     mapped_input = node.container_mapped_input(input_def.name)
-                    unresolveable_input_defs.append(mapped_input.definition)
+                    unresolveable_input_defs.append(mapped_input.get_definition())
         return unresolveable_input_defs
 
     @property
@@ -283,7 +283,7 @@ class GraphDefinition(NodeDefinition):
         return True
 
     @property
-    def solids(self) -> List[Node]:
+    def solids(self) -> Sequence[Node]:
         return list(set(self._node_dict.values()))
 
     @property
@@ -382,7 +382,7 @@ class GraphDefinition(NodeDefinition):
 
         check.str_param(input_name, "input_name")
         for mapping in self._input_mappings:
-            if mapping.definition.name == input_name:
+            if mapping.graph_input_name == input_name:
                 return mapping
         check.failed(f"Could not find input mapping {input_name}")
 
@@ -399,16 +399,14 @@ class GraphDefinition(NodeDefinition):
     def get_output_mapping(self, output_name: str) -> OutputMapping:
         check.str_param(output_name, "output_name")
         for mapping in self._output_mappings:
-            if mapping.definition.name == output_name:
+            if mapping.graph_output_name == output_name:
                 return mapping
         check.failed(f"Could not find output mapping {output_name}")
 
     T_Handle = TypeVar("T_Handle", bound=Optional[NodeHandle])
 
     def resolve_output_to_origin(
-        self,
-        output_name: str,
-        handle: Optional[NodeHandle],
+        self, output_name: str, handle: Optional[NodeHandle]
     ) -> Tuple[OutputDefinition, Optional[NodeHandle]]:
         check.str_param(output_name, "output_name")
         check.opt_inst_param(handle, "handle", NodeHandle)
@@ -421,6 +419,13 @@ class GraphDefinition(NodeDefinition):
             NodeHandle(mapped_solid.name, handle),  # type: ignore
         )
 
+    def resolve_output_to_origin_op_def(self, output_name: str) -> "SolidDefinition":
+        mapping = self.get_output_mapping(output_name)
+        check.invariant(mapping, "Can only resolve outputs for valid output names")
+        return self.solid_named(
+            mapping.maps_from.solid_name
+        ).definition.resolve_output_to_origin_op_def(output_name)
+
     def default_value_for_input(self, input_name: str) -> object:
         check.str_param(input_name, "input_name")
 
@@ -521,7 +526,7 @@ class GraphDefinition(NodeDefinition):
         hooks: Optional[AbstractSet[HookDefinition]] = None,
         op_retry_policy: Optional[RetryPolicy] = None,
         version_strategy: Optional[VersionStrategy] = None,
-        op_selection: Optional[List[str]] = None,
+        op_selection: Optional[Sequence[str]] = None,
         partitions_def: Optional["PartitionsDefinition"] = None,
         asset_layer: Optional["AssetLayer"] = None,
         input_values: Optional[Mapping[str, object]] = None,
@@ -626,7 +631,7 @@ class GraphDefinition(NodeDefinition):
         instance: Optional["DagsterInstance"] = None,
         resources: Optional[Mapping[str, object]] = None,
         raise_on_error: bool = True,
-        op_selection: Optional[List[str]] = None,
+        op_selection: Optional[Sequence[str]] = None,
         run_id: Optional[str] = None,
         input_values: Optional[Mapping[str, object]] = None,
     ) -> "ExecuteInProcessResult":
@@ -666,7 +671,7 @@ class GraphDefinition(NodeDefinition):
         from .job_definition import JobDefinition
 
         instance = check.opt_inst_param(instance, "instance", DagsterInstance)
-        resources = check.opt_dict_param(resources, "resources", key_type=str)
+        resources = check.opt_mapping_param(resources, "resources", key_type=str)
         input_values = check.opt_mapping_param(input_values, "input_values")
 
         resource_defs = wrap_resources_for_execution(resources)
@@ -680,7 +685,7 @@ class GraphDefinition(NodeDefinition):
         ).get_job_def_for_subset_selection(op_selection)
 
         run_config = run_config if run_config is not None else {}
-        op_selection = check.opt_list_param(op_selection, "op_selection", str)
+        op_selection = check.opt_sequence_param(op_selection, "op_selection", str)
 
         return core_execute_in_process(
             ephemeral_pipeline=ephemeral_job,
@@ -723,7 +728,7 @@ class GraphDefinition(NodeDefinition):
         return super(GraphDefinition, self).alias(name)
 
     @public
-    def tag(self, tags: Optional[Dict[str, str]]) -> "PendingNodeInvocation":
+    def tag(self, tags: Optional[Mapping[str, str]]) -> "PendingNodeInvocation":
         return super(GraphDefinition, self).tag(tags)
 
     @public
@@ -784,7 +789,7 @@ class SubselectedGraphDefinition(GraphDefinition):
     def parent_graph_def(self) -> GraphDefinition:
         return self._parent_graph_def
 
-    def get_top_level_omitted_nodes(self) -> List[Node]:
+    def get_top_level_omitted_nodes(self) -> Sequence[Node]:
         return [
             solid for solid in self.parent_graph_def.solids if not self.has_solid_named(solid.name)
         ]
@@ -796,71 +801,56 @@ class SubselectedGraphDefinition(GraphDefinition):
 
 def _validate_in_mappings(
     input_mappings: Sequence[InputMapping],
-    solid_dict: Mapping[str, Node],
+    nodes_by_name: Mapping[str, Node],
     dependency_structure: DependencyStructure,
     name: str,
     class_name: str,
-) -> Tuple[Sequence[InputMapping], Sequence[InputDefinition]]:
+) -> Sequence[InputDefinition]:
     from .composition import MappedInputPlaceholder
 
-    input_def_dict: Dict[str, InputDefinition] = OrderedDict()
+    input_defs_by_name: Dict[str, InputDefinition] = OrderedDict()
     mapping_keys = set()
 
+    target_input_types_by_graph_input_name: Dict[str, Set[DagsterType]] = defaultdict(set)
+
     for mapping in input_mappings:
         # handle incorrect objects passed in as mappings
         if not isinstance(mapping, InputMapping):
             if isinstance(mapping, InputDefinition):
                 raise DagsterInvalidDefinitionError(
-                    "In {class_name} '{name}' you passed an InputDefinition "
-                    "named '{input_name}' directly in to input_mappings. Return "
-                    "an InputMapping by calling mapping_to on the InputDefinition.".format(
-                        name=name, input_name=mapping.name, class_name=class_name
-                    )
+                    f"In {class_name} '{name}' you passed an InputDefinition "
+                    f"named '{mapping.name}' directly in to input_mappings. Return "
+                    "an InputMapping by calling mapping_to on the InputDefinition."
                 )
             else:
                 raise DagsterInvalidDefinitionError(
-                    "In {class_name} '{name}' received unexpected type '{type}' in input_mappings. "
-                    "Provide an OutputMapping using InputDefinition(...).mapping_to(...)".format(
-                        type=type(mapping), name=name, class_name=class_name
-                    )
+                    f"In {class_name} '{name}' received unexpected type '{type(mapping)}' in input_mappings. "
+                    "Provide an InputMapping using InputMapping(...)"
                 )
 
-        if input_def_dict.get(mapping.definition.name):
-            if input_def_dict[mapping.definition.name] != mapping.definition:
-                raise DagsterInvalidDefinitionError(
-                    "In {class_name} '{name}' multiple input mappings with same "
-                    "definition name but different definitions".format(
-                        name=name, class_name=class_name
-                    ),
-                )
-        else:
-            input_def_dict[mapping.definition.name] = mapping.definition
+        input_defs_by_name[mapping.graph_input_name] = mapping.get_definition()
 
-        target_solid = solid_dict.get(mapping.maps_to.solid_name)
-        if target_solid is None:
+        target_node = nodes_by_name.get(mapping.maps_to.node_name)
+        if target_node is None:
             raise DagsterInvalidDefinitionError(
-                "In {class_name} '{name}' input mapping references solid "
-                "'{solid_name}' which it does not contain.".format(
-                    name=name, solid_name=mapping.maps_to.solid_name, class_name=class_name
-                )
+                f"In {class_name} '{name}' input mapping references node "
+                f"'{mapping.maps_to.node_name}' which it does not contain."
             )
-        if not target_solid.has_input(mapping.maps_to.input_name):
+        if not target_node.has_input(mapping.maps_to.input_name):
             raise DagsterInvalidDefinitionError(
-                "In {class_name} '{name}' input mapping to solid '{mapping.maps_to.solid_name}' "
-                "which contains no input named '{mapping.maps_to.input_name}'".format(
-                    name=name, mapping=mapping, class_name=class_name
-                )
+                f"In {class_name} '{name}' input mapping to node '{mapping.maps_to.node_name}' "
+                f"which contains no input named '{mapping.maps_to.input_name}'"
             )
 
-        target_input = target_solid.input_def_named(mapping.maps_to.input_name)
-        solid_input_handle = SolidInputHandle(target_solid, target_input)
+        target_input_def = target_node.input_def_named(mapping.maps_to.input_name)
+        solid_input_handle = SolidInputHandle(target_node, target_input_def)
 
         if mapping.maps_to_fan_in:
             maps_to = cast(FanInInputPointer, mapping.maps_to)
             if not dependency_structure.has_fan_in_deps(solid_input_handle):
                 raise DagsterInvalidDefinitionError(
                     f"In {class_name} '{name}' input mapping target "
-                    f'"{maps_to.solid_name}.{maps_to.input_name}" (index {maps_to.fan_in_index} of fan-in) '
+                    f'"{maps_to.node_name}.{maps_to.input_name}" (index {maps_to.fan_in_index} of fan-in) '
                     f"is not a MultiDependencyDefinition."
                 )
             inner_deps = dependency_structure.get_fan_in_deps(solid_input_handle)
@@ -869,74 +859,57 @@ def _validate_in_mappings(
             ):
                 raise DagsterInvalidDefinitionError(
                     f"In {class_name} '{name}' input mapping target "
-                    f'"{maps_to.solid_name}.{maps_to.input_name}" index {maps_to.fan_in_index} in '
+                    f'"{maps_to.node_name}.{maps_to.input_name}" index {maps_to.fan_in_index} in '
                     f"the MultiDependencyDefinition is not a MappedInputPlaceholder"
                 )
-            mapping_keys.add(f"{maps_to.solid_name}.{maps_to.input_name}.{maps_to.fan_in_index}")
-            target_type = target_input.dagster_type.get_inner_type_for_fan_in()
-            fan_in_msg = " (index {} of fan-in)".format(maps_to.fan_in_index)
+            mapping_keys.add(f"{maps_to.node_name}.{maps_to.input_name}.{maps_to.fan_in_index}")
+            target_input_types_by_graph_input_name[mapping.graph_input_name].add(
+                target_input_def.dagster_type.get_inner_type_for_fan_in()
+            )
         else:
             if dependency_structure.has_deps(solid_input_handle):
                 raise DagsterInvalidDefinitionError(
-                    "In {class_name} '{name}' input mapping target "
-                    '"{mapping.maps_to.solid_name}.{mapping.maps_to.input_name}" '
-                    "is already satisfied by output".format(
-                        name=name, mapping=mapping, class_name=class_name
-                    )
+                    f"In {class_name} '{name}' input mapping target "
+                    f'"{mapping.maps_to.node_name}.{mapping.maps_to.input_name}" '
+                    "is already satisfied by output"
                 )
 
-            mapping_keys.add(
-                "{mapping.maps_to.solid_name}.{mapping.maps_to.input_name}".format(mapping=mapping)
-            )
-            target_type = target_input.dagster_type
-            fan_in_msg = ""
-
-        if (
-            # no need to check mapping type for graphs because users can't specify ins/out type on graphs
-            class_name not in (GraphDefinition.__name__, SubselectedGraphDefinition.__name__)
-            and target_type != mapping.definition.dagster_type
-        ):
-            raise DagsterInvalidDefinitionError(
-                "In {class_name} '{name}' input "
-                "'{mapping.definition.name}' of type {mapping.definition.dagster_type.display_name} maps to "
-                "{mapping.maps_to.solid_name}.{mapping.maps_to.input_name}{fan_in_msg} of different type "
-                "{target_type.display_name}. InputMapping source and "
-                "destination must have the same type.".format(
-                    mapping=mapping,
-                    name=name,
-                    target_type=target_type,
-                    class_name=class_name,
-                    fan_in_msg=fan_in_msg,
-                )
+            mapping_keys.add(f"{mapping.maps_to.node_name}.{mapping.maps_to.input_name}")
+            target_input_types_by_graph_input_name[mapping.graph_input_name].add(
+                target_input_def.dagster_type
             )
 
     for input_handle in dependency_structure.input_handles():
         if dependency_structure.has_fan_in_deps(input_handle):
             for idx, dep in enumerate(dependency_structure.get_fan_in_deps(input_handle)):
                 if dep is MappedInputPlaceholder:
-                    mapping_str = (
-                        "{input_handle.solid_name}.{input_handle.input_name}.{idx}".format(
-                            input_handle=input_handle, idx=idx
-                        )
-                    )
+                    mapping_str = f"{input_handle.node_name}.{input_handle.input_name}.{idx}"
                     if mapping_str not in mapping_keys:
                         raise DagsterInvalidDefinitionError(
-                            "Unsatisfied MappedInputPlaceholder at index {idx} in "
-                            "MultiDependencyDefinition for '{input_handle.solid_name}.{input_handle.input_name}'".format(
-                                input_handle=input_handle, idx=idx
-                            )
+                            f"Unsatisfied MappedInputPlaceholder at index {idx} in "
+                            f"MultiDependencyDefinition for '{input_handle.node_name}.{input_handle.input_name}'"
                         )
 
-    return input_mappings, list(input_def_dict.values())
+    # if the dagster type on a graph input is Any and all its target inputs have the
+    # same dagster type, then use that dagster type for the graph input
+    for graph_input_name, graph_input_def in input_defs_by_name.items():
+        if graph_input_def.dagster_type.kind == DagsterTypeKind.ANY:
+            target_input_types = target_input_types_by_graph_input_name[graph_input_name]
+            if len(target_input_types) == 1:
+                input_defs_by_name[graph_input_name] = graph_input_def.with_dagster_type(
+                    next(iter(target_input_types))
+                )
+
+    return list(input_defs_by_name.values())
 
 
 def _validate_out_mappings(
     output_mappings: Sequence[OutputMapping],
     solid_dict: Mapping[str, Node],
-    dependency_structure: DependencyStructure,
     name: str,
     class_name: str,
-) -> Sequence[OutputMapping]:
+) -> Tuple[Sequence[OutputMapping], Sequence[OutputDefinition]]:
+    output_defs = []
     for mapping in output_mappings:
         if isinstance(mapping, OutputMapping):
 
@@ -960,15 +933,18 @@ def _validate_out_mappings(
                 )
 
             target_output = target_solid.output_def_named(mapping.maps_from.output_name)
+            output_def = mapping.get_definition(is_dynamic=target_output.is_dynamic)
+            output_defs.append(output_def)
 
             if (
-                mapping.definition.dagster_type.kind != DagsterTypeKind.ANY
-                and (target_output.dagster_type != mapping.definition.dagster_type)
+                mapping.dagster_type
+                and mapping.dagster_type.kind != DagsterTypeKind.ANY
+                and (target_output.dagster_type != mapping.dagster_type)
                 and class_name != "GraphDefinition"
             ):
                 raise DagsterInvalidDefinitionError(
                     "In {class_name} '{name}' output "
-                    "'{mapping.definition.name}' of type {mapping.definition.dagster_type.display_name} "
+                    "'{mapping.graph_output_name}' of type {mapping.dagster_type.display_name} "
                     "maps from {mapping.maps_from.solid_name}.{mapping.maps_from.output_name} of different type "
                     "{target_output.dagster_type.display_name}. OutputMapping source "
                     "and destination must have the same type.".format(
@@ -979,23 +955,6 @@ def _validate_out_mappings(
                     )
                 )
 
-            if target_output.is_dynamic and not mapping.definition.is_dynamic:
-                raise DagsterInvalidDefinitionError(
-                    f'In {class_name} "{name}" can not map from {target_output.__class__.__name__} '
-                    f'"{target_output.name}" to {mapping.definition.__class__.__name__} '
-                    f'"{mapping.definition.name}". Definition types must align.'
-                )
-
-            dynamic_handle = dependency_structure.get_upstream_dynamic_handle_for_solid(
-                target_solid.name
-            )
-            if dynamic_handle and not mapping.definition.is_dynamic:
-                raise DagsterInvalidDefinitionError(
-                    f'In {class_name} "{name}" output "{mapping.definition.name}" mapping from '
-                    f"{target_solid.describe_node()} must be a DynamicOutputDefinition since it is "
-                    f'downstream of dynamic output "{dynamic_handle.describe()}".'
-                )
-
         elif isinstance(mapping, OutputDefinition):
             raise DagsterInvalidDefinitionError(
                 "You passed an OutputDefinition named '{output_name}' directly "
@@ -1009,4 +968,4 @@ def _validate_out_mappings(
                     type=type(mapping)
                 )
             )
-    return output_mappings
+    return output_mappings, output_defs
