@@ -1,3 +1,6 @@
+import importlib
+import os
+import warnings
 from functools import update_wrapper
 from typing import (
     TYPE_CHECKING,
@@ -17,7 +20,7 @@ from typing import (
 
 import dagster._check as check
 from dagster._annotations import public
-from dagster._config import Field, Shape
+from dagster._config import Field, Shape, StringSource
 from dagster._config.config_type import ConfigType
 from dagster._config.validate import validate_config
 from dagster._core.definitions.composition import MappedInputPlaceholder
@@ -47,7 +50,7 @@ from dagster._core.selector.subset_selector import (
     OpSelectionData,
     parse_op_selection,
 )
-from dagster._core.storage.io_manager import io_manager
+from dagster._core.storage.io_manager import IOManagerDefinition, io_manager
 from dagster._core.utils import str_format_set
 from dagster._utils import merge_dicts
 
@@ -171,48 +174,40 @@ class JobDefinition(PipelineDefinition):
         presets = []
         config_mapping = None
         partitioned_config = None
+        self._explicit_config = False
 
         if partitions_def:
-            check.invariant(
-                not isinstance(config, ConfigMapping),
-                "Can't supply a ConfigMappi/ng for 'config' when 'partitions_def' is supplied.",
-            )
-
-            if isinstance(config, PartitionedConfig):
-                check.invariant(
-                    config.partitions_def == partitions_def,
-                    "Can't supply a PartitionedConfig for 'config' with a different "
-                    "PartitionsDefinition than supplied for 'partitions_def'.",
-                )
+            partitioned_config = PartitionedConfig.from_flexible_config(config, partitions_def)
+        else:
+            if isinstance(config, ConfigMapping):
+                config_mapping = config
+            elif isinstance(config, PartitionedConfig):
                 partitioned_config = config
-            else:
-                hardcoded_config = config if config else {}
-                partitioned_config = PartitionedConfig(partitions_def, lambda _: hardcoded_config)
-
-        if isinstance(config, ConfigMapping):
-            config_mapping = config
-        elif isinstance(config, PartitionedConfig):
-            partitioned_config = config
-        elif isinstance(config, dict):
-            check.invariant(
-                len(_preset_defs) == 0,
-                "Bad state: attempted to pass preset definitions to job alongside config dictionary.",
-            )
-            presets = [PresetDefinition(name="default", run_config=config)]
-            # Using config mapping here is a trick to make it so that the preset will be used even
-            # when no config is supplied for the job.
-            config_mapping = _config_mapping_with_default_value(
-                get_run_config_schema_for_job(
-                    graph_def, resource_defs_with_defaults, executor_def, logger_defs, asset_layer
-                ),
-                config,
-                name,
-            )
-        elif config is not None:
-            check.failed(
-                f"config param must be a ConfigMapping, a PartitionedConfig, or a dictionary, but "
-                f"is an object of type {type(config)}"
-            )
+            elif isinstance(config, dict):
+                check.invariant(
+                    len(_preset_defs) == 0,
+                    "Bad state: attempted to pass preset definitions to job alongside config dictionary.",
+                )
+                presets = [PresetDefinition(name="default", run_config=config)]
+                # Using config mapping here is a trick to make it so that the preset will be used even
+                # when no config is supplied for the job.
+                config_mapping = _config_mapping_with_default_value(
+                    get_run_config_schema_for_job(
+                        graph_def,
+                        resource_defs_with_defaults,
+                        executor_def,
+                        logger_defs,
+                        asset_layer,
+                    ),
+                    config,
+                    name,
+                )
+                self._explicit_config = True
+            elif config is not None:
+                check.failed(
+                    f"config param must be a ConfigMapping, a PartitionedConfig, or a dictionary, but "
+                    f"is an object of type {type(config)}"
+                )
 
         # Exists for backcompat - JobDefinition is implemented as a single-mode pipeline.
         mode_def = ModeDefinition(
@@ -294,6 +289,7 @@ class JobDefinition(PipelineDefinition):
         asset_selection: Optional[Sequence[AssetKey]] = None,
         run_id: Optional[str] = None,
         input_values: Optional[Mapping[str, object]] = None,
+        tags: Optional[Mapping[str, str]] = None,
     ) -> "ExecuteInProcessResult":
         """
         Execute the Job in-process, gathering results in-memory.
@@ -370,16 +366,12 @@ class JobDefinition(PipelineDefinition):
             op_selection, frozenset(asset_selection) if asset_selection else None
         )
 
-        tags = None
+        merged_tags = merge_dicts(self.tags, tags or {})
         if partition_key:
             if not self.partitioned_config:
                 check.failed(
                     f"Provided partition key `{partition_key}` for job `{self._name}` without a partitioned config"
                 )
-            check.invariant(
-                not run_config,
-                "Cannot provide both run_config and partition_key arguments to `execute_in_process`",
-            )
             partition_set = self.get_partition_set_def()
             if not partition_set:
                 check.failed(
@@ -387,8 +379,10 @@ class JobDefinition(PipelineDefinition):
                 )
 
             partition = partition_set.get_partition(partition_key)
-            run_config = partition_set.run_config_for_partition(partition)
-            tags = partition_set.tags_for_partition(partition)
+            run_config = (
+                run_config if run_config else partition_set.run_config_for_partition(partition)
+            )
+            merged_tags.update(partition_set.tags_for_partition(partition))
 
         return core_execute_in_process(
             ephemeral_pipeline=ephemeral_job,
@@ -396,7 +390,7 @@ class JobDefinition(PipelineDefinition):
             instance=instance,
             output_capturing_enabled=True,
             raise_on_error=raise_on_error,
-            run_tags=tags,
+            run_tags=merged_tags,
             run_id=run_id,
             asset_selection=frozenset(asset_selection),
         )
@@ -445,20 +439,22 @@ class JobDefinition(PipelineDefinition):
     ) -> "JobDefinition":
         asset_selection = check.opt_set_param(asset_selection, "asset_selection", AssetKey)
 
-        for asset in asset_selection:
-            nonexistent_assets = [
-                asset for asset in asset_selection if asset not in self.asset_layer.asset_keys
-            ]
-            nonexistent_asset_strings = [
-                asset_str
-                for asset_str in (asset.to_string() for asset in nonexistent_assets)
-                if asset_str
-            ]
-            if nonexistent_assets:
-                raise DagsterInvalidSubsetError(
-                    "Assets provided in asset_selection argument "
-                    f"{', '.join(nonexistent_asset_strings)} do not exist in parent asset group or job."
-                )
+        nonexistent_assets = [
+            asset
+            for asset in asset_selection
+            if asset not in self.asset_layer.asset_keys
+            and asset not in self.asset_layer.source_assets_by_key
+        ]
+        nonexistent_asset_strings = [
+            asset_str
+            for asset_str in (asset.to_string() for asset in nonexistent_assets)
+            if asset_str
+        ]
+        if nonexistent_assets:
+            raise DagsterInvalidSubsetError(
+                "Assets provided in asset_selection argument "
+                f"{', '.join(nonexistent_asset_strings)} do not exist in parent asset group or job."
+            )
         asset_selection_data = AssetSelectionData(
             asset_selection=asset_selection,
             parent_job_def=self,
@@ -479,7 +475,7 @@ class JobDefinition(PipelineDefinition):
             tags=self.tags,
             asset_selection=asset_selection,
             asset_selection_data=asset_selection_data,
-            config=self.config_mapping,
+            config=self.config_mapping or self.partitioned_config,
         )
         return new_job
 
@@ -490,20 +486,27 @@ class JobDefinition(PipelineDefinition):
         if not op_selection:
             return self
 
-        op_selection = check.opt_list_param(op_selection, "op_selection", str)
+        op_selection = check.opt_sequence_param(op_selection, "op_selection", str)
 
         resolved_op_selection_dict = parse_op_selection(self, op_selection)
 
         try:
             sub_graph = get_subselected_graph_definition(self.graph, resolved_op_selection_dict)
 
+            # if explicit config was passed the config_mapping that resolves the defaults implicitly is
+            # very unlikely to work. The preset will still present the default config in dagit.
+            if self._explicit_config:
+                config_arg = None
+            else:
+                config_arg = self.config_mapping or self.partitioned_config
+
             return JobDefinition(
                 name=self.name,
                 description=self.description,
                 resource_defs=dict(self.resource_defs),
                 logger_defs=dict(self.loggers),
                 executor_def=self.executor_def,
-                config=self.config_mapping or self.partitioned_config,
+                config=config_arg,
                 tags=self.tags,
                 hook_defs=self.hook_defs,
                 op_retry_policy=self._solid_retry_policy,
@@ -567,15 +570,34 @@ class JobDefinition(PipelineDefinition):
     def run_request_for_partition(
         self,
         partition_key: str,
-        run_key: Optional[str],
+        run_key: Optional[str] = None,
         tags: Optional[Mapping[str, str]] = None,
+        asset_selection: Optional[Sequence[AssetKey]] = None,
+        run_config: Optional[Mapping[str, Any]] = None,
     ) -> RunRequest:
+        """
+        Creates a RunRequest object for a run that processes the given partition.
+
+        Args:
+            partition_key: The key of the partition to request a run for.
+            run_key (Optional[str]): A string key to identify this launched run. For sensors, ensures that
+                only one run is created per run key across all sensor evaluations.  For schedules,
+                ensures that one run is created per tick, across failure recoveries. Passing in a `None`
+                value means that a run will always be launched per evaluation.
+            tags (Optional[Dict[str, str]]): A dictionary of tags (string key-value pairs) to attach
+                to the launched run.
+            run_config (Optional[Mapping[str, Any]]: Configuration for the run. If the job has
+                a :py:class:`PartitionedConfig`, this value will override replace the config
+                provided by it.
+
+        Returns:
+            RunRequest: an object that requests a run to process the given partition.
+        """
         partition_set = self.get_partition_set_def()
         if not partition_set:
             check.failed("Called run_request_for_partition on a non-partitioned job")
 
         partition = partition_set.get_partition(partition_key)
-        run_config = partition_set.run_config_for_partition(partition)
         run_request_tags = (
             {**tags, **partition_set.tags_for_partition(partition)}
             if tags
@@ -583,7 +605,13 @@ class JobDefinition(PipelineDefinition):
         )
 
         return RunRequest(
-            run_key=run_key, run_config=run_config, tags=run_request_tags, job_name=self.name
+            run_key=run_key,
+            run_config=run_config
+            if run_config is not None
+            else partition_set.run_config_for_partition(partition),
+            tags=run_request_tags,
+            job_name=self.name,
+            asset_selection=asset_selection,
         )
 
     @public
@@ -812,15 +840,82 @@ def get_direct_input_values_from_job(target: PipelineDefinition) -> Mapping[str,
     description="Built-in filesystem IO manager that stores and retrieves values using pickling."
 )
 def default_job_io_manager(init_context: "InitResourceContext"):
+    # support overriding the default io manager via environment variables
+    module_name = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_MODULE")
+    attribute_name = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_ATTRIBUTE")
+    silence_failures = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_SILENCE_FAILURES")
+
+    if module_name and attribute_name:
+        from dagster._core.execution.build_resources import build_resources
+
+        try:
+            module = importlib.import_module(module_name)
+            attr = getattr(module, attribute_name)
+            check.invariant(
+                isinstance(attr, IOManagerDefinition),
+                "DAGSTER_DEFAULT_IO_MANAGER_MODULE and DAGSTER_DEFAULT_IO_MANAGER_ATTRIBUTE must specify an IOManagerDefinition",
+            )
+            with build_resources({"io_manager": attr}, instance=init_context.instance) as resources:
+                return resources.io_manager
+        except Exception as e:
+            if not silence_failures:
+                raise
+            else:
+                warnings.warn(
+                    f"Failed to load io manager override with module: {module_name} attribute: {attribute_name}: {e}\n"
+                    "Falling back to default io manager."
+                )
+
+    # normally, default to the fs_io_manager
     from dagster._core.storage.fs_io_manager import PickledObjectFilesystemIOManager
 
     instance = check.not_none(init_context.instance)
     return PickledObjectFilesystemIOManager(base_dir=instance.storage_directory())
 
 
+@io_manager(
+    description="Built-in filesystem IO manager that stores and retrieves values using pickling.",
+    config_schema={"base_dir": Field(StringSource, is_required=False)},
+)
+def default_job_io_manager_with_fs_io_manager_schema(init_context: "InitResourceContext"):
+    # support overriding the default io manager via environment variables
+    module_name = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_MODULE")
+    attribute_name = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_ATTRIBUTE")
+    silence_failures = os.getenv("DAGSTER_DEFAULT_IO_MANAGER_SILENCE_FAILURES")
+
+    if module_name and attribute_name:
+        from dagster._core.execution.build_resources import build_resources
+
+        try:
+            module = importlib.import_module(module_name)
+            attr = getattr(module, attribute_name)
+            check.invariant(
+                isinstance(attr, IOManagerDefinition),
+                "DAGSTER_DEFAULT_IO_MANAGER_MODULE and DAGSTER_DEFAULT_IO_MANAGER_ATTRIBUTE must specify an IOManagerDefinition",
+            )
+            with build_resources({"io_manager": attr}, instance=init_context.instance) as resources:
+                return resources.io_manager
+        except Exception as e:
+            if not silence_failures:
+                raise
+            else:
+                warnings.warn(
+                    f"Failed to load io manager override with module: {module_name} attribute: {attribute_name}: {e}\n"
+                    "Falling back to default io manager."
+                )
+    from dagster._core.storage.fs_io_manager import PickledObjectFilesystemIOManager
+
+    # normally, default to the fs_io_manager
+    base_dir = init_context.resource_config.get(
+        "base_dir", init_context.instance.storage_directory() if init_context.instance else None
+    )
+
+    return PickledObjectFilesystemIOManager(base_dir=base_dir)
+
+
 def _config_mapping_with_default_value(
     inner_schema: ConfigType,
-    default_config: Dict[str, Any],
+    default_config: Mapping[str, Any],
     job_name: str,
 ) -> ConfigMapping:
     if not isinstance(inner_schema, Shape):
