@@ -1,5 +1,6 @@
 import copy
 import inspect
+import json
 from abc import ABC, abstractmethod
 from datetime import datetime, time, timedelta
 from enum import Enum
@@ -7,6 +8,7 @@ from typing import (
     Any,
     Callable,
     Generic,
+    Iterable,
     List,
     Mapping,
     NamedTuple,
@@ -23,6 +25,7 @@ from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import PublicAttr, public
+from dagster._core.definitions.partition_key_range import PartitionKeyRange
 from dagster._core.definitions.target import ExecutableDefinition
 from dagster._serdes import whitelist_for_serdes
 from dagster._seven.compat.pendulum import PendulumDateTime, to_timezone
@@ -39,6 +42,7 @@ from ..errors import (
     user_code_error_boundary,
 )
 from ..storage.pipeline_run import PipelineRun
+from .config import ConfigMapping
 from .mode import DEFAULT_MODE_NAME
 from .run_request import RunRequest, SkipReason
 from .schedule_definition import (
@@ -221,12 +225,39 @@ class PartitionsDefinition(ABC, Generic[T]):
 
         return IdentityPartitionMapping()
 
+    def get_partition_keys_in_range(self, partition_key_range: PartitionKeyRange) -> Sequence[str]:
+        partition_keys = self.get_partition_keys()
+
+        keys_exist = {
+            partition_key_range.start: partition_key_range.start in partition_keys,
+            partition_key_range.end: partition_key_range.end in partition_keys,
+        }
+        if not all(keys_exist.values()):
+            raise DagsterInvalidInvocationError(
+                f"""Partition range {partition_key_range.start} to {partition_key_range.end} is
+                not a valid range. Nonexistent partition keys:
+                {list(key for key in keys_exist if keys_exist[key] is False)}"""
+            )
+
+        return partition_keys[
+            partition_keys.index(partition_key_range.start) : partition_keys.index(
+                partition_key_range.end
+            )
+            + 1
+        ]
+
+    def empty_subset(self) -> "PartitionsSubset":
+        return DefaultPartitionsSubset(self, [])
+
+    def deserialize_subset(self, serialized: str) -> "PartitionsSubset":
+        return DefaultPartitionsSubset.from_serialized(self, serialized)
+
 
 class StaticPartitionsDefinition(
     PartitionsDefinition[str],
 ):  # pylint: disable=unsubscriptable-object
     def __init__(self, partition_keys: Sequence[str]):
-        check.list_param(partition_keys, "partition_keys", of_type=str)
+        check.sequence_param(partition_keys, "partition_keys", of_type=str)
 
         # Dagit selects partition ranges following the format '2022-01-13...2022-01-14'
         # "..." is an invalid substring in partition keys
@@ -342,7 +373,12 @@ class ScheduleTimeBasedPartitionsDefinition(
         )
 
     def get_cron_schedule(self) -> str:
-        return get_cron_schedule(self.schedule_type, self.execution_time, self.execution_day)
+        return cron_schedule_from_schedule_type_and_offsets(
+            schedule_type=self.schedule_type,
+            minute_offset=self.execution_time.minute,
+            hour_offset=self.execution_time.hour,
+            day_offset=self.execution_day,
+        )
 
     def get_execution_time_to_partition_fn(self) -> Callable[[datetime], datetime]:
         if self.schedule_type is ScheduleType.HOURLY:
@@ -622,7 +658,7 @@ class PartitionSetDefinition(Generic[T]):
         check.str_param(schedule_name, "schedule_name")
         check.str_param(cron_schedule, "cron_schedule")
         check.opt_callable_param(should_execute, "should_execute")
-        check.opt_dict_param(environment_vars, "environment_vars", key_type=str, value_type=str)
+        check.opt_mapping_param(environment_vars, "environment_vars", key_type=str, value_type=str)
         check.callable_param(partition_selector, "partition_selector")
         check.opt_str_param(execution_timezone, "execution_timezone")
         check.opt_str_param(description, "description")
@@ -842,6 +878,28 @@ class PartitionedConfig(Generic[T]):
             raise DagsterInvalidInvocationError(f"No partition for partition key {partition_key}.")
         return self.run_config_for_partition_fn(partition[0])
 
+    @classmethod
+    def from_flexible_config(
+        cls,
+        config: Optional[Union[ConfigMapping, Mapping[str, object], "PartitionedConfig"]],
+        partitions_def: PartitionsDefinition,
+    ) -> "PartitionedConfig":
+        check.invariant(
+            not isinstance(config, ConfigMapping),
+            "Can't supply a ConfigMapping for 'config' when 'partitions_def' is supplied.",
+        )
+
+        if isinstance(config, PartitionedConfig):
+            check.invariant(
+                config.partitions_def == partitions_def,
+                "Can't supply a PartitionedConfig for 'config' with a different "
+                "PartitionsDefinition than supplied for 'partitions_def'.",
+            )
+            return config
+        else:
+            hardcoded_config = config if config else {}
+            return cls(partitions_def, lambda _: cast(Mapping, hardcoded_config))
+
     def __call__(self, *args, **kwargs):
         if self._decorated_fn is None:
             raise DagsterInvalidInvocationError(
@@ -876,7 +934,7 @@ def static_partitioned_config(
     Returns:
         PartitionedConfig
     """
-    check.list_param(partition_keys, "partition_keys", str)
+    check.sequence_param(partition_keys, "partition_keys", str)
 
     def inner(fn: Callable[[str], Mapping[str, Any]]) -> PartitionedConfig:
         check.callable_param(fn, "fn")
@@ -937,21 +995,60 @@ def dynamic_partitioned_config(
     return inner
 
 
-def get_cron_schedule(
+def cron_schedule_from_schedule_type_and_offsets(
     schedule_type: ScheduleType,
-    time_of_day: time = time(0, 0),
-    execution_day: Optional[int] = None,
-) -> str:
-    minute = time_of_day.minute
-    hour = time_of_day.hour
-
+    minute_offset: int,
+    hour_offset: int,
+    day_offset: Optional[int],
+):
     if schedule_type is ScheduleType.HOURLY:
-        return f"{minute} * * * *"
+        return f"{minute_offset} * * * *"
     elif schedule_type is ScheduleType.DAILY:
-        return f"{minute} {hour} * * *"
+        return f"{minute_offset} {hour_offset} * * *"
     elif schedule_type is ScheduleType.WEEKLY:
-        return f"{minute} {hour} * * {execution_day if execution_day != None else 0}"
+        return f"{minute_offset} {hour_offset} * * {day_offset if day_offset != None else 0}"
     elif schedule_type is ScheduleType.MONTHLY:
-        return f"{minute} {hour} {execution_day if execution_day != None else 1} * *"
+        return f"{minute_offset} {hour_offset} {day_offset if day_offset != None else 1} * *"
     else:
         check.assert_never(schedule_type)
+
+
+class PartitionsSubset(ABC):
+    """Represents a subset of the partitions within a PartitionsDefinition"""
+
+    @abstractmethod
+    def get_partition_keys_not_in_subset(
+        self, current_time: Optional[datetime] = None
+    ) -> Iterable[str]:
+        raise NotImplementedError()
+
+    @abstractmethod
+    def with_partition_keys(self, partition_keys: Iterable[str]) -> "PartitionsSubset":
+        raise NotImplementedError()
+
+    @abstractmethod
+    def serialize(self) -> str:
+        raise NotImplementedError()
+
+
+class DefaultPartitionsSubset(PartitionsSubset):
+    def __init__(self, partitions_def: PartitionsDefinition, subset=None):
+        self._partitions_def = partitions_def
+        self._subset = subset or set()
+
+    def get_partition_keys_not_in_subset(
+        self, current_time: Optional[datetime] = None
+    ) -> Iterable[str]:
+        return set(self._partitions_def.get_partition_keys()) - self._subset
+
+    def with_partition_keys(self, partition_keys: Iterable[str]) -> "DefaultPartitionsSubset":
+        return DefaultPartitionsSubset(self._partitions_def, self._subset | set(partition_keys))
+
+    def serialize(self) -> str:
+        return json.dumps(self._subset)
+
+    @staticmethod
+    def from_serialized(
+        partitions_def: PartitionsDefinition, serialized: str
+    ) -> "DefaultPartitionsSubset":
+        return DefaultPartitionsSubset(subset=json.loads(serialized), partitions_def=partitions_def)
