@@ -6,10 +6,14 @@ from typing import (
     Any,
     Callable,
     Dict,
+    FrozenSet,
     Generic,
+    Iterable,
     List,
     Mapping,
+    NamedTuple,
     Optional,
+    Sequence,
     Set,
     Type,
     TypeVar,
@@ -20,9 +24,15 @@ from typing import (
 import dagster._check as check
 from dagster._annotations import public
 from dagster._core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
-from dagster._utils import merge_dicts
+from dagster._core.instance import DagsterInstance
+from dagster._core.selector import parse_solid_selection
+from dagster._serdes import whitelist_for_serdes
+from dagster._utils import make_readonly_value, merge_dicts
 
-from .events import AssetKey
+from .asset_selection import AssetGraph
+from .assets_job import ASSET_BASE_JOB_PREFIX, get_base_asset_jobs, is_base_asset_job_name
+from .cacheable_assets import AssetsDefinitionCacheableData
+from .events import AssetKey, CoercibleToAssetKey
 from .executor_definition import ExecutorDefinition
 from .graph_definition import GraphDefinition, SubselectedGraphDefinition
 from .job_definition import JobDefinition
@@ -36,7 +46,9 @@ from .unresolved_asset_job_definition import UnresolvedAssetJobDefinition
 from .utils import check_valid_name
 
 if TYPE_CHECKING:
-    from dagster._core.definitions import AssetGroup
+    from dagster._core.definitions import AssetGroup, AssetsDefinition
+    from dagster._core.definitions.cacheable_assets import CacheableAssetsDefinition
+    from dagster._core.storage.asset_value_loader import AssetValueLoader
 
 VALID_REPOSITORY_DATA_DICT_KEYS = {
     "pipelines",
@@ -55,6 +67,41 @@ RepositoryLevelDefinition = TypeVar(
     SensorDefinition,
 )
 
+RepositoryListDefinition = Union[
+    "AssetsDefinition",
+    "AssetGroup",
+    GraphDefinition,
+    PartitionSetDefinition,
+    PipelineDefinition,
+    ScheduleDefinition,
+    SensorDefinition,
+    SourceAsset,
+    UnresolvedAssetJobDefinition,
+]
+
+
+@whitelist_for_serdes
+class RepositoryLoadData(
+    NamedTuple(
+        "_RepositoryLoadData",
+        [
+            ("cached_data_by_key", Mapping[str, Sequence[AssetsDefinitionCacheableData]]),
+        ],
+    )
+):
+    def __new__(cls, cached_data_by_key: Mapping[str, Sequence[AssetsDefinitionCacheableData]]):
+        return super(RepositoryLoadData, cls).__new__(
+            cls,
+            cached_data_by_key=make_readonly_value(
+                check.mapping_param(
+                    cached_data_by_key,
+                    "cached_data_by_key",
+                    key_type=str,
+                    value_type=list,
+                )
+            ),
+        )
+
 
 class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
     def __init__(
@@ -66,7 +113,7 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
             str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
         ],
         validation_fn: Callable[[RepositoryLevelDefinition], RepositoryLevelDefinition],
-        lazy_definitions_fn: Optional[Callable[[], List[RepositoryLevelDefinition]]] = None,
+        lazy_definitions_fn: Optional[Callable[[], Sequence[RepositoryLevelDefinition]]] = None,
     ):
         """
         Args:
@@ -100,16 +147,16 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
             str, Union[RepositoryLevelDefinition, Callable[[], RepositoryLevelDefinition]]
         ] = definitions
         self._definition_cache: Dict[str, RepositoryLevelDefinition] = {}
-        self._definition_names: Optional[List[str]] = None
+        self._definition_names: Optional[Sequence[str]] = None
 
         self._lazy_definitions_fn: Callable[
-            [], List[RepositoryLevelDefinition]
+            [], Sequence[RepositoryLevelDefinition]
         ] = lazy_definitions_fn or (lambda: [])
-        self._lazy_definitions: Optional[List[RepositoryLevelDefinition]] = None
+        self._lazy_definitions: Optional[Sequence[RepositoryLevelDefinition]] = None
 
-        self._all_definitions: Optional[List[RepositoryLevelDefinition]] = None
+        self._all_definitions: Optional[Sequence[RepositoryLevelDefinition]] = None
 
-    def _get_lazy_definitions(self) -> List[RepositoryLevelDefinition]:
+    def _get_lazy_definitions(self) -> Sequence[RepositoryLevelDefinition]:
         if self._lazy_definitions is None:
             self._lazy_definitions = self._lazy_definitions_fn()
             for definition in self._lazy_definitions:
@@ -117,7 +164,7 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
 
         return self._lazy_definitions
 
-    def get_definition_names(self) -> List[str]:
+    def get_definition_names(self) -> Sequence[str]:
         if self._definition_names:
             return self._definition_names
 
@@ -140,7 +187,7 @@ class _CacheingDefinitionIndex(Generic[RepositoryLevelDefinition]):
 
         return definition_name in self.get_definition_names()
 
-    def get_all_definitions(self) -> List[RepositoryLevelDefinition]:
+    def get_all_definitions(self) -> Sequence[RepositoryLevelDefinition]:
         if self._all_definitions is not None:
             return self._all_definitions
 
@@ -218,7 +265,7 @@ class RepositoryData(ABC):
     """
 
     @abstractmethod
-    def get_all_pipelines(self) -> List[PipelineDefinition]:
+    def get_all_pipelines(self) -> Sequence[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Returns:
@@ -226,7 +273,7 @@ class RepositoryData(ABC):
         """
 
     @public
-    def get_all_jobs(self) -> List[JobDefinition]:
+    def get_all_jobs(self) -> Sequence[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Returns:
@@ -234,7 +281,7 @@ class RepositoryData(ABC):
         """
         return [job for job in self.get_all_pipelines() if isinstance(job, JobDefinition)]
 
-    def get_pipeline_names(self) -> List[str]:
+    def get_pipeline_names(self) -> Sequence[str]:
         """Get the names of all pipelines/jobs in the repository.
 
         Returns:
@@ -243,7 +290,7 @@ class RepositoryData(ABC):
         return [pipeline_def.name for pipeline_def in self.get_all_pipelines()]
 
     @public
-    def get_job_names(self) -> List[str]:
+    def get_job_names(self) -> Sequence[str]:
         """Get the names of all jobs in the repository.
 
         Returns:
@@ -326,7 +373,7 @@ class RepositoryData(ABC):
         """
         return partition_set_name in self.get_partition_set_names()
 
-    def get_all_partition_sets(self) -> List[PartitionSetDefinition]:
+    def get_all_partition_sets(self) -> Sequence[PartitionSetDefinition]:
         """Return all partition sets in the repository as a list.
 
         Returns:
@@ -355,7 +402,7 @@ class RepositoryData(ABC):
         return partition_sets_with_name[0]
 
     @public
-    def get_schedule_names(self) -> List[str]:
+    def get_schedule_names(self) -> Sequence[str]:
         """Get the names of all schedules in the repository.
 
         Returns:
@@ -364,7 +411,7 @@ class RepositoryData(ABC):
         return [schedule.name for schedule in self.get_all_schedules()]
 
     @public
-    def get_all_schedules(self) -> List[ScheduleDefinition]:
+    def get_all_schedules(self) -> Sequence[ScheduleDefinition]:
         """Return all schedules in the repository as a list.
 
         Returns:
@@ -395,11 +442,11 @@ class RepositoryData(ABC):
         return schedule_name in self.get_schedule_names()
 
     @public
-    def get_all_sensors(self) -> List[SensorDefinition]:
+    def get_all_sensors(self) -> Sequence[SensorDefinition]:
         return []
 
     @public
-    def get_sensor_names(self) -> List[str]:
+    def get_sensor_names(self) -> Sequence[str]:
         return [sensor.name for sensor in self.get_all_sensors()]
 
     @public
@@ -438,8 +485,8 @@ Resolvable = Callable[[], T]
 class CachingRepositoryData(RepositoryData):
     """Default implementation of RepositoryData used by the :py:func:`@repository <repository>` decorator."""
 
-    _all_jobs: Optional[List[JobDefinition]]
-    _all_pipelines: Optional[List[PipelineDefinition]]
+    _all_jobs: Optional[Sequence[JobDefinition]]
+    _all_pipelines: Optional[Sequence[PipelineDefinition]]
 
     def __init__(
         self,
@@ -451,6 +498,7 @@ class CachingRepositoryData(RepositoryData):
         schedules: Mapping[str, Union[ScheduleDefinition, Resolvable[ScheduleDefinition]]],
         sensors: Mapping[str, Union[SensorDefinition, Resolvable[SensorDefinition]]],
         source_assets_by_key: Mapping[AssetKey, SourceAsset],
+        assets_defs_by_key: Mapping[AssetKey, "AssetsDefinition"],
     ):
         """Constructs a new CachingRepositoryData object.
 
@@ -475,7 +523,11 @@ class CachingRepositoryData(RepositoryData):
             sensors (Mapping[str, Union[SensorDefinition, Callable[[], SensorDefinition]]]):
                 The sensors belonging to a repository.
             source_assets_by_key (Mapping[AssetKey, SourceAsset]): The source assets belonging to a repository.
+            assets_defs_by_key (Mapping[AssetKey, AssetsDefinition]): The assets definitions
+                belonging to a repository.
         """
+        from dagster._core.definitions import AssetsDefinition
+
         check.mapping_param(
             pipelines, "pipelines", key_type=str, value_type=(PipelineDefinition, FunctionType)
         )
@@ -495,6 +547,9 @@ class CachingRepositoryData(RepositoryData):
         check.mapping_param(
             source_assets_by_key, "source_assets_by_key", key_type=AssetKey, value_type=SourceAsset
         )
+        check.mapping_param(
+            assets_defs_by_key, "assets_defs_by_key", key_type=AssetKey, value_type=AssetsDefinition
+        )
 
         self._pipelines = _CacheingDefinitionIndex(
             PipelineDefinition,
@@ -527,8 +582,9 @@ class CachingRepositoryData(RepositoryData):
             ],
         )
         self._source_assets_by_key = source_assets_by_key
+        self._assets_defs_by_key = assets_defs_by_key
 
-        def load_partition_sets_from_pipelines() -> List[PartitionSetDefinition]:
+        def load_partition_sets_from_pipelines() -> Sequence[PartitionSetDefinition]:
             job_partition_sets = []
             for pipeline in self.get_all_pipelines():
                 if isinstance(pipeline, JobDefinition):
@@ -633,22 +689,14 @@ class CachingRepositoryData(RepositoryData):
                     f"Object mapped to {key} is not an instance of JobDefinition or GraphDefinition."
                 )
 
-        return CachingRepositoryData(**repository_definitions, source_assets_by_key={})
+        return CachingRepositoryData(
+            **repository_definitions, source_assets_by_key={}, assets_defs_by_key={}
+        )
 
     @classmethod
     def from_list(
         cls,
-        repository_definitions: List[
-            Union[
-                PipelineDefinition,
-                PartitionSetDefinition,
-                ScheduleDefinition,
-                SensorDefinition,
-                "AssetGroup",
-                GraphDefinition,
-                UnresolvedAssetJobDefinition,
-            ]
-        ],
+        repository_definitions: Sequence[RepositoryListDefinition],
         default_executor_def: Optional[ExecutorDefinition] = None,
         default_logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
     ) -> "CachingRepositoryData":
@@ -680,7 +728,7 @@ class CachingRepositoryData(RepositoryData):
                     raise DagsterInvalidDefinitionError(
                         f"Duplicate {definition.target_type} definition found for {definition.describe_target()}"
                     )
-                if AssetGroup.is_base_job_name(definition.name):
+                if is_base_asset_job_name(definition.name):
                     raise DagsterInvalidDefinitionError(
                         f"Attempted to provide job called {definition.name} to repository, which "
                         "is a reserved name. Please rename the job."
@@ -759,15 +807,24 @@ class CachingRepositoryData(RepositoryData):
             )
 
         if combined_asset_group:
-            for job_def in combined_asset_group.get_base_jobs():
+            for job_def in get_base_asset_jobs(
+                assets=combined_asset_group.assets,
+                source_assets=combined_asset_group.source_assets,
+                executor_def=combined_asset_group.executor_def,
+                resource_defs=combined_asset_group.resource_defs,
+            ):
                 pipelines_or_jobs[job_def.name] = job_def
 
             source_assets_by_key = {
                 source_asset.key: source_asset
                 for source_asset in combined_asset_group.source_assets
             }
+            assets_defs_by_key = {
+                key: asset for asset in combined_asset_group.assets for key in asset.keys
+            }
         else:
             source_assets_by_key = {}
+            assets_defs_by_key = {}
 
         for name, sensor_def in sensors.items():
             if sensor_def.has_loadable_targets():
@@ -823,17 +880,18 @@ class CachingRepositoryData(RepositoryData):
             schedules=schedules,
             sensors=sensors,
             source_assets_by_key=source_assets_by_key,
+            assets_defs_by_key=assets_defs_by_key,
         )
 
-    def get_pipeline_names(self) -> List[str]:
+    def get_pipeline_names(self) -> Sequence[str]:
         """Get the names of all pipelines/jobs in the repository.
 
         Returns:
             List[str]
         """
-        return self._pipelines.get_definition_names() + self.get_job_names()
+        return [*self._pipelines.get_definition_names(), *self.get_job_names()]
 
-    def get_job_names(self) -> List[str]:
+    def get_job_names(self) -> Sequence[str]:
         """Get the names of all jobs in the repository.
 
         Returns:
@@ -868,7 +926,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(job_name, "job_name")
         return self._jobs.has_definition(job_name)
 
-    def get_all_pipelines(self) -> List[PipelineDefinition]:
+    def get_all_pipelines(self) -> Sequence[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Note that this will construct any pipeline/job that has not yet been constructed.
@@ -888,7 +946,7 @@ class CachingRepositoryData(RepositoryData):
         self._all_pipelines = pipelines
         return self._all_pipelines
 
-    def get_all_jobs(self) -> List[JobDefinition]:
+    def get_all_jobs(self) -> Sequence[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Note that this will construct any job that has not yet been constructed.
@@ -905,7 +963,7 @@ class CachingRepositoryData(RepositoryData):
         self.get_all_pipelines()
 
         # The `get_all_pipelines` call ensures _all_jobs is set.
-        return cast(List[JobDefinition], self._all_jobs)
+        return cast(Sequence[JobDefinition], self._all_jobs)
 
     def get_pipeline(self, pipeline_name: str) -> PipelineDefinition:
         """Get a pipeline/job by name.
@@ -943,7 +1001,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(job_name, "job_name")
         return self._jobs.get_definition(job_name)
 
-    def get_partition_set_names(self) -> List[str]:
+    def get_partition_set_names(self) -> Sequence[str]:
         """Get the names of all partition sets in the repository.
 
         Returns:
@@ -963,7 +1021,7 @@ class CachingRepositoryData(RepositoryData):
         check.str_param(partition_set_name, "partition_set_name")
         return self._partition_sets.has_definition(partition_set_name)
 
-    def get_all_partition_sets(self) -> List[PartitionSetDefinition]:
+    def get_all_partition_sets(self) -> Sequence[PartitionSetDefinition]:
         """Return all partition sets in the repository as a list.
 
         Note that this will construct any partition set that has not yet been constructed.
@@ -990,7 +1048,7 @@ class CachingRepositoryData(RepositoryData):
 
         return self._partition_sets.get_definition(partition_set_name)
 
-    def get_schedule_names(self) -> List[str]:
+    def get_schedule_names(self) -> Sequence[str]:
         """Get the names of all schedules in the repository.
 
         Returns:
@@ -998,7 +1056,7 @@ class CachingRepositoryData(RepositoryData):
         """
         return self._schedules.get_definition_names()
 
-    def get_all_schedules(self) -> List[ScheduleDefinition]:
+    def get_all_schedules(self) -> Sequence[ScheduleDefinition]:
         """Return all schedules in the repository as a list.
 
         Note that this will construct any schedule that has not yet been constructed.
@@ -1030,10 +1088,10 @@ class CachingRepositoryData(RepositoryData):
 
         return self._schedules.has_definition(schedule_name)
 
-    def get_all_sensors(self) -> List[SensorDefinition]:
+    def get_all_sensors(self) -> Sequence[SensorDefinition]:
         return self._sensors.get_all_definitions()
 
-    def get_sensor_names(self) -> List[str]:
+    def get_sensor_names(self) -> Sequence[str]:
         return self._sensors.get_definition_names()
 
     def get_sensor(self, sensor_name: str) -> SensorDefinition:
@@ -1045,7 +1103,10 @@ class CachingRepositoryData(RepositoryData):
     def get_source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
         return self._source_assets_by_key
 
-    def _check_solid_defs(self, pipelines: List[PipelineDefinition]) -> None:
+    def get_assets_defs_by_key(self) -> Mapping[AssetKey, "AssetsDefinition"]:
+        return self._assets_defs_by_key
+
+    def _check_solid_defs(self, pipelines: Sequence[PipelineDefinition]) -> None:
         solid_defs = {}
         solid_to_pipeline = {}
         for pipeline in pipelines:
@@ -1065,7 +1126,7 @@ class CachingRepositoryData(RepositoryData):
                     raise DagsterInvalidDefinitionError(
                         (
                             f"Conflicting definitions found in repository with name '{solid_def.name}'. "
-                            "Op/Graph/Solid definition names must be unique within a "
+                            "Op/Graph definition names must be unique within a "
                             f"repository. {solid_def.__class__.__name__} is defined in {pipeline.target_type} "
                             f"'{first_name}' and in {pipeline.target_type} '{second_name}'."
                         )
@@ -1127,10 +1188,18 @@ class RepositoryDefinition:
         *,
         repository_data,
         description=None,
+        repository_load_data=None,
     ):
         self._name = check_valid_name(name)
         self._description = check.opt_str_param(description, "description")
         self._repository_data = check.inst_param(repository_data, "repository_data", RepositoryData)
+        self._repository_load_data = check.opt_inst_param(
+            repository_load_data, "repository_load_data", RepositoryLoadData
+        )
+
+    @property
+    def repository_load_data(self) -> Optional[RepositoryLoadData]:
+        return self._repository_load_data
 
     @public  # type: ignore
     @property
@@ -1147,13 +1216,13 @@ class RepositoryDefinition:
         self._repository_data.load_all_definitions()
 
     @property
-    def pipeline_names(self) -> List[str]:
+    def pipeline_names(self) -> Sequence[str]:
         """List[str]: Names of all pipelines/jobs in the repository"""
         return self._repository_data.get_pipeline_names()
 
     @public  # type: ignore
     @property
-    def job_names(self) -> List[str]:
+    def job_names(self) -> Sequence[str]:
         """List[str]: Names of all jobs in the repository"""
         return self._repository_data.get_job_names()
 
@@ -1183,7 +1252,7 @@ class RepositoryDefinition:
         """
         return self._repository_data.get_pipeline(name)
 
-    def get_all_pipelines(self) -> List[PipelineDefinition]:
+    def get_all_pipelines(self) -> Sequence[PipelineDefinition]:
         """Return all pipelines/jobs in the repository as a list.
 
         Note that this will construct any pipeline/job in the lazily evaluated dictionary that
@@ -1223,8 +1292,8 @@ class RepositoryDefinition:
         """
         return self._repository_data.get_job(name)
 
-    @public  # type: ignore
-    def get_all_jobs(self) -> List[JobDefinition]:
+    @public
+    def get_all_jobs(self) -> Sequence[JobDefinition]:
         """Return all jobs in the repository as a list.
 
         Note that this will construct any job in the lazily evaluated dictionary that has
@@ -1236,7 +1305,7 @@ class RepositoryDefinition:
         return self._repository_data.get_all_jobs()
 
     @property
-    def partition_set_defs(self) -> List[PartitionSetDefinition]:
+    def partition_set_defs(self) -> Sequence[PartitionSetDefinition]:
         return self._repository_data.get_all_partition_sets()
 
     def get_partition_set_def(self, name: str) -> PartitionSetDefinition:
@@ -1244,7 +1313,7 @@ class RepositoryDefinition:
 
     @public  # type: ignore
     @property
-    def schedule_defs(self) -> List[ScheduleDefinition]:
+    def schedule_defs(self) -> Sequence[ScheduleDefinition]:
         return self._repository_data.get_all_schedules()
 
     @public  # type: ignore
@@ -1257,7 +1326,7 @@ class RepositoryDefinition:
 
     @public  # type: ignore
     @property
-    def sensor_defs(self) -> List[SensorDefinition]:
+    def sensor_defs(self) -> Sequence[SensorDefinition]:
         return self._repository_data.get_all_sensors()
 
     @public  # type: ignore
@@ -1269,15 +1338,214 @@ class RepositoryDefinition:
         return self._repository_data.has_sensor(name)
 
     @property
-    def source_assets_by_key(self) -> Dict[AssetKey, SourceAsset]:
+    def source_assets_by_key(self) -> Mapping[AssetKey, SourceAsset]:
         return self._repository_data.get_source_assets_by_key()
 
+    @property
+    def _assets_defs_by_key(self) -> Mapping[AssetKey, "AssetsDefinition"]:
+        return self._repository_data.get_assets_defs_by_key()
+
+    def get_base_asset_job_names(self) -> Sequence[str]:
+        return [
+            job_name for job_name in self.job_names if job_name.startswith(ASSET_BASE_JOB_PREFIX)
+        ]
+
+    def get_base_job_for_assets(self, asset_keys: Iterable[AssetKey]) -> Optional[JobDefinition]:
+        """
+        Returns the asset base job that contains all the given assets, or None if there is no such
+        job.
+        """
+        if self.has_job(ASSET_BASE_JOB_PREFIX):
+            base_job = self.get_job(ASSET_BASE_JOB_PREFIX)
+            if all(key in base_job.asset_layer.assets_defs_by_key for key in asset_keys):
+                return base_job
+        else:
+            i = 0
+            while self.has_job(f"{ASSET_BASE_JOB_PREFIX}_{i}"):
+                base_job = self.get_job(f"{ASSET_BASE_JOB_PREFIX}_{i}")
+                if all(key in base_job.asset_layer.assets_defs_by_key for key in asset_keys):
+                    return base_job
+
+                i += 1
+
+        return None
+
+    def get_maybe_subset_job_def(
+        self,
+        job_name: str,
+        op_selection: Optional[Sequence[str]] = None,
+        asset_selection: Optional[FrozenSet[AssetKey]] = None,
+        solids_to_execute: Optional[FrozenSet[str]] = None,
+    ):
+        # named job forward expecting pipeline distinction to be removed soon
+        defn = self.get_pipeline(job_name)
+        if isinstance(defn, JobDefinition):
+            return defn.get_job_def_for_subset_selection(op_selection, asset_selection)
+
+        check.invariant(
+            asset_selection is None,
+            f"Asset selection cannot be provided with a pipeline {asset_selection}",
+        )
+        # pipelines use post-resolved selection, should be removed soon
+        if op_selection and solids_to_execute is None:
+            solids_to_execute = parse_solid_selection(defn, op_selection)
+
+        return defn.get_pipeline_subset_def(solids_to_execute)
+
+    @public
+    def load_asset_value(
+        self,
+        asset_key: CoercibleToAssetKey,
+        *,
+        python_type: Optional[Type] = None,
+        instance: Optional[DagsterInstance] = None,
+        partition_key: Optional[str] = None,
+    ) -> object:
+        """
+        Loads the contents of an asset as a Python object.
+
+        Invokes `load_input` on the :py:class:`IOManager` associated with the asset.
+
+        If you want to load the values of multiple assets, it's more efficient to use
+        :py:meth:`~dagster.RepositoryDefinition.get_asset_value_loader`, which avoids spinning up
+        resources separately for each asset.
+
+        Args:
+            asset_key (Union[AssetKey, Sequence[str], str]): The key of the asset to load.
+            python_type (Optional[Type]): The python type to load the asset as. This is what will
+                be returned inside `load_input` by `context.dagster_type.typing_type`.
+            partition_key (Optional[str]): The partition of the asset to load.
+
+        Returns:
+            The contents of an asset as a Python object.
+        """
+        from dagster._core.storage.asset_value_loader import AssetValueLoader
+
+        with AssetValueLoader(self._assets_defs_by_key, instance=instance) as loader:
+            return loader.load_asset_value(
+                asset_key, python_type=python_type, partition_key=partition_key
+            )
+
+    @public
+    def get_asset_value_loader(
+        self, instance: Optional[DagsterInstance] = None
+    ) -> "AssetValueLoader":
+        """
+        Returns an object that can load the contents of assets as Python objects.
+
+        Invokes `load_input` on the :py:class:`IOManager` associated with the assets. Avoids
+        spinning up resources separately for each asset.
+
+        Usage:
+
+            .. code-block:: python
+
+                with my_repo.get_asset_value_loader() as loader:
+                    asset1 = loader.load_asset_value()
+                    asset1 = loader.load_asset_value()
+
+        """
+        from dagster._core.storage.asset_value_loader import AssetValueLoader
+
+        return AssetValueLoader(self._assets_defs_by_key, instance=instance)
+
+    @property
+    def asset_graph(self) -> AssetGraph:
+        return AssetGraph.from_assets(
+            [*self._assets_defs_by_key.values(), *self.source_assets_by_key.values()]
+        )
+
     # If definition comes from the @repository decorator, then the __call__ method will be
     # overwritten. Therefore, we want to maintain the call-ability of repository definitions.
     def __call__(self, *args, **kwargs):
         return self
 
 
+class PendingRepositoryDefinition:
+    def __init__(
+        self,
+        name: str,
+        repository_definitions: Sequence[
+            Union[RepositoryListDefinition, "CacheableAssetsDefinition"]
+        ],
+        description: Optional[str] = None,
+        default_logger_defs: Optional[Mapping[str, LoggerDefinition]] = None,
+        default_executor_def: Optional[ExecutorDefinition] = None,
+    ):
+        self._repository_definitions = check.list_param(
+            repository_definitions,
+            "repository_definition",
+            additional_message="PendingRepositoryDefinition supports only list-based repository data at this time.",
+        )
+        self._name = name
+        self._description = description
+        self._default_logger_defs = default_logger_defs
+        self._default_executor_def = default_executor_def
+
+    @property
+    def name(self) -> str:
+        return self._name
+
+    def _compute_repository_load_data(self) -> RepositoryLoadData:
+        from dagster._core.definitions.cacheable_assets import CacheableAssetsDefinition
+
+        return RepositoryLoadData(
+            cached_data_by_key={
+                defn.unique_id: defn.compute_cacheable_data()
+                for defn in self._repository_definitions
+                if isinstance(defn, CacheableAssetsDefinition)
+            }
+        )
+
+    def _get_repository_definition(
+        self, repository_load_data: RepositoryLoadData
+    ) -> RepositoryDefinition:
+
+        from dagster._core.definitions.cacheable_assets import CacheableAssetsDefinition
+
+        resolved_definitions: List[RepositoryListDefinition] = []
+        for defn in self._repository_definitions:
+            if isinstance(defn, CacheableAssetsDefinition):
+                # should always have metadata for each cached defn at this point
+                check.invariant(
+                    defn.unique_id in repository_load_data.cached_data_by_key,
+                    f"No metadata found for CacheableAssetsDefinition with unique_id {defn.unique_id}.",
+                )
+                # use the emtadata to generate definitions
+                resolved_definitions.extend(
+                    defn.build_definitions(
+                        data=repository_load_data.cached_data_by_key[defn.unique_id]
+                    )
+                )
+            else:
+                resolved_definitions.append(defn)
+
+        repository_data = CachingRepositoryData.from_list(
+            resolved_definitions,
+            default_executor_def=self._default_executor_def,
+            default_logger_defs=self._default_logger_defs,
+        )
+
+        return RepositoryDefinition(
+            self._name,
+            repository_data=repository_data,
+            description=self._description,
+            repository_load_data=repository_load_data,
+        )
+
+    def reconstruct_repository_definition(
+        self, repository_load_data: RepositoryLoadData
+    ) -> RepositoryDefinition:
+        """Use the provided RepositoryLoadData to construct and return a RepositoryDefinition"""
+        check.inst_param(repository_load_data, "repository_load_data", RepositoryLoadData)
+        return self._get_repository_definition(repository_load_data)
+
+    def compute_repository_definition(self) -> RepositoryDefinition:
+        """Compute the required RepositoryLoadData and use it to construct and return a RepositoryDefinition"""
+        repository_load_data = self._compute_repository_load_data()
+        return self._get_repository_definition(repository_load_data)
+
+
 def _process_and_validate_target(
     schedule_or_sensor_def: Union[SensorDefinition, ScheduleDefinition],
     coerced_graphs: Dict[str, JobDefinition],
