@@ -5,6 +5,7 @@ import dagster._check as check
 from dagster._annotations import PublicAttr
 from dagster._core.definitions.events import AssetKey
 from dagster._core.storage.pipeline_run import PipelineRun, PipelineRunStatus
+from dagster._core.storage.tags import PARTITION_NAME_TAG
 from dagster._serdes.serdes import register_serdes_enum_fallbacks, whitelist_for_serdes
 from dagster._utils.error import SerializableErrorInfo
 
@@ -55,6 +56,9 @@ class RunRequest(
     Represents all the information required to launch a single run.  Must be returned by a
     SensorDefinition or ScheduleDefinition's evaluation function for a run to be launched.
 
+    To build a run request for a particular partitition, use
+    :py:func:`~JobDefinition.run_request_for_partition`.
+
     Attributes:
         run_key (Optional[str]): A string key to identify this launched run. For sensors, ensures that
             only one run is created per run key across all sensor evaluations.  For schedules,
@@ -72,7 +76,7 @@ class RunRequest(
 
     def __new__(
         cls,
-        run_key: Optional[str],
+        run_key: Optional[str] = None,
         run_config: Optional[Mapping[str, Any]] = None,
         tags: Optional[Mapping[str, str]] = None,
         job_name: Optional[str] = None,
@@ -81,14 +85,29 @@ class RunRequest(
         return super(RunRequest, cls).__new__(
             cls,
             run_key=check.opt_str_param(run_key, "run_key"),
-            run_config=check.opt_dict_param(run_config, "run_config", key_type=str),
-            tags=check.opt_dict_param(tags, "tags", key_type=str, value_type=str),
+            run_config=check.opt_mapping_param(run_config, "run_config", key_type=str),
+            tags=check.opt_mapping_param(tags, "tags", key_type=str, value_type=str),
             job_name=check.opt_str_param(job_name, "job_name"),
-            asset_selection=check.opt_sequence_param(
+            asset_selection=check.opt_nullable_sequence_param(
                 asset_selection, "asset_selection", of_type=AssetKey
             ),
         )
 
+    @property
+    def partition_key(self) -> Optional[str]:
+        return self.tags.get(PARTITION_NAME_TAG)
+
+    def with_replaced_attrs(
+        self, job_name: Optional[str] = None, asset_selection: Optional[Sequence[AssetKey]] = None
+    ) -> "RunRequest":
+        return RunRequest(
+            run_key=self.run_key,
+            run_config=self.run_config,
+            tags=self.tags,
+            job_name=job_name or self.job_name,
+            asset_selection=asset_selection or self.asset_selection,
+        )
+
 
 @whitelist_for_serdes
 class PipelineRunReaction(
