@@ -1,12 +1,13 @@
 import warnings
 from datetime import datetime
-from typing import TYPE_CHECKING, Any, Callable, List, NamedTuple, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Callable, NamedTuple, Optional, Sequence, Union, cast
 
 import pendulum
 
 import dagster._check as check
 from dagster._annotations import PublicAttr
 from dagster._core.errors import (
+    DagsterInvalidDefinitionError,
     DagsterInvalidInvocationError,
     RunStatusSensorExecutionError,
     user_code_error_boundary,
@@ -193,7 +194,7 @@ def run_failure_sensor(
     minimum_interval_seconds: Optional[int] = None,
     description: Optional[str] = None,
     monitored_jobs: Optional[
-        List[
+        Sequence[
             Union[
                 PipelineDefinition,
                 GraphDefinition,
@@ -204,7 +205,7 @@ def run_failure_sensor(
         ]
     ] = None,
     job_selection: Optional[
-        List[
+        Sequence[
             Union[
                 PipelineDefinition,
                 GraphDefinition,
@@ -214,6 +215,7 @@ def run_failure_sensor(
             ]
         ]
     ] = None,
+    monitor_all_repositories: bool = False,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
     request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
     request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
@@ -233,12 +235,17 @@ def run_failure_sensor(
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition, RepositorySelector, JobSelector]]]): The jobs
-            in the current repository that will be monitored by this failure sensor. Defaults to None, which means the alert will
-            be sent when any job in the current repository fails. Jobs in external repositories can be specified using RepositorySelector and JobSelector.
-        job_selection (Optional[List[Union[JobDefinition, GraphDefinition, RepositorySelector, JobSelector]]]): (deprecated in favor of monitored_jobs)
-            The jobs in the current repository that will be monitored by this failure sensor. Defaults to None, which means
-            the alert will be sent when any job in the repository fails.
+        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition, RepositorySelector, JobSelector]]]):
+            The jobs in the current repository that will be monitored by this failure sensor.
+            Defaults to None, which means the alert will be sent when any job in the current
+            repository fails.
+        monitor_all_repositories (bool): If set to True, the sensor will monitor all runs in the
+            Dagster instance. If set to True, an error will be raised if you also specify
+            monitored_jobs or job_selection. Defaults to False.
+        job_selection (Optional[List[Union[JobDefinition, GraphDefinition, RepositorySelector, JobSelector]]]):
+            (deprecated in favor of monitored_jobs) The jobs in the current repository that will be
+            monitored by this failure sensor. Defaults to None, which means the alert will be sent
+            when any job in the repository fails.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
         request_job (Optional[Union[GraphDefinition, JobDefinition]]): The job a RunRequest should
@@ -266,6 +273,7 @@ def run_failure_sensor(
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
             monitored_jobs=jobs,
+            monitor_all_repositories=monitor_all_repositories,
             default_status=default_status,
             request_job=request_job,
             request_jobs=request_jobs,
@@ -296,9 +304,12 @@ class RunStatusSensorDefinition(SensorDefinition):
         minimum_interval_seconds (Optional[int]): The minimum number of seconds that will elapse
             between sensor evaluations.
         description (Optional[str]): A human-readable description of the sensor.
-        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition]]]): The jobs
-            in the current repository that will be monitored by this sensor. Defaults to None, which means the alert will be sent
-            when any job in the repository fails.
+        monitored_jobs (Optional[List[Union[JobDefinition, GraphDefinition, UnresolvedAssetJobDefinition, JobSelector, RepositorySelector]]]):
+            The jobs in the current repository that will be monitored by this sensor. Defaults to
+            None, which means the alert will be sent when any job in the repository fails.
+        monitor_all_repositories (bool): If set to True, the sensor will monitor all runs in the
+            Dagster instance. If set to True, an error will be raised if you also specify
+            monitored_jobs or job_selection. Defaults to False.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
         request_job (Optional[Union[GraphDefinition, JobDefinition]]): The job a RunRequest should
@@ -315,7 +326,7 @@ class RunStatusSensorDefinition(SensorDefinition):
         minimum_interval_seconds: Optional[int] = None,
         description: Optional[str] = None,
         monitored_jobs: Optional[
-            List[
+            Sequence[
                 Union[
                     PipelineDefinition,
                     GraphDefinition,
@@ -325,6 +336,7 @@ class RunStatusSensorDefinition(SensorDefinition):
                 ]
             ]
         ] = None,
+        monitor_all_repositories: bool = False,
         default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
         request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
         request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
@@ -445,8 +457,14 @@ class RunStatusSensorDefinition(SensorDefinition):
 
                 job_match = False
 
+                # if monitor_all_repositories is provided, then we want to run the sensor for all jobs in all repositories
+                if monitor_all_repositories:
+                    job_match = True
+
                 # check if the run is in the current repository and (if provided) one of jobs specified in monitored_jobs
                 if (
+                    not job_match
+                    and
                     # the pipeline has a repository (not manually executed)
                     pipeline_run.external_pipeline_origin
                     and
@@ -454,31 +472,36 @@ class RunStatusSensorDefinition(SensorDefinition):
                     pipeline_run.external_pipeline_origin.external_repository_origin.repository_name
                     == context.repository_name
                 ):
-                    if current_repo_jobs:
+                    if monitored_jobs:
                         if pipeline_run.pipeline_name in map(lambda x: x.name, current_repo_jobs):
                             job_match = True
                     else:
                         job_match = True
 
-                # check if the run is one of the jobs specified by monitored (ie in another repo)
-                # make a JobSelector for the run in question
-                run_job_selector = JobSelector(
-                    location_name=pipeline_run.external_pipeline_origin.external_repository_origin.repository_location_origin.location_name,
-                    repository_name=pipeline_run.external_pipeline_origin.external_repository_origin.repository_name,
-                    job_name=pipeline_run.pipeline_name,
-                )
-                if run_job_selector in other_repo_jobs:
-                    job_match = True
+                if not job_match:
+                    # check if the run is one of the jobs specified by JobSelector or RepositorySelector (ie in another repo)
+                    # make a JobSelector for the run in question
+                    external_repository_origin = check.not_none(
+                        pipeline_run.external_pipeline_origin
+                    ).external_repository_origin
+                    run_job_selector = JobSelector(
+                        location_name=external_repository_origin.repository_location_origin.location_name,
+                        repository_name=external_repository_origin.repository_name,
+                        job_name=pipeline_run.pipeline_name,
+                    )
+                    if run_job_selector in other_repo_jobs:
+                        job_match = True
 
-                # make a RepositorySelector for the run in question
-                run_repo_selector = RepositorySelector(
-                    location_name=pipeline_run.external_pipeline_origin.external_repository_origin.repository_location_origin.location_name,
-                    repository_name=pipeline_run.external_pipeline_origin.external_repository_origin.repository_name,
-                )
-                if run_repo_selector in other_repos:
-                    job_match = True
+                    # make a RepositorySelector for the run in question
+                    run_repo_selector = RepositorySelector(
+                        location_name=external_repository_origin.repository_location_origin.location_name,
+                        repository_name=external_repository_origin.repository_name,
+                    )
+                    if run_repo_selector in other_repos:
+                        job_match = True
 
                 if not job_match:
+                    # the run in question doesn't match any of the criteria for we advance the cursor and move on
                     context.update_cursor(
                         RunStatusSensorCursor(
                             record_id=storage_id, update_timestamp=update_timestamp.isoformat()
@@ -598,7 +621,7 @@ def run_status_sensor(
     minimum_interval_seconds: Optional[int] = None,
     description: Optional[str] = None,
     monitored_jobs: Optional[
-        List[
+        Sequence[
             Union[
                 PipelineDefinition,
                 GraphDefinition,
@@ -609,7 +632,7 @@ def run_status_sensor(
         ]
     ] = None,
     job_selection: Optional[
-        List[
+        Sequence[
             Union[
                 PipelineDefinition,
                 GraphDefinition,
@@ -619,6 +642,7 @@ def run_status_sensor(
             ]
         ]
     ] = None,
+    monitor_all_repositories: bool = False,
     default_status: DefaultSensorStatus = DefaultSensorStatus.STOPPED,
     request_job: Optional[Union[GraphDefinition, JobDefinition]] = None,
     request_jobs: Optional[Sequence[Union[GraphDefinition, JobDefinition]]] = None,
@@ -643,8 +667,12 @@ def run_status_sensor(
             Jobs in the current repository that will be monitored by this sensor. Defaults to None, which means the alert will
             be sent when any job in the repository matches the requested run_status. Jobs in external repositories can be monitored by using
             RepositorySelector or JobSelector.
-        job_selection (Optional[List[Union[PipelineDefinition, GraphDefinition, RepositorySelector, JobSelector]]]): (deprecated in favor of monitored_jobs)
-            Jobs in the current repository that will be monitored by this sensor. Defaults to None, which means the alert will be sent when
+        monitor_all_repositories (bool): If set to True, the sensor will monitor all runs in the Dagster instance.
+            If set to True, an error will be raised if you also specify monitored_jobs or job_selection.
+            Defaults to False.
+        job_selection (Optional[List[Union[PipelineDefinition, GraphDefinition, RepositorySelector, JobSelector]]]):
+            (deprecated in favor of monitored_jobs) Jobs in the current repository that will be
+            monitored by this sensor. Defaults to None, which means the alert will be sent when
             any job in the repository matches the requested run_status.
         default_status (DefaultSensorStatus): Whether the sensor starts as running or not. The default
             status can be overridden from Dagit or via the GraphQL API.
@@ -665,6 +693,11 @@ def run_status_sensor(
             deprecation_warning("job_selection", "2.0.0", "Use monitored_jobs instead.")
         jobs = monitored_jobs if monitored_jobs else job_selection
 
+        if jobs and monitor_all_repositories:
+            DagsterInvalidDefinitionError(
+                f"Cannot specify both monitor_all_repositories and {'monitored_jobs' if monitored_jobs else 'job_selection'}."
+            )
+
         return RunStatusSensorDefinition(
             name=sensor_name,
             run_status=run_status,
@@ -672,6 +705,7 @@ def run_status_sensor(
             minimum_interval_seconds=minimum_interval_seconds,
             description=description,
             monitored_jobs=jobs,
+            monitor_all_repositories=monitor_all_repositories,
             default_status=default_status,
             request_job=request_job,
             request_jobs=request_jobs,
