@@ -4,7 +4,6 @@ from typing import (
     Any,
     Callable,
     Iterator,
-    List,
     Mapping,
     Optional,
     Sequence,
@@ -40,7 +39,6 @@ from .resource_requirement import (
     SolidDefinitionResourceRequirement,
 )
 from .solid_invocation import solid_invocation_result
-from .utils import DEFAULT_IO_MANAGER_KEY
 
 if TYPE_CHECKING:
     from .asset_layer import AssetLayer
@@ -143,10 +141,10 @@ class SolidDefinition(NodeDefinition):
 
         super(SolidDefinition, self).__init__(
             name=name,
-            input_defs=check.list_param(input_defs, "input_defs", InputDefinition),
-            output_defs=check.list_param(output_defs, "output_defs", OutputDefinition),
+            input_defs=check.sequence_param(input_defs, "input_defs", InputDefinition),
+            output_defs=check.sequence_param(output_defs, "output_defs", OutputDefinition),
             description=description,
-            tags=check.opt_dict_param(tags, "tags", key_type=str),
+            tags=check.opt_mapping_param(tags, "tags", key_type=str),
             positional_inputs=positional_inputs,
         )
 
@@ -260,9 +258,12 @@ class SolidDefinition(NodeDefinition):
     ) -> Tuple[OutputDefinition, T_Handle]:
         return self.output_def_named(output_name), handle
 
+    def resolve_output_to_origin_op_def(self, output_name: str) -> "SolidDefinition":
+        return self
+
     def get_inputs_must_be_resolved_top_level(
         self, asset_layer: "AssetLayer", handle: Optional[NodeHandle] = None
-    ) -> List[InputDefinition]:
+    ) -> Sequence[InputDefinition]:
         handle = cast(NodeHandle, check.inst_param(handle, "handle", NodeHandle))
         unresolveable_input_defs = []
         for input_def in self.input_defs:
@@ -433,7 +434,6 @@ class CompositeSolidDefinition(GraphDefinition):
         tags: Optional[Mapping[str, str]] = None,
         positional_inputs: Optional[Sequence[str]] = None,
     ):
-        _check_io_managers_on_composite_solid(name, input_mappings, output_mappings)
 
         super(CompositeSolidDefinition, self).__init__(
             name=name,
@@ -491,30 +491,3 @@ class CompositeSolidDefinition(GraphDefinition):
     @property
     def is_graph_job_op_node(self) -> bool:
         return False
-
-
-def _check_io_managers_on_composite_solid(
-    name: str,
-    input_mappings: Optional[Sequence[InputMapping]],
-    output_mappings: Optional[Sequence[OutputMapping]],
-) -> None:
-    # Ban root_manager_key on composite solids
-    if input_mappings:
-        for input_mapping in input_mappings:
-            input_def = input_mapping.definition
-            if input_def.root_manager_key:
-                raise DagsterInvalidDefinitionError(
-                    "Root input manager cannot be set on a composite solid: "
-                    f'root_manager_key "{input_def.root_manager_key}" '
-                    f'is set on InputDefinition "{input_def.name}" of composite solid "{name}". '
-                )
-    # Ban io_manager_key on composite solids
-    if output_mappings:
-        for output_mapping in output_mappings:
-            output_def = output_mapping.definition
-            if output_def.io_manager_key != DEFAULT_IO_MANAGER_KEY:
-                raise DagsterInvalidDefinitionError(
-                    "IO manager cannot be set on a composite solid: "
-                    f'io_manager_key "{output_def.io_manager_key}" '
-                    f'is set  on OutputtDefinition "{output_def.name}" of composite solid "{name}". '
-                )
