@@ -1,8 +1,14 @@
-from typing import Dict, Iterator, Mapping, NamedTuple, Optional, Sequence, Union, cast
+from __future__ import annotations
+
+import warnings
+from typing import TYPE_CHECKING, Dict, Iterator, Mapping, Optional, Sequence, Union, cast
+
+from typing_extensions import Protocol, TypeAlias
 
 import dagster._check as check
 from dagster._annotations import PublicAttr, public
-from dagster._core.definitions.events import AssetKey, CoercibleToAssetKey
+from dagster._core.definitions.events import AssetKey, AssetObservation, CoercibleToAssetKey
+from dagster._core.definitions.logical_version import LOGICAL_VERSION_TAG_KEY, LogicalVersion
 from dagster._core.definitions.metadata import (
     MetadataEntry,
     MetadataMapping,
@@ -10,6 +16,7 @@ from dagster._core.definitions.metadata import (
     PartitionMetadataEntry,
     normalize_metadata,
 )
+from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.definitions.partition import PartitionsDefinition
 from dagster._core.definitions.resource_definition import ResourceDefinition
 from dagster._core.definitions.resource_requirement import (
@@ -26,24 +33,39 @@ from dagster._core.definitions.utils import (
 from dagster._core.errors import DagsterInvalidDefinitionError, DagsterInvalidInvocationError
 from dagster._core.storage.io_manager import IOManagerDefinition
 from dagster._utils import merge_dicts
-from dagster._utils.backcompat import experimental_arg_warning
+from dagster._utils.backcompat import ExperimentalWarning, experimental_arg_warning
+
+if TYPE_CHECKING:
+    from dagster._core.execution.context.compute import (
+        OpExecutionContext,
+        SourceAssetObserveContext,
+    )
 
 
-class SourceAsset(
-    NamedTuple(
-        "_SourceAsset",
-        [
-            ("key", PublicAttr[AssetKey]),
-            ("metadata_entries", Sequence[Union[MetadataEntry, PartitionMetadataEntry]]),
-            ("io_manager_key", PublicAttr[Optional[str]]),
-            ("description", PublicAttr[Optional[str]]),
-            ("partitions_def", PublicAttr[Optional[PartitionsDefinition]]),
-            ("group_name", PublicAttr[str]),
-            ("resource_defs", PublicAttr[Dict[str, ResourceDefinition]]),
-        ],
-    ),
-    ResourceAddable,
-):
+class SourceAssetObserveFunctionWithContext(Protocol):
+    @property
+    def __name__(self) -> str:
+        ...
+
+    def __call__(self, context: "SourceAssetObserveContext") -> LogicalVersion:
+        ...
+
+
+class SourceAssetObserveFunctionNoContext(Protocol):
+    @property
+    def __name__(self) -> str:
+        ...
+
+    def __call__(self) -> LogicalVersion:
+        ...
+
+
+SourceAssetObserveFunction: TypeAlias = Union[
+    SourceAssetObserveFunctionWithContext, SourceAssetObserveFunctionNoContext
+]
+
+
+class SourceAsset(ResourceAddable):
     """A SourceAsset represents an asset that will be loaded by (but not updated by) Dagster.
 
     Attributes:
@@ -57,10 +79,22 @@ class SourceAsset(
         description (Optional[str]): The description of the asset.
         partitions_def (Optional[PartitionsDefinition]): Defines the set of partition keys that
             compose the asset.
+        observe_fn (Optional[SourceAssetObserveFunction]) Observation function for the source asset.
     """
 
-    def __new__(
-        cls,
+    key: PublicAttr[AssetKey]
+    metadata_entries: Sequence[Union[MetadataEntry, PartitionMetadataEntry]]
+    io_manager_key: PublicAttr[Optional[str]]
+    _io_manager_def: PublicAttr[Optional[IOManagerDefinition]]
+    description: PublicAttr[Optional[str]]
+    partitions_def: PublicAttr[Optional[PartitionsDefinition]]
+    group_name: PublicAttr[str]
+    resource_defs: PublicAttr[Dict[str, ResourceDefinition]]
+    observe_fn: PublicAttr[Optional[SourceAssetObserveFunction]]
+    _node_def: Optional[OpDefinition]  # computed lazily
+
+    def __init__(
+        self,
         key: CoercibleToAssetKey,
         metadata: Optional[MetadataUserInput] = None,
         io_manager_key: Optional[str] = None,
@@ -70,6 +104,7 @@ class SourceAsset(
         _metadata_entries: Optional[Sequence[Union[MetadataEntry, PartitionMetadataEntry]]] = None,
         group_name: Optional[str] = None,
         resource_defs: Optional[Mapping[str, ResourceDefinition]] = None,
+        observe_fn: Optional[SourceAssetObserveFunction] = None,
         # Add additional fields to with_resources and with_group below
     ):
 
@@ -79,35 +114,37 @@ class SourceAsset(
         if io_manager_def is not None:
             experimental_arg_warning("io_manager_def", "SourceAsset.__new__")
 
-        key = AssetKey.from_coerceable(key)
+        self.key = AssetKey.from_coerceable(key)
         metadata = check.opt_dict_param(metadata, "metadata", key_type=str)
-        metadata_entries = _metadata_entries or normalize_metadata(metadata, [], allow_invalid=True)
-        resource_defs = dict(check.opt_mapping_param(resource_defs, "resource_defs"))
-        io_manager_def = check.opt_inst_param(io_manager_def, "io_manager_def", IOManagerDefinition)
-        if io_manager_def:
+        self.metadata_entries = _metadata_entries or normalize_metadata(
+            metadata, [], allow_invalid=True
+        )
+        self.resource_defs = dict(check.opt_mapping_param(resource_defs, "resource_defs"))
+        self._io_manager_def = check.opt_inst_param(  # type: ignore
+            io_manager_def, "io_manager_def", IOManagerDefinition  # type: ignore
+        )
+        if self._io_manager_def:
             if not io_manager_key:
-                source_asset_path = "__".join(key.path)
-                io_manager_key = f"{source_asset_path}__io_manager"
+                io_manager_key = self.key.to_python_identifier("io_manager")
 
-            if io_manager_key in resource_defs and resource_defs[io_manager_key] != io_manager_def:
+            if (
+                io_manager_key in self.resource_defs
+                and self.resource_defs[io_manager_key] != io_manager_def
+            ):
                 raise DagsterInvalidDefinitionError(
                     f"Provided conflicting definitions for io manager key '{io_manager_key}'. Please provide only one definition per key."
                 )
 
-            resource_defs[io_manager_key] = io_manager_def
+            self.resource_defs[io_manager_key] = self._io_manager_def
 
-        return super().__new__(
-            cls,
-            key=key,
-            metadata_entries=metadata_entries,
-            io_manager_key=check.opt_str_param(io_manager_key, "io_manager_key"),
-            description=check.opt_str_param(description, "description"),
-            partitions_def=check.opt_inst_param(
-                partitions_def, "partitions_def", PartitionsDefinition
-            ),
-            group_name=validate_group_name(group_name),
-            resource_defs=resource_defs,
+        self.io_manager_key = check.opt_str_param(io_manager_key, "io_manager_key")
+        self.partitions_def = check.opt_inst_param(
+            partitions_def, "partitions_def", PartitionsDefinition
         )
+        self.group_name = validate_group_name(group_name)
+        self.description = check.opt_str_param(description, "description")
+        self.observe_fn = check.opt_callable_param(observe_fn, "observe_fn")
+        self._node_def = None
 
     @public  # type: ignore
     @property
@@ -126,6 +163,56 @@ class SourceAsset(
             self.resource_defs.get(io_manager_key) if io_manager_key else None,
         )
 
+    @public  # type: ignore
+    @property
+    def op(self) -> OpDefinition:
+        check.invariant(
+            isinstance(self.node_def, OpDefinition),
+            "The NodeDefinition for this AssetsDefinition is not of type OpDefinition.",
+        )
+        return cast(OpDefinition, self.node_def)
+
+    @public  # type: ignore
+    @property
+    def is_observable(self) -> bool:
+        return self.node_def is not None
+
+    @property
+    def node_def(self) -> Optional[OpDefinition]:
+        """Op that generates observation metadata for a source asset."""
+        from dagster._core.definitions.decorators.solid_decorator import DecoratedSolidFunction
+
+        if self.observe_fn is None:
+            return None
+        elif self._node_def is None:
+            observe_fn = self.observe_fn
+
+            def fn(context: OpExecutionContext):
+                logical_version = observe_fn(context)  # type: ignore
+                check.inst(
+                    logical_version,
+                    LogicalVersion,
+                    "Source asset observation function must return a LogicalVersion",
+                )
+                tags = {LOGICAL_VERSION_TAG_KEY: logical_version.value}
+                context.log_event(
+                    AssetObservation(
+                        asset_key=self.key,
+                        tags=tags,
+                    )
+                )
+
+            compute_fn = DecoratedSolidFunction(decorated_fn=fn)
+
+            self._node_def = OpDefinition(
+                compute_fn=compute_fn,
+                name="__".join(self.key.path).replace("-", "_"),
+                description=self.description,
+            )
+            return self._node_def
+        else:
+            return self._node_def
+
     def with_resources(self, resource_defs) -> "SourceAsset":
         from dagster._core.execution.resources_init import get_transitive_required_resource_keys
 
@@ -161,15 +248,19 @@ class SourceAsset(
             if self.get_io_manager_key() != DEFAULT_IO_MANAGER_KEY
             else None
         )
-        return SourceAsset(
-            key=self.key,
-            io_manager_key=io_manager_key,
-            description=self.description,
-            partitions_def=self.partitions_def,
-            _metadata_entries=self.metadata_entries,
-            resource_defs=relevant_resource_defs,
-            group_name=self.group_name,
-        )
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=ExperimentalWarning)
+
+            return SourceAsset(
+                key=self.key,
+                io_manager_key=io_manager_key,
+                description=self.description,
+                partitions_def=self.partitions_def,
+                _metadata_entries=self.metadata_entries,
+                resource_defs=relevant_resource_defs,
+                group_name=self.group_name,
+                observe_fn=self.observe_fn,
+            )
 
     def with_group_name(self, group_name: str) -> "SourceAsset":
         if self.group_name != DEFAULT_GROUP_NAME:
@@ -177,16 +268,20 @@ class SourceAsset(
                 f"A group name has already been provided to source asset {self.key.to_user_string()}"
             )
 
-        return SourceAsset(
-            key=self.key,
-            _metadata_entries=self.metadata_entries,
-            io_manager_key=self.io_manager_key,
-            io_manager_def=self.io_manager_def,
-            description=self.description,
-            partitions_def=self.partitions_def,
-            group_name=group_name,
-            resource_defs=self.resource_defs,
-        )
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", category=ExperimentalWarning)
+
+            return SourceAsset(
+                key=self.key,
+                _metadata_entries=self.metadata_entries,
+                io_manager_key=self.io_manager_key,
+                io_manager_def=self.io_manager_def,
+                description=self.description,
+                partitions_def=self.partitions_def,
+                group_name=group_name,
+                resource_defs=self.resource_defs,
+                observe_fn=self.observe_fn,
+            )
 
     def get_resource_requirements(self) -> Iterator[ResourceRequirement]:
         yield SourceAssetIOManagerRequirement(
@@ -194,3 +289,17 @@ class SourceAsset(
         )
         for source_key, resource_def in self.resource_defs.items():
             yield from resource_def.get_resource_requirements(outer_context=source_key)
+
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, SourceAsset):
+            return False
+        else:
+            return (
+                self.key == other.key
+                and self.metadata_entries == other.metadata_entries
+                and self.io_manager_key == other.io_manager_key
+                and self.description == other.description
+                and self.group_name == other.group_name
+                and self.resource_defs == other.resource_defs
+                and self.observe_fn == other.observe_fn
+            )
