@@ -5,21 +5,23 @@ from typing import (
     Dict,
     FrozenSet,
     Iterator,
-    List,
     Mapping,
     NamedTuple,
     Optional,
     Sequence,
     Tuple,
     Union,
+    cast,
 )
 
 import dagster._check as check
 from dagster._annotations import experimental
 from dagster._core.definitions import IPipeline, JobDefinition, PipelineDefinition
+from dagster._core.definitions.events import AssetKey
 from dagster._core.definitions.pipeline_base import InMemoryPipeline
 from dagster._core.definitions.pipeline_definition import PipelineSubsetDefinition
 from dagster._core.definitions.reconstruct import ReconstructableJob, ReconstructablePipeline
+from dagster._core.definitions.repository_definition import RepositoryLoadData
 from dagster._core.errors import DagsterExecutionInterruptedError, DagsterInvariantViolationError
 from dagster._core.events import DagsterEvent, EngineEventData
 from dagster._core.execution.context.system import PlanOrchestrationContext
@@ -161,6 +163,8 @@ def execute_run_iterator(
             )
 
     execution_plan = _get_execution_plan_from_run(pipeline, pipeline_run, instance)
+    if isinstance(pipeline, ReconstructablePipeline):
+        pipeline = pipeline.with_repository_load_data(execution_plan.repository_load_data)
 
     return iter(
         ExecuteRunWithPlanIterable(
@@ -257,6 +261,9 @@ def execute_run(
             )
 
     execution_plan = _get_execution_plan_from_run(pipeline, pipeline_run, instance)
+    if isinstance(pipeline, ReconstructablePipeline):
+        pipeline = pipeline.with_repository_load_data(execution_plan.repository_load_data)
+
     output_capture: Optional[Dict[StepOutputHandle, Any]] = {}
 
     _execute_run_iterable = ExecuteRunWithPlanIterable(
@@ -276,8 +283,6 @@ def execute_run(
     )
     event_list = list(_execute_run_iterable)
 
-    pipeline_def = pipeline.get_definition()
-
     return PipelineExecutionResult(
         pipeline.get_definition(),
         pipeline_run.run_id,
@@ -295,11 +300,11 @@ def execute_run(
 
 def execute_pipeline_iterator(
     pipeline: Union[PipelineDefinition, IPipeline],
-    run_config: Optional[dict] = None,
+    run_config: Optional[Mapping[str, object]] = None,
     mode: Optional[str] = None,
     preset: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
-    solid_selection: Optional[List[str]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
+    solid_selection: Optional[Sequence[str]] = None,
     instance: Optional[DagsterInstance] = None,
 ) -> Iterator[DagsterEvent]:
     """Execute a pipeline iteratively.
@@ -338,6 +343,8 @@ def execute_pipeline_iterator(
     """
 
     with ephemeral_instance_if_missing(instance) as execute_instance:
+        pipeline, repository_load_data = _pipeline_with_repository_load_data(pipeline)
+
         (
             pipeline,
             run_config,
@@ -361,6 +368,7 @@ def execute_pipeline_iterator(
             solid_selection=solid_selection,
             solids_to_execute=solids_to_execute,
             tags=tags,
+            repository_load_data=repository_load_data,
         )
 
         return execute_run_iterator(pipeline, pipeline_run, execute_instance)
@@ -409,14 +417,14 @@ class ReexecutionOptions(NamedTuple):
         """
         from dagster._core.execution.plan.resume_retry import get_retry_steps_from_parent_run
 
-        parent_run = instance.get_run_by_id(run_id)
+        parent_run = check.not_none(instance.get_run_by_id(run_id))
         check.invariant(
             parent_run.status == DagsterRunStatus.FAILURE,
             "Cannot reexecute from failure a run that is not failed",
         )
         # Tried to thread through KnownExecutionState to execution plan creation, but little benefit. It is recalculated later by the re-execution machinery.
         step_keys_to_execute, _ = get_retry_steps_from_parent_run(
-            instance, parent_run=instance.get_run_by_id(run_id)
+            instance, parent_run=cast(DagsterRun, instance.get_run_by_id(run_id))
         )
         return ReexecutionOptions(parent_run_id=run_id, step_selection=step_keys_to_execute)
 
@@ -426,10 +434,11 @@ def execute_job(
     job: ReconstructableJob,
     instance: "DagsterInstance",
     run_config: Any = None,
-    tags: Optional[Dict[str, Any]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
     raise_on_error: bool = False,
-    op_selection: Optional[List[str]] = None,
+    op_selection: Optional[Sequence[str]] = None,
     reexecution_options: Optional[ReexecutionOptions] = None,
+    asset_selection: Optional[Sequence[AssetKey]] = None,
 ) -> ExecuteJobResult:
     """Execute a job synchronously.
 
@@ -526,6 +535,10 @@ def execute_job(
 
     check.inst_param(job, "job", ReconstructablePipeline)
     check.inst_param(instance, "instance", DagsterInstance)
+    check.opt_sequence_param(asset_selection, "asset_selection", of_type=AssetKey)
+
+    # get the repository load data here because we call job.get_definition() later in this fn
+    job, _ = _pipeline_with_repository_load_data(job)
 
     if reexecution_options is not None and op_selection is not None:
         raise DagsterInvariantViolationError(
@@ -534,7 +547,8 @@ def execute_job(
 
     if reexecution_options:
         if run_config is None:
-            run_config = instance.get_run_by_id(reexecution_options.parent_run_id).run_config
+            run = check.not_none(instance.get_run_by_id(reexecution_options.parent_run_id))
+            run_config = run.run_config
         result = reexecute_pipeline(
             pipeline=job,
             parent_run_id=reexecution_options.parent_run_id,
@@ -556,11 +570,12 @@ def execute_job(
             tags=tags,
             solid_selection=op_selection,
             raise_on_error=raise_on_error,
+            asset_selection=asset_selection,
         )
 
     # We use PipelineExecutionResult to construct the JobExecutionResult.
     return ExecuteJobResult(
-        job_def=job.get_definition(),
+        job_def=cast(ReconstructableJob, job).get_definition(),
         reconstruct_context=result.reconstruct_context(),
         event_list=result.event_list,
         dagster_run=instance.get_run_by_id(result.run_id),
@@ -569,11 +584,11 @@ def execute_job(
 
 def execute_pipeline(
     pipeline: Union[PipelineDefinition, IPipeline],
-    run_config: Optional[dict] = None,
+    run_config: Optional[Mapping] = None,
     mode: Optional[str] = None,
     preset: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
-    solid_selection: Optional[List[str]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
+    solid_selection: Optional[Sequence[str]] = None,
     instance: Optional[DagsterInstance] = None,
     raise_on_error: bool = True,
 ) -> PipelineExecutionResult:
@@ -632,11 +647,15 @@ def _logged_execute_pipeline(
     run_config: Optional[dict] = None,
     mode: Optional[str] = None,
     preset: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
-    solid_selection: Optional[List[str]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
+    solid_selection: Optional[Sequence[str]] = None,
     raise_on_error: bool = True,
+    asset_selection: Optional[Sequence[AssetKey]] = None,
 ) -> PipelineExecutionResult:
     check.inst_param(instance, "instance", DagsterInstance)
+
+    pipeline, repository_load_data = _pipeline_with_repository_load_data(pipeline)
+
     (
         pipeline,
         run_config,
@@ -665,6 +684,8 @@ def _logged_execute_pipeline(
         pipeline_code_origin=(
             pipeline.get_python_origin() if isinstance(pipeline, ReconstructablePipeline) else None
         ),
+        repository_load_data=repository_load_data,
+        asset_selection=frozenset(asset_selection) if asset_selection else None,
     )
 
     return execute_run(
@@ -678,11 +699,11 @@ def _logged_execute_pipeline(
 def reexecute_pipeline(
     pipeline: Union[IPipeline, PipelineDefinition],
     parent_run_id: str,
-    run_config: Optional[dict] = None,
-    step_selection: Optional[List[str]] = None,
+    run_config: Optional[Mapping] = None,
+    step_selection: Optional[Sequence[str]] = None,
     mode: Optional[str] = None,
     preset: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
     instance: Optional[DagsterInstance] = None,
     raise_on_error: bool = True,
 ) -> PipelineExecutionResult:
@@ -724,11 +745,13 @@ def reexecute_pipeline(
     For the asynchronous version, see :py:func:`reexecute_pipeline_iterator`.
     """
 
-    check.opt_list_param(step_selection, "step_selection", of_type=str)
+    check.opt_sequence_param(step_selection, "step_selection", of_type=str)
 
     check.str_param(parent_run_id, "parent_run_id")
 
     with ephemeral_instance_if_missing(instance) as execute_instance:
+        pipeline, repository_load_data = _pipeline_with_repository_load_data(pipeline)
+
         (pipeline, run_config, mode, tags, _, _) = _check_execute_pipeline_args(
             pipeline=pipeline,
             run_config=run_config,
@@ -753,7 +776,7 @@ def reexecute_pipeline(
                 pipeline,
                 mode,
                 run_config,
-                parent_pipeline_run,
+                cast(DagsterRun, parent_pipeline_run),
                 step_selection,
             )
 
@@ -773,6 +796,12 @@ def reexecute_pipeline(
             solids_to_execute=parent_pipeline_run.solids_to_execute,
             root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
             parent_run_id=parent_pipeline_run.run_id,
+            pipeline_code_origin=(
+                pipeline.get_python_origin()
+                if isinstance(pipeline, ReconstructablePipeline)
+                else None
+            ),
+            repository_load_data=repository_load_data,
         )
 
         return execute_run(
@@ -786,11 +815,11 @@ def reexecute_pipeline(
 def reexecute_pipeline_iterator(
     pipeline: Union[IPipeline, PipelineDefinition],
     parent_run_id: str,
-    run_config: Optional[dict] = None,
-    step_selection: Optional[List[str]] = None,
+    run_config: Optional[Mapping[str, object]] = None,
+    step_selection: Optional[Sequence[str]] = None,
     mode: Optional[str] = None,
     preset: Optional[str] = None,
-    tags: Optional[Dict[str, Any]] = None,
+    tags: Optional[Mapping[str, Any]] = None,
     instance: Optional[DagsterInstance] = None,
 ) -> Iterator[DagsterEvent]:
     """Reexecute a pipeline iteratively.
@@ -831,11 +860,13 @@ def reexecute_pipeline_iterator(
       Iterator[DagsterEvent]: The stream of events resulting from pipeline reexecution.
     """
 
-    check.opt_list_param(step_selection, "step_selection", of_type=str)
+    check.opt_sequence_param(step_selection, "step_selection", of_type=str)
 
     check.str_param(parent_run_id, "parent_run_id")
 
     with ephemeral_instance_if_missing(instance) as execute_instance:
+        pipeline, repository_load_data = _pipeline_with_repository_load_data(pipeline)
+
         (pipeline, run_config, mode, tags, _, _) = _check_execute_pipeline_args(
             pipeline=pipeline,
             run_config=run_config,
@@ -860,7 +891,7 @@ def reexecute_pipeline_iterator(
                 pipeline,
                 mode,
                 run_config,
-                parent_pipeline_run,
+                cast(DagsterRun, parent_pipeline_run),
                 step_selection,
             )
 
@@ -874,6 +905,7 @@ def reexecute_pipeline_iterator(
             solids_to_execute=parent_pipeline_run.solids_to_execute,
             root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
             parent_run_id=parent_pipeline_run.run_id,
+            repository_load_data=repository_load_data,
         )
 
         return execute_run_iterator(pipeline, pipeline_run, execute_instance)
@@ -894,6 +926,9 @@ def execute_plan_iterator(
     retry_mode = check.opt_inst_param(retry_mode, "retry_mode", RetryMode, RetryMode.DISABLED)
     run_config = check.opt_mapping_param(run_config, "run_config")
 
+    if isinstance(pipeline, ReconstructablePipeline):
+        pipeline = pipeline.with_repository_load_data(execution_plan.repository_load_data)
+
     return iter(
         ExecuteRunWithPlanIterable(
             execution_plan=execution_plan,
@@ -915,9 +950,9 @@ def execute_plan(
     pipeline: IPipeline,
     instance: DagsterInstance,
     pipeline_run: PipelineRun,
-    run_config: Optional[Dict] = None,
+    run_config: Optional[Mapping] = None,
     retry_mode: Optional[RetryMode] = None,
-) -> List[DagsterEvent]:
+) -> Sequence[DagsterEvent]:
     """This is the entry point of dagster-graphql executions. For the dagster CLI entry point, see
     execute_pipeline() above.
     """
@@ -925,7 +960,7 @@ def execute_plan(
     check.inst_param(pipeline, "pipeline", IPipeline)
     check.inst_param(instance, "instance", DagsterInstance)
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
-    run_config = check.opt_dict_param(run_config, "run_config")
+    run_config = check.opt_mapping_param(run_config, "run_config")
     check.opt_inst_param(retry_mode, "retry_mode", RetryMode)
 
     return list(
@@ -953,8 +988,8 @@ def _get_execution_plan_from_run(
     pipeline: IPipeline, pipeline_run: PipelineRun, instance: DagsterInstance
 ) -> ExecutionPlan:
 
+    execution_plan_snapshot = None
     if (
-        # need to rebuild execution plan so it matches the subsetted graph
         pipeline.solids_to_execute is None
         and pipeline.asset_selection is None
         and pipeline_run.execution_plan_snapshot_id
@@ -967,12 +1002,22 @@ def _get_execution_plan_from_run(
                 pipeline_run.pipeline_name,
                 execution_plan_snapshot,
             )
+
+    if pipeline_run.has_repository_load_data:
+        # if you haven't fetched it already, get the snapshot now
+        execution_plan_snapshot = execution_plan_snapshot or instance.get_execution_plan_snapshot(
+            check.not_none(pipeline_run.execution_plan_snapshot_id)
+        )
+    # need to rebuild execution plan so it matches the subsetted graph
     return create_execution_plan(
         pipeline,
         run_config=pipeline_run.run_config,
         mode=pipeline_run.mode,
         step_keys_to_execute=pipeline_run.step_keys_to_execute,
         instance_ref=instance.get_ref() if instance.is_persistent else None,
+        repository_load_data=execution_plan_snapshot.repository_load_data
+        if execution_plan_snapshot
+        else None,
     )
 
 
@@ -980,25 +1025,35 @@ def create_execution_plan(
     pipeline: Union[IPipeline, PipelineDefinition],
     run_config: Optional[Mapping[str, object]] = None,
     mode: Optional[str] = None,
-    step_keys_to_execute: Optional[List[str]] = None,
+    step_keys_to_execute: Optional[Sequence[str]] = None,
     known_state: Optional[KnownExecutionState] = None,
     instance_ref: Optional[InstanceRef] = None,
-    tags: Optional[Dict[str, str]] = None,
+    tags: Optional[Mapping[str, str]] = None,
+    repository_load_data: Optional[RepositoryLoadData] = None,
 ) -> ExecutionPlan:
+
     pipeline = _check_pipeline(pipeline)
+
+    # If you have repository_load_data, make sure to use it when building plan
+    if isinstance(pipeline, ReconstructablePipeline) and repository_load_data is not None:
+        pipeline = pipeline.with_repository_load_data(repository_load_data)
+
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
     run_config = check.opt_mapping_param(run_config, "run_config", key_type=str)
     mode = check.opt_str_param(mode, "mode", default=pipeline_def.get_default_mode_name())
-    check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+    check.opt_nullable_sequence_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
     check.opt_inst_param(instance_ref, "instance_ref", InstanceRef)
-    tags = check.opt_dict_param(tags, "tags", key_type=str, value_type=str)
+    tags = check.opt_mapping_param(tags, "tags", key_type=str, value_type=str)
     known_state = check.opt_inst_param(
         known_state,
         "known_state",
         KnownExecutionState,
         default=KnownExecutionState(),
     )
+    repository_load_data = check.opt_inst_param(
+        repository_load_data, "repository_load_data", RepositoryLoadData
+    )
 
     resolved_run_config = ResolvedRunConfig.build(pipeline_def, run_config, mode=mode)
 
@@ -1009,6 +1064,7 @@ def create_execution_plan(
         known_state=known_state,
         instance_ref=instance_ref,
         tags=tags,
+        repository_load_data=repository_load_data,
     )
 
 
@@ -1035,6 +1091,8 @@ def pipeline_execution_iterator(
         for event in pipeline_context.executor.execute(pipeline_context, execution_plan):
             if event.is_step_failure:
                 failed_steps.append(event.step_key)
+            elif event.is_resource_init_failure and event.step_key:
+                failed_steps.append(event.step_key)
 
             yield event
     except GeneratorExit:
@@ -1143,24 +1201,24 @@ class ExecuteRunWithPlanIterable:
 
 def _check_execute_pipeline_args(
     pipeline: Union[PipelineDefinition, IPipeline],
-    run_config: Optional[dict],
+    run_config: Optional[Mapping],
     mode: Optional[str],
     preset: Optional[str],
-    tags: Optional[Dict[str, Any]],
-    solid_selection: Optional[List[str]] = None,
+    tags: Optional[Mapping[str, Any]],
+    solid_selection: Optional[Sequence[str]] = None,
 ) -> Tuple[
     IPipeline,
-    Optional[dict],
+    Optional[Mapping],
     Optional[str],
-    Dict[str, Any],
+    Mapping[str, Any],
     Optional[FrozenSet[str]],
-    Optional[List[str]],
+    Optional[Sequence[str]],
 ]:
     pipeline = _check_pipeline(pipeline)
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
 
-    run_config = check.opt_dict_param(run_config, "run_config")
+    run_config = check.opt_mapping_param(run_config, "run_config")
     check.opt_str_param(mode, "mode")
     check.opt_str_param(preset, "preset")
     check.invariant(
@@ -1170,8 +1228,8 @@ def _check_execute_pipeline_args(
         ),
     )
 
-    tags = check.opt_dict_param(tags, "tags", key_type=str)
-    check.opt_list_param(solid_selection, "solid_selection", of_type=str)
+    tags = check.opt_mapping_param(tags, "tags", key_type=str)
+    check.opt_sequence_param(solid_selection, "solid_selection", of_type=str)
 
     if preset is not None:
         pipeline_preset = pipeline_def.get_preset(preset)
@@ -1251,9 +1309,9 @@ def _resolve_reexecute_step_selection(
     instance: DagsterInstance,
     pipeline: IPipeline,
     mode: Optional[str],
-    run_config: Optional[dict],
+    run_config: Optional[Mapping],
     parent_pipeline_run: DagsterRun,
-    step_selection: List[str],
+    step_selection: Sequence[str],
 ) -> ExecutionPlan:
     if parent_pipeline_run.solid_selection:
         pipeline = pipeline.subset_for_execution(parent_pipeline_run.solid_selection, None)
@@ -1276,3 +1334,17 @@ def _resolve_reexecute_step_selection(
         tags=parent_pipeline_run.tags,
     )
     return execution_plan
+
+
+def _pipeline_with_repository_load_data(
+    pipeline: Union[PipelineDefinition, IPipeline],
+) -> Tuple[Union[PipelineDefinition, IPipeline], Optional[RepositoryLoadData]]:
+    """For ReconstructablePipeline, generate and return any required RepositoryLoadData, alongside
+    a ReconstructablePipeline with this repository load data baked in.
+    """
+    if isinstance(pipeline, ReconstructablePipeline):
+        # Unless this ReconstructablePipeline alread has repository_load_data attached, this will
+        # force the repository_load_data to be computed from scratch.
+        repository_load_data = pipeline.repository.get_definition().repository_load_data
+        return pipeline.with_repository_load_data(repository_load_data), repository_load_data
+    return pipeline, None
