@@ -1,5 +1,7 @@
 from abc import ABC, abstractmethod
-from typing import AbstractSet, Any, Dict, Iterator, List, Mapping, Optional, cast
+from typing import AbstractSet, Any, Dict, Iterator, List, Mapping, Optional, Sequence, cast
+
+from typing_extensions import TypeAlias
 
 import dagster._check as check
 from dagster._annotations import public
@@ -16,6 +18,7 @@ from dagster._core.definitions.job_definition import JobDefinition
 from dagster._core.definitions.mode import ModeDefinition
 from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.definitions.partition import PartitionsDefinition
+from dagster._core.definitions.partition_key_range import PartitionKeyRange
 from dagster._core.definitions.pipeline_definition import PipelineDefinition
 from dagster._core.definitions.solid_definition import SolidDefinition
 from dagster._core.definitions.step_launcher import StepLauncher
@@ -292,6 +295,15 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
         """
         return self._step_execution_context.partition_key
 
+    @public  # type: ignore
+    @property
+    def asset_partition_key_range(self) -> PartitionKeyRange:
+        """The asset partition key for the current run.
+
+        Raises an error if the current run is not a partitioned run.
+        """
+        return self._step_execution_context.asset_partition_key_range
+
     @public  # type: ignore
     @property
     def partition_time_window(self) -> str:
@@ -317,11 +329,20 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
         selected_asset_keys = self.selected_asset_keys
         selected_outputs = set()
         for output_name in self.op.output_dict.keys():
+
             asset_info = self.job_def.asset_layer.asset_info_for_output(
                 self.solid_handle, output_name
             )
-            if asset_info and asset_info.key in selected_asset_keys:
+            if any(  #  For graph-backed assets, check if a downstream asset is selected
+                [
+                    asset_key in selected_asset_keys
+                    for asset_key in self.job_def.asset_layer.downstream_dep_assets(
+                        self.solid_handle, output_name
+                    )
+                ]
+            ) or (asset_info and asset_info.key in selected_asset_keys):
                 selected_outputs.add(output_name)
+
         return selected_outputs
 
     @public
@@ -379,11 +400,19 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
         """
         return self._step_execution_context.asset_partitions_time_window_for_output(output_name)
 
+    @public
+    def asset_partition_key_range_for_output(
+        self, output_name: str = "result"
+    ) -> PartitionKeyRange:
+        return self._step_execution_context.asset_partition_key_range_for_output(output_name)
+
+    @public
+    def asset_partition_key_range_for_input(self, input_name: str) -> PartitionKeyRange:
+        return self._step_execution_context.asset_partition_key_range_for_input(input_name)
+
     @public
     def asset_partition_key_for_input(self, input_name: str) -> str:
-        """Returns the asset partition key for the given output. Defaults to "result", which is the
-        name of the default output.
-        """
+        """Returns the partition key of the upstream asset corresponding to the given input."""
         return self._step_execution_context.asset_partition_key_for_input(input_name)
 
     @public
@@ -414,6 +443,21 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
 
         return result
 
+    @public
+    def asset_partition_keys_for_output(self, output_name: str) -> Sequence[str]:
+        """Returns a list of the partition keys for the given output."""
+        return self.asset_partitions_def_for_output(output_name).get_partition_keys_in_range(
+            self._step_execution_context.asset_partition_key_range_for_output(output_name)
+        )
+
+    @public
+    def asset_partition_keys_for_input(self, input_name: str) -> Sequence[str]:
+        """Returns a list of the partition keys of the upstream asset corresponding to the
+        given input."""
+        return self.asset_partitions_def_for_input(input_name).get_partition_keys_in_range(
+            self._step_execution_context.asset_partition_key_range_for_input(input_name)
+        )
+
     @public
     def has_tag(self, key: str) -> bool:
         """Check if a logging tag is set.
@@ -521,7 +565,7 @@ class SolidExecutionContext(AbstractComputeExecutionContext):
                 return ("dog", 5)
 
         """
-        metadata = check.dict_param(metadata, "metadata", key_type=str)
+        metadata = check.mapping_param(metadata, "metadata", key_type=str)
         output_name = check.opt_str_param(output_name, "output_name")
         mapping_key = check.opt_str_param(mapping_key, "mapping_key")
 
@@ -587,3 +631,6 @@ class OpExecutionContext(SolidExecutionContext):
             context.log.info("Hello, world!")
 
     """
+
+
+SourceAssetObserveContext: TypeAlias = OpExecutionContext
