@@ -1,4 +1,4 @@
-from typing import TYPE_CHECKING, Any, Dict, Iterator, List, Optional, Sequence, Union, cast
+from typing import TYPE_CHECKING, Any, Iterator, List, Mapping, Optional, Sequence, Union, cast
 
 import dagster._check as check
 from dagster._annotations import public
@@ -36,9 +36,17 @@ class InputContext:
         config (Optional[Any]): The config attached to the input that we're loading.
         metadata (Optional[Dict[str, Any]]): A dict of metadata that is assigned to the
             InputDefinition that we're loading for.
+            This property only contains metadata passed in explicitly with :py:class:`AssetIn`
+            or :py:class:`In`. To access metadata of an upstream asset or operation definition,
+            use the metadata in :py:attr:`.InputContext.upstream_output`.
         upstream_output (Optional[OutputContext]): Info about the output that produced the object
             we're loading.
         dagster_type (Optional[DagsterType]): The type of this input.
+            Dagster types do not propagate from an upstream output to downstream inputs,
+            and this property only captures type information for the input that is either
+            passed in explicitly with :py:class:`AssetIn` or :py:class:`In`, or can be
+            infered from type hints. For an asset input, the Dagster type from the upstream
+            asset definition is ignored.
         log (Optional[DagsterLogManager]): The log manager to use for this input.
         resource_config (Optional[Dict[str, Any]]): The config associated with the resource that
             initializes the RootInputManager.
@@ -65,14 +73,16 @@ class InputContext:
         pipeline_name: Optional[str] = None,
         solid_def: Optional["SolidDefinition"] = None,
         config: Optional[Any] = None,
-        metadata: Optional[Dict[str, Any]] = None,
+        metadata: Optional[Mapping[str, Any]] = None,
         upstream_output: Optional["OutputContext"] = None,
         dagster_type: Optional["DagsterType"] = None,
         log_manager: Optional["DagsterLogManager"] = None,
-        resource_config: Optional[Dict[str, Any]] = None,
-        resources: Optional[Union["Resources", Dict[str, Any]]] = None,
+        resource_config: Optional[Mapping[str, Any]] = None,
+        resources: Optional[Union["Resources", Mapping[str, Any]]] = None,
         step_context: Optional["StepExecutionContext"] = None,
         op_def: Optional["OpDefinition"] = None,
+        asset_key: Optional[AssetKey] = None,
+        partition_key: Optional[str] = None,
     ):
         from dagster._core.definitions.resource_definition import IContainsGenerator, Resources
         from dagster._core.execution.build_resources import build_resources
@@ -90,6 +100,11 @@ class InputContext:
         self._log = log_manager
         self._resource_config = resource_config
         self._step_context = step_context
+        self._asset_key = asset_key
+        if self._step_context and self._step_context.has_partition_key:
+            self._partition_key: Optional[str] = self._step_context.partition_key
+        else:
+            self._partition_key = partition_key
 
         if isinstance(resources, Resources):
             self._resources_cm = None
@@ -177,7 +192,7 @@ class InputContext:
 
     @public  # type: ignore
     @property
-    def metadata(self) -> Optional[Dict[str, Any]]:
+    def metadata(self) -> Optional[Mapping[str, Any]]:
         return self._metadata
 
     @public  # type: ignore
@@ -209,7 +224,7 @@ class InputContext:
 
     @public  # type: ignore
     @property
-    def resource_config(self) -> Optional[Dict[str, Any]]:
+    def resource_config(self) -> Optional[Mapping[str, Any]]:
         return self._resource_config
 
     @public  # type: ignore
@@ -232,27 +247,17 @@ class InputContext:
     @public  # type: ignore
     @property
     def has_asset_key(self) -> bool:
-        return (
-            self._step_context is not None
-            and self._name is not None
-            and self._step_context.pipeline_def.asset_layer.asset_key_for_input(
-                node_handle=self.step_context.solid_handle, input_name=self._name
-            )
-            is not None
-        )
+        return self._asset_key is not None
 
     @public  # type: ignore
     @property
     def asset_key(self) -> AssetKey:
-        result = self.step_context.pipeline_def.asset_layer.asset_key_for_input(
-            node_handle=self.step_context.solid_handle, input_name=self.name
-        )
-        if result is None:
+        if self._asset_key is None:
             raise DagsterInvariantViolationError(
                 "Attempting to access asset_key, but no asset is associated with this input"
             )
 
-        return result
+        return self._asset_key
 
     @public  # type: ignore
     @property
@@ -281,7 +286,7 @@ class InputContext:
     @property
     def has_partition_key(self) -> bool:
         """Whether the current run is a partitioned run"""
-        return self.step_context.has_partition_key
+        return self._partition_key is not None
 
     @public  # type: ignore
     @property
@@ -290,7 +295,12 @@ class InputContext:
 
         Raises an error if the current run is not a partitioned run.
         """
-        return self.step_context.partition_key
+        if self._partition_key is None:
+            check.failed(
+                "Tried to access partition_key on a non-partitioned run.",
+            )
+
+        return self._partition_key
 
     @public  # type: ignore
     @property
@@ -319,6 +329,15 @@ class InputContext:
         """
         return self.step_context.asset_partition_key_range_for_input(self.name)
 
+    @public  # type: ignore
+    @property
+    def asset_partition_keys(self) -> Sequence[str]:
+        """The partition keys for input asset.
+
+        Raises an error if the input asset has no partitioning.
+        """
+        return self.asset_partitions_def.get_partition_keys_in_range(self.asset_partition_key_range)
+
     @public  # type: ignore
     @property
     def asset_partitions_time_window(self) -> TimeWindow:
@@ -384,7 +403,7 @@ class InputContext:
     def get_asset_identifier(self) -> Sequence[str]:
         if self.asset_key is not None:
             if self.has_asset_partitions:
-                return self.asset_key.path + [self.asset_partition_key]
+                return [*self.asset_key.path, self.asset_partition_key]
             else:
                 return self.asset_key.path
         else:
@@ -402,7 +421,7 @@ class InputContext:
 
     def add_input_metadata(
         self,
-        metadata: Dict[str, Any],
+        metadata: Mapping[str, Any],
         description: Optional[str] = None,
     ) -> None:
         """Accepts a dictionary of metadata. Metadata entries will appear on the LOADED_INPUT event.
@@ -414,7 +433,7 @@ class InputContext:
         from dagster._core.definitions.metadata import normalize_metadata
         from dagster._core.events import DagsterEvent
 
-        metadata = check.dict_param(metadata, "metadata", key_type=str)
+        metadata = check.mapping_param(metadata, "metadata", key_type=str)
         self._metadata_entries.extend(normalize_metadata(metadata, []))
         if self.has_asset_key:
             check.opt_str_param(description, "description")
@@ -431,7 +450,7 @@ class InputContext:
 
     def get_observations(
         self,
-    ) -> List[AssetObservation]:
+    ) -> Sequence[AssetObservation]:
         """Retrieve the list of user-generated asset observations that were observed via the context.
 
         User-generated events that were yielded will not appear in this list.
@@ -455,7 +474,7 @@ class InputContext:
         """
         return self._observations
 
-    def consume_metadata_entries(self) -> List[Union[MetadataEntry, PartitionMetadataEntry]]:
+    def consume_metadata_entries(self) -> Sequence[Union[MetadataEntry, PartitionMetadataEntry]]:
         result = self._metadata_entries
         self._metadata_entries = []
         return result
@@ -464,13 +483,15 @@ class InputContext:
 def build_input_context(
     name: Optional[str] = None,
     config: Optional[Any] = None,
-    metadata: Optional[Dict[str, Any]] = None,
+    metadata: Optional[Mapping[str, Any]] = None,
     upstream_output: Optional["OutputContext"] = None,
     dagster_type: Optional["DagsterType"] = None,
-    resource_config: Optional[Dict[str, Any]] = None,
-    resources: Optional[Dict[str, Any]] = None,
+    resource_config: Optional[Mapping[str, Any]] = None,
+    resources: Optional[Mapping[str, Any]] = None,
     op_def: Optional["OpDefinition"] = None,
     step_context: Optional["StepExecutionContext"] = None,
+    asset_key: Optional["AssetKey"] = None,
+    partition_key: Optional[str] = None,
 ) -> "InputContext":
     """Builds input context from provided parameters.
 
@@ -495,6 +516,7 @@ def build_input_context(
         asset_key (Optional[AssetKey]): The asset key attached to the InputDefinition.
         op_def (Optional[OpDefinition]): The definition of the op that's loading the input.
         step_context (Optional[StepExecutionContext]): For internal use.
+        partition_key (Optional[str]): String value representing partition key to execute with.
 
     Examples:
 
@@ -512,13 +534,15 @@ def build_input_context(
     from dagster._core.types.dagster_type import DagsterType
 
     name = check.opt_str_param(name, "name")
-    metadata = check.opt_dict_param(metadata, "metadata", key_type=str)
+    metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
     upstream_output = check.opt_inst_param(upstream_output, "upstream_output", OutputContext)
     dagster_type = check.opt_inst_param(dagster_type, "dagster_type", DagsterType)
-    resource_config = check.opt_dict_param(resource_config, "resource_config", key_type=str)
-    resources = check.opt_dict_param(resources, "resources", key_type=str)
+    resource_config = check.opt_mapping_param(resource_config, "resource_config", key_type=str)
+    resources = check.opt_mapping_param(resources, "resources", key_type=str)
     op_def = check.opt_inst_param(op_def, "op_def", OpDefinition)
     step_context = check.opt_inst_param(step_context, "step_context", StepExecutionContext)
+    asset_key = check.opt_inst_param(asset_key, "asset_key", AssetKey)
+    partition_key = check.opt_str_param(partition_key, "partition_key")
 
     return InputContext(
         name=name,
@@ -532,4 +556,6 @@ def build_input_context(
         resources=resources,
         step_context=step_context,
         op_def=op_def,
+        asset_key=asset_key,
+        partition_key=partition_key,
     )
