@@ -325,7 +325,7 @@ class OutputContext:
     @public  # type: ignore
     @property
     def asset_partitions_def(self) -> "PartitionsDefinition":
-        """The PartitionsDefinition on the upstream asset corresponding to this input."""
+        """The PartitionsDefinition on the asset corresponding to this output."""
         asset_key = self.asset_key
         result = self.step_context.pipeline_def.asset_layer.partitions_def_for_asset(asset_key)
         if result is None:
@@ -382,11 +382,12 @@ class OutputContext:
                 "For more details: https://github.com/dagster-io/dagster/issues/7900"
             )
 
-        check.invariant(
-            self._partition_key is not None,
-            "Tried to access partition_key on a non-partitioned run.",
-        )
-        return cast(str, self._partition_key)
+        if self._partition_key is None:
+            check.failed(
+                "Tried to access partition_key on a non-partitioned run.",
+            )
+
+        return self._partition_key
 
     @public  # type: ignore
     @property
@@ -439,6 +440,25 @@ class OutputContext:
 
         return self.step_context.asset_partition_key_range_for_output(self.name)
 
+    @public  # type: ignore
+    @property
+    def asset_partition_keys(self) -> Sequence[str]:
+        """The partition keys for the output asset.
+
+        Raises an error if the output asset has no partitioning.
+        """
+        if self._warn_on_step_context_use:
+            warnings.warn(
+                "You are using InputContext.upstream_output.asset_partition_keys"
+                "This use on upstream_output is deprecated and will fail in the future"
+                "Try to obtain what you need directly from InputContext"
+                "For more details: https://github.com/dagster-io/dagster/issues/7900"
+            )
+
+        return self.asset_partitions_def.get_partition_keys_in_range(
+            self.step_context.asset_partition_key_range_for_output(self.name)
+        )
+
     @public  # type: ignore
     @property
     def asset_partitions_time_window(self) -> TimeWindow:
@@ -551,7 +571,7 @@ class OutputContext:
     def get_asset_identifier(self) -> Sequence[str]:
         if self.asset_key is not None:
             if self.has_asset_partitions:
-                return self.asset_key.path + [self.asset_partition_key]
+                return [*self.asset_key.path, self.asset_partition_key]
             else:
                 return self.asset_key.path
         else:
@@ -828,13 +848,13 @@ def build_output_context(
 
     step_key = check.opt_str_param(step_key, "step_key")
     name = check.opt_str_param(name, "name")
-    metadata = check.opt_dict_param(metadata, "metadata", key_type=str)
+    metadata = check.opt_mapping_param(metadata, "metadata", key_type=str)
     run_id = check.opt_str_param(run_id, "run_id", default=RUN_ID_PLACEHOLDER)
     mapping_key = check.opt_str_param(mapping_key, "mapping_key")
     dagster_type = check.opt_inst_param(dagster_type, "dagster_type", DagsterType)
     version = check.opt_str_param(version, "version")
-    resource_config = check.opt_dict_param(resource_config, "resource_config", key_type=str)
-    resources = check.opt_dict_param(resources, "resources", key_type=str)
+    resource_config = check.opt_mapping_param(resource_config, "resource_config", key_type=str)
+    resources = check.opt_mapping_param(resources, "resources", key_type=str)
     op_def = check.opt_inst_param(op_def, "op_def", OpDefinition)
     asset_key = AssetKey.from_coerceable(asset_key) if asset_key else None
     partition_key = check.opt_str_param(partition_key, "partition_key")
