@@ -11,10 +11,10 @@ from typing import (
     Generator,
     Generic,
     Iterable,
-    List,
     Mapping,
     NamedTuple,
     Optional,
+    Sequence,
     Type,
     TypeVar,
     Union,
@@ -63,7 +63,7 @@ def initialize_console_manager(
 ) -> DagsterLogManager:
     # initialize default colored console logger
     loggers = []
-    for logger_def, logger_config in default_system_loggers():
+    for logger_def, logger_config in default_system_loggers(instance):
         loggers.append(
             logger_def.logger_fn(
                 InitLoggerContext(
@@ -124,6 +124,7 @@ def create_context_creation_data(
     pipeline_run: PipelineRun,
     instance: DagsterInstance,
 ) -> "ContextCreationData":
+
     pipeline_def = pipeline.get_definition()
     resolved_run_config = ResolvedRunConfig.build(pipeline_def, run_config, mode=pipeline_run.mode)
 
@@ -227,7 +228,7 @@ def execution_context_event_generator(
     execution_plan = check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
     pipeline_def = pipeline.get_definition()
 
-    run_config = check.dict_param(run_config, "run_config", key_type=str)
+    run_config = check.mapping_param(run_config, "run_config", key_type=str)
     pipeline_run = check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     instance = check.inst_param(instance, "instance", DagsterInstance)
 
@@ -285,7 +286,7 @@ class PlanOrchestrationContextManager(ExecutionContextManager[PlanOrchestrationC
         instance: DagsterInstance,
         raise_on_error: Optional[bool] = False,
         output_capture: Optional[Dict["StepOutputHandle", Any]] = None,
-        executor_defs: Optional[List[ExecutorDefinition]] = None,
+        executor_defs: Optional[Sequence[ExecutorDefinition]] = None,
         resume_from_failure=False,
     ):
         event_generator = context_event_generator(
@@ -313,8 +314,8 @@ def orchestration_context_event_generator(
     pipeline_run: PipelineRun,
     instance: DagsterInstance,
     raise_on_error: bool,
-    executor_defs: Optional[List[ExecutorDefinition]],
-    output_capture: Optional[Dict["StepOutputHandle", Any]],
+    executor_defs: Optional[Sequence[ExecutorDefinition]],
+    output_capture: Optional[Mapping["StepOutputHandle", Any]],
     resume_from_failure: bool = False,
 ) -> Generator[Union[DagsterEvent, PlanOrchestrationContext], None, None]:
     check.invariant(executor_defs is None)
@@ -445,7 +446,7 @@ def scoped_pipeline_context(
     """
     check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
     check.inst_param(pipeline, "pipeline", IPipeline)
-    check.dict_param(run_config, "run_config", key_type=str)
+    check.mapping_param(run_config, "run_config", key_type=str)
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     check.inst_param(instance, "instance", DagsterInstance)
     check.callable_param(scoped_resources_builder_cm, "scoped_resources_builder_cm")
@@ -483,7 +484,7 @@ def create_log_manager(
     )
 
     # The following logic is tightly coupled to the processing of logger config in
-    # python_modules/dagster/dagster/core/system_config/objects.py#config_map_loggers
+    # python_modules/dagster/dagster/_core/system_config/objects.py#config_map_loggers
     # Changes here should be accompanied checked against that function, which applies config mapping
     # via ConfigurableDefinition (@configured) to incoming logger configs. See docstring for more details.
 
@@ -502,7 +503,7 @@ def create_log_manager(
             )
 
     if not loggers:
-        for (logger_def, logger_config) in default_system_loggers():
+        for (logger_def, logger_config) in default_system_loggers(context_creation_data.instance):
             loggers.append(
                 logger_def.logger_fn(
                     InitLoggerContext(
@@ -533,7 +534,7 @@ def create_context_free_log_manager(
 
     loggers = []
     # Use the default logger
-    for (logger_def, logger_config) in default_system_loggers():
+    for (logger_def, logger_config) in default_system_loggers(instance):
         loggers += [
             logger_def.logger_fn(
                 InitLoggerContext(
