@@ -1,17 +1,19 @@
 import sys
 from contextlib import ExitStack
-from typing import Iterator, List, cast
+from typing import Iterator, Sequence, cast
 
 import dagster._check as check
 from dagster._core.definitions import Failure, HookExecutionResult, RetryRequested
 from dagster._core.errors import (
     DagsterError,
     DagsterExecutionInterruptedError,
+    DagsterMaxRetriesExceededError,
     DagsterUserCodeExecutionError,
     HookExecutionError,
     user_code_error_boundary,
 )
 from dagster._core.events import DagsterEvent, EngineEventData
+from dagster._core.execution.compute_logs import create_compute_log_file_key
 from dagster._core.execution.context.system import PlanExecutionContext, StepExecutionContext
 from dagster._core.execution.plan.execute_step import core_dagster_event_sequence_for_step
 from dagster._core.execution.plan.objects import (
@@ -22,6 +24,7 @@ from dagster._core.execution.plan.objects import (
     step_failure_event_from_exc_info,
 )
 from dagster._core.execution.plan.plan import ExecutionPlan
+from dagster._core.storage.captured_log_manager import CapturedLogManager
 from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 
 
@@ -30,8 +33,19 @@ def inner_plan_execution_iterator(
 ) -> Iterator[DagsterEvent]:
     check.inst_param(pipeline_context, "pipeline_context", PlanExecutionContext)
     check.inst_param(execution_plan, "execution_plan", ExecutionPlan)
+    compute_log_manager = pipeline_context.instance.compute_log_manager
+    step_keys = [step.key for step in execution_plan.get_steps_to_execute_in_topo_order()]
+    with ExitStack() as plan_stack:
+        active_execution = plan_stack.enter_context(
+            execution_plan.start(retry_mode=pipeline_context.retry_mode)
+        )
 
-    with execution_plan.start(retry_mode=pipeline_context.retry_mode) as active_execution:
+        # begin capturing logs for the whole process if this is a captured log manager
+        if isinstance(compute_log_manager, CapturedLogManager):
+            file_key = create_compute_log_file_key()
+            log_key = compute_log_manager.build_log_key_for_run(pipeline_context.run_id, file_key)
+            log_context = plan_stack.enter_context(compute_log_manager.capture_logs(log_key))
+            yield DagsterEvent.capture_logs(pipeline_context, step_keys, log_key, log_context)
 
         # It would be good to implement a reference tracking algorithm here to
         # garbage collect results that are no longer needed by any steps
@@ -57,48 +71,61 @@ def inner_plan_execution_iterator(
                 ).format(solid_name=step_context.solid.name, missing_resources=missing_resources),
             )
 
-            # capture all of the logs for this step
-            with ExitStack() as stack:
-                log_capture_error = None
-                try:
-                    stack.enter_context(
-                        pipeline_context.instance.compute_log_manager.watch(
-                            step_context.pipeline_run, step_context.step.key
+            with ExitStack() as step_stack:
+                if not isinstance(compute_log_manager, CapturedLogManager):
+                    # capture all of the logs for individual steps
+                    log_capture_error = None
+                    try:
+                        step_stack.enter_context(
+                            pipeline_context.instance.compute_log_manager.watch(
+                                step_context.pipeline_run, step_context.step.key
+                            )
                         )
-                    )
-                except Exception as e:
-                    yield DagsterEvent.engine_event(
-                        plan_context=step_context,
-                        message="Exception while setting up compute log capture",
-                        event_specific_data=EngineEventData(
-                            error=serializable_error_info_from_exc_info(sys.exc_info())
-                        ),
-                    )
-                    log_capture_error = e
+                    except Exception as e:
+                        yield DagsterEvent.engine_event(
+                            plan_context=step_context,
+                            message="Exception while setting up compute log capture",
+                            event_specific_data=EngineEventData(
+                                error=serializable_error_info_from_exc_info(sys.exc_info())
+                            ),
+                        )
+                        log_capture_error = e
 
-                if not log_capture_error:
-                    yield DagsterEvent.capture_logs(
-                        step_context, log_key=step_context.step.key, steps=[step_context.step]
-                    )
+                    if not log_capture_error:
+                        yield DagsterEvent.legacy_compute_log_step_event(step_context)
 
-                for step_event in check.generator(dagster_event_sequence_for_step(step_context)):
-                    check.inst(step_event, DagsterEvent)
-                    step_event_list.append(step_event)
-                    yield step_event
-                    active_execution.handle_event(step_event)
+                    for step_event in check.generator(
+                        dagster_event_sequence_for_step(step_context)
+                    ):
+                        check.inst(step_event, DagsterEvent)
+                        step_event_list.append(step_event)
+                        yield step_event
+                        active_execution.handle_event(step_event)
 
-                active_execution.verify_complete(pipeline_context, step.key)
+                    active_execution.verify_complete(pipeline_context, step.key)
 
-                try:
-                    stack.close()
-                except Exception:
-                    yield DagsterEvent.engine_event(
-                        plan_context=step_context,
-                        message="Exception while cleaning up compute log capture",
-                        event_specific_data=EngineEventData(
-                            error=serializable_error_info_from_exc_info(sys.exc_info())
-                        ),
-                    )
+                    try:
+                        step_stack.close()
+                    except Exception:
+                        yield DagsterEvent.engine_event(
+                            plan_context=step_context,
+                            message="Exception while cleaning up compute log capture",
+                            event_specific_data=EngineEventData(
+                                error=serializable_error_info_from_exc_info(sys.exc_info())
+                            ),
+                        )
+                else:
+                    # we have already set up the log capture at the process level, just handle the
+                    # step events
+                    for step_event in check.generator(
+                        dagster_event_sequence_for_step(step_context)
+                    ):
+                        check.inst(step_event, DagsterEvent)
+                        step_event_list.append(step_event)
+                        yield step_event
+                        active_execution.handle_event(step_event)
+
+                    active_execution.verify_complete(pipeline_context, step.key)
 
             # process skips from failures or uncovered inputs
             for event in active_execution.plan_events_iterator(pipeline_context):
@@ -111,7 +138,7 @@ def inner_plan_execution_iterator(
 
 
 def _trigger_hook(
-    step_context: StepExecutionContext, step_event_list: List[DagsterEvent]
+    step_context: StepExecutionContext, step_event_list: Sequence[DagsterEvent]
 ) -> Iterator[DagsterEvent]:
     """Trigger hooks and record hook's operatonal events"""
     hook_defs = step_context.pipeline_def.get_all_hooks_for_handle(step_context.solid_handle)
@@ -259,6 +286,8 @@ def dagster_event_sequence_for_step(
                         error_source=ErrorSource.USER_CODE_ERROR if fail_err.cause else None,
                     ),
                 )
+                if step_context.raise_on_error:
+                    raise DagsterMaxRetriesExceededError.from_error_info(fail_err)
             else:
                 yield DagsterEvent.step_retry_event(
                     step_context,
