@@ -1,7 +1,7 @@
 import multiprocessing
 import os
 import sys
-from typing import List, Optional
+from typing import Optional, Sequence
 
 from dagster import MetadataEntry
 from dagster import _check as check
@@ -45,6 +45,7 @@ class MultiprocessExecutorChildProcessCommand(ChildProcessCommand):
         recon_pipeline,
         retry_mode,
         known_state,
+        repository_load_data,
     ):
         self.run_config = run_config
         self.pipeline_run = pipeline_run
@@ -54,6 +55,7 @@ class MultiprocessExecutorChildProcessCommand(ChildProcessCommand):
         self.recon_pipeline = recon_pipeline
         self.retry_mode = retry_mode
         self.known_state = known_state
+        self.repository_load_data = repository_load_data
 
     def execute(self):
         pipeline = self.recon_pipeline
@@ -65,6 +67,7 @@ class MultiprocessExecutorChildProcessCommand(ChildProcessCommand):
                 mode=self.pipeline_run.mode,
                 step_keys_to_execute=[self.step_key],
                 known_state=self.known_state,
+                repository_load_data=self.repository_load_data,
             )
 
             log_manager = create_context_free_log_manager(instance, self.pipeline_run)
@@ -95,7 +98,7 @@ class MultiprocessExecutor(Executor):
         retries: RetryMode,
         max_concurrent: int,
         start_method: Optional[str] = None,
-        explicit_forkserver_preload: Optional[List[str]] = None,
+        explicit_forkserver_preload: Optional[Sequence[str]] = None,
     ):
         self._retries = check.inst_param(retries, "retries", RetryMode)
         max_concurrent = max_concurrent if max_concurrent else multiprocessing.cpu_count()
@@ -201,6 +204,7 @@ class MultiprocessExecutor(Executor):
                                 term_events,
                                 self.retries,
                                 active_execution.get_known_state(),
+                                execution_plan.repository_load_data,
                             )
 
                     # process active iterators
@@ -303,6 +307,7 @@ def execute_step_out_of_process(
     term_events,
     retries,
     known_state,
+    repository_load_data,
 ):
     command = MultiprocessExecutorChildProcessCommand(
         run_config=step_context.run_config,
@@ -313,6 +318,7 @@ def execute_step_out_of_process(
         recon_pipeline=pipeline,
         retry_mode=retries,
         known_state=known_state,
+        repository_load_data=repository_load_data,
     )
 
     yield DagsterEvent.step_worker_starting(
