@@ -6,9 +6,10 @@ for that.
 """
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from datetime import datetime
 from typing import Dict, List, Mapping, NamedTuple, Optional, Sequence, Set, Tuple, Union, cast
 
+import pendulum
+
 from dagster import StaticPartitionsDefinition
 from dagster import _check as check
 from dagster._core.definitions import (
@@ -22,17 +23,18 @@ from dagster._core.definitions import (
     SourceAsset,
 )
 from dagster._core.definitions.asset_layer import AssetOutputInfo
+from dagster._core.definitions.asset_sensor_definition import AssetSensorDefinition
+from dagster._core.definitions.assets_job import ASSET_BASE_JOB_PREFIX
 from dagster._core.definitions.dependency import NodeOutputHandle
 from dagster._core.definitions.events import AssetKey
+from dagster._core.definitions.freshness_policy import FreshnessPolicy
 from dagster._core.definitions.metadata import MetadataEntry, MetadataUserInput, normalize_metadata
 from dagster._core.definitions.mode import DEFAULT_MODE_NAME
+from dagster._core.definitions.multi_dimensional_partitions import MultiPartitionsDefinition
+from dagster._core.definitions.op_definition import OpDefinition
 from dagster._core.definitions.partition import PartitionScheduleDefinition, ScheduleType
 from dagster._core.definitions.schedule_definition import DefaultScheduleStatus
-from dagster._core.definitions.sensor_definition import (
-    AssetSensorDefinition,
-    DefaultSensorStatus,
-    SensorDefinition,
-)
+from dagster._core.definitions.sensor_definition import DefaultSensorStatus, SensorDefinition
 from dagster._core.definitions.time_window_partitions import TimeWindowPartitionsDefinition
 from dagster._core.definitions.utils import DEFAULT_GROUP_NAME
 from dagster._core.errors import DagsterInvalidDefinitionError
@@ -47,29 +49,28 @@ class ExternalRepositoryData(
         "_ExternalRepositoryData",
         [
             ("name", str),
-            ("external_pipeline_datas", Sequence["ExternalPipelineData"]),
             ("external_schedule_datas", Sequence["ExternalScheduleData"]),
             ("external_partition_set_datas", Sequence["ExternalPartitionSetData"]),
             ("external_sensor_datas", Sequence["ExternalSensorData"]),
             ("external_asset_graph_data", Sequence["ExternalAssetNode"]),
+            ("external_pipeline_datas", Optional[Sequence["ExternalPipelineData"]]),
+            ("external_job_refs", Optional[Sequence["ExternalJobRef"]]),
         ],
     )
 ):
     def __new__(
         cls,
         name: str,
-        external_pipeline_datas: Sequence["ExternalPipelineData"],
         external_schedule_datas: Sequence["ExternalScheduleData"],
         external_partition_set_datas: Sequence["ExternalPartitionSetData"],
         external_sensor_datas: Optional[Sequence["ExternalSensorData"]] = None,
         external_asset_graph_data: Optional[Sequence["ExternalAssetNode"]] = None,
+        external_pipeline_datas: Optional[Sequence["ExternalPipelineData"]] = None,
+        external_job_refs: Optional[Sequence["ExternalJobRef"]] = None,
     ):
         return super(ExternalRepositoryData, cls).__new__(
             cls,
             name=check.str_param(name, "name"),
-            external_pipeline_datas=check.sequence_param(
-                external_pipeline_datas, "external_pipeline_datas", of_type=ExternalPipelineData
-            ),
             external_schedule_datas=check.sequence_param(
                 external_schedule_datas, "external_schedule_datas", of_type=ExternalScheduleData
             ),
@@ -88,10 +89,31 @@ class ExternalRepositoryData(
                 "external_asset_graph_dats",
                 of_type=ExternalAssetNode,
             ),
+            external_pipeline_datas=check.opt_nullable_sequence_param(
+                external_pipeline_datas, "external_pipeline_datas", of_type=ExternalPipelineData
+            ),
+            external_job_refs=check.opt_nullable_sequence_param(
+                external_job_refs, "external_job_refs", of_type=ExternalJobRef
+            ),
         )
 
+    def has_pipeline_data(self):
+        return self.external_pipeline_datas is not None
+
+    def get_external_pipeline_datas(self) -> Sequence["ExternalPipelineData"]:
+        if self.external_pipeline_datas is None:
+            check.failed("Snapshots were deferred, external_pipeline_data not loaded")
+        return self.external_pipeline_datas
+
+    def get_external_job_refs(self) -> Sequence["ExternalJobRef"]:
+        if self.external_job_refs is None:
+            check.failed("Snapshots were not deferred, external_job_refs not loaded")
+        return self.external_job_refs
+
     def get_pipeline_snapshot(self, name):
         check.str_param(name, "name")
+        if self.external_pipeline_datas is None:
+            check.failed("Snapshots were deferred, external_pipeline_data not loaded")
 
         for external_pipeline_data in self.external_pipeline_datas:
             if external_pipeline_data.name == name:
@@ -101,6 +123,8 @@ class ExternalRepositoryData(
 
     def get_external_pipeline_data(self, name):
         check.str_param(name, "name")
+        if self.external_pipeline_datas is None:
+            check.failed("Snapshots were deferred, external_pipeline_data not loaded")
 
         for external_pipeline_data in self.external_pipeline_datas:
             if external_pipeline_data.name == name:
@@ -193,13 +217,46 @@ class ExternalPipelineData(
             parent_pipeline_snapshot=check.opt_inst_param(
                 parent_pipeline_snapshot, "parent_pipeline_snapshot", PipelineSnapshot
             ),
-            active_presets=check.list_param(
+            active_presets=check.sequence_param(
                 active_presets, "active_presets", of_type=ExternalPresetData
             ),
             is_job=check.bool_param(is_job, "is_job"),
         )
 
 
+@whitelist_for_serdes
+class ExternalJobRef(
+    NamedTuple(
+        "_ExternalJobRef",
+        [
+            ("name", str),
+            ("snapshot_id", str),
+            ("active_presets", Sequence["ExternalPresetData"]),
+            ("parent_snapshot_id", Optional[str]),
+            ("is_legacy_pipeline", bool),
+        ],
+    )
+):
+    def __new__(
+        cls,
+        name: str,
+        snapshot_id: str,
+        active_presets: Sequence["ExternalPresetData"],
+        parent_snapshot_id: Optional[str],
+        is_legacy_pipeline: bool = False,
+    ):
+        return super(ExternalJobRef, cls).__new__(
+            cls,
+            name=check.str_param(name, "name"),
+            snapshot_id=check.str_param(snapshot_id, "snapshot_id"),
+            active_presets=check.list_param(
+                active_presets, "active_presets", of_type=ExternalPresetData
+            ),
+            parent_snapshot_id=check.opt_str_param(parent_snapshot_id, "parent_snapshot_id"),
+            is_legacy_pipeline=check.bool_param(is_legacy_pipeline, "is_legacy"),
+        )
+
+
 @whitelist_for_serdes
 class ExternalPresetData(
     NamedTuple(
@@ -229,7 +286,7 @@ class ExternalPresetData(
                 solid_selection, "solid_selection", of_type=str
             ),
             mode=check.str_param(mode, "mode"),
-            tags=check.opt_dict_param(tags, "tags", key_type=str, value_type=str),
+            tags=check.opt_mapping_param(tags, "tags", key_type=str, value_type=str),
         )
 
 
@@ -245,7 +302,7 @@ class ExternalScheduleData(
         "_ExternalScheduleData",
         [
             ("name", str),
-            ("cron_schedule", str),
+            ("cron_schedule", Union[str, Sequence[str]]),
             ("pipeline_name", str),
             ("solid_selection", Optional[Sequence[str]]),
             ("mode", Optional[str]),
@@ -270,10 +327,14 @@ class ExternalScheduleData(
         description=None,
         default_status=None,
     ):
+        cron_schedule = check.inst_param(cron_schedule, "cron_schedule", (str, Sequence))
+        if not isinstance(cron_schedule, str):
+            cron_schedule = check.sequence_param(cron_schedule, "cron_schedule", of_type=str)
+
         return super(ExternalScheduleData, cls).__new__(
             cls,
             name=check.str_param(name, "name"),
-            cron_schedule=check.str_param(cron_schedule, "cron_schedule"),
+            cron_schedule=cron_schedule,
             pipeline_name=check.str_param(pipeline_name, "pipeline_name"),
             solid_selection=check.opt_nullable_list_param(solid_selection, "solid_selection", str),
             mode=check.opt_str_param(mode, "mode"),
@@ -392,7 +453,9 @@ class ExternalSensorData(
             mode=check.opt_str_param(mode, "mode"),  # keep legacy field populated
             min_interval=check.opt_int_param(min_interval, "min_interval"),
             description=check.opt_str_param(description, "description"),
-            target_dict=check.opt_dict_param(target_dict, "target_dict", str, ExternalTargetData),
+            target_dict=check.opt_mapping_param(
+                target_dict, "target_dict", str, ExternalTargetData
+            ),
             metadata=check.opt_inst_param(metadata, "metadata", ExternalSensorMetadata),
             default_status=DefaultSensorStatus.RUNNING
             if default_status == DefaultSensorStatus.RUNNING
@@ -464,51 +527,68 @@ class ExternalTimeWindowPartitionsDefinitionData(
     NamedTuple(
         "_ExternalTimeWindowPartitionsDefinitionData",
         [
-            ("schedule_type", ScheduleType),
             ("start", float),
             ("timezone", Optional[str]),
             ("fmt", str),
             ("end_offset", int),
-            ("minute_offset", int),
-            ("hour_offset", int),
+            ("cron_schedule", Optional[str]),
+            # superseded by cron_schedule, but kept around for backcompat
+            ("schedule_type", Optional[ScheduleType]),
+            # superseded by cron_schedule, but kept around for backcompat
+            ("minute_offset", Optional[int]),
+            # superseded by cron_schedule, but kept around for backcompat
+            ("hour_offset", Optional[int]),
+            # superseded by cron_schedule, but kept around for backcompat
             ("day_offset", Optional[int]),
         ],
     ),
 ):
     def __new__(
         cls,
-        schedule_type: ScheduleType,
         start: float,
         timezone: Optional[str],
         fmt: str,
         end_offset: int,
-        minute_offset: int = 0,
-        hour_offset: int = 0,
+        cron_schedule: Optional[str] = None,
+        schedule_type: Optional[ScheduleType] = None,
+        minute_offset: Optional[int] = None,
+        hour_offset: Optional[int] = None,
         day_offset: Optional[int] = None,
     ):
         return super(ExternalTimeWindowPartitionsDefinitionData, cls).__new__(
             cls,
-            schedule_type=check.inst_param(schedule_type, "schedule_type", ScheduleType),
+            schedule_type=check.opt_inst_param(schedule_type, "schedule_type", ScheduleType),
             start=check.float_param(start, "start"),
             timezone=check.opt_str_param(timezone, "timezone"),
             fmt=check.str_param(fmt, "fmt"),
             end_offset=check.int_param(end_offset, "end_offset"),
-            minute_offset=check.int_param(minute_offset, "minute_offset"),
-            hour_offset=check.int_param(hour_offset, "hour_offset"),
+            minute_offset=check.opt_int_param(minute_offset, "minute_offset"),
+            hour_offset=check.opt_int_param(hour_offset, "hour_offset"),
             day_offset=check.opt_int_param(day_offset, "day_offset"),
+            cron_schedule=check.opt_str_param(cron_schedule, "cron_schedule"),
         )
 
     def get_partitions_definition(self):
-        return TimeWindowPartitionsDefinition(
-            self.schedule_type,
-            datetime.fromtimestamp(self.start),
-            self.timezone,
-            self.fmt,
-            self.end_offset,
-            self.minute_offset,
-            self.hour_offset,
-            self.day_offset,
-        )
+        if self.cron_schedule is not None:
+            return TimeWindowPartitionsDefinition(
+                cron_schedule=self.cron_schedule,
+                start=pendulum.from_timestamp(self.start, tz=self.timezone),
+                timezone=self.timezone,
+                fmt=self.fmt,
+                end_offset=self.end_offset,
+            )
+        else:
+            # backcompat case
+            return TimeWindowPartitionsDefinition(
+                schedule_type=self.schedule_type,
+                start=pendulum.from_timestamp(self.start, tz=self.timezone),
+                timezone=self.timezone,
+                fmt=self.fmt,
+                end_offset=self.end_offset,
+                minute_offset=self.minute_offset,
+                hour_offset=self.hour_offset,
+                day_offset=self.day_offset,
+            )
 
 
 @whitelist_for_serdes
@@ -518,13 +598,54 @@ class ExternalStaticPartitionsDefinitionData(
 ):
     def __new__(cls, partition_keys: Sequence[str]):
         return super(ExternalStaticPartitionsDefinitionData, cls).__new__(
-            cls, partition_keys=check.list_param(partition_keys, "partition_keys", str)
+            cls, partition_keys=check.sequence_param(partition_keys, "partition_keys", str)
         )
 
     def get_partitions_definition(self):
         return StaticPartitionsDefinition(self.partition_keys)
 
 
+@whitelist_for_serdes
+class ExternalPartitionDimensionDefinition(
+    NamedTuple(
+        "_ExternalPartitionDimensionDefinition",
+        [("name", str), ("external_partitions_def_data", ExternalPartitionsDefinitionData)],
+    )
+):
+    def __new__(cls, name: str, external_partitions_def_data: ExternalPartitionsDefinitionData):
+        return super(ExternalPartitionDimensionDefinition, cls).__new__(
+            cls,
+            name=check.str_param(name, "name"),
+            external_partitions_def_data=check.inst_param(
+                external_partitions_def_data,
+                "external_partitions_def_data",
+                ExternalPartitionsDefinitionData,
+            ),
+        )
+
+
+@whitelist_for_serdes
+class ExternalMultiPartitionsDefinitionData(
+    ExternalPartitionsDefinitionData,
+    NamedTuple(
+        "_ExternalMultiPartitionsDefinitionData",
+        [
+            (
+                "external_partition_dimension_definitions",
+                Sequence[ExternalPartitionDimensionDefinition],
+            )
+        ],
+    ),
+):
+    def get_partitions_definition(self):
+        return MultiPartitionsDefinition(
+            {
+                partition_dimension.name: partition_dimension.external_partitions_def_data.get_partitions_definition()
+                for partition_dimension in self.external_partition_dimension_definitions
+            }
+        )
+
+
 @whitelist_for_serdes
 class ExternalPartitionSetData(
     NamedTuple(
@@ -534,6 +655,7 @@ class ExternalPartitionSetData(
             ("pipeline_name", str),
             ("solid_selection", Optional[Sequence[str]]),
             ("mode", Optional[str]),
+            ("external_partitions_data", Optional[ExternalPartitionsDefinitionData]),
         ],
     )
 ):
@@ -543,6 +665,7 @@ class ExternalPartitionSetData(
         pipeline_name: str,
         solid_selection: Optional[Sequence[str]],
         mode: Optional[str],
+        external_partitions_data: Optional[ExternalPartitionsDefinitionData] = None,
     ):
         return super(ExternalPartitionSetData, cls).__new__(
             cls,
@@ -552,6 +675,11 @@ class ExternalPartitionSetData(
                 solid_selection, "solid_selection", str
             ),
             mode=check.opt_str_param(mode, "mode"),
+            external_partitions_data=check.opt_inst_param(
+                external_partitions_data,
+                "external_partitions_data",
+                ExternalPartitionsDefinitionData,
+            ),
         )
 
 
@@ -562,7 +690,7 @@ class ExternalPartitionNamesData(
     def __new__(cls, partition_names: Optional[Sequence[str]] = None):
         return super(ExternalPartitionNamesData, cls).__new__(
             cls,
-            partition_names=check.opt_list_param(partition_names, "partition_names", str),
+            partition_names=check.opt_sequence_param(partition_names, "partition_names", str),
         )
 
 
@@ -618,7 +746,7 @@ class ExternalPartitionSetExecutionParamData(
     def __new__(cls, partition_data: Sequence[ExternalPartitionExecutionParamData]):
         return super(ExternalPartitionSetExecutionParamData, cls).__new__(
             cls,
-            partition_data=check.list_param(
+            partition_data=check.sequence_param(
                 partition_data, "partition_data", of_type=ExternalPartitionExecutionParamData
             ),
         )
@@ -708,6 +836,7 @@ class ExternalAssetNode(
             ("compute_kind", Optional[str]),
             ("op_name", Optional[str]),
             ("op_names", Optional[Sequence[str]]),
+            ("op_version", Optional[str]),
             ("node_definition_name", Optional[str]),
             ("graph_name", Optional[str]),
             ("op_description", Optional[str]),
@@ -717,6 +846,9 @@ class ExternalAssetNode(
             ("output_description", Optional[str]),
             ("metadata_entries", Sequence[MetadataEntry]),
             ("group_name", Optional[str]),
+            ("freshness_policy", Optional[FreshnessPolicy]),
+            ("is_source", bool),
+            ("is_observable", bool),
         ],
     )
 ):
@@ -733,6 +865,7 @@ class ExternalAssetNode(
         compute_kind: Optional[str] = None,
         op_name: Optional[str] = None,
         op_names: Optional[Sequence[str]] = None,
+        op_version: Optional[str] = None,
         node_definition_name: Optional[str] = None,
         graph_name: Optional[str] = None,
         op_description: Optional[str] = None,
@@ -742,10 +875,20 @@ class ExternalAssetNode(
         output_description: Optional[str] = None,
         metadata_entries: Optional[Sequence[MetadataEntry]] = None,
         group_name: Optional[str] = None,
+        freshness_policy: Optional[FreshnessPolicy] = None,
+        is_source: Optional[bool] = None,
+        is_observable: bool = False,
     ):
         # backcompat logic to handle ExternalAssetNodes serialized without op_names/graph_name
         if not op_names:
             op_names = list(filter(None, [op_name]))
+
+        # backcompat logic to handle ExternalAssetNodes serialzied without is_source
+        if is_source is None:
+            # prior to this field being added, all non-source assets must be part of at least one
+            # job, and no source assets could be part of any job
+            is_source = len(job_names or []) == 0
+
         return super(ExternalAssetNode, cls).__new__(
             cls,
             asset_key=check.inst_param(asset_key, "asset_key", AssetKey),
@@ -757,7 +900,8 @@ class ExternalAssetNode(
             ),
             compute_kind=check.opt_str_param(compute_kind, "compute_kind"),
             op_name=check.opt_str_param(op_name, "op_name"),
-            op_names=check.opt_list_param(op_names, "op_names"),
+            op_names=check.opt_sequence_param(op_names, "op_names"),
+            op_version=check.opt_str_param(op_version, "op_version"),
             node_definition_name=check.opt_str_param(node_definition_name, "node_definition_name"),
             graph_name=check.opt_str_param(graph_name, "graph_name"),
             op_description=check.opt_str_param(
@@ -773,21 +917,36 @@ class ExternalAssetNode(
                 metadata_entries, "metadata_entries", of_type=MetadataEntry
             ),
             group_name=check.opt_str_param(group_name, "group_name"),
+            freshness_policy=check.opt_inst_param(
+                freshness_policy, "freshness_policy", FreshnessPolicy
+            ),
+            is_source=check.bool_param(is_source, "is_source"),
+            is_observable=check.bool_param(is_observable, "is_observable"),
         )
 
 
 def external_repository_data_from_def(
     repository_def: RepositoryDefinition,
+    defer_snapshots: bool = False,
 ) -> ExternalRepositoryData:
     check.inst_param(repository_def, "repository_def", RepositoryDefinition)
 
     pipelines = repository_def.get_all_pipelines()
-    return ExternalRepositoryData(
-        name=repository_def.name,
-        external_pipeline_datas=sorted(
+    if defer_snapshots:
+        pipeline_datas = None
+        job_refs = sorted(
+            list(map(external_job_ref_from_def, pipelines)),
+            key=lambda pd: pd.name,
+        )
+    else:
+        pipeline_datas = sorted(
             list(map(external_pipeline_data_from_def, pipelines)),
             key=lambda pd: pd.name,
-        ),
+        )
+        job_refs = None
+
+    return ExternalRepositoryData(
+        name=repository_def.name,
         external_schedule_datas=sorted(
             list(map(external_schedule_data_from_def, repository_def.schedule_defs)),
             key=lambda sd: sd.name,
@@ -797,12 +956,17 @@ def external_repository_data_from_def(
             key=lambda psd: psd.name,
         ),
         external_sensor_datas=sorted(
-            list(map(external_sensor_data_from_def, repository_def.sensor_defs)),
+            [
+                external_sensor_data_from_def(sensor_def, repository_def)
+                for sensor_def in repository_def.sensor_defs
+            ],
             key=lambda sd: sd.name,
         ),
         external_asset_graph_data=external_asset_graph_from_defs(
             pipelines, source_assets_by_key=repository_def.source_assets_by_key
         ),
+        external_pipeline_datas=pipeline_datas,
+        external_job_refs=job_refs,
     )
 
 
@@ -813,13 +977,15 @@ def external_asset_graph_from_defs(
         AssetKey, List[Tuple[NodeOutputHandle, PipelineDefinition]]
     ] = defaultdict(list)
     asset_info_by_asset_key: Dict[AssetKey, AssetOutputInfo] = dict()
+    freshness_policy_by_asset_key: Dict[AssetKey, FreshnessPolicy] = dict()
     metadata_by_asset_key: Dict[AssetKey, MetadataUserInput] = dict()
 
     deps: Dict[AssetKey, Dict[AssetKey, ExternalAssetDependency]] = defaultdict(dict)
     dep_by: Dict[AssetKey, Dict[AssetKey, ExternalAssetDependedBy]] = defaultdict(dict)
     all_upstream_asset_keys: Set[AssetKey] = set()
     op_names_by_asset_key: Dict[AssetKey, Sequence[str]] = {}
-    group_names: Dict[AssetKey, str] = {}
+    op_version_by_asset_key: Dict[AssetKey, Optional[str]] = dict()
+    group_name_by_asset_key: Dict[AssetKey, str] = {}
 
     for pipeline_def in pipelines:
         asset_info_by_node_output = pipeline_def.asset_layer.asset_info_by_node_output_handle
@@ -840,6 +1006,10 @@ def external_asset_graph_from_defs(
             node_defs_by_asset_key[output_key].append((node_output_handle, pipeline_def))
             asset_info_by_asset_key[output_key] = asset_info
 
+            node_def = pipeline_def.get_solid(node_output_handle.node_handle).definition
+            if isinstance(node_def, OpDefinition):
+                op_version_by_asset_key[output_key] = node_def.version
+
             for upstream_key in upstream_asset_keys:
                 deps[output_key][upstream_key] = ExternalAssetDependency(
                     upstream_asset_key=upstream_key
@@ -848,9 +1018,11 @@ def external_asset_graph_from_defs(
                     downstream_asset_key=output_key
                 )
 
+        # for assets_def in pipeline_def.asset_layer.assets_defs_by_key.values():
         for assets_def in pipeline_def.asset_layer.assets_defs_by_key.values():
             metadata_by_asset_key.update(assets_def.metadata_by_key)
-        group_names.update(pipeline_def.asset_layer.group_names_by_assets())
+            freshness_policy_by_asset_key.update(assets_def.freshness_policies_by_key)
+        group_name_by_asset_key.update(pipeline_def.asset_layer.group_names_by_assets())
 
     asset_keys_without_definitions = all_upstream_asset_keys.difference(
         node_defs_by_asset_key.keys()
@@ -862,7 +1034,8 @@ def external_asset_graph_from_defs(
             dependencies=list(deps[asset_key].values()),
             depended_by=list(dep_by[asset_key].values()),
             job_names=[],
-            group_name=group_names.get(asset_key),
+            group_name=group_name_by_asset_key.get(asset_key),
+            op_version=op_version_by_asset_key.get(asset_key),
         )
         for asset_key in asset_keys_without_definitions
     ]
@@ -878,10 +1051,12 @@ def external_asset_graph_from_defs(
                     asset_key=source_asset.key,
                     dependencies=list(deps[source_asset.key].values()),
                     depended_by=list(dep_by[source_asset.key].values()),
-                    job_names=[],
+                    job_names=[ASSET_BASE_JOB_PREFIX] if source_asset.node_def is not None else [],
                     op_description=source_asset.description,
                     metadata_entries=metadata_entries,
                     group_name=source_asset.group_name,
+                    is_source=True,
+                    is_observable=source_asset.is_observable,
                 )
             )
 
@@ -908,26 +1083,11 @@ def external_asset_graph_from_defs(
 
         job_names = [job_def.name for _, job_def in node_tuple_list]
 
-        # temporary workaround to retrieve asset partition definition from job
-        partitions_def_data: Optional[
-            Union[
-                ExternalTimeWindowPartitionsDefinitionData,
-                ExternalStaticPartitionsDefinitionData,
-            ]
-        ] = None
+        partitions_def_data: Optional[ExternalPartitionsDefinitionData] = None
 
         partitions_def = asset_info.partitions_def
         if partitions_def:
-            if isinstance(partitions_def, TimeWindowPartitionsDefinition):
-                partitions_def_data = external_time_window_partitions_definition_from_def(
-                    partitions_def
-                )
-            elif isinstance(partitions_def, StaticPartitionsDefinition):
-                partitions_def_data = external_static_partitions_definition_from_def(partitions_def)
-            else:
-                raise DagsterInvalidDefinitionError(
-                    "Only static partition and time window partitions are currently supported."
-                )
+            partitions_def_data = external_partitions_definition_from_def(partitions_def)
 
         # if the asset is produced by an op at the top level of the graph, graph_name should be None
         graph_name = None
@@ -948,17 +1108,18 @@ def external_asset_graph_from_defs(
                 or node_def.name,
                 graph_name=graph_name,
                 op_names=op_names_by_asset_key[asset_key],
-                op_description=node_def.description,
+                op_version=op_version_by_asset_key.get(asset_key),
+                op_description=node_def.description or output_def.description,
                 node_definition_name=node_def.name,
                 job_names=job_names,
                 partitions_def_data=partitions_def_data,
                 output_name=output_def.name,
-                output_description=output_def.description,
                 metadata_entries=asset_metadata_entries,
                 # assets defined by Out(asset_key="k") do not have any group
                 # name specified we default to DEFAULT_GROUP_NAME here to ensure
                 # such assets are part of the default group
-                group_name=group_names.get(asset_key, DEFAULT_GROUP_NAME),
+                group_name=group_name_by_asset_key.get(asset_key, DEFAULT_GROUP_NAME),
+                freshness_policy=freshness_policy_by_asset_key.get(asset_key),
             )
         )
 
@@ -979,6 +1140,27 @@ def external_pipeline_data_from_def(pipeline_def: PipelineDefinition) -> Externa
     )
 
 
+def external_job_ref_from_def(pipeline_def: PipelineDefinition) -> ExternalJobRef:
+    check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
+
+    parent = pipeline_def.parent_pipeline_def
+    if parent:
+        parent_snapshot_id = parent.get_pipeline_snapshot_id()
+    else:
+        parent_snapshot_id = None
+
+    return ExternalJobRef(
+        name=pipeline_def.name,
+        snapshot_id=pipeline_def.get_pipeline_snapshot_id(),
+        parent_snapshot_id=parent_snapshot_id,
+        active_presets=sorted(
+            list(map(external_preset_data_from_def, pipeline_def.preset_defs)),
+            key=lambda pd: pd.name,
+        ),
+        is_legacy_pipeline=not isinstance(pipeline_def, JobDefinition),
+    )
+
+
 def external_schedule_data_from_def(schedule_def: ScheduleDefinition) -> ExternalScheduleData:
     check.inst_param(schedule_def, "schedule_def", ScheduleDefinition)
     return ExternalScheduleData(
@@ -997,19 +1179,31 @@ def external_schedule_data_from_def(schedule_def: ScheduleDefinition) -> Externa
     )
 
 
+def external_partitions_definition_from_def(
+    partitions_def: PartitionsDefinition,
+) -> ExternalPartitionsDefinitionData:
+    if isinstance(partitions_def, TimeWindowPartitionsDefinition):
+        return external_time_window_partitions_definition_from_def(partitions_def)
+    elif isinstance(partitions_def, StaticPartitionsDefinition):
+        return external_static_partitions_definition_from_def(partitions_def)
+    elif isinstance(partitions_def, MultiPartitionsDefinition):
+        return external_multi_partitions_definition_from_def(partitions_def)
+    else:
+        raise DagsterInvalidDefinitionError(
+            "Only static, time window, and multi-dimensional partitions are currently supported."
+        )
+
+
 def external_time_window_partitions_definition_from_def(
     partitions_def: TimeWindowPartitionsDefinition,
 ) -> ExternalTimeWindowPartitionsDefinitionData:
     check.inst_param(partitions_def, "partitions_def", TimeWindowPartitionsDefinition)
     return ExternalTimeWindowPartitionsDefinitionData(
-        schedule_type=partitions_def.schedule_type,
-        start=partitions_def.start.timestamp(),
+        cron_schedule=partitions_def.cron_schedule,
+        start=pendulum.instance(partitions_def.start, tz=partitions_def.timezone).timestamp(),
         timezone=partitions_def.timezone,
         fmt=partitions_def.fmt,
         end_offset=partitions_def.end_offset,
-        minute_offset=partitions_def.minute_offset,
-        hour_offset=partitions_def.hour_offset,
-        day_offset=partitions_def.day_offset,
     )
 
 
@@ -1022,38 +1216,90 @@ def external_static_partitions_definition_from_def(
     )
 
 
+def external_multi_partitions_definition_from_def(
+    partitions_def: MultiPartitionsDefinition,
+) -> ExternalMultiPartitionsDefinitionData:
+    check.inst_param(partitions_def, "partitions_def", MultiPartitionsDefinition)
+
+    if any(
+        [
+            not (
+                isinstance(dimension.partitions_def, TimeWindowPartitionsDefinition)
+                or isinstance(dimension.partitions_def, StaticPartitionsDefinition)
+            )
+            for dimension in partitions_def.partitions_defs
+        ]
+    ):
+        raise DagsterInvalidDefinitionError(
+            "Only static and time window partition dimensions are currently supported."
+        )
+
+    return ExternalMultiPartitionsDefinitionData(
+        external_partition_dimension_definitions=[
+            ExternalPartitionDimensionDefinition(
+                dimension.name, external_partitions_definition_from_def(dimension.partitions_def)
+            )
+            for dimension in partitions_def.partitions_defs
+        ]
+    )
+
+
 def external_partition_set_data_from_def(
     partition_set_def: PartitionSetDefinition,
 ) -> ExternalPartitionSetData:
     check.inst_param(partition_set_def, "partition_set_def", PartitionSetDefinition)
+
+    partitions_def = partition_set_def._partitions_def  # pylint: disable=protected-access
+
+    partitions_def_data: Optional[ExternalPartitionsDefinitionData] = None
+    if isinstance(partitions_def, TimeWindowPartitionsDefinition):
+        partitions_def_data = external_time_window_partitions_definition_from_def(partitions_def)
+    elif isinstance(partitions_def, StaticPartitionsDefinition):
+        partitions_def_data = external_static_partitions_definition_from_def(partitions_def)
+    else:
+        partitions_def_data = None
+
     return ExternalPartitionSetData(
         name=partition_set_def.name,
         pipeline_name=partition_set_def.pipeline_or_job_name,
         solid_selection=partition_set_def.solid_selection,
         mode=partition_set_def.mode,
+        external_partitions_data=partitions_def_data,
     )
 
 
-def external_sensor_data_from_def(sensor_def: SensorDefinition) -> ExternalSensorData:
+def external_sensor_data_from_def(
+    sensor_def: SensorDefinition, repository_def: RepositoryDefinition
+) -> ExternalSensorData:
     first_target = sensor_def.targets[0] if sensor_def.targets else None
 
     asset_keys = None
     if isinstance(sensor_def, AssetSensorDefinition):
         asset_keys = [sensor_def.asset_key]
 
-    return ExternalSensorData(
-        name=sensor_def.name,
-        pipeline_name=first_target.pipeline_name if first_target else None,
-        mode=first_target.mode if first_target else None,
-        solid_selection=first_target.solid_selection if first_target else None,
-        target_dict={
+    if sensor_def.asset_selection is not None:
+        target_dict = {
+            base_asset_job_name: ExternalTargetData(
+                pipeline_name=base_asset_job_name, mode=DEFAULT_MODE_NAME, solid_selection=None
+            )
+            for base_asset_job_name in repository_def.get_base_asset_job_names()
+        }
+    else:
+        target_dict = {
             target.pipeline_name: ExternalTargetData(
                 pipeline_name=target.pipeline_name,
                 mode=target.mode,
                 solid_selection=target.solid_selection,
             )
             for target in sensor_def.targets
-        },
+        }
+
+    return ExternalSensorData(
+        name=sensor_def.name,
+        pipeline_name=first_target.pipeline_name if first_target else None,
+        mode=first_target.mode if first_target else None,
+        solid_selection=first_target.solid_selection if first_target else None,
+        target_dict=target_dict,
         min_interval=sensor_def.minimum_interval_seconds,
         description=sensor_def.description,
         metadata=ExternalSensorMetadata(asset_keys=asset_keys),
