@@ -7,11 +7,11 @@ from typing import (
     Any,
     Dict,
     Generator,
-    List,
     Mapping,
     NamedTuple,
     NoReturn,
     Optional,
+    Sequence,
     Set,
     Type,
     cast,
@@ -19,6 +19,7 @@ from typing import (
 
 import dagster._check as check
 from dagster._core.errors import DagsterInvariantViolationError, DagsterUserCodeUnreachableError
+from dagster._core.instance.config import DEFAULT_LOCAL_CODE_SERVER_STARTUP_TIMEOUT
 from dagster._core.origin import DEFAULT_DAGSTER_ENTRY_POINT
 from dagster._core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster._serdes import (
@@ -99,7 +100,7 @@ class RepositoryLocationOrigin(ABC, tuple):
         raise NotImplementedError
 
     @abstractmethod
-    def get_display_metadata(self) -> Dict[str, Any]:
+    def get_display_metadata(self) -> Mapping[str, Any]:
         pass
 
     def get_id(self) -> str:
@@ -114,10 +115,6 @@ class RepositoryLocationOrigin(ABC, tuple):
     def create_location(self) -> "RepositoryLocation":
         pass
 
-    @property
-    def supports_server_watch(self):
-        return False
-
 
 @whitelist_for_serdes
 class RegisteredRepositoryLocationOrigin(
@@ -132,7 +129,7 @@ class RegisteredRepositoryLocationOrigin(
     def __new__(cls, location_name: str):
         return super(RegisteredRepositoryLocationOrigin, cls).__new__(cls, location_name)
 
-    def get_display_metadata(self) -> Dict[str, Any]:
+    def get_display_metadata(self) -> Mapping[str, Any]:
         return {}
 
     def create_location(self) -> NoReturn:
@@ -149,8 +146,9 @@ class InProcessRepositoryLocationOrigin(
         [
             ("loadable_target_origin", LoadableTargetOrigin),
             ("container_image", Optional[str]),
-            ("entry_point", List[str]),
-            ("container_context", Optional[Dict[str, Any]]),
+            ("entry_point", Sequence[str]),
+            ("container_context", Optional[Mapping[str, Any]]),
+            ("location_name", str),
         ],
     ),
     RepositoryLocationOrigin,
@@ -164,8 +162,9 @@ class InProcessRepositoryLocationOrigin(
         cls,
         loadable_target_origin: LoadableTargetOrigin,
         container_image: Optional[str] = None,
-        entry_point: Optional[List[str]] = None,
+        entry_point: Optional[Sequence[str]] = None,
         container_context=None,
+        location_name: Optional[str] = None,
     ):
         return super(InProcessRepositoryLocationOrigin, cls).__new__(
             cls,
@@ -174,22 +173,21 @@ class InProcessRepositoryLocationOrigin(
             ),
             container_image=check.opt_str_param(container_image, "container_image"),
             entry_point=(
-                check.opt_list_param(entry_point, "entry_point")
+                check.opt_sequence_param(entry_point, "entry_point")
                 if entry_point
                 else DEFAULT_DAGSTER_ENTRY_POINT
             ),
             container_context=check.opt_dict_param(container_context, "container_context"),
+            location_name=check.opt_str_param(
+                location_name, "location_name", default=IN_PROCESS_NAME
+            ),
         )
 
-    @property
-    def location_name(self) -> str:
-        return IN_PROCESS_NAME
-
     @property
     def is_reload_supported(self) -> bool:
         return False
 
-    def get_display_metadata(self) -> Dict[str, Any]:
+    def get_display_metadata(self) -> Mapping[str, Any]:
         return {}
 
     def create_location(self) -> "InProcessRepositoryLocation":
@@ -225,7 +223,7 @@ class ManagedGrpcPythonEnvRepositoryLocationOrigin(
             else _assign_loadable_target_origin_name(loadable_target_origin),
         )
 
-    def get_display_metadata(self) -> Dict[str, str]:
+    def get_display_metadata(self) -> Mapping[str, str]:
         metadata = {
             "python_file": self.loadable_target_origin.python_file,
             "module_name": self.loadable_target_origin.module_name,
@@ -244,7 +242,8 @@ class ManagedGrpcPythonEnvRepositoryLocationOrigin(
 
     @contextmanager
     def create_single_location(
-        self, instance: "DagsterInstance"
+        self,
+        instance: "DagsterInstance",
     ) -> Generator["RepositoryLocation", None, None]:
         from dagster._core.workspace.context import DAGIT_GRPC_SERVER_HEARTBEAT_TTL
 
@@ -252,9 +251,12 @@ class ManagedGrpcPythonEnvRepositoryLocationOrigin(
         from .repository_location import GrpcServerRepositoryLocation
 
         with ProcessGrpcServerRegistry(
+            instance=instance,
             reload_interval=0,
             heartbeat_ttl=DAGIT_GRPC_SERVER_HEARTBEAT_TTL,
-            startup_timeout=instance.code_server_process_startup_timeout,
+            startup_timeout=instance.code_server_process_startup_timeout
+            if instance
+            else DEFAULT_LOCAL_CODE_SERVER_STARTUP_TIMEOUT,
         ) as grpc_server_registry:
             endpoint = grpc_server_registry.get_grpc_endpoint(self)
             with GrpcServerRepositoryLocation(
@@ -313,7 +315,7 @@ class GrpcServerRepositoryLocationOrigin(
             use_ssl if check.opt_bool_param(use_ssl, "use_ssl") else None,
         )
 
-    def get_display_metadata(self) -> Dict[str, str]:
+    def get_display_metadata(self) -> Mapping[str, str]:
         metadata = {
             "host": self.host,
             "port": str(self.port) if self.port else None,
@@ -328,10 +330,6 @@ class GrpcServerRepositoryLocationOrigin(
 
         return GrpcServerRepositoryLocation(self)
 
-    @property
-    def supports_server_watch(self) -> bool:
-        return True
-
     def create_client(self) -> "DagsterGrpcClient":
         from dagster._grpc.client import DagsterGrpcClient
 
@@ -420,6 +418,10 @@ class ExternalPipelineOrigin(
     def get_id(self) -> str:
         return create_snapshot_id(self)
 
+    @property
+    def location_name(self) -> str:
+        return self.external_repository_origin.repository_location_origin.location_name
+
 
 class ExternalInstigatorOriginSerializer(DefaultNamedTupleSerializer):
     @classmethod
