@@ -3,7 +3,7 @@ import sys
 import threading
 from abc import abstractmethod
 from contextlib import AbstractContextManager
-from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Optional, Union, cast
+from typing import TYPE_CHECKING, Any, Dict, Mapping, Optional, Sequence, Tuple, Union, cast
 
 import dagster._check as check
 from dagster._api.get_server_id import sync_get_server_id
@@ -22,15 +22,22 @@ from dagster._api.snapshot_schedule import sync_get_external_schedule_execution_
 from dagster._api.snapshot_sensor import sync_get_external_sensor_execution_data_grpc
 from dagster._core.code_pointer import CodePointer
 from dagster._core.definitions.reconstruct import ReconstructablePipeline
-from dagster._core.errors import DagsterInvariantViolationError
+from dagster._core.definitions.repository_definition import RepositoryDefinition
+from dagster._core.errors import DagsterInvariantViolationError, DagsterUserCodeProcessError
 from dagster._core.execution.api import create_execution_plan
 from dagster._core.execution.plan.state import KnownExecutionState
 from dagster._core.host_representation import ExternalPipelineSubsetResult
 from dagster._core.host_representation.external import (
     ExternalExecutionPlan,
+    ExternalPartitionSet,
     ExternalPipeline,
     ExternalRepository,
 )
+from dagster._core.host_representation.external_data import (
+    ExternalPartitionNamesData,
+    ExternalScheduleExecutionErrorData,
+    ExternalSensorExecutionErrorData,
+)
 from dagster._core.host_representation.grpc_server_registry import GrpcServerRegistry
 from dagster._core.host_representation.handle import PipelineHandle, RepositoryHandle
 from dagster._core.host_representation.origin import (
@@ -63,12 +70,9 @@ if TYPE_CHECKING:
     from dagster._core.host_representation import (
         ExternalPartitionConfigData,
         ExternalPartitionExecutionErrorData,
-        ExternalPartitionNamesData,
         ExternalPartitionSetExecutionParamData,
         ExternalPartitionTagsData,
-        ExternalScheduleExecutionErrorData,
     )
-    from dagster._core.host_representation.external_data import ExternalSensorExecutionErrorData
 
 
 class RepositoryLocation(AbstractContextManager):
@@ -100,10 +104,10 @@ class RepositoryLocation(AbstractContextManager):
         pass
 
     @abstractmethod
-    def get_repositories(self) -> Dict[str, ExternalRepository]:
+    def get_repositories(self) -> Mapping[str, ExternalRepository]:
         pass
 
-    def get_repository_names(self) -> List[str]:
+    def get_repository_names(self) -> Sequence[str]:
         return list(self.get_repositories().keys())
 
     @property
@@ -116,7 +120,7 @@ class RepositoryLocation(AbstractContextManager):
         external_pipeline: ExternalPipeline,
         run_config: Mapping[str, object],
         mode: str,
-        step_keys_to_execute: Optional[List[str]],
+        step_keys_to_execute: Optional[Sequence[str]],
         known_state: Optional[KnownExecutionState],
         instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
@@ -128,15 +132,20 @@ class RepositoryLocation(AbstractContextManager):
         a solid selection is specified, which requires access to the underlying PipelineDefinition
         to generate the subsetted pipeline snapshot."""
         if not selector.solid_selection and not selector.asset_selection:
-            return self.get_repository(selector.repository_name).get_full_external_pipeline(
+            return self.get_repository(selector.repository_name).get_full_external_job(
                 selector.pipeline_name
             )
 
         repo_handle = self.get_repository(selector.repository_name).handle
 
-        return ExternalPipeline(
-            self.get_subset_external_pipeline_result(selector).external_pipeline_data, repo_handle
-        )
+        subset_result = self.get_subset_external_pipeline_result(selector)
+        external_data = subset_result.external_pipeline_data
+        if external_data is None:
+            check.failed(
+                f"Failed to fetch subset data, success: {subset_result.success} error: {subset_result.error}"
+            )
+
+        return ExternalPipeline(external_data, repo_handle)
 
     @abstractmethod
     def get_subset_external_pipeline_result(
@@ -160,7 +169,7 @@ class RepositoryLocation(AbstractContextManager):
 
     @abstractmethod
     def get_external_partition_names(
-        self, repository_handle: RepositoryHandle, partition_set_name: str
+        self, external_partition_set: ExternalPartitionSet
     ) -> Union["ExternalPartitionNamesData", "ExternalPartitionExecutionErrorData"]:
         pass
 
@@ -169,7 +178,7 @@ class RepositoryLocation(AbstractContextManager):
         self,
         repository_handle: RepositoryHandle,
         partition_set_name: str,
-        partition_names: List[str],
+        partition_names: Sequence[str],
     ) -> Union["ExternalPartitionSetExecutionParamData", "ExternalPartitionExecutionErrorData"]:
         pass
 
@@ -180,7 +189,7 @@ class RepositoryLocation(AbstractContextManager):
         repository_handle: RepositoryHandle,
         schedule_name: str,
         scheduled_execution_time,
-    ) -> Union["ScheduleExecutionData", "ExternalScheduleExecutionErrorData"]:
+    ) -> "ScheduleExecutionData":
         pass
 
     @abstractmethod
@@ -192,7 +201,7 @@ class RepositoryLocation(AbstractContextManager):
         last_completion_time: Optional[float],
         last_run_key: Optional[str],
         cursor: Optional[str],
-    ) -> Union["SensorExecutionData", "ExternalSensorExecutionErrorData"]:
+    ) -> "SensorExecutionData":
         pass
 
     @abstractmethod
@@ -218,7 +227,7 @@ class RepositoryLocation(AbstractContextManager):
     def origin(self) -> RepositoryLocationOrigin:
         pass
 
-    def get_display_metadata(self) -> Dict[str, str]:
+    def get_display_metadata(self) -> Mapping[str, str]:
         return merge_dicts(
             self.origin.get_display_metadata(),
             ({"image": self.container_image} if self.container_image else {}),
@@ -235,17 +244,17 @@ class RepositoryLocation(AbstractContextManager):
         pass
 
     @property
-    def container_context(self) -> Optional[Dict[str, Any]]:
+    def container_context(self) -> Optional[Mapping[str, Any]]:
         return None
 
     @property
     @abstractmethod
-    def entry_point(self) -> Optional[List[str]]:
+    def entry_point(self) -> Optional[Sequence[str]]:
         pass
 
     @property
     @abstractmethod
-    def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
+    def repository_code_pointer_dict(self) -> Mapping[str, CodePointer]:
         pass
 
     def get_repository_python_origin(self, repository_name: str) -> "RepositoryPythonOrigin":
@@ -280,15 +289,8 @@ class InProcessRepositoryLocation(RepositoryLocation):
 
         self._repository_code_pointer_dict = self._loaded_repositories.code_pointers_by_repo_name
 
-        self._recon_repos = {
-            repo_name: self._loaded_repositories.get_recon_repo(repo_name)
-            for repo_name in self._repository_code_pointer_dict
-        }
-
-        self._repositories = {}
-        for repo_name in self._repository_code_pointer_dict:
-            recon_repo = self._loaded_repositories.get_recon_repo(repo_name)
-            repo_def = recon_repo.get_definition()
+        self._repositories: Dict[str, ExternalRepository] = {}
+        for repo_name, repo_def in self._loaded_repositories.definitions_by_name.items():
             self._repositories[repo_name] = external_repo_from_def(
                 repo_def,
                 RepositoryHandle(repository_name=repo_name, repository_location=self),
@@ -311,19 +313,26 @@ class InProcessRepositoryLocation(RepositoryLocation):
         return self._origin.container_image
 
     @property
-    def container_context(self) -> Optional[Dict[str, Any]]:
+    def container_context(self) -> Optional[Mapping[str, Any]]:
         return self._origin.container_context
 
     @property
-    def entry_point(self) -> Optional[List[str]]:
+    def entry_point(self) -> Optional[Sequence[str]]:
         return self._origin.entry_point
 
     @property
-    def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
+    def repository_code_pointer_dict(self) -> Mapping[str, CodePointer]:
         return self._repository_code_pointer_dict
 
-    def get_reconstructable_pipeline(self, name: str) -> ReconstructablePipeline:
-        return self._recon_repos[name].get_reconstructable_pipeline(name)
+    def get_reconstructable_pipeline(
+        self, repository_name: str, name: str
+    ) -> ReconstructablePipeline:
+        return self._loaded_repositories.reconstructables_by_name[
+            repository_name
+        ].get_reconstructable_pipeline(name)
+
+    def _get_repo_def(self, name: str) -> RepositoryDefinition:
+        return self._loaded_repositories.definitions_by_name[name]
 
     def get_repository(self, name: str) -> ExternalRepository:
         return self._repositories[name]
@@ -331,7 +340,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
     def has_repository(self, name: str) -> bool:
         return name in self._repositories
 
-    def get_repositories(self) -> Dict[str, ExternalRepository]:
+    def get_repositories(self) -> Mapping[str, ExternalRepository]:
         return self._repositories
 
     def get_subset_external_pipeline_result(
@@ -348,7 +357,8 @@ class InProcessRepositoryLocation(RepositoryLocation):
         from dagster._grpc.impl import get_external_pipeline_subset_result
 
         return get_external_pipeline_subset_result(
-            self.get_reconstructable_pipeline(selector.pipeline_name),
+            self._get_repo_def(selector.repository_name),
+            selector.pipeline_name,
             selector.solid_selection,
             selector.asset_selection,
         )
@@ -358,22 +368,25 @@ class InProcessRepositoryLocation(RepositoryLocation):
         external_pipeline: ExternalPipeline,
         run_config: Mapping[str, object],
         mode: str,
-        step_keys_to_execute: Optional[List[str]],
+        step_keys_to_execute: Optional[Sequence[str]],
         known_state: Optional[KnownExecutionState],
         instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-        check.dict_param(run_config, "run_config")
+        check.mapping_param(run_config, "run_config")
         check.str_param(mode, "mode")
-        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        step_keys_to_execute = check.opt_nullable_sequence_param(
+            step_keys_to_execute, "step_keys_to_execute", of_type=str
+        )
         check.opt_inst_param(known_state, "known_state", KnownExecutionState)
         check.opt_inst_param(instance, "instance", DagsterInstance)
 
         execution_plan = create_execution_plan(
             pipeline=self.get_reconstructable_pipeline(
-                external_pipeline.name
+                external_pipeline.repository_handle.repository_name, external_pipeline.name
             ).subset_for_execution_from_existing_pipeline(
-                external_pipeline.solids_to_execute, external_pipeline.asset_selection
+                external_pipeline.solids_to_execute,
+                external_pipeline.asset_selection,
             ),
             run_config=run_config,
             mode=mode,
@@ -396,7 +409,7 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.str_param(partition_name, "partition_name")
 
         return get_partition_config(
-            recon_repo=self._recon_repos[repository_handle.repository_name],
+            self._get_repo_def(repository_handle.repository_name),
             partition_set_name=partition_set_name,
             partition_name=partition_name,
         )
@@ -409,20 +422,26 @@ class InProcessRepositoryLocation(RepositoryLocation):
         check.str_param(partition_name, "partition_name")
 
         return get_partition_tags(
-            recon_repo=self._recon_repos[repository_handle.repository_name],
+            self._get_repo_def(repository_handle.repository_name),
             partition_set_name=partition_set_name,
             partition_name=partition_name,
         )
 
     def get_external_partition_names(
-        self, repository_handle: RepositoryHandle, partition_set_name: str
+        self, external_partition_set: ExternalPartitionSet
     ) -> Union["ExternalPartitionNamesData", "ExternalPartitionExecutionErrorData"]:
-        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-        check.str_param(partition_set_name, "partition_set_name")
+        check.inst_param(external_partition_set, "external_partition_set", ExternalPartitionSet)
+
+        # Prefer to return the names without calling out to user code if the
+        # partition set allows it
+        if external_partition_set.has_partition_name_data():
+            return ExternalPartitionNamesData(
+                partition_names=external_partition_set.get_partition_names()
+            )
 
         return get_partition_names(
-            recon_repo=self._recon_repos[repository_handle.repository_name],
-            partition_set_name=partition_set_name,
+            self._get_repo_def(external_partition_set.repository_handle.repository_name),
+            partition_set_name=external_partition_set.name,
         )
 
     def get_external_schedule_execution_data(
@@ -431,14 +450,14 @@ class InProcessRepositoryLocation(RepositoryLocation):
         repository_handle: RepositoryHandle,
         schedule_name: str,
         scheduled_execution_time,
-    ) -> Union["ScheduleExecutionData", "ExternalScheduleExecutionErrorData"]:
+    ) -> "ScheduleExecutionData":
         check.inst_param(instance, "instance", DagsterInstance)
         check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
         check.str_param(schedule_name, "schedule_name")
         check.opt_inst_param(scheduled_execution_time, "scheduled_execution_time", PendulumDateTime)
 
-        return get_external_schedule_execution(
-            recon_repo=self._recon_repos[repository_handle.repository_name],
+        result = get_external_schedule_execution(
+            self._get_repo_def(repository_handle.repository_name),
             instance_ref=instance.get_ref(),
             schedule_name=schedule_name,
             scheduled_execution_timestamp=scheduled_execution_time.timestamp()
@@ -448,6 +467,10 @@ class InProcessRepositoryLocation(RepositoryLocation):
             if scheduled_execution_time
             else None,
         )
+        if isinstance(result, ExternalScheduleExecutionErrorData):
+            raise DagsterUserCodeProcessError.from_error_info(result.error)
+
+        return result
 
     def get_external_sensor_execution_data(
         self,
@@ -457,28 +480,32 @@ class InProcessRepositoryLocation(RepositoryLocation):
         last_completion_time: Optional[float],
         last_run_key: Optional[str],
         cursor: Optional[str],
-    ) -> Union["SensorExecutionData", "ExternalSensorExecutionErrorData"]:
-        return get_external_sensor_execution(
-            self._recon_repos[repository_handle.repository_name],
+    ) -> "SensorExecutionData":
+        result = get_external_sensor_execution(
+            self._get_repo_def(repository_handle.repository_name),
             instance.get_ref(),
             name,
             last_completion_time,
             last_run_key,
             cursor,
         )
+        if isinstance(result, ExternalSensorExecutionErrorData):
+            raise DagsterUserCodeProcessError.from_error_info(result.error)
+
+        return result
 
     def get_external_partition_set_execution_param_data(
         self,
         repository_handle: RepositoryHandle,
         partition_set_name: str,
-        partition_names: List[str],
+        partition_names: Sequence[str],
     ) -> Union["ExternalPartitionSetExecutionParamData", "ExternalPartitionExecutionErrorData"]:
         check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
         check.str_param(partition_set_name, "partition_set_name")
-        check.list_param(partition_names, "partition_names", of_type=str)
+        check.sequence_param(partition_names, "partition_names", of_type=str)
 
         return get_partition_set_execution_param_data(
-            recon_repo=self._recon_repos[repository_handle.repository_name],
+            self._get_repo_def(repository_handle.repository_name),
             partition_set_name=partition_set_name,
             partition_names=partition_names,
         )
@@ -499,6 +526,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         heartbeat: Optional[bool] = False,
         watch_server: Optional[bool] = True,
         grpc_server_registry: Optional[GrpcServerRegistry] = None,
+        grpc_metadata: Optional[Sequence[Tuple[str, str]]] = None,
     ):
         from dagster._grpc.client import DagsterGrpcClient, client_heartbeat_thread
 
@@ -519,9 +547,6 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
             self._host = check.str_param(host, "host")
             self._use_ssl = False
 
-        self._watch_thread_shutdown_event = None
-        self._watch_thread = None
-
         self._heartbeat_shutdown_event = None
         self._heartbeat_thread = None
 
@@ -543,6 +568,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
                 socket=self._socket,
                 host=self._host,
                 use_ssl=self._use_ssl,
+                metadata=grpc_metadata,
             )
             list_repositories_response = sync_list_repositories_grpc(self.client)
 
@@ -606,11 +632,11 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         return cast(str, self._container_image)
 
     @property
-    def container_context(self) -> Optional[Dict[str, Any]]:
+    def container_context(self) -> Optional[Mapping[str, Any]]:
         return self._container_context
 
     @property
-    def repository_code_pointer_dict(self) -> Dict[str, CodePointer]:
+    def repository_code_pointer_dict(self) -> Mapping[str, CodePointer]:
         return cast(Dict[str, CodePointer], self._repository_code_pointer_dict)
 
     @property
@@ -618,7 +644,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         return self._executable_path
 
     @property
-    def entry_point(self) -> Optional[List[str]]:
+    def entry_point(self) -> Optional[Sequence[str]]:
         return self._entry_point
 
     @property
@@ -648,18 +674,10 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
             self._heartbeat_shutdown_event.set()
             self._heartbeat_shutdown_event = None
 
-        if self._watch_thread_shutdown_event:
-            self._watch_thread_shutdown_event.set()
-            self._watch_thread_shutdown_event = None
-
         if self._heartbeat_thread:
             self._heartbeat_thread.join()
             self._heartbeat_thread = None
 
-        if self._watch_thread:
-            self._watch_thread.join()
-            self._watch_thread = None
-
     @property
     def is_reload_supported(self) -> bool:
         return True
@@ -671,7 +689,7 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
     def has_repository(self, name: str) -> bool:
         return name in self.get_repositories()
 
-    def get_repositories(self) -> Dict[str, ExternalRepository]:
+    def get_repositories(self) -> Mapping[str, ExternalRepository]:
         return self.external_repositories
 
     def get_external_execution_plan(
@@ -679,24 +697,30 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         external_pipeline: ExternalPipeline,
         run_config: Mapping[str, Any],
         mode: str,
-        step_keys_to_execute: Optional[List[str]],
+        step_keys_to_execute: Optional[Sequence[str]],
         known_state: Optional[KnownExecutionState],
         instance: Optional[DagsterInstance] = None,
     ) -> ExternalExecutionPlan:
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
-        run_config = check.dict_param(run_config, "run_config")
+        run_config = check.mapping_param(run_config, "run_config")
         check.str_param(mode, "mode")
-        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        check.opt_nullable_sequence_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
         check.opt_inst_param(known_state, "known_state", KnownExecutionState)
         check.opt_inst_param(instance, "instance", DagsterInstance)
 
+        asset_selection = (
+            frozenset(check.opt_set_param(external_pipeline.asset_selection, "asset_selection"))
+            if external_pipeline.asset_selection is not None
+            else None
+        )
+
         execution_plan_snapshot_or_error = sync_get_external_execution_plan_grpc(
             api_client=self.client,
             pipeline_origin=external_pipeline.get_external_origin(),
             run_config=run_config,
             mode=mode,
             pipeline_snapshot_id=external_pipeline.identifying_pipeline_snapshot_id,
-            asset_selection=external_pipeline.asset_selection,
+            asset_selection=asset_selection,
             solid_selection=external_pipeline.solid_selection,
             step_keys_to_execute=step_keys_to_execute,
             known_state=known_state,
@@ -748,13 +772,19 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         )
 
     def get_external_partition_names(
-        self, repository_handle: RepositoryHandle, partition_set_name: str
+        self, external_partition_set: ExternalPartitionSet
     ) -> "ExternalPartitionNamesData":
-        check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
-        check.str_param(partition_set_name, "partition_set_name")
+        check.inst_param(external_partition_set, "external_partition_set", ExternalPartitionSet)
+
+        # Prefer to return the names without calling out to user code if the
+        # partition set allows it
+        if external_partition_set.has_partition_name_data():
+            return ExternalPartitionNamesData(
+                partition_names=external_partition_set.get_partition_names()
+            )
 
         return sync_get_external_partition_names_grpc(
-            self.client, repository_handle, partition_set_name
+            self.client, external_partition_set.repository_handle, external_partition_set.name
         )
 
     def get_external_schedule_execution_data(
@@ -800,11 +830,11 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
         self,
         repository_handle: RepositoryHandle,
         partition_set_name: str,
-        partition_names: List[str],
+        partition_names: Sequence[str],
     ) -> "ExternalPartitionSetExecutionParamData":
         check.inst_param(repository_handle, "repository_handle", RepositoryHandle)
         check.str_param(partition_set_name, "partition_set_name")
-        check.list_param(partition_names, "partition_names", of_type=str)
+        check.sequence_param(partition_names, "partition_names", of_type=str)
 
         return sync_get_external_partition_set_execution_param_data_grpc(
             self.client, repository_handle, partition_set_name, partition_names
@@ -812,4 +842,4 @@ class GrpcServerRepositoryLocation(RepositoryLocation):
 
     def get_external_notebook_data(self, notebook_path: str) -> bytes:
         check.str_param(notebook_path, "notebook_path")
-        return sync_get_streaming_external_notebook_data_grpc(self.client, notebook_path)
+        return sync_get_streaming_external_notebook_data_grpc(self.client, notebook_path)  # type: ignore
