@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import logging.config
 import os
@@ -42,6 +44,7 @@ from dagster._core.errors import (
     DagsterInvariantViolationError,
     DagsterRunAlreadyExists,
     DagsterRunConflict,
+    DagsterUndefinedLogicalVersionError,
 )
 from dagster._core.storage.pipeline_run import (
     IN_PROGRESS_RUN_STATUSES,
@@ -61,7 +64,7 @@ from dagster._core.utils import str_format_list
 from dagster._serdes import ConfigurableClass
 from dagster._seven import get_current_datetime_in_utc
 from dagster._utils import merge_dicts, traced
-from dagster._utils.backcompat import experimental_functionality_warning
+from dagster._utils.backcompat import deprecation_warning, experimental_functionality_warning
 from dagster._utils.error import serializable_error_info_from_exc_info
 
 from .config import (
@@ -86,6 +89,7 @@ if TYPE_CHECKING:
     from dagster._core.definitions.run_request import InstigatorType
     from dagster._core.events import DagsterEvent, DagsterEventType
     from dagster._core.events.log import EventLogEntry
+    from dagster._core.execution.backfill import PartitionBackfill
     from dagster._core.execution.plan.resume_retry import ReexecutionStrategy
     from dagster._core.execution.stats import RunStepKeyStatsSnapshot
     from dagster._core.host_representation import (
@@ -97,8 +101,15 @@ if TYPE_CHECKING:
     from dagster._core.launcher import RunLauncher
     from dagster._core.run_coordinator import RunCoordinator
     from dagster._core.scheduler import Scheduler
-    from dagster._core.scheduler.instigation import InstigatorState, InstigatorTick, TickStatus
+    from dagster._core.scheduler.instigation import (
+        InstigatorState,
+        InstigatorTick,
+        TickData,
+        TickStatus,
+    )
+    from dagster._core.secrets import SecretsLoader
     from dagster._core.snap import ExecutionPlanSnapshot, PipelineSnapshot
+    from dagster._core.storage.captured_log_manager import CapturedLogManager
     from dagster._core.storage.compute_log_manager import ComputeLogManager
     from dagster._core.storage.event_log import EventLogStorage
     from dagster._core.storage.event_log.base import AssetRecord, EventLogRecord, EventRecordsFilter
@@ -111,7 +122,7 @@ if TYPE_CHECKING:
 
 def _check_run_equality(
     pipeline_run: PipelineRun, candidate_run: PipelineRun
-) -> Dict[str, Tuple[Any, Any]]:
+) -> Mapping[str, Tuple[Any, Any]]:
     field_diff = {}
     for field in pipeline_run._fields:
         expected_value = getattr(pipeline_run, field)
@@ -122,7 +133,7 @@ def _check_run_equality(
     return field_diff
 
 
-def _format_field_diff(field_diff: Dict[str, Tuple[Any, Any]]) -> str:
+def _format_field_diff(field_diff: Mapping[str, Tuple[Any, Any]]) -> str:
     return "\n".join(
         [
             (
@@ -284,12 +295,15 @@ class DagsterInstance:
         run_launcher: "RunLauncher",
         scheduler: Optional["Scheduler"] = None,
         schedule_storage: Optional["ScheduleStorage"] = None,
-        settings: Optional[Dict[str, Any]] = None,
+        settings: Optional[Mapping[str, Any]] = None,
+        secrets_loader: Optional["SecretsLoader"] = None,
         ref: Optional[InstanceRef] = None,
     ):
         from dagster._core.launcher import RunLauncher
         from dagster._core.run_coordinator import RunCoordinator
         from dagster._core.scheduler import Scheduler
+        from dagster._core.secrets import SecretsLoader
+        from dagster._core.storage.captured_log_manager import CapturedLogManager
         from dagster._core.storage.compute_log_manager import ComputeLogManager
         from dagster._core.storage.event_log import EventLogStorage
         from dagster._core.storage.root import LocalArtifactStorage
@@ -309,6 +323,10 @@ class DagsterInstance:
         self._compute_log_manager = check.inst_param(
             compute_log_manager, "compute_log_manager", ComputeLogManager
         )
+        if not isinstance(self._compute_log_manager, CapturedLogManager):
+            deprecation_warning(
+                "ComputeLogManager", "1.2.0", "Implement the CapturedLogManager interface instead."
+            )
         self._compute_log_manager.register_instance(self)
         self._scheduler = check.opt_inst_param(scheduler, "scheduler", Scheduler)
 
@@ -324,7 +342,12 @@ class DagsterInstance:
         self._run_launcher = check.inst_param(run_launcher, "run_launcher", RunLauncher)
         self._run_launcher.register_instance(self)
 
-        self._settings = check.opt_dict_param(settings, "settings")
+        self._settings = check.opt_mapping_param(settings, "settings")
+
+        self._secrets_loader = check.opt_inst_param(secrets_loader, "secrets_loader", SecretsLoader)
+
+        if self._secrets_loader:
+            self._secrets_loader.register_instance(self)
 
         self._ref = check.opt_inst_param(ref, "ref", InstanceRef)
 
@@ -362,7 +385,7 @@ class DagsterInstance:
     @public
     @staticmethod
     def ephemeral(
-        tempdir: Optional[str] = None, preload: Optional[List["DebugRunPayload"]] = None
+        tempdir: Optional[str] = None, preload: Optional[Sequence["DebugRunPayload"]] = None
     ) -> "DagsterInstance":
         from dagster._core.launcher.sync_in_memory_run_launcher import SyncInMemoryRunLauncher
         from dagster._core.run_coordinator import DefaultRunCoordinator
@@ -472,6 +495,7 @@ class DagsterInstance:
             run_coordinator=instance_ref.run_coordinator,
             run_launcher=instance_ref.run_launcher,
             settings=instance_ref.settings,
+            secrets_loader=instance_ref.secrets_loader,
             ref=instance_ref,
             **kwargs,
         )
@@ -640,8 +664,6 @@ class DagsterInstance:
 
         if "enabled" in telemetry_settings:
             return telemetry_settings["enabled"]
-        elif "experimental_dagit" in telemetry_settings:
-            return telemetry_settings["experimental_dagit"]
         else:
             return dagster_telemetry_enabled_default
 
@@ -652,7 +674,7 @@ class DagsterInstance:
         return self._run_monitoring_enabled
 
     @property
-    def run_monitoring_settings(self) -> Dict:
+    def run_monitoring_settings(self) -> Mapping:
         return self.get_settings("run_monitoring")
 
     @property
@@ -660,7 +682,7 @@ class DagsterInstance:
         return self.run_monitoring_settings.get("start_timeout_seconds", 180)
 
     @property
-    def code_server_settings(self) -> Dict:
+    def code_server_settings(self) -> Mapping:
         return self.get_settings("code_servers")
 
     @property
@@ -697,7 +719,7 @@ class DagsterInstance:
     # python logs
 
     @property
-    def managed_python_loggers(self) -> List[str]:
+    def managed_python_loggers(self) -> Sequence[str]:
         python_log_settings = self.get_settings("python_logs") or {}
         return python_log_settings.get("managed_python_loggers", [])
 
@@ -726,11 +748,17 @@ class DagsterInstance:
             self._schedule_storage.upgrade()
             self._schedule_storage.migrate(print_fn)
 
-    def optimize_for_dagit(self, statement_timeout):
+    def optimize_for_dagit(self, statement_timeout, pool_recycle):
         if self._schedule_storage:
-            self._schedule_storage.optimize_for_dagit(statement_timeout=statement_timeout)
-        self._run_storage.optimize_for_dagit(statement_timeout=statement_timeout)
-        self._event_storage.optimize_for_dagit(statement_timeout=statement_timeout)
+            self._schedule_storage.optimize_for_dagit(
+                statement_timeout=statement_timeout, pool_recycle=pool_recycle
+            )
+        self._run_storage.optimize_for_dagit(
+            statement_timeout=statement_timeout, pool_recycle=pool_recycle
+        )
+        self._event_storage.optimize_for_dagit(
+            statement_timeout=statement_timeout, pool_recycle=pool_recycle
+        )
 
     def reindex(self, print_fn=lambda _: None):
         print_fn("Checking for reindexing...")
@@ -746,6 +774,8 @@ class DagsterInstance:
         self._run_launcher.dispose()
         self._event_storage.dispose()
         self._compute_log_manager.dispose()
+        if self._secrets_loader:
+            self._secrets_loader.dispose()
 
     # run storage
     @public
@@ -790,11 +820,11 @@ class DagsterInstance:
         return self._event_storage.get_stats_for_run(run_id)
 
     @traced
-    def get_run_step_stats(self, run_id, step_keys=None) -> List["RunStepKeyStatsSnapshot"]:
+    def get_run_step_stats(self, run_id, step_keys=None) -> Sequence["RunStepKeyStatsSnapshot"]:
         return self._event_storage.get_step_stats_for_run(run_id, step_keys)
 
     @traced
-    def get_run_tags(self) -> List[Tuple[str, Set[str]]]:
+    def get_run_tags(self) -> Sequence[Tuple[str, Set[str]]]:
         return self._run_storage.get_run_tags()
 
     @traced
@@ -817,7 +847,8 @@ class DagsterInstance:
         asset_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
-    ):
+        repository_load_data=None,
+    ) -> PipelineRun:
         from dagster._core.definitions.job_definition import JobDefinition
         from dagster._core.execution.api import create_execution_plan
         from dagster._core.execution.plan.plan import ExecutionPlan
@@ -869,6 +900,7 @@ class DagsterInstance:
                 mode=mode,
                 instance_ref=self.get_ref() if self.is_persistent else None,
                 tags=tags,
+                repository_load_data=repository_load_data,
             )
 
         return self.create_run(
@@ -913,7 +945,7 @@ class DagsterInstance:
         solid_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
-    ):
+    ) -> DagsterRun:
 
         # https://github.com/dagster-io/dagster/issues/2403
         if tags and IS_AIRFLOW_INGEST_PIPELINE_STR in tags:
@@ -958,6 +990,8 @@ class DagsterInstance:
             execution_plan_snapshot_id=execution_plan_snapshot_id,
             external_pipeline_origin=external_pipeline_origin,
             pipeline_code_origin=pipeline_code_origin,
+            has_repository_load_data=execution_plan_snapshot is not None
+            and execution_plan_snapshot.repository_load_data is not None,
         )
 
     def _ensure_persisted_pipeline_snapshot(self, pipeline_snapshot, parent_pipeline_snapshot):
@@ -1029,8 +1063,11 @@ class DagsterInstance:
         return execution_plan_snapshot_id
 
     def _log_asset_materialization_planned_events(self, pipeline_run, execution_plan_snapshot):
-        from dagster._core.events import DagsterEvent
-        from dagster._core.execution.context_creation_pipeline import initialize_console_manager
+        from dagster._core.events import (
+            AssetMaterializationPlannedData,
+            DagsterEvent,
+            DagsterEventType,
+        )
 
         pipeline_name = pipeline_run.pipeline_name
 
@@ -1040,9 +1077,13 @@ class DagsterInstance:
                     asset_key = output.properties.asset_key
                     if asset_key:
                         # Logs and stores asset_materialization_planned event
-                        DagsterEvent.asset_materialization_planned(
-                            pipeline_name, asset_key, initialize_console_manager(pipeline_run, self)
+                        event = DagsterEvent(
+                            event_type_value=DagsterEventType.ASSET_MATERIALIZATION_PLANNED.value,
+                            pipeline_name=pipeline_name,
+                            message=f"{pipeline_name} intends to materialize asset {asset_key.to_string()}",
+                            event_specific_data=AssetMaterializationPlannedData(asset_key),
                         )
+                        self.report_dagster_event(event, pipeline_run.run_id, logging.DEBUG)
 
     def create_run(
         self,
@@ -1063,7 +1104,7 @@ class DagsterInstance:
         solid_selection=None,
         external_pipeline_origin=None,
         pipeline_code_origin=None,
-    ):
+    ) -> PipelineRun:
 
         pipeline_run = self._construct_run_with_snapshots(
             pipeline_name=pipeline_name,
@@ -1098,11 +1139,11 @@ class DagsterInstance:
         repo_location: "RepositoryLocation",
         external_pipeline: "ExternalPipeline",
         strategy: "ReexecutionStrategy",
-        extra_tags: Optional[Dict[str, Any]] = None,
-        run_config: Optional[Dict[str, Any]] = None,
+        extra_tags: Optional[Mapping[str, Any]] = None,
+        run_config: Optional[Mapping[str, Any]] = None,
         mode: Optional[str] = None,
         use_parent_run_tags: bool = False,
-    ) -> DagsterRun:
+    ) -> PipelineRun:
         from dagster._core.execution.plan.resume_retry import (
             ReexecutionStrategy,
             get_retry_steps_from_parent_run,
@@ -1113,8 +1154,8 @@ class DagsterInstance:
         check.inst_param(repo_location, "repo_location", RepositoryLocation)
         check.inst_param(external_pipeline, "external_pipeline", ExternalPipeline)
         check.inst_param(strategy, "strategy", ReexecutionStrategy)
-        check.opt_dict_param(extra_tags, "extra_tags", key_type=str)
-        check.opt_dict_param(run_config, "run_config", key_type=str)
+        check.opt_mapping_param(extra_tags, "extra_tags", key_type=str)
+        check.opt_mapping_param(run_config, "run_config", key_type=str)
         check.opt_str_param(mode, "mode")
 
         check.bool_param(use_parent_run_tags, "use_parent_run_tags")
@@ -1252,7 +1293,7 @@ class DagsterInstance:
             return get_run()
 
     @traced
-    def add_run(self, pipeline_run: PipelineRun):
+    def add_run(self, pipeline_run: PipelineRun) -> PipelineRun:
         return self._run_storage.add_run(pipeline_run)
 
     @traced
@@ -1264,7 +1305,7 @@ class DagsterInstance:
         return self._run_storage.handle_run_event(run_id, event)
 
     @traced
-    def add_run_tags(self, run_id: str, new_tags: Dict[str, str]):
+    def add_run_tags(self, run_id: str, new_tags: Mapping[str, str]):
         return self._run_storage.add_run_tags(run_id, new_tags)
 
     @traced
@@ -1291,7 +1332,7 @@ class DagsterInstance:
         filters: Optional[RunsFilter] = None,
         cursor: Optional[str] = None,
         limit: Optional[int] = None,
-    ) -> Dict[str, Dict[str, Union[Iterable[PipelineRun], int]]]:
+    ) -> Mapping[str, Mapping[str, Union[Iterable[PipelineRun], int]]]:
         return self._run_storage.get_run_groups(filters=filters, cursor=cursor, limit=limit)
 
     @public
@@ -1304,7 +1345,7 @@ class DagsterInstance:
         ascending: bool = False,
         cursor: Optional[str] = None,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
-    ) -> List[RunRecord]:
+    ) -> Sequence[RunRecord]:
         """Return a list of run records stored in the run storage, sorted by the given column in given order.
 
         Args:
@@ -1326,7 +1367,7 @@ class DagsterInstance:
         return self._run_storage.supports_bucket_queries
 
     @traced
-    def get_run_partition_data(self, runs_filter: RunsFilter) -> List[RunPartitionData]:
+    def get_run_partition_data(self, runs_filter: RunsFilter) -> Sequence[RunPartitionData]:
         """Get run partition data for a given partitioned job."""
         return self._run_storage.get_run_partition_data(runs_filter)
 
@@ -1400,6 +1441,11 @@ class DagsterInstance:
     ) -> Mapping[AssetKey, Optional["EventLogEntry"]]:
         return self._event_storage.get_latest_materialization_events(asset_keys)
 
+    @public
+    @traced
+    def get_latest_materialization_event(self, asset_key: AssetKey) -> Optional["EventLogEntry"]:
+        return self._event_storage.get_latest_materialization_events([asset_key]).get(asset_key)
+
     @public
     @traced
     def get_event_records(
@@ -1429,6 +1475,23 @@ class DagsterInstance:
     ) -> Iterable["AssetRecord"]:
         return self._event_storage.get_asset_records(asset_keys)
 
+    @traced
+    def get_event_tags_for_asset(
+        self, asset_key: AssetKey, filter_tags: Optional[Mapping[str, str]] = None
+    ) -> Sequence[Mapping[str, str]]:
+        """
+        Fetches asset event tags for the given asset key.
+
+        If filter_tags is provided, searches for events containing all of the filter tags. Then,
+        returns all tags for those events. This enables searching for multipartitioned asset
+        partition tags with a fixed dimension value, e.g. all of the tags for events where
+        "country" == "US".
+
+        Returns a list of dicts, where each dict is a mapping of tag key to tag value for a
+        single event.
+        """
+        return self._event_storage.get_event_tags_for_asset(asset_key, filter_tags)
+
     @traced
     def run_ids_for_asset_key(self, asset_key):
         check.inst_param(asset_key, "asset_key", AssetKey)
@@ -1895,10 +1958,11 @@ class DagsterInstance:
         )
 
         stored_state = self.get_instigator_state(instigator_origin_id, selector_id)
+        computed_state: InstigatorState
         if external_sensor:
             computed_state = external_sensor.get_current_instigator_state(stored_state)
         else:
-            computed_state = stored_state
+            computed_state = check.not_none(stored_state)
 
         if not computed_state.is_running:
             return computed_state
@@ -1920,6 +1984,8 @@ class DagsterInstance:
     def all_instigator_state(
         self, repository_origin_id=None, repository_selector_id=None, instigator_type=None
     ):
+        if not self._schedule_storage:
+            check.failed("Schedule storage not available")
         return self._schedule_storage.all_instigator_state(
             repository_origin_id, repository_selector_id, instigator_type
         )
@@ -1971,11 +2037,11 @@ class DagsterInstance:
             origin_id, selector_id, before=before, after=after, limit=limit, statuses=statuses
         )
 
-    def create_tick(self, tick_data):
-        return self._schedule_storage.create_tick(tick_data)
+    def create_tick(self, tick_data: "TickData") -> "InstigatorTick":
+        return check.not_none(self._schedule_storage).create_tick(tick_data)
 
-    def update_tick(self, tick):
-        return self._schedule_storage.update_tick(tick)
+    def update_tick(self, tick: "InstigatorTick"):
+        return check.not_none(self._schedule_storage).update_tick(tick)
 
     def purge_ticks(self, origin_id, selector_id, before, tick_statuses=None):
         self._schedule_storage.purge_ticks(origin_id, selector_id, before, tick_statuses)
@@ -2002,7 +2068,7 @@ class DagsterInstance:
         """Called on a regular interval by the daemon"""
         self._run_storage.add_daemon_heartbeat(daemon_heartbeat)
 
-    def get_daemon_heartbeats(self) -> Dict[str, "DaemonHeartbeat"]:
+    def get_daemon_heartbeats(self) -> Mapping[str, "DaemonHeartbeat"]:
         """Latest heartbeats of all daemon types"""
         return self._run_storage.get_daemon_heartbeats()
 
@@ -2038,15 +2104,15 @@ class DagsterInstance:
         return daemons
 
     def get_daemon_statuses(
-        self, daemon_types: Optional[List[str]] = None
-    ) -> Dict[str, "DaemonStatus"]:
+        self, daemon_types: Optional[Sequence[str]] = None
+    ) -> Mapping[str, "DaemonStatus"]:
         """
         Get the current status of the daemons. If daemon_types aren't provided, defaults to all
         required types. Returns a dict of daemon type to status.
         """
         from dagster._daemon.controller import get_daemon_statuses
 
-        check.opt_list_param(daemon_types, "daemon_types", of_type=str)
+        check.opt_sequence_param(daemon_types, "daemon_types", of_type=str)
         return get_daemon_statuses(
             self, daemon_types=daemon_types or self.get_required_daemon_types(), ignore_errors=True
         )
@@ -2059,17 +2125,17 @@ class DagsterInstance:
         return False
 
     # backfill
-    def get_backfills(self, status=None, cursor=None, limit=None):
+    def get_backfills(self, status=None, cursor=None, limit=None) -> Sequence["PartitionBackfill"]:
         return self._run_storage.get_backfills(status=status, cursor=cursor, limit=limit)
 
-    def get_backfill(self, backfill_id):
+    def get_backfill(self, backfill_id: str) -> Optional["PartitionBackfill"]:
         return self._run_storage.get_backfill(backfill_id)
 
-    def add_backfill(self, partition_backfill):
+    def add_backfill(self, partition_backfill: "PartitionBackfill") -> None:
         self._run_storage.add_backfill(partition_backfill)
 
-    def update_backfill(self, partition_backfill):
-        return self._run_storage.update_backfill(partition_backfill)
+    def update_backfill(self, partition_backfill: "PartitionBackfill") -> None:
+        self._run_storage.update_backfill(partition_backfill)
 
     @property
     def should_start_background_run_thread(self) -> bool:
@@ -2080,7 +2146,7 @@ class DagsterInstance:
 
     def get_tick_retention_settings(
         self, instigator_type: "InstigatorType"
-    ) -> Dict["TickStatus", int]:
+    ) -> Mapping["TickStatus", int]:
         from dagster._core.definitions.run_request import InstigatorType
 
         retention_settings = self.get_settings("retention")
@@ -2092,13 +2158,46 @@ class DagsterInstance:
         default_tick_settings = get_default_tick_retention_settings(instigator_type)
         return get_tick_retention_settings(tick_settings, default_tick_settings)
 
+    def inject_env_vars(self, location_name: Optional[str]):
+        if not self._secrets_loader:
+            return
 
-def is_dagit_telemetry_enabled(instance):
-    telemetry_settings = instance.get_settings("telemetry")
-    if not telemetry_settings:
-        return False
+        new_env = self._secrets_loader.get_secrets_for_environment(location_name)
+        for k, v in new_env.items():
+            os.environ[k] = v
 
-    if "experimental_dagit" in telemetry_settings:
-        return telemetry_settings["experimental_dagit"]
-    else:
-        return False
+    def get_latest_logical_version_record(
+        self,
+        key: AssetKey,
+        is_source: Optional[bool] = None,
+    ) -> Optional[EventLogRecord]:
+        from dagster._core.event_api import EventRecordsFilter
+        from dagster._core.events import DagsterEventType
+
+        # When we cant don't know whether the requested key corresponds to a source or regular
+        # asset, we need to retrieve both the latest observation and materialization for all assets.
+        # If there is a materialization, it's a regular asset and we can ignore the observation.
+
+        observation: Optional[EventLogRecord] = None
+        if is_source or is_source is None:
+            observations = self.get_event_records(
+                EventRecordsFilter(
+                    event_type=DagsterEventType.ASSET_OBSERVATION,
+                    asset_key=key,
+                ),
+                limit=1,
+            )
+            observation = next(iter(observations), None)
+
+        materialization: Optional[EventLogRecord] = None
+        if not is_source:
+            materializations = self.get_event_records(
+                EventRecordsFilter(
+                    event_type=DagsterEventType.ASSET_MATERIALIZATION,
+                    asset_key=key,
+                ),
+                limit=1,
+            )
+            materialization = next(iter(materializations), None)
+
+        return materialization or observation
