@@ -1,6 +1,6 @@
 import datetime
 import logging
-from typing import TYPE_CHECKING, Any, Dict, List, NamedTuple, Optional, Union
+from typing import TYPE_CHECKING, Any, Mapping, NamedTuple, Optional, Sequence, Union
 
 import dagster._check as check
 from dagster._core.utils import coerce_valid_log_level, make_new_run_id
@@ -87,7 +87,7 @@ class DagsterLoggingMetadata(
         [
             ("run_id", Optional[str]),
             ("pipeline_name", Optional[str]),
-            ("pipeline_tags", Dict[str, str]),
+            ("pipeline_tags", Mapping[str, str]),
             ("step_key", Optional[str]),
             ("solid_name", Optional[str]),
             ("resource_name", Optional[str]),
@@ -103,7 +103,7 @@ class DagsterLoggingMetadata(
         cls,
         run_id: Optional[str] = None,
         pipeline_name: Optional[str] = None,
-        pipeline_tags: Optional[Dict[str, str]] = None,
+        pipeline_tags: Optional[Mapping[str, str]] = None,
         step_key: Optional[str] = None,
         solid_name: Optional[str] = None,
         resource_name: Optional[str] = None,
@@ -126,7 +126,7 @@ class DagsterLoggingMetadata(
             return self.pipeline_name or "system"
         return f"resource:{self.resource_name}"
 
-    def to_tags(self) -> Dict[str, str]:
+    def to_tags(self) -> Mapping[str, str]:
         # converts all values into strings
         return {k: str(v) for k, v in self._asdict().items()}
 
@@ -159,7 +159,7 @@ def construct_log_string(
 
 def get_dagster_meta_dict(
     logging_metadata: DagsterLoggingMetadata, dagster_message_props: DagsterMessageProps
-) -> Dict[str, Any]:
+) -> Mapping[str, Any]:
     # combine all dagster meta information into a single dictionary
     meta_dict = {
         **logging_metadata._asdict(),
@@ -186,8 +186,8 @@ class DagsterLogHandler(logging.Handler):
     def __init__(
         self,
         logging_metadata: DagsterLoggingMetadata,
-        loggers: List[logging.Logger],
-        handlers: List[logging.Handler],
+        loggers: Sequence[logging.Logger],
+        handlers: Sequence[logging.Handler],
     ):
         self._logging_metadata = logging_metadata
         self._loggers = loggers
@@ -206,7 +206,7 @@ class DagsterLogHandler(logging.Handler):
             handlers=self._handlers,
         )
 
-    def _extract_extra(self, record: logging.LogRecord) -> Dict[str, Any]:
+    def _extract_extra(self, record: logging.LogRecord) -> Mapping[str, Any]:
         """In the logging.Logger log() implementation, the elements of the `extra` dictionary
         argument are smashed into the __dict__ of the underlying logging.LogRecord.
         This function figures out what the original `extra` values of the log call were by
@@ -302,10 +302,10 @@ class DagsterLogManager(logging.Logger):
         self,
         dagster_handler: DagsterLogHandler,
         level: int = logging.NOTSET,
-        managed_loggers: Optional[List[logging.Logger]] = None,
+        managed_loggers: Optional[Sequence[logging.Logger]] = None,
     ):
         super().__init__(name="dagster", level=coerce_valid_log_level(level))
-        self._managed_loggers = check.opt_list_param(
+        self._managed_loggers = check.opt_sequence_param(
             managed_loggers, "managed_loggers", of_type=logging.Logger
         )
         self._dagster_handler = dagster_handler
@@ -314,14 +314,14 @@ class DagsterLogManager(logging.Logger):
     @classmethod
     def create(
         cls,
-        loggers: List[logging.Logger],
-        handlers: Optional[List[logging.Handler]] = None,
+        loggers: Sequence[logging.Logger],
+        handlers: Optional[Sequence[logging.Handler]] = None,
         instance: Optional["DagsterInstance"] = None,
         pipeline_run: Optional["PipelineRun"] = None,
     ) -> "DagsterLogManager":
         """Create a DagsterLogManager with a set of subservient loggers."""
 
-        handlers = check.opt_list_param(handlers, "handlers", of_type=logging.Handler)
+        handlers = check.opt_sequence_param(handlers, "handlers", of_type=logging.Handler)
 
         managed_loggers = [get_dagster_logger()]
         python_log_level = logging.NOTSET
