@@ -17,7 +17,8 @@ from watchdog.observers import Observer
 import dagster._check as check
 import dagster._seven as seven
 from dagster._config import StringSource
-from dagster._core.events import DagsterEventType
+from dagster._core.errors import DagsterInvariantViolationError
+from dagster._core.events import ASSET_EVENTS
 from dagster._core.events.log import EventLogEntry
 from dagster._core.storage.event_log.base import EventLogCursor, EventLogRecord, EventRecordsFilter
 from dagster._core.storage.pipeline_run import PipelineRunStatus, RunsFilter
@@ -131,6 +132,11 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
             if os.path.splitext(os.path.basename(filename))[0] != INDEX_SHARD_NAME
         ]
 
+    def has_table(self, table_name: str) -> bool:
+        conn_string = self.conn_string_for_shard(INDEX_SHARD_NAME)
+        engine = create_engine(conn_string, poolclass=NullPool)
+        return bool(engine.dialect.has_table(engine.connect(), table_name))
+
     def path_for_shard(self, run_id):
         return os.path.join(self._base_dir, "{run_id}.db".format(run_id=run_id))
 
@@ -227,22 +233,25 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
 
         if event.is_dagster_event and event.dagster_event.asset_key:
             check.invariant(
-                event.dagster_event_type == DagsterEventType.ASSET_MATERIALIZATION
-                or event.dagster_event_type == DagsterEventType.ASSET_OBSERVATION
-                or event.dagster_event_type == DagsterEventType.ASSET_MATERIALIZATION_PLANNED,
+                event.dagster_event_type in ASSET_EVENTS,
                 "Can only store asset materializations, materialization_planned, and observations in index database",
             )
 
+            event_id = None
+
             # mirror the event in the cross-run index database
             with self.index_connection() as conn:
-                conn.execute(insert_event_statement)
+                result = conn.execute(insert_event_statement)
+                event_id = result.inserted_primary_key[0]
 
-            if (
-                event.dagster_event.is_step_materialization
-                or event.dagster_event.is_asset_observation
-                or event.dagster_event.is_asset_materialization_planned
-            ):
-                self.store_asset_event(event)
+            self.store_asset_event(event)
+
+            if event_id is None:
+                raise DagsterInvariantViolationError(
+                    "Cannot store asset event tags for null event id."
+                )
+
+            self.store_asset_event_tags(event, event_id)
 
     def get_event_records(
         self,
@@ -259,13 +268,10 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         check.opt_int_param(limit, "limit")
         check.bool_param(ascending, "ascending")
 
-        is_asset_query = event_records_filter and (
-            event_records_filter.event_type == DagsterEventType.ASSET_MATERIALIZATION
-            or event_records_filter.event_type == DagsterEventType.ASSET_OBSERVATION
-        )
+        is_asset_query = event_records_filter and event_records_filter.event_type in ASSET_EVENTS
         if is_asset_query:
-            # asset materializations and observations get mirrored into the index shard, so no
-            # custom run shard-aware cursor logic needed
+            # asset materializations, observations and materialization planned events
+            # get mirrored into the index shard, so no custom run shard-aware cursor logic needed
             return super(SqliteEventLogStorage, self).get_event_records(
                 event_records_filter=event_records_filter, limit=limit, ascending=ascending
             )
@@ -409,6 +415,10 @@ class SqliteEventLogStorage(SqlEventLogStorage, ConfigurableClass):
         with self.index_connection() as conn:
             return check_alembic_revision(alembic_config, conn)
 
+    @property
+    def is_run_sharded(self):
+        return True
+
 
 class SqliteEventLogStorageWatchdog(PatternMatchingEventHandler):
     def __init__(self, event_log_storage, run_id, callback, cursor, **kwargs):
