@@ -11,6 +11,7 @@ from typing import (
     Mapping,
     NamedTuple,
     Optional,
+    Sequence,
     Type,
     Union,
 )
@@ -43,6 +44,7 @@ from .tags import (
 )
 
 if TYPE_CHECKING:
+    from dagster._core.definitions.repository_definition import RepositoryLoadData
     from dagster._core.host_representation.origin import ExternalPipelineOrigin
 
 
@@ -222,6 +224,7 @@ def pipeline_run_from_storage(
     reexecution_config=None,  # pylint: disable=unused-argument
     external_pipeline_origin=None,
     pipeline_code_origin=None,
+    has_repository_load_data=None,
     **kwargs,
 ):
 
@@ -235,6 +238,7 @@ def pipeline_run_from_storage(
     # * renamed solid_subset -> solid_selection, added solids_to_execute
     # * renamed environment_dict -> run_config
     # * added asset_selection
+    # * added has_repository_load_data
 
     # back compat for environment dict => run_config
     if environment_dict:
@@ -305,6 +309,7 @@ def pipeline_run_from_storage(
         execution_plan_snapshot_id=execution_plan_snapshot_id,
         external_pipeline_origin=external_pipeline_origin,
         pipeline_code_origin=pipeline_code_origin,
+        has_repository_load_data=has_repository_load_data,
     )
 
 
@@ -317,17 +322,18 @@ class PipelineRun(
             ("run_config", Mapping[str, object]),
             ("mode", Optional[str]),
             ("asset_selection", Optional[FrozenSet[AssetKey]]),
-            ("solid_selection", Optional[List[str]]),
+            ("solid_selection", Optional[Sequence[str]]),
             ("solids_to_execute", Optional[FrozenSet[str]]),
-            ("step_keys_to_execute", Optional[List[str]]),
+            ("step_keys_to_execute", Optional[Sequence[str]]),
             ("status", PipelineRunStatus),
-            ("tags", Dict[str, str]),
+            ("tags", Mapping[str, str]),
             ("root_run_id", Optional[str]),
             ("parent_run_id", Optional[str]),
             ("pipeline_snapshot_id", Optional[str]),
             ("execution_plan_snapshot_id", Optional[str]),
             ("external_pipeline_origin", Optional["ExternalPipelineOrigin"]),
             ("pipeline_code_origin", Optional[PipelinePythonOrigin]),
+            ("has_repository_load_data", bool),
         ],
     )
 ):
@@ -342,17 +348,18 @@ class PipelineRun(
         run_config: Optional[Mapping[str, object]] = None,
         mode: Optional[str] = None,
         asset_selection: Optional[FrozenSet[AssetKey]] = None,
-        solid_selection: Optional[List[str]] = None,
+        solid_selection: Optional[Sequence[str]] = None,
         solids_to_execute: Optional[FrozenSet[str]] = None,
-        step_keys_to_execute: Optional[List[str]] = None,
+        step_keys_to_execute: Optional[Sequence[str]] = None,
         status: Optional[PipelineRunStatus] = None,
-        tags: Optional[Dict[str, str]] = None,
+        tags: Optional[Mapping[str, str]] = None,
         root_run_id: Optional[str] = None,
         parent_run_id: Optional[str] = None,
         pipeline_snapshot_id: Optional[str] = None,
         execution_plan_snapshot_id: Optional[str] = None,
         external_pipeline_origin: Optional["ExternalPipelineOrigin"] = None,
         pipeline_code_origin: Optional[PipelinePythonOrigin] = None,
+        has_repository_load_data: Optional[bool] = None,
     ):
         check.invariant(
             (root_run_id is not None and parent_run_id is not None)
@@ -368,10 +375,10 @@ class PipelineRun(
         )
         # a list of solid queries provided by the user
         # possible to be None when only solids_to_execute is set by the user directly
-        solid_selection = check.opt_nullable_list_param(
+        solid_selection = check.opt_nullable_sequence_param(
             solid_selection, "solid_selection", of_type=str
         )
-        check.opt_nullable_list_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
+        check.opt_nullable_sequence_param(step_keys_to_execute, "step_keys_to_execute", of_type=str)
 
         asset_selection = check.opt_nullable_set_param(
             asset_selection, "asset_selection", of_type=AssetKey
@@ -405,7 +412,7 @@ class PipelineRun(
             status=check.opt_inst_param(
                 status, "status", PipelineRunStatus, PipelineRunStatus.NOT_STARTED
             ),
-            tags=check.opt_dict_param(tags, "tags", key_type=str, value_type=str),
+            tags=check.opt_mapping_param(tags, "tags", key_type=str, value_type=str),
             root_run_id=check.opt_str_param(root_run_id, "root_run_id"),
             parent_run_id=check.opt_str_param(parent_run_id, "parent_run_id"),
             pipeline_snapshot_id=check.opt_str_param(pipeline_snapshot_id, "pipeline_snapshot_id"),
@@ -418,6 +425,9 @@ class PipelineRun(
             pipeline_code_origin=check.opt_inst_param(
                 pipeline_code_origin, "pipeline_code_origin", PipelinePythonOrigin
             ),
+            has_repository_load_data=check.opt_bool_param(
+                has_repository_load_data, "has_repository_load_data", default=False
+            ),
         )
 
     def with_status(self, status):
@@ -434,6 +444,12 @@ class PipelineRun(
 
         return self._replace(status=status)
 
+    def with_job_origin(self, origin):
+        from dagster._core.host_representation.origin import ExternalPipelineOrigin
+
+        check.inst_param(origin, "origin", ExternalPipelineOrigin)
+        return self._replace(external_pipeline_origin=origin)
+
     def with_mode(self, mode):
         return self._replace(mode=mode)
 
@@ -509,7 +525,18 @@ class PipelineRun(
 
     @staticmethod
     def tags_for_partition_set(partition_set, partition):
-        return {PARTITION_NAME_TAG: partition.name, PARTITION_SET_TAG: partition_set.name}
+        from dagster._core.definitions.multi_dimensional_partitions import (
+            MultiPartitionKey,
+            get_tags_from_multi_partition_key,
+        )
+
+        tags = {PARTITION_SET_TAG: partition_set.name}
+        if isinstance(partition.name, MultiPartitionKey):
+            tags.update(get_tags_from_multi_partition_key(partition.name))
+        else:
+            tags[PARTITION_NAME_TAG] = partition.name
+
+        return tags
 
 
 @whitelist_for_serdes(serializer=DagsterRunSerializer)
@@ -566,10 +593,10 @@ class RunsFilter(
     NamedTuple(
         "_RunsFilter",
         [
-            ("run_ids", List[str]),
+            ("run_ids", Sequence[str]),
             ("job_name", Optional[str]),
-            ("statuses", List[PipelineRunStatus]),
-            ("tags", Dict[str, Union[str, List[str]]]),
+            ("statuses", Sequence[DagsterRunStatus]),
+            ("tags", Mapping[str, Union[str, Sequence[str]]]),
             ("snapshot_id", Optional[str]),
             ("updated_after", Optional[datetime]),
             ("mode", Optional[str]),
@@ -577,12 +604,35 @@ class RunsFilter(
         ],
     )
 ):
+    """Defines a filter across job runs, for use when querying storage directly.
+
+    Each field of the RunsFilter represents a logical AND with each other. For
+    example, if you specify job_name and tags, then you will receive only runs
+    with the specified job_name AND the specified tags. If left blank, then
+    all values will be permitted for that field.
+
+    Args:
+        run_ids (Optional[List[str]]): A list of job run_id values.
+        job_name (Optional[str]):
+            Name of the job to query for. If blank, all job_names will be accepted.
+        statuses (Optional[List[DagsterRunStatus]]):
+            A list of run statuses to filter by. If blank, all run statuses will be allowed.
+        tags (Optional[Dict[str, Union[str, List[str]]]]):
+            A dictionary of run tags to query by. All tags specified here must be present for a given run to pass the filter.
+        snapshot_id (Optional[str]): The ID of the job snapshot to query for. Intended for internal use.
+        updated_after (Optional[DateTime]): Filter by runs that were last updated before this datetime.
+        created_before (Optional[DateTime]): Filter by runs that were created before this datetime.
+        mode (Optional[str]): (deprecated)
+        pipeline_name (Optional[str]): (deprecated)
+
+    """
+
     def __new__(
         cls,
-        run_ids: Optional[List[str]] = None,
+        run_ids: Optional[Sequence[str]] = None,
         job_name: Optional[str] = None,
-        statuses: Optional[List[PipelineRunStatus]] = None,
-        tags: Optional[Dict[str, Union[str, List[str]]]] = None,
+        statuses: Optional[Sequence[DagsterRunStatus]] = None,
+        tags: Optional[Mapping[str, Union[str, Sequence[str]]]] = None,
         snapshot_id: Optional[str] = None,
         updated_after: Optional[datetime] = None,
         mode: Optional[str] = None,
@@ -595,10 +645,10 @@ class RunsFilter(
 
         return super(RunsFilter, cls).__new__(
             cls,
-            run_ids=check.opt_list_param(run_ids, "run_ids", of_type=str),
+            run_ids=check.opt_sequence_param(run_ids, "run_ids", of_type=str),
             job_name=check.opt_str_param(job_name, "job_name"),
-            statuses=check.opt_list_param(statuses, "statuses", of_type=PipelineRunStatus),
-            tags=check.opt_dict_param(tags, "tags", key_type=str),
+            statuses=check.opt_sequence_param(statuses, "statuses", of_type=PipelineRunStatus),
+            tags=check.opt_mapping_param(tags, "tags", key_type=str),
             snapshot_id=check.opt_str_param(snapshot_id, "snapshot_id"),
             updated_after=check.opt_inst_param(updated_after, "updated_after", datetime),
             mode=check.opt_str_param(mode, "mode"),
@@ -731,17 +781,17 @@ class RunPartitionData(
 
 @whitelist_for_serdes
 class ExecutionSelector(
-    NamedTuple("_ExecutionSelector", [("name", str), ("solid_subset", Optional[List[str]])])
+    NamedTuple("_ExecutionSelector", [("name", str), ("solid_subset", Optional[Sequence[str]])])
 ):
     """
     Kept here to maintain loading of PipelineRuns from when it was still alive.
     """
 
-    def __new__(cls, name: str, solid_subset: Optional[List[str]] = None):
+    def __new__(cls, name: str, solid_subset: Optional[Sequence[str]] = None):
         return super(ExecutionSelector, cls).__new__(
             cls,
             name=check.str_param(name, "name"),
             solid_subset=None
             if solid_subset is None
-            else check.list_param(solid_subset, "solid_subset", of_type=str),
+            else check.sequence_param(solid_subset, "solid_subset", of_type=str),
         )
