@@ -20,13 +20,19 @@ from dagster._core.errors import (
 from dagster._core.events import EVENT_TYPE_TO_PIPELINE_RUN_STATUS, DagsterEvent, DagsterEventType
 from dagster._core.execution.backfill import BulkActionStatus, PartitionBackfill
 from dagster._core.execution.bulk_actions import BulkActionType
+from dagster._core.host_representation.origin import ExternalPipelineOrigin
 from dagster._core.snap import (
     ExecutionPlanSnapshot,
     PipelineSnapshot,
     create_execution_plan_snapshot_id,
     create_pipeline_snapshot_id,
 )
-from dagster._core.storage.tags import PARTITION_NAME_TAG, PARTITION_SET_TAG, ROOT_RUN_ID_TAG
+from dagster._core.storage.tags import (
+    PARTITION_NAME_TAG,
+    PARTITION_SET_TAG,
+    REPOSITORY_LABEL_TAG,
+    ROOT_RUN_ID_TAG,
+)
 from dagster._daemon.types import DaemonHeartbeat
 from dagster._serdes import (
     deserialize_as,
@@ -189,7 +195,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         # overriden with an old value.
         return run.with_status(status)
 
-    def _rows_to_runs(self, rows: Iterable[Tuple]) -> List[PipelineRun]:
+    def _rows_to_runs(self, rows: Iterable[Tuple]) -> Sequence[PipelineRun]:
         return list(map(self._row_to_run, rows))
 
     def _add_cursor_limit_to_query(
@@ -241,36 +247,30 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         if filters.created_before:
             query = query.where(RunsTable.c.create_timestamp < filters.created_before)
 
-        return query
+        if filters.tags:
+            intersections = [
+                db.select([RunTagsTable.c.run_id]).where(
+                    db.and_(
+                        RunTagsTable.c.key == key,
+                        (
+                            RunTagsTable.c.value == value
+                            if isinstance(value, str)
+                            else RunTagsTable.c.value.in_(value)
+                        ),
+                    )
+                )
+                for key, value in filters.tags.items()
+            ]
 
-    def _apply_tags_table_joins(
-        self,
-        table: db.Table,
-        tags: Mapping[str, Union[str, Sequence[str]]],
-    ):
-        multi_join = len(tags) > 1
-        for key, value in tags.items():
-            tags_table = RunTagsTable.alias() if multi_join else RunTagsTable
-            table = table.join(
-                tags_table,
-                db.and_(
-                    RunsTable.c.run_id == tags_table.c.run_id,
-                    tags_table.c.key == key,
-                    (
-                        tags_table.c.value == value
-                        if isinstance(value, str)
-                        else tags_table.c.value.in_(value)
-                    ),
-                ),
-            )
-        return table
+            query = query.where(RunsTable.c.run_id.in_(db.intersect(*intersections)))
+        return query
 
     def _runs_query(
         self,
         filters: Optional[RunsFilter] = None,
         cursor: Optional[str] = None,
         limit: Optional[int] = None,
-        columns: Optional[List[str]] = None,
+        columns: Optional[Sequence[str]] = None,
         order_by: Optional[str] = None,
         ascending: bool = False,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
@@ -278,7 +278,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         filters = check.opt_inst_param(filters, "filters", RunsFilter, default=RunsFilter())
         check.opt_str_param(cursor, "cursor")
         check.opt_int_param(limit, "limit")
-        check.opt_list_param(columns, "columns")
+        check.opt_sequence_param(columns, "columns")
         check.opt_str_param(order_by, "order_by")
         check.opt_bool_param(ascending, "ascending")
 
@@ -290,13 +290,8 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
                 check.failed("cannot specify bucket_by and limit/cursor at the same time")
             return self._bucketed_runs_query(bucket_by, filters, columns, order_by, ascending)
 
-        if filters.tags:
-            table = self._apply_tags_table_joins(RunsTable, filters.tags)
-        else:
-            table = RunsTable
-
         base_query = db.select([getattr(RunsTable.c, column) for column in columns]).select_from(
-            table
+            RunsTable
         )
         base_query = self._add_filters_to_query(base_query, filters)
         return self._add_cursor_limit_to_query(base_query, cursor, limit, order_by, ascending)
@@ -321,7 +316,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         self,
         bucket_by: Union[JobBucket, TagBucket],
         filters: RunsFilter,
-        columns: List[str],
+        columns: Sequence[str],
         order_by: Optional[str] = None,
         ascending: bool = False,
     ):
@@ -329,37 +324,37 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         query_columns = [getattr(RunsTable.c, column) for column in columns] + [bucket_rank]
 
         if isinstance(bucket_by, JobBucket):
-            # bucketing by job
-            if filters.tags:
-                table = self._apply_tags_table_joins(RunsTable, filters.tags)
-            else:
-                table = RunsTable
-
-            base_query = db.select(query_columns).select_from(table)
+            base_query = db.select(query_columns).select_from(RunsTable)
             base_query = base_query.where(RunsTable.c.pipeline_name.in_(bucket_by.job_names))
             base_query = self._add_filters_to_query(base_query, filters)
 
         elif not filters.tags:
             # bucketing by tag, no tag filters
             base_query = db.select(query_columns).select_from(
-                self._apply_tags_table_joins(
-                    RunsTable,
-                    {bucket_by.tag_key: bucket_by.tag_values},
+                RunsTable.join(
+                    RunTagsTable,
+                    db.and_(
+                        RunsTable.c.run_id == RunTagsTable.c.run_id,
+                        RunTagsTable.c.key == bucket_by.tag_key,
+                        RunTagsTable.c.value.in_(bucket_by.tag_values),
+                    ),
                 )
             )
             base_query = self._add_filters_to_query(base_query, filters)
-
         else:
             # there are tag filters as well as tag buckets, so we have to apply the tag filters in
             # a separate join
-            filtered_query = db.select([RunsTable.c.run_id]).select_from(
-                self._apply_tags_table_joins(RunsTable, filters.tags)
-            )
+            filtered_query = db.select([RunsTable.c.run_id])
             filtered_query = self._add_filters_to_query(filtered_query, filters)
             filtered_query = filtered_query.alias("filtered_query")
             base_query = db.select(query_columns).select_from(
-                self._apply_tags_table_joins(
-                    RunsTable, {bucket_by.tag_key: bucket_by.tag_values}
+                RunsTable.join(
+                    RunTagsTable,
+                    db.and_(
+                        RunsTable.c.run_id == RunTagsTable.c.run_id,
+                        RunTagsTable.c.key == bucket_by.tag_key,
+                        RunTagsTable.c.value.in_(bucket_by.tag_values),
+                    ),
                 ).join(filtered_query, RunsTable.c.run_id == filtered_query.c.run_id)
             )
 
@@ -380,7 +375,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         cursor: Optional[str] = None,
         limit: Optional[int] = None,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
-    ) -> List[PipelineRun]:
+    ) -> Sequence[PipelineRun]:
         query = self._runs_query(filters, cursor, limit, bucket_by=bucket_by)
         rows = self.fetchall(query)
         return self._rows_to_runs(rows)
@@ -422,7 +417,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         ascending: bool = False,
         cursor: Optional[str] = None,
         bucket_by: Optional[Union[JobBucket, TagBucket]] = None,
-    ) -> List[RunRecord]:
+    ) -> Sequence[RunRecord]:
         filters = check.opt_inst_param(filters, "filters", RunsFilter, default=RunsFilter())
         check.opt_int_param(limit, "limit")
 
@@ -456,7 +451,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             for row in rows
         ]
 
-    def get_run_tags(self) -> List[Tuple[str, Set[str]]]:
+    def get_run_tags(self) -> Sequence[Tuple[str, Set[str]]]:
         result = defaultdict(set)
         query = db.select([RunTagsTable.c.key, RunTagsTable.c.value]).distinct(
             RunTagsTable.c.key, RunTagsTable.c.value
@@ -466,9 +461,9 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             result[r[0]].add(r[1])
         return sorted(list([(k, v) for k, v in result.items()]), key=lambda x: x[0])
 
-    def add_run_tags(self, run_id: str, new_tags: Dict[str, str]):
+    def add_run_tags(self, run_id: str, new_tags: Mapping[str, str]):
         check.str_param(run_id, "run_id")
-        check.dict_param(new_tags, "new_tags", key_type=str, value_type=str)
+        check.mapping_param(new_tags, "new_tags", key_type=str, value_type=str)
 
         run = self.get_run_by_id(run_id)
         if not run:
@@ -560,14 +555,14 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             res = conn.execute(run_group_query)
             run_group = self._rows_to_runs(res)
 
-        return (root_run_id, [root_run] + run_group)
+        return (root_run_id, [root_run, *run_group])
 
     def get_run_groups(
         self,
         filters: Optional[RunsFilter] = None,
         cursor: Optional[str] = None,
         limit: Optional[int] = None,
-    ) -> Dict[str, Dict[str, Union[Iterable[PipelineRun], int]]]:
+    ) -> Mapping[str, Mapping[str, Union[Iterable[PipelineRun], int]]]:
         # The runs that would be returned by calling RunStorage.get_runs with the same arguments
         runs = self._runs_query(
             filters=filters, cursor=cursor, limit=limit, columns=["run_body", "status", "run_id"]
@@ -807,7 +802,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
         return defensively_unpack_pipeline_snapshot_query(logging, row) if row else None
 
-    def get_run_partition_data(self, runs_filter: RunsFilter) -> List[RunPartitionData]:
+    def get_run_partition_data(self, runs_filter: RunsFilter) -> Sequence[RunPartitionData]:
         if self.has_built_index(RUN_PARTITIONS) and self.has_run_stats_index_cols():
             query = self._runs_query(
                 filters=runs_filter,
@@ -852,7 +847,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
 
     def _get_partition_runs(
         self, partition_set_name: str, partition_name: str
-    ) -> List[PipelineRun]:
+    ) -> Sequence[PipelineRun]:
         # utility method to help test reads off of the partition column
         if not self.has_built_index(RUN_PARTITIONS):
             # query by tags
@@ -966,7 +961,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
                     )
                 )
 
-    def get_daemon_heartbeats(self) -> Dict[str, DaemonHeartbeat]:
+    def get_daemon_heartbeats(self) -> Mapping[str, DaemonHeartbeat]:
 
         with self.connect() as conn:
             rows = conn.execute(db.select(DaemonHeartbeatsTable.columns))
@@ -983,6 +978,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             conn.execute(RunTagsTable.delete())  # pylint: disable=no-value-for-parameter
             conn.execute(SnapshotsTable.delete())  # pylint: disable=no-value-for-parameter
             conn.execute(DaemonHeartbeatsTable.delete())  # pylint: disable=no-value-for-parameter
+            conn.execute(BulkActionsTable.delete())  # pylint: disable=no-value-for-parameter
 
     def wipe_daemon_heartbeats(self):
         with self.connect() as conn:
@@ -994,7 +990,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
         status: Optional[BulkActionStatus] = None,
         cursor: Optional[str] = None,
         limit: Optional[int] = None,
-    ) -> List[PartitionBackfill]:
+    ) -> Sequence[PartitionBackfill]:
         check.opt_inst_param(status, "status", BulkActionStatus)
         query = db.select([BulkActionsTable.c.body])
         if status:
@@ -1054,7 +1050,7 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
     def supports_kvs(self):
         return True
 
-    def kvs_get(self, keys: Set[str]) -> Dict[str, str]:
+    def kvs_get(self, keys: Set[str]) -> Mapping[str, str]:
         check.set_param(keys, "keys", of_type=str)
 
         with self.connect() as conn:
@@ -1063,8 +1059,8 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
             )
             return {row.key: row.value for row in rows}
 
-    def kvs_set(self, pairs: Dict[str, str]) -> None:
-        check.dict_param(pairs, "pairs", key_type=str, value_type=str)
+    def kvs_set(self, pairs: Mapping[str, str]) -> None:
+        check.mapping_param(pairs, "pairs", key_type=str, value_type=str)
         db_values = [{"key": k, "value": v} for k, v in pairs.items()]
 
         with self.connect() as conn:
@@ -1077,6 +1073,24 @@ class SqlRunStorage(RunStorage):  # pylint: disable=no-init
                     .values(value=db.sql.case(pairs, value=KeyValueStoreTable.c.key))
                 )
 
+    # Migrating run history
+    def replace_job_origin(self, run: PipelineRun, job_origin: ExternalPipelineOrigin):
+        new_label = job_origin.external_repository_origin.get_label()
+        with self.connect() as conn:
+            conn.execute(
+                RunsTable.update()  # pylint: disable=no-value-for-parameter
+                .where(RunsTable.c.run_id == run.run_id)
+                .values(
+                    run_body=serialize_dagster_namedtuple(run.with_job_origin(job_origin)),
+                )
+            )
+            conn.execute(
+                RunTagsTable.update()  # pylint: disable=no-value-for-parameter
+                .where(RunTagsTable.c.run_id == run.run_id)
+                .where(RunTagsTable.c.key == REPOSITORY_LABEL_TAG)
+                .values(value=new_label)
+            )
+
 
 GET_PIPELINE_SNAPSHOT_QUERY_ID = "get-pipeline-snapshot"
 
