@@ -20,7 +20,7 @@ import sys
 import uuid
 from functools import wraps
 from logging.handlers import RotatingFileHandler
-from typing import Dict, NamedTuple, Optional
+from typing import Mapping, NamedTuple, Optional
 
 import click
 import yaml
@@ -136,7 +136,7 @@ class TelemetryEntry(
             ("event_id", str),
             ("elapsed_time", str),
             ("instance_id", str),
-            ("metadata", Dict[str, str]),
+            ("metadata", Mapping[str, str]),
             ("python_version", str),
             ("dagster_version", str),
             ("os_desc", str),
@@ -174,7 +174,7 @@ class TelemetryEntry(
         client_time: str,
         event_id: str,
         instance_id: str,
-        metadata: Optional[Dict[str, str]] = None,
+        metadata: Optional[Mapping[str, str]] = None,
         elapsed_time: Optional[str] = None,
         run_storage_id: Optional[str] = None,
     ):
@@ -183,7 +183,7 @@ class TelemetryEntry(
         elapsed_time = check.opt_str_param(elapsed_time, "elapsed_time", "")
         event_id = check.str_param(event_id, "event_id")
         instance_id = check.str_param(instance_id, "instance_id")
-        metadata = check.opt_dict_param(metadata, "metadata")
+        metadata = check.opt_mapping_param(metadata, "metadata")
         run_storage_id = check.opt_str_param(run_storage_id, "run_storage_id", default="")
 
         return super(TelemetryEntry, cls).__new__(
@@ -340,7 +340,7 @@ def _get_instance_telemetry_enabled(instance):
 
 def get_or_set_instance_id():
     instance_id = _get_telemetry_instance_id()
-    if instance_id == None:
+    if instance_id is None:
         instance_id = _set_telemetry_instance_id()
     return instance_id
 
@@ -395,7 +395,7 @@ def log_external_repo_stats(instance, source, external_repo, external_pipeline=N
 
         pipeline_name_hash = hash_name(external_pipeline.name) if external_pipeline else ""
         repo_hash = hash_name(external_repo.name)
-        num_pipelines_in_repo = len(external_repo.get_all_external_pipelines())
+        num_pipelines_in_repo = len(external_repo.get_all_external_jobs())
         num_schedules_in_repo = len(external_repo.get_external_schedules())
         num_sensors_in_repo = len(external_repo.get_external_sensors())
 
