@@ -8,11 +8,12 @@ import time
 from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor
 from contextlib import ExitStack, contextmanager
+from typing import NamedTuple, Optional, Sequence, TypeVar
 
 import pendulum
 import yaml
 
-from dagster import Shape
+from dagster import Permissive, Shape
 from dagster import _check as check
 from dagster import fs_io_manager
 from dagster._config import Array, Field
@@ -23,11 +24,11 @@ from dagster._core.host_representation.origin import (
 from dagster._core.instance import DagsterInstance
 from dagster._core.launcher import RunLauncher
 from dagster._core.run_coordinator import RunCoordinator, SubmitRunContext
+from dagster._core.secrets import SecretsLoader
 from dagster._core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter
 from dagster._core.workspace.context import WorkspaceProcessContext
 from dagster._core.workspace.load_target import WorkspaceLoadTarget
 from dagster._daemon.controller import create_daemon_grpc_server_registry
-from dagster._daemon.workspace import DaemonWorkspace
 from dagster._legacy import ModeDefinition, composite_solid, pipeline, solid
 from dagster._serdes import ConfigurableClass
 from dagster._seven.compat.pendulum import create_pendulum_time, mock_pendulum_timezone
@@ -35,6 +36,26 @@ from dagster._utils import Counter, merge_dicts, traced, traced_counter
 from dagster._utils.error import serializable_error_info_from_exc_info
 from dagster._utils.log import configure_loggers
 
+T_NamedTuple = TypeVar("T_NamedTuple", bound=NamedTuple)
+
+
+def assert_namedtuple_lists_equal(
+    t1_list: Sequence[T_NamedTuple],
+    t2_list: Sequence[T_NamedTuple],
+    exclude_fields: Optional[Sequence[str]] = None,
+) -> None:
+    for t1, t2 in zip(t1_list, t2_list):
+        assert_namedtuples_equal(t1, t2, exclude_fields)
+
+
+def assert_namedtuples_equal(
+    t1: T_NamedTuple, t2: T_NamedTuple, exclude_fields: Optional[Sequence[str]] = None
+) -> None:
+    exclude_fields = exclude_fields or []
+    for field in type(t1)._fields:
+        if not field in exclude_fields:
+            assert getattr(t1, field) == getattr(t2, field)
+
 
 def step_output_event_filter(pipe_iterator):
     for step_event in pipe_iterator:
@@ -415,6 +436,31 @@ class MockedRunCoordinator(RunCoordinator, ConfigurableClass):
         check.not_implemented("Cancellation not supported")
 
 
+class TestSecretsLoader(SecretsLoader, ConfigurableClass):
+    def __init__(
+        self,
+        inst_data,
+        env_vars,
+    ):
+        self._inst_data = inst_data
+        self.env_vars = env_vars
+
+    def get_secrets_for_environment(self, location_name):
+        return self.env_vars.copy()
+
+    @property
+    def inst_data(self):
+        return self._inst_data
+
+    @classmethod
+    def config_type(cls):
+        return {"env_vars": Field(Permissive())}
+
+    @staticmethod
+    def from_config_value(inst_data, config_value):
+        return TestSecretsLoader(inst_data=inst_data, **config_value)
+
+
 def get_terminate_signal():
     if sys.platform == "win32":
         return signal.SIGTERM
@@ -466,12 +512,19 @@ def in_process_test_workspace(instance, loadable_target_origin, container_image=
 
 
 @contextmanager
-def create_test_daemon_workspace(workspace_load_target, instance):
+def create_test_daemon_workspace_context(
+    workspace_load_target: WorkspaceLoadTarget,
+    instance: DagsterInstance,
+):
     """Creates a DynamicWorkspace suitable for passing into a DagsterDaemon loop when running tests."""
     configure_loggers()
     with create_daemon_grpc_server_registry(instance) as grpc_server_registry:
-        with DaemonWorkspace(grpc_server_registry, workspace_load_target) as workspace:
-            yield workspace
+        with WorkspaceProcessContext(
+            instance,
+            workspace_load_target,
+            grpc_server_registry=grpc_server_registry,
+        ) as workspace_process_context:
+            yield workspace_process_context
 
 
 def remove_none_recursively(obj):
