@@ -109,8 +109,8 @@ class DagsterType(RequiresResources):
         required_resource_keys: t.Optional[t.Set[str]] = None,
         kind: DagsterTypeKind = DagsterTypeKind.REGULAR,
         typing_type: t.Any = t.Any,
-        metadata_entries: t.Optional[t.List[MetadataEntry]] = None,
-        metadata: t.Optional[t.Dict[str, RawMetadataValue]] = None,
+        metadata_entries: t.Optional[t.Sequence[MetadataEntry]] = None,
+        metadata: t.Optional[t.Mapping[str, RawMetadataValue]] = None,
     ):
         check.opt_str_param(key, "key")
         check.opt_str_param(name, "name")
@@ -183,13 +183,16 @@ class DagsterType(RequiresResources):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return hash(self.key)
+
     @staticmethod
     def from_builtin_enum(builtin_enum) -> "DagsterType":
         check.invariant(BuiltinEnum.contains(builtin_enum), "must be member of BuiltinEnum")
         return _RUNTIME_MAP[builtin_enum]
 
     @property
-    def metadata_entries(self) -> t.List[MetadataEntry]:
+    def metadata_entries(self) -> t.Sequence[MetadataEntry]:
         return self._metadata_entries  # type: ignore
 
     @public  # type: ignore
@@ -235,7 +238,7 @@ class DagsterType(RequiresResources):
         return self._description
 
     @property
-    def inner_types(self) -> t.List["DagsterType"]:
+    def inner_types(self) -> t.Sequence["DagsterType"]:
         return []
 
     @property
@@ -247,7 +250,7 @@ class DagsterType(RequiresResources):
         return self.materializer.schema_type.key if self.materializer else None
 
     @property
-    def type_param_keys(self) -> t.List[str]:
+    def type_param_keys(self) -> t.Sequence[str]:
         return []
 
     @property
