@@ -3,9 +3,9 @@ import threading
 import time
 import warnings
 from abc import ABC, abstractmethod
-from collections import OrderedDict
 from contextlib import ExitStack
-from typing import TYPE_CHECKING, Dict, List, Optional, Union, cast
+from itertools import count
+from typing import TYPE_CHECKING, Dict, Mapping, Optional, Sequence, Union, cast
 
 import dagster._check as check
 from dagster._core.errors import (
@@ -15,6 +15,7 @@ from dagster._core.errors import (
 from dagster._core.execution.plan.state import KnownExecutionState
 from dagster._core.host_representation import (
     ExternalExecutionPlan,
+    ExternalPartitionSet,
     ExternalPipeline,
     GrpcServerRepositoryLocation,
     PipelineSelector,
@@ -40,8 +41,6 @@ from .permissions import PermissionResult, get_user_permissions
 from .workspace import IWorkspace, WorkspaceLocationEntry, WorkspaceLocationLoadStatus
 
 if TYPE_CHECKING:
-    from rx.subjects import Subject
-
     from dagster._core.host_representation import (
         ExternalPartitionConfigData,
         ExternalPartitionExecutionErrorData,
@@ -72,7 +71,7 @@ class BaseWorkspaceRequestContext(IWorkspace):
         pass
 
     @abstractmethod
-    def get_workspace_snapshot(self) -> Dict[str, WorkspaceLocationEntry]:
+    def get_workspace_snapshot(self) -> Mapping[str, WorkspaceLocationEntry]:
         pass
 
     @abstractmethod
@@ -91,7 +90,7 @@ class BaseWorkspaceRequestContext(IWorkspace):
 
     @property
     @abstractmethod
-    def permissions(self) -> Dict[str, PermissionResult]:
+    def permissions(self) -> Mapping[str, PermissionResult]:
         pass
 
     @abstractmethod
@@ -124,7 +123,7 @@ class BaseWorkspaceRequestContext(IWorkspace):
         )
 
     @property
-    def repository_locations(self) -> List[RepositoryLocation]:
+    def repository_locations(self) -> Sequence[RepositoryLocation]:
         return [
             entry.repository_location
             for entry in self.get_workspace_snapshot().values()
@@ -132,10 +131,10 @@ class BaseWorkspaceRequestContext(IWorkspace):
         ]
 
     @property
-    def repository_location_names(self) -> List[str]:
+    def repository_location_names(self) -> Sequence[str]:
         return list(self.get_workspace_snapshot())
 
-    def repository_location_errors(self) -> List[SerializableErrorInfo]:
+    def repository_location_errors(self) -> Sequence[SerializableErrorInfo]:
         return [
             entry.load_error for entry in self.get_workspace_snapshot().values() if entry.load_error
         ]
@@ -175,7 +174,7 @@ class BaseWorkspaceRequestContext(IWorkspace):
         self.process_context.reload_workspace()
         return self.process_context.create_request_context()
 
-    def has_external_pipeline(self, selector: PipelineSelector) -> bool:
+    def has_external_job(self, selector: PipelineSelector) -> bool:
         check.inst_param(selector, "selector", PipelineSelector)
         if not self.has_repository_location(selector.location_name):
             return False
@@ -183,21 +182,21 @@ class BaseWorkspaceRequestContext(IWorkspace):
         loc = self.get_repository_location(selector.location_name)
         return loc.has_repository(selector.repository_name) and loc.get_repository(
             selector.repository_name
-        ).has_external_pipeline(selector.pipeline_name)
+        ).has_external_job(selector.pipeline_name)
 
-    def get_full_external_pipeline(self, selector: PipelineSelector) -> ExternalPipeline:
+    def get_full_external_job(self, selector: PipelineSelector) -> ExternalPipeline:
         return (
             self.get_repository_location(selector.location_name)
             .get_repository(selector.repository_name)
-            .get_full_external_pipeline(selector.pipeline_name)
+            .get_full_external_job(selector.pipeline_name)
         )
 
     def get_external_execution_plan(
         self,
         external_pipeline: ExternalPipeline,
-        run_config: dict,
+        run_config: Mapping[str, object],
         mode: str,
-        step_keys_to_execute: List[str],
+        step_keys_to_execute: Sequence[str],
         known_state: KnownExecutionState,
     ) -> ExternalExecutionPlan:
         return self.get_repository_location(
@@ -234,17 +233,17 @@ class BaseWorkspaceRequestContext(IWorkspace):
         )
 
     def get_external_partition_names(
-        self, repository_handle: RepositoryHandle, partition_set_name: str
+        self, external_partition_set: ExternalPartitionSet
     ) -> Union["ExternalPartitionNamesData", "ExternalPartitionExecutionErrorData"]:
         return self.get_repository_location(
-            repository_handle.location_name
-        ).get_external_partition_names(repository_handle, partition_set_name)
+            external_partition_set.repository_handle.location_name
+        ).get_external_partition_names(external_partition_set)
 
     def get_external_partition_set_execution_param_data(
         self,
         repository_handle: RepositoryHandle,
         partition_set_name: str,
-        partition_names: List[str],
+        partition_names: Sequence[str],
     ) -> Union["ExternalPartitionSetExecutionParamData", "ExternalPartitionExecutionErrorData"]:
         return self.get_repository_location(
             repository_handle.location_name
@@ -265,22 +264,24 @@ class WorkspaceRequestContext(BaseWorkspaceRequestContext):
     def __init__(
         self,
         instance: DagsterInstance,
-        workspace_snapshot: Dict[str, WorkspaceLocationEntry],
-        process_context: "WorkspaceProcessContext",
+        workspace_snapshot: Mapping[str, WorkspaceLocationEntry],
+        process_context: "IWorkspaceProcessContext",
         version: Optional[str],
         source: Optional[object],
+        read_only: bool,
     ):
         self._instance = instance
         self._workspace_snapshot = workspace_snapshot
         self._process_context = process_context
         self._version = version
         self._source = source
+        self._read_only = read_only
 
     @property
     def instance(self) -> DagsterInstance:
         return self._instance
 
-    def get_workspace_snapshot(self) -> Dict[str, WorkspaceLocationEntry]:
+    def get_workspace_snapshot(self) -> Mapping[str, WorkspaceLocationEntry]:
         return self._workspace_snapshot
 
     def get_location_entry(self, name) -> Optional[WorkspaceLocationEntry]:
@@ -296,14 +297,14 @@ class WorkspaceRequestContext(BaseWorkspaceRequestContext):
 
     @property
     def read_only(self) -> bool:
-        return self._process_context.read_only
+        return self._read_only
 
     @property
-    def permissions(self) -> Dict[str, PermissionResult]:
-        return self._process_context.permissions
+    def permissions(self) -> Mapping[str, PermissionResult]:
+        return get_user_permissions(self._read_only)
 
     def has_permission(self, permission: str) -> bool:
-        permissions = self._process_context.permissions
+        permissions = self.permissions
         check.invariant(
             permission in permissions, f"Permission {permission} not listed in permissions map"
         )
@@ -342,11 +343,6 @@ class IWorkspaceProcessContext(ABC):
     def version(self) -> str:
         pass
 
-    @property
-    @abstractmethod
-    def location_state_events(self) -> "Subject":
-        pass
-
     @abstractmethod
     def reload_repository_location(self, name: str) -> None:
         pass
@@ -360,7 +356,7 @@ class IWorkspaceProcessContext(ABC):
 
     @property
     @abstractmethod
-    def instance(self):
+    def instance(self) -> DagsterInstance:
         pass
 
     def __enter__(self):
@@ -395,19 +391,11 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
         check.opt_str_param(version, "version")
         check.bool_param(read_only, "read_only")
 
-        # lazy import for perf
-        from rx.subjects import Subject
-
         self._instance = check.inst_param(instance, "instance", DagsterInstance)
         self._workspace_load_target = check.opt_inst_param(
             workspace_load_target, "workspace_load_target", WorkspaceLoadTarget
         )
 
-        self._location_state_events = Subject()
-        self._location_state_subscriber = LocationStateSubscriber(
-            self._location_state_events_handler
-        )
-
         self._read_only = read_only
 
         self._version = version
@@ -419,8 +407,9 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
         self._watch_thread_shutdown_events: Dict[str, threading.Event] = {}
         self._watch_threads: Dict[str, threading.Thread] = {}
 
-        self._state_subscribers: List[LocationStateSubscriber] = []
-        self.add_state_subscriber(self._location_state_subscriber)
+        self._state_subscriber_id_iter = count()
+        self._state_subscribers: Dict[int, LocationStateSubscriber] = {}
+        self.add_state_subscriber(LocationStateSubscriber(self._location_state_events_handler))
 
         if grpc_server_registry:
             self._grpc_server_registry: GrpcServerRegistry = check.inst_param(
@@ -429,49 +418,34 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
         else:
             self._grpc_server_registry = self._stack.enter_context(
                 ProcessGrpcServerRegistry(
+                    instance=self._instance,
                     reload_interval=0,
                     heartbeat_ttl=DAGIT_GRPC_SERVER_HEARTBEAT_TTL,
                     startup_timeout=instance.code_server_process_startup_timeout,
                 )
             )
 
-        self._location_entry_dict: Dict[str, WorkspaceLocationEntry] = OrderedDict()
-
-        with self._lock:
-            self._load_workspace()
+        self._location_entry_dict: Dict[str, WorkspaceLocationEntry] = {}
+        self._update_workspace(
+            {origin.location_name: self._load_location(origin) for origin in self._origins}
+        )
 
     @property
     def workspace_load_target(self):
         return self._workspace_load_target
 
-    def add_state_subscriber(self, subscriber):
-        self._state_subscribers.append(subscriber)
+    @property
+    def _origins(self) -> Sequence[RepositoryLocationOrigin]:
+        return self._workspace_load_target.create_origins() if self._workspace_load_target else []
 
-    def _load_workspace(self):
-        assert self._lock.locked()
-        repository_location_origins = (
-            self._workspace_load_target.create_origins() if self._workspace_load_target else []
-        )
+    def add_state_subscriber(self, subscriber: LocationStateSubscriber) -> int:
+        token = next(self._state_subscriber_id_iter)
+        self._state_subscribers[token] = subscriber
+        return token
 
-        check.list_param(
-            repository_location_origins,
-            "repository_location_origins",
-            of_type=RepositoryLocationOrigin,
-        )
-
-        self._location_entry_dict = OrderedDict()
-
-        for origin in repository_location_origins:
-            check.invariant(
-                self._location_entry_dict.get(origin.location_name) is None,
-                'Cannot have multiple locations with the same name, got multiple "{name}"'.format(
-                    name=origin.location_name,
-                ),
-            )
-
-            if origin.supports_server_watch:
-                self._start_watch_thread(origin)
-            self._location_entry_dict[origin.location_name] = self._load_location(origin)
+    def rm_state_subscriber(self, token: int) -> None:
+        if token in self._state_subscribers:
+            del self._state_subscribers[token]
 
     def _create_location_from_origin(
         self, origin: RepositoryLocationOrigin
@@ -505,8 +479,8 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
         return self._read_only
 
     @property
-    def permissions(self) -> Dict[str, PermissionResult]:
-        return get_user_permissions(self)
+    def permissions(self) -> Mapping[str, PermissionResult]:
+        return get_user_permissions(True)
 
     @property
     def version(self) -> str:
@@ -514,7 +488,7 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
 
     def _send_state_event_to_subscribers(self, event: LocationStateChangeEvent) -> None:
         check.inst_param(event, "event", LocationStateChangeEvent)
-        for subscriber in self._state_subscribers:
+        for subscriber in self._state_subscribers.values():
             subscriber.handle_event(event)
 
     def _start_watch_thread(self, origin: GrpcServerRepositoryLocationOrigin) -> None:
@@ -547,8 +521,7 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
         self._watch_threads[location_name] = watch_thread
         watch_thread.start()
 
-    def _load_location(self, origin):
-        assert self._lock.locked()
+    def _load_location(self, origin: RepositoryLocationOrigin) -> WorkspaceLocationEntry:
         location_name = origin.location_name
         location = None
         error = None
@@ -606,39 +579,50 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
 
     def reload_repository_location(self, name: str) -> None:
         # Can be called from a background thread
+        new = self._load_location(self._location_entry_dict[name].origin)
         with self._lock:
             # Relying on GC to clean up the old location once nothing else
             # is referencing it
-            self._location_entry_dict[name] = self._load_location(
-                self._location_entry_dict[name].origin
-            )
+            self._location_entry_dict[name] = new
 
     def shutdown_repository_location(self, name: str):
         with self._lock:
             self._location_entry_dict[name].origin.shutdown_server()
 
     def reload_workspace(self):
-        # Can be called from a background thread
-        with self._lock:
-            self._cleanup_locations()
-            self._load_workspace()
+        updated_locations = {
+            origin.location_name: self._load_location(origin) for origin in self._origins
+        }
+        self._update_workspace(updated_locations)
 
-    def _cleanup_locations(self):
-        assert self._lock.locked()
-        for _, event in self._watch_thread_shutdown_events.items():
+    def _update_workspace(self, new_locations: Dict[str, WorkspaceLocationEntry]):
+        # minimize lock time by only holding while swapping data old to new
+        with self._lock:
+            previous_events = self._watch_thread_shutdown_events
+            self._watch_thread_shutdown_events = {}
+
+            previous_threads = self._watch_threads
+            self._watch_threads = {}
+
+            previous_locations = self._location_entry_dict
+            self._location_entry_dict = new_locations
+
+            # start monitoring for new locations
+            for entry in self._location_entry_dict.values():
+                if isinstance(entry.origin, GrpcServerRepositoryLocationOrigin):
+                    self._start_watch_thread(entry.origin)
+
+        # clean up previous locations
+        for event in previous_events.values():
             event.set()
-        for _, watch_thread in self._watch_threads.items():
+
+        for watch_thread in previous_threads.values():
             watch_thread.join()
 
-        self._watch_thread_shutdown_events = {}
-        self._watch_threads = {}
-
-        for entry in self._location_entry_dict.values():
+        for entry in previous_locations.values():
             if entry.repository_location:
                 entry.repository_location.cleanup()
 
-        self._location_entry_dict = OrderedDict()
-
     def create_request_context(self, source=None) -> WorkspaceRequestContext:
         return WorkspaceRequestContext(
             instance=self._instance,
@@ -646,12 +630,9 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
             process_context=self,
             version=self.version,
             source=source,
+            read_only=self._read_only,
         )
 
-    @property
-    def location_state_events(self) -> "Subject":
-        return self._location_state_events
-
     def _location_state_events_handler(self, event: LocationStateChangeEvent) -> None:
         # If the server was updated or we were not able to reconnect, we immediately reload the
         # location handle
@@ -665,12 +646,19 @@ class WorkspaceProcessContext(IWorkspaceProcessContext):
             # with the correct error messages
             self.reload_repository_location(event.location_name)
 
-        self._location_state_events.on_next(event)
-
     def __enter__(self):
         return self
 
     def __exit__(self, exception_type, exception_value, traceback):
-        with self._lock:
-            self._cleanup_locations()
+        self._update_workspace({})  # update to empty to close all current locations
         self._stack.close()
+
+    def copy_for_test_instance(self, instance: DagsterInstance) -> "WorkspaceProcessContext":
+        """make a copy with a different instance, created for tests"""
+        return WorkspaceProcessContext(
+            instance=instance,
+            workspace_load_target=self.workspace_load_target,
+            version=self.version,
+            read_only=self.read_only,
+            grpc_server_registry=self._grpc_server_registry,
+        )
