@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import TYPE_CHECKING, Dict, NamedTuple, Optional
+from typing import TYPE_CHECKING, Mapping, NamedTuple, Optional
 
 from dagster._utils.error import SerializableErrorInfo
 
@@ -18,7 +18,7 @@ class WorkspaceLocationEntry(NamedTuple):
     repository_location: Optional["RepositoryLocation"]
     load_error: Optional[SerializableErrorInfo]
     load_status: WorkspaceLocationLoadStatus
-    display_metadata: Dict[str, str]
+    display_metadata: Mapping[str, str]
     update_timestamp: float
 
 
@@ -32,5 +32,5 @@ class IWorkspace(ABC):
         """Return the RepositoryLocation for the given location name, or raise an error if there is an error loading it."""
 
     @abstractmethod
-    def get_workspace_snapshot(self) -> Dict[str, WorkspaceLocationEntry]:
+    def get_workspace_snapshot(self) -> Mapping[str, WorkspaceLocationEntry]:
         """Return an entry for each location in the workspace."""
