@@ -1,10 +1,11 @@
 import sys
 from collections import defaultdict
-from typing import Dict
+from typing import Dict, Tuple
 
 from dagster import DagsterEvent, DagsterEventType
 from dagster import _check as check
 from dagster._core.instance import DagsterInstance
+from dagster._core.run_coordinator.queued_run_coordinator import QueuedRunCoordinator
 from dagster._core.storage.pipeline_run import (
     IN_PROGRESS_RUN_STATUSES,
     PipelineRun,
@@ -12,7 +13,7 @@ from dagster._core.storage.pipeline_run import (
     RunsFilter,
 )
 from dagster._core.storage.tags import PRIORITY_TAG
-from dagster._core.workspace.context import IWorkspace
+from dagster._core.workspace.context import IWorkspaceProcessContext
 from dagster._daemon.daemon import IntervalDaemon
 from dagster._utils.error import serializable_error_info_from_exc_info
 
@@ -27,7 +28,7 @@ class _TagConcurrencyLimitsCounter:
         check.list_param(in_progress_runs, "in_progress_runs", of_type=PipelineRun)
 
         self._key_limits: Dict[str, int] = {}
-        self._key_value_limits: Dict[(str, str), int] = {}
+        self._key_value_limits: Dict[Tuple[str, str], int] = {}
         self._unique_value_limits: Dict[str, int] = {}
 
         for tag_limit in tag_concurrency_limits:
@@ -43,8 +44,8 @@ class _TagConcurrencyLimitsCounter:
                 self._unique_value_limits[key] = limit
 
         self._key_counts: Dict[str, int] = defaultdict(lambda: 0)
-        self._key_value_counts: Dict[(str, str), int] = defaultdict(lambda: 0)
-        self._unique_value_counts: Dict[(str, str), int] = defaultdict(lambda: 0)
+        self._key_value_counts: Dict[Tuple[str, str], int] = defaultdict(lambda: 0)
+        self._unique_value_counts: Dict[Tuple[str, str], int] = defaultdict(lambda: 0)
 
         # initialize counters based on current in progress runs
         for run in in_progress_runs:
@@ -99,9 +100,13 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
     def daemon_type(cls):
         return "QUEUED_RUN_COORDINATOR"
 
-    def run_iteration(self, instance, workspace):
-        check.inst_param(instance, "instance", DagsterInstance)
-        check.inst_param(workspace, "workspace", IWorkspace)
+    def run_iteration(
+        self,
+        workspace_process_context: IWorkspaceProcessContext,
+    ):
+        instance = workspace_process_context.instance
+        if not isinstance(instance.run_coordinator, QueuedRunCoordinator):
+            check.failed(f"Expected QueuedRunCoordinator, got {instance.run_coordinator}")
 
         run_queue_config = instance.run_coordinator.get_run_queue_config()
 
@@ -149,7 +154,7 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
             error_info = None
 
             try:
-                self._dequeue_run(instance, run, workspace)
+                self._dequeue_run(instance, run, workspace_process_context)
             except Exception:
                 error_info = serializable_error_info_from_exc_info(sys.exc_info())
 
@@ -197,9 +202,14 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
         # sorted is stable, so fifo is maintained
         return sorted(runs, key=get_priority, reverse=True)
 
-    def _dequeue_run(self, instance, run, workspace):
+    def _dequeue_run(
+        self,
+        instance: DagsterInstance,
+        run: PipelineRun,
+        workspace_process_context: IWorkspaceProcessContext,
+    ):
         # double check that the run is still queued before dequeing
-        reloaded_run = instance.get_run_by_id(run.run_id)
+        reloaded_run = check.not_none(instance.get_run_by_id(run.run_id))
 
         if reloaded_run.status != PipelineRunStatus.QUEUED:
             self._logger.info(
@@ -214,4 +224,4 @@ class QueuedRunCoordinatorDaemon(IntervalDaemon):
             pipeline_name=run.pipeline_name,
         )
         instance.report_dagster_event(dequeued_event, run_id=run.run_id)
-        instance.launch_run(run.run_id, workspace)
+        instance.launch_run(run.run_id, workspace_process_context.create_request_context())
