@@ -1,11 +1,13 @@
-import concurrent.futures
+import datetime
+import logging
 import os
 import sys
 import threading
 import time
 from collections import defaultdict
+from concurrent.futures import Future, ThreadPoolExecutor
 from contextlib import ExitStack
-from typing import Dict, NamedTuple, Optional, Sequence
+from typing import Dict, Generator, List, Mapping, NamedTuple, Optional, Sequence, Union
 
 import pendulum
 
@@ -16,11 +18,14 @@ from dagster._core.definitions.sensor_definition import DefaultSensorStatus, Sen
 from dagster._core.definitions.utils import validate_tags
 from dagster._core.errors import DagsterError
 from dagster._core.host_representation import PipelineSelector
-from dagster._core.host_representation.external import ExternalSensor
+from dagster._core.host_representation.external import ExternalPipeline, ExternalSensor
+from dagster._core.host_representation.external_data import ExternalTargetData
+from dagster._core.host_representation.repository_location import RepositoryLocation
 from dagster._core.instance import DagsterInstance
 from dagster._core.scheduler.instigation import (
     InstigatorState,
     InstigatorStatus,
+    InstigatorTick,
     SensorInstigatorData,
     TickData,
     TickStatus,
@@ -28,28 +33,42 @@ from dagster._core.scheduler.instigation import (
 from dagster._core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter
 from dagster._core.storage.tags import RUN_KEY_TAG, SENSOR_NAME_TAG
 from dagster._core.telemetry import SENSOR_RUN_CREATED, hash_name, log_action
-from dagster._core.workspace.context import IWorkspace
+from dagster._core.workspace.context import IWorkspaceProcessContext
 from dagster._utils import merge_dicts
-from dagster._utils.error import serializable_error_info_from_exc_info
+from dagster._utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 
 MIN_INTERVAL_LOOP_TIME = 5
 
 FINISHED_TICK_STATES = [TickStatus.SKIPPED, TickStatus.SUCCESS, TickStatus.FAILURE]
 
+TDaemonGenerator = Generator[Union[None, SerializableErrorInfo], None, None]
+
 
 class DagsterSensorDaemonError(DagsterError):
     """Error when running the SensorDaemon"""
 
 
 class SkippedSensorRun(
-    NamedTuple("SkippedSensorRun", [("run_key", Optional[str]), ("existing_run", PipelineRun)])
+    NamedTuple(
+        "SkippedSensorRun",
+        [
+            ("run_key", Optional[str]),
+            ("existing_run", PipelineRun),
+        ],
+    )
 ):
     """Placeholder for runs that are skipped during the run_key idempotence check"""
 
 
 class SensorLaunchContext:
     def __init__(
-        self, external_sensor, tick, instance, logger, tick_retention_settings, sensor_state_lock
+        self,
+        external_sensor: ExternalSensor,
+        tick: InstigatorTick,
+        instance: DagsterInstance,
+        logger: logging.Logger,
+        tick_retention_settings,
+        sensor_state_lock: threading.Lock,
     ):
         self._external_sensor = external_sensor
         self._instance = instance
@@ -184,24 +203,28 @@ def _check_for_debug_crash(debug_crash_flags, key):
     raise Exception("Process didn't terminate after sending crash signal")
 
 
-RELOAD_WORKSPACE = 60
+VERBOSE_LOGS_INTERVAL = 60
 
 
-def execute_sensor_iteration_loop(instance, workspace, logger, until=None):
+def execute_sensor_iteration_loop(
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    until=None,
+) -> TDaemonGenerator:
     """
     Helper function that performs sensor evaluations on a tighter loop, while reusing grpc locations
     within a given daemon interval.  Rather than relying on the daemon machinery to run the
     iteration loop every 30 seconds, sensors are continuously evaluated, every 5 seconds. We rely on
     each sensor definition's min_interval to check that sensor evaluations are spaced appropriately.
     """
-    workspace_loaded_time = pendulum.now("UTC").timestamp()
 
     sensor_state_lock = threading.Lock()
+    sensor_tick_futures: Dict[str, Future] = {}
     with ExitStack() as stack:
-        settings = instance.get_settings("sensors")
+        settings = workspace_process_context.instance.get_settings("sensors")
         if settings.get("use_threads"):
             threadpool_executor = stack.enter_context(
-                concurrent.futures.ThreadPoolExecutor(
+                ThreadPoolExecutor(
                     max_workers=settings.get("num_workers"),
                     thread_name_prefix="sensor_daemon_worker",
                 )
@@ -209,56 +232,56 @@ def execute_sensor_iteration_loop(instance, workspace, logger, until=None):
         else:
             threadpool_executor = None
 
-        workspace_iteration = 0
-        start_time = pendulum.now("UTC").timestamp()
+        last_verbose_time = None
         while True:
             start_time = pendulum.now("UTC").timestamp()
             if until and start_time >= until:
                 # provide a way of organically ending the loop to support test environment
                 break
 
-            if start_time - workspace_loaded_time > RELOAD_WORKSPACE:
-                workspace.cleanup(
-                    cleanup_locations=False
-                )  # rely on GC to clean locations once no thread is using them anymore
-                workspace_loaded_time = pendulum.now("UTC").timestamp()
-                workspace_iteration = 0
-
-            yield from execute_sensor_iteration(
-                instance,
-                logger,
-                workspace.get_workspace_copy_for_iteration(),
-                threadpool_executor,
-                sensor_state_lock,
-                log_verbose_checks=(workspace_iteration == 0),
+            # occasionally enable verbose logging (doing it always would be too much)
+            verbose_logs_iteration = (
+                last_verbose_time is None or start_time - last_verbose_time > VERBOSE_LOGS_INTERVAL
             )
+            yield from execute_sensor_iteration(
+                workspace_process_context,
+                logger,
+                threadpool_executor=threadpool_executor,
+                sensor_tick_futures=sensor_tick_futures,
+                sensor_state_lock=sensor_state_lock,
+                log_verbose_checks=verbose_logs_iteration,
+            )
+            end_time = pendulum.now("UTC").timestamp()
 
-            loop_duration = pendulum.now("UTC").timestamp() - start_time
+            if verbose_logs_iteration:
+                last_verbose_time = end_time
+
+            loop_duration = end_time - start_time
             sleep_time = max(0, MIN_INTERVAL_LOOP_TIME - loop_duration)
             time.sleep(sleep_time)
-            yield
-            workspace_iteration += 1
+            yield None
 
 
 def execute_sensor_iteration(
-    instance,
-    logger,
-    workspace,
-    threadpool_executor=None,
-    sensor_state_lock=None,
-    log_verbose_checks=True,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    threadpool_executor: Optional[ThreadPoolExecutor] = None,
+    sensor_tick_futures: Optional[Dict[str, Future]] = None,
+    sensor_state_lock: Optional[threading.Lock] = None,
+    log_verbose_checks: bool = True,
     debug_crash_flags=None,
-    debug_futures=None,
 ):
-    check.inst_param(workspace, "workspace", IWorkspace)
-    check.inst_param(instance, "instance", DagsterInstance)
+
+    instance = workspace_process_context.instance
 
     if not sensor_state_lock:
         sensor_state_lock = threading.Lock()
 
     workspace_snapshot = {
         location_entry.origin.location_name: location_entry
-        for location_entry in workspace.get_workspace_snapshot().values()
+        for location_entry in workspace_process_context.create_request_context()
+        .get_workspace_snapshot()
+        .values()
     }
 
     all_sensor_states = {
@@ -268,7 +291,7 @@ def execute_sensor_iteration(
 
     tick_retention_settings = instance.get_tick_retention_settings(InstigatorType.SENSOR)
 
-    sensors = {}
+    sensors: Dict[str, ExternalSensor] = {}
     for location_entry in workspace_snapshot.values():
         repo_location = location_entry.repository_location
         if repo_location:
@@ -308,9 +331,9 @@ def execute_sensor_iteration(
                     f"{repo_location_name} that can no longer be found in the workspace. "
                     "You can turn off this sensor in the Dagit UI from the Status tab."
                 )
-            elif not workspace_snapshot[repo_location_name].repository_location.has_repository(
-                repo_name
-            ):
+            elif not check.not_none(  # checked above
+                workspace_snapshot[repo_location_name].repository_location
+            ).has_repository(repo_name):
                 logger.warning(
                     f"Could not find repository {repo_name} in location {repo_location_name} to "
                     + f"run sensor {sensor_name}. If this repository no longer exists, you can "
@@ -346,31 +369,35 @@ def execute_sensor_iteration(
             continue
 
         if threadpool_executor:
-            # add the sensor evaluations to a threadpool
+            if sensor_tick_futures is None:
+                check.failed("sensor_tick_futures dict must be passed with threadpool_executor")
+
+            # only allow one tick per sensor to be in flight
+            if (
+                external_sensor.selector_id in sensor_tick_futures
+                and not sensor_tick_futures[external_sensor.selector_id].done()
+            ):
+                continue
+
             future = threadpool_executor.submit(
                 _process_tick,
+                workspace_process_context,
                 logger,
-                instance,
-                workspace,
                 external_sensor,
                 sensor_state,
                 sensor_state_lock,
                 sensor_debug_crash_flags,
                 tick_retention_settings,
             )
-
-            # for tests, add the futures to enable for waiting
-            if debug_futures is not None:
-                debug_futures[external_sensor.selector_id] = future
+            sensor_tick_futures[external_sensor.selector_id] = future
             yield
 
         else:
             # evaluate the sensors in a loop, synchronously, yielding to allow the sensor daemon to
             # heartbeat
             yield from _process_tick_generator(
+                workspace_process_context,
                 logger,
-                instance,
-                workspace,
                 external_sensor,
                 sensor_state,
                 sensor_state_lock,
@@ -380,12 +407,11 @@ def execute_sensor_iteration(
 
 
 def _process_tick(
-    logger,
-    instance,
-    workspace,
-    external_sensor,
-    sensor_state,
-    sensor_state_lock,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    external_sensor: ExternalSensor,
+    sensor_state: InstigatorState,
+    sensor_state_lock: threading.Lock,
     sensor_debug_crash_flags,
     tick_retention_settings,
 ):
@@ -393,9 +419,8 @@ def _process_tick(
     # heartbeat to keep the daemon alive
     list(
         _process_tick_generator(
+            workspace_process_context,
             logger,
-            instance,
-            workspace,
             external_sensor,
             sensor_state,
             sensor_state_lock,
@@ -406,23 +431,25 @@ def _process_tick(
 
 
 def _process_tick_generator(
-    logger,
-    instance,
-    workspace,
-    external_sensor,
-    sensor_state,
-    sensor_state_lock,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    external_sensor: ExternalSensor,
+    sensor_state: InstigatorState,
+    sensor_state_lock: threading.Lock,
     sensor_debug_crash_flags,
     tick_retention_settings,
 ):
+    instance = workspace_process_context.instance
     error_info = None
     with sensor_state_lock:
         # acquire the lock to avoid a race condition where we're updating the recently touched
         # timestamp on the sensor state, but clobbering it with an older timestamp which might open
         # us up to a new evaluation being delegated within the minimum interval
         now = pendulum.now("UTC")
-        sensor_state = instance.get_instigator_state(
-            external_sensor.get_external_origin_id(), external_sensor.selector_id
+        sensor_state = check.not_none(
+            instance.get_instigator_state(
+                external_sensor.get_external_origin_id(), external_sensor.selector_id
+            )
         )
         if _is_under_min_interval(sensor_state, external_sensor):
             # check the since we might have been queued before processing
@@ -449,9 +476,8 @@ def _process_tick_generator(
         ) as tick_context:
             _check_for_debug_crash(sensor_debug_crash_flags, "TICK_HELD")
             yield from _evaluate_sensor(
+                workspace_process_context,
                 tick_context,
-                instance,
-                workspace,
                 external_sensor,
                 sensor_state,
                 sensor_debug_crash_flags,
@@ -466,8 +492,21 @@ def _process_tick_generator(
     yield error_info
 
 
-def _mark_sensor_state_for_tick(instance, external_sensor, sensor_state, now):
-    instigator_data = sensor_state.instigator_data
+def _sensor_instigator_data(state: InstigatorState) -> Optional[SensorInstigatorData]:
+    instigator_data = state.instigator_data
+    if instigator_data is None or isinstance(instigator_data, SensorInstigatorData):
+        return instigator_data
+    else:
+        check.failed(f"Expected SensorInstigatorData, got {instigator_data}")
+
+
+def _mark_sensor_state_for_tick(
+    instance: DagsterInstance,
+    external_sensor: ExternalSensor,
+    sensor_state: InstigatorState,
+    now: datetime.datetime,
+):
+    instigator_data = _sensor_instigator_data(sensor_state)
     instance.update_instigator_state(
         sensor_state.with_data(
             SensorInstigatorData(
@@ -484,28 +523,30 @@ def _mark_sensor_state_for_tick(instance, external_sensor, sensor_state, now):
 
 
 def _evaluate_sensor(
-    context,
-    instance,
-    workspace,
-    external_sensor,
-    state,
+    workspace_process_context: IWorkspaceProcessContext,
+    context: SensorLaunchContext,
+    external_sensor: ExternalSensor,
+    state: InstigatorState,
     sensor_debug_crash_flags=None,
 ):
+    instance = workspace_process_context.instance
     context.logger.info(f"Checking for new runs for sensor: {external_sensor.name}")
 
     sensor_origin = external_sensor.get_external_origin()
     repository_handle = external_sensor.handle.repository_handle
-    repo_location = workspace.get_repository_location(
+    repo_location = workspace_process_context.create_request_context().get_repository_location(
         sensor_origin.external_repository_origin.repository_location_origin.location_name
     )
 
+    instigator_data = _sensor_instigator_data(state)
+
     sensor_runtime_data = repo_location.get_external_sensor_execution_data(
         instance,
         repository_handle,
         external_sensor.name,
-        state.instigator_data.last_tick_timestamp if state.instigator_data else None,
-        state.instigator_data.last_run_key if state.instigator_data else None,
-        state.instigator_data.cursor if state.instigator_data else None,
+        instigator_data.last_tick_timestamp if instigator_data else None,
+        instigator_data.last_run_key if instigator_data else None,
+        instigator_data.cursor if instigator_data else None,
     )
 
     yield
@@ -514,7 +555,7 @@ def _evaluate_sensor(
     if not sensor_runtime_data.run_requests:
         if sensor_runtime_data.pipeline_run_reactions:
             for pipeline_run_reaction in sensor_runtime_data.pipeline_run_reactions:
-                origin_run_id = pipeline_run_reaction.pipeline_run.run_id
+                origin_run_id = check.not_none(pipeline_run_reaction.pipeline_run).run_id
                 if pipeline_run_reaction.error:
                     context.logger.error(
                         f"Got a reaction request for run {origin_run_id} but execution errorred: {pipeline_run_reaction.error}"
@@ -534,7 +575,7 @@ def _evaluate_sensor(
                     status = (
                         pipeline_run_reaction.run_status.value
                         if pipeline_run_reaction.run_status
-                        else pipeline_run_reaction.pipeline_run.status.value
+                        else check.not_none(pipeline_run_reaction.pipeline_run).status.value
                     )
                     # log to the original pipeline run
                     message = (
@@ -574,7 +615,9 @@ def _evaluate_sensor(
     )
 
     for run_request in sensor_runtime_data.run_requests:
-        target_data = external_sensor.get_target_data(run_request.job_name)
+        target_data: ExternalTargetData = check.not_none(
+            external_sensor.get_target_data(run_request.job_name)
+        )
 
         pipeline_selector = PipelineSelector(
             location_name=repo_location.name,
@@ -608,7 +651,7 @@ def _evaluate_sensor(
             context.logger.info(
                 "Launching run for {sensor_name}".format(sensor_name=external_sensor.name)
             )
-            instance.submit_run(run.run_id, workspace)
+            instance.submit_run(run.run_id, workspace_process_context.create_request_context())
             context.logger.info(
                 "Completed launch of run {run_id} for {sensor_name}".format(
                     run_id=run.run_id, sensor_name=external_sensor.name
@@ -641,22 +684,20 @@ def _evaluate_sensor(
     yield
 
 
-def _is_under_min_interval(state, external_sensor):
-    if not state.instigator_data:
+def _is_under_min_interval(state: InstigatorState, external_sensor: ExternalSensor) -> bool:
+    instigator_data = _sensor_instigator_data(state)
+    if not instigator_data:
         return False
 
-    if (
-        not state.instigator_data.last_tick_start_timestamp
-        and not state.instigator_data.last_tick_timestamp
-    ):
+    if not instigator_data.last_tick_start_timestamp and not instigator_data.last_tick_timestamp:
         return False
 
     if not external_sensor.min_interval_seconds:
         return False
 
     elapsed = pendulum.now("UTC").timestamp() - max(
-        state.instigator_data.last_tick_timestamp or 0,
-        state.instigator_data.last_tick_start_timestamp or 0,
+        instigator_data.last_tick_timestamp or 0,
+        instigator_data.last_tick_start_timestamp or 0,
     )
     return elapsed < external_sensor.min_interval_seconds
 
@@ -676,7 +717,7 @@ def _fetch_existing_runs(
     runs_with_run_keys = instance.get_runs(filters=RunsFilter(tags={RUN_KEY_TAG: run_keys}))
 
     # filter down to runs with run_key that match the sensor name and its namespace (repository)
-    valid_runs = []
+    valid_runs: List[PipelineRun] = []
     for run in runs_with_run_keys:
         # if the run doesn't have a set origin, just match on sensor name
         if (
@@ -686,7 +727,8 @@ def _fetch_existing_runs(
             valid_runs.append(run)
         # otherwise prevent the same named sensor across repos from effecting each other
         elif (
-            run.external_pipeline_origin.external_repository_origin.get_selector_id()
+            run.external_pipeline_origin is not None
+            and run.external_pipeline_origin.external_repository_origin.get_selector_id()
             == external_sensor.get_external_origin().external_repository_origin.get_selector_id()
             and run.tags.get(SENSOR_NAME_TAG) == external_sensor.name
         ):
@@ -702,14 +744,14 @@ def _fetch_existing_runs(
 
 
 def _get_or_create_sensor_run(
-    context,
+    context: SensorLaunchContext,
     instance: DagsterInstance,
-    repo_location,
-    external_sensor,
-    external_pipeline,
-    run_request,
-    target_data,
-    existing_runs_by_key: Dict[str, PipelineRun],
+    repo_location: RepositoryLocation,
+    external_sensor: ExternalSensor,
+    external_pipeline: ExternalPipeline,
+    run_request: RunRequest,
+    target_data: ExternalTargetData,
+    existing_runs_by_key: Mapping[str, PipelineRun],
 ):
 
     if not run_request.run_key:
@@ -739,7 +781,12 @@ def _get_or_create_sensor_run(
 
 
 def _create_sensor_run(
-    instance, repo_location, external_sensor, external_pipeline, run_request, target_data
+    instance: DagsterInstance,
+    repo_location: RepositoryLocation,
+    external_sensor: ExternalSensor,
+    external_pipeline: ExternalPipeline,
+    run_request: RunRequest,
+    target_data: ExternalTargetData,
 ):
     from dagster._daemon.daemon import get_telemetry_daemon_session_id
 
@@ -789,5 +836,7 @@ def _create_sensor_run(
         parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
         external_pipeline_origin=external_pipeline.get_external_origin(),
         pipeline_code_origin=external_pipeline.get_python_origin(),
-        asset_selection=set(run_request.asset_selection),
+        asset_selection=frozenset(run_request.asset_selection)
+        if run_request.asset_selection
+        else None,
     )
