@@ -3,7 +3,7 @@ import subprocess
 import sys
 import warnings
 from contextlib import contextmanager
-from typing import Iterator, Optional
+from typing import Iterator, Optional, Sequence, Tuple
 
 import grpc
 from grpc_health.v1 import health_pb2
@@ -53,7 +53,14 @@ def client_heartbeat_thread(client, shutdown_event):
 
 
 class DagsterGrpcClient:
-    def __init__(self, port=None, socket=None, host="localhost", use_ssl=False):
+    def __init__(
+        self,
+        port=None,
+        socket=None,
+        host="localhost",
+        use_ssl=False,
+        metadata: Optional[Sequence[Tuple[str, str]]] = None,
+    ):
         self.port = check.opt_int_param(port, "port")
         self.socket = check.opt_str_param(socket, "socket")
         self.host = check.opt_str_param(host, "host")
@@ -61,6 +68,8 @@ class DagsterGrpcClient:
 
         self._ssl_creds = grpc.ssl_channel_credentials() if use_ssl else None
 
+        self._metadata = check.opt_list_param(metadata, "metadata")
+
         check.invariant(
             port is not None if seven.IS_WINDOWS else True,
             "You must pass a valid `port` on Windows: `socket` not supported.",
@@ -79,6 +88,10 @@ class DagsterGrpcClient:
         else:
             self._server_address = "unix:" + os.path.abspath(socket)
 
+    @property
+    def metadata(self) -> Sequence[Tuple[str, str]]:
+        return self._metadata
+
     @property
     def use_ssl(self) -> bool:
         return self._use_ssl
@@ -105,21 +118,50 @@ class DagsterGrpcClient:
         ) as channel:
             yield channel
 
-    def _query(self, method, request_type, timeout=DEFAULT_GRPC_TIMEOUT, **kwargs):
+    def _get_response(
+        self,
+        method,
+        request,
+        timeout=DEFAULT_GRPC_TIMEOUT,
+    ):
+        with self._channel() as channel:
+            stub = DagsterApiStub(channel)
+            return getattr(stub, method)(request, metadata=self._metadata, timeout=timeout)
+
+    def _query(
+        self,
+        method,
+        request_type,
+        timeout=DEFAULT_GRPC_TIMEOUT,
+        **kwargs,
+    ):
         try:
-            with self._channel() as channel:
-                stub = DagsterApiStub(channel)
-                response = getattr(stub, method)(request_type(**kwargs), timeout=timeout)
-            return response
+            return self._get_response(method, request=request_type(**kwargs), timeout=timeout)
         except Exception as e:
             raise DagsterUserCodeUnreachableError("Could not reach user code server") from e
 
-    def _streaming_query(self, method, request_type, timeout=DEFAULT_GRPC_TIMEOUT, **kwargs):
+    def _get_streaming_response(
+        self,
+        method,
+        request,
+        timeout=DEFAULT_GRPC_TIMEOUT,
+    ):
+
+        with self._channel() as channel:
+            stub = DagsterApiStub(channel)
+            yield from getattr(stub, method)(request, metadata=self._metadata, timeout=timeout)
+
+    def _streaming_query(
+        self,
+        method,
+        request_type,
+        timeout=DEFAULT_GRPC_TIMEOUT,
+        **kwargs,
+    ):
         try:
-            with self._channel() as channel:
-                stub = DagsterApiStub(channel)
-                response_stream = getattr(stub, method)(request_type(**kwargs), timeout=timeout)
-                yield from response_stream
+            yield from self._get_streaming_response(
+                method, request=request_type(**kwargs), timeout=timeout
+            )
         except Exception as e:
             raise DagsterUserCodeUnreachableError("Could not reach user code server") from e
 
@@ -238,7 +280,11 @@ class DagsterGrpcClient:
 
         return res.serialized_external_pipeline_subset_result
 
-    def external_repository(self, external_repository_origin):
+    def external_repository(
+        self,
+        external_repository_origin: ExternalRepositoryOrigin,
+        defer_snapshots: bool = False,
+    ):
         check.inst_param(
             external_repository_origin,
             "external_repository_origin",
@@ -252,11 +298,34 @@ class DagsterGrpcClient:
             serialized_repository_python_origin=serialize_dagster_namedtuple(
                 external_repository_origin
             ),
+            defer_snapshots=defer_snapshots,
         )
 
         return res.serialized_external_repository_data
 
-    def streaming_external_repository(self, external_repository_origin):
+    def external_job(
+        self,
+        external_repository_origin: ExternalRepositoryOrigin,
+        job_name: str,
+    ):
+        check.inst_param(
+            external_repository_origin,
+            "external_repository_origin",
+            ExternalRepositoryOrigin,
+        )
+
+        return self._query(
+            "ExternalJob",
+            api_pb2.ExternalJobRequest,
+            serialized_repository_origin=serialize_dagster_namedtuple(external_repository_origin),
+            job_name=job_name,
+        )
+
+    def streaming_external_repository(
+        self,
+        external_repository_origin: ExternalRepositoryOrigin,
+        defer_snapshots: bool = False,
+    ):
         for res in self._streaming_query(
             "StreamingExternalRepository",
             api_pb2.ExternalRepositoryRequest,
@@ -264,6 +333,7 @@ class DagsterGrpcClient:
             serialized_repository_python_origin=serialize_dagster_namedtuple(
                 external_repository_origin
             ),
+            defer_snapshots=defer_snapshots,
         ):
             yield {
                 "sequence_number": res.sequence_number,
@@ -444,10 +514,14 @@ def ephemeral_grpc_api_client(
     check.bool_param(force_port, "force_port")
     check.int_param(max_retries, "max_retries")
 
-    with GrpcServerProcess(
-        loadable_target_origin=loadable_target_origin,
-        force_port=force_port,
-        max_retries=max_retries,
-        max_workers=max_workers,
-    ).create_ephemeral_client() as client:
-        yield client
+    from dagster._core.test_utils import instance_for_test
+
+    with instance_for_test() as instance:
+        with GrpcServerProcess(
+            instance_ref=instance.get_ref(),
+            loadable_target_origin=loadable_target_origin,
+            force_port=force_port,
+            max_retries=max_retries,
+            max_workers=max_workers,
+        ).create_ephemeral_client() as client:
+            yield client
