@@ -3,19 +3,16 @@
 import os
 import sys
 from contextlib import ExitStack
-from typing import Generator, List, Optional
+from typing import Generator, Optional, Sequence, Union
 
 import pendulum
 
 import dagster._check as check
 from dagster._core.definitions import ScheduleEvaluationContext
 from dagster._core.definitions.events import AssetKey
-from dagster._core.definitions.reconstruct import ReconstructablePipeline, ReconstructableRepository
-from dagster._core.definitions.sensor_definition import (
-    MultiAssetSensorDefinition,
-    MultiAssetSensorEvaluationContext,
-    SensorEvaluationContext,
-)
+from dagster._core.definitions.reconstruct import ReconstructablePipeline
+from dagster._core.definitions.repository_definition import RepositoryDefinition
+from dagster._core.definitions.sensor_definition import SensorEvaluationContext
 from dagster._core.errors import (
     DagsterExecutionInterruptedError,
     DagsterRunNotFoundError,
@@ -39,6 +36,7 @@ from dagster._core.host_representation.external_data import (
     ExternalSensorExecutionErrorData,
 )
 from dagster._core.instance import DagsterInstance
+from dagster._core.instance.ref import InstanceRef
 from dagster._core.snap.execution_plan_snapshot import (
     ExecutionPlanSnapshotErrorData,
     snapshot_from_execution_plan,
@@ -76,14 +74,41 @@ def core_execute_run(
     recon_pipeline: ReconstructablePipeline,
     pipeline_run: PipelineRun,
     instance: DagsterInstance,
+    inject_env_vars: bool,
     resume_from_failure: bool = False,
 ) -> Generator[DagsterEvent, None, None]:
     check.inst_param(recon_pipeline, "recon_pipeline", ReconstructablePipeline)
     check.inst_param(pipeline_run, "pipeline_run", PipelineRun)
     check.inst_param(instance, "instance", DagsterInstance)
 
+    if inject_env_vars:
+        try:
+            location_name = (
+                pipeline_run.external_pipeline_origin.location_name
+                if pipeline_run.external_pipeline_origin
+                else None
+            )
+
+            instance.inject_env_vars(location_name)
+        except Exception:
+            yield instance.report_engine_event(
+                "Error while loading environment variables.",
+                pipeline_run,
+                EngineEventData.engine_error(serializable_error_info_from_exc_info(sys.exc_info())),
+            )
+            yield from _report_run_failed_if_not_finished(instance, pipeline_run.run_id)
+            raise
+
     # try to load the pipeline definition early
     try:
+        # add in cached metadata to load repository more efficiently
+        if pipeline_run.has_repository_load_data:
+            execution_plan_snapshot = instance.get_execution_plan_snapshot(
+                check.not_none(pipeline_run.execution_plan_snapshot_id)
+            )
+            recon_pipeline = recon_pipeline.with_repository_load_data(
+                execution_plan_snapshot.repository_load_data,
+            )
         recon_pipeline.get_definition()
     except Exception:
         yield instance.report_engine_event(
@@ -95,15 +120,13 @@ def core_execute_run(
         raise
 
     # Reload the run to verify that its status didn't change while the pipeline was loaded
-    pipeline_run = instance.get_run_by_id(pipeline_run.run_id)
-    check.inst(
-        pipeline_run,
-        PipelineRun,
+    dagster_run = check.not_none(
+        instance.get_run_by_id(pipeline_run.run_id),
         f"Pipeline run with id '{pipeline_run.run_id}' was deleted after the run worker started.",
     )
 
     try:
-        pipeline_run = instance.get_run_by_id(pipeline_run.run_id)
+        dagster_run = check.not_none(instance.get_run_by_id(dagster_run.run_id))
         yield from execute_run_iterator(
             recon_pipeline, pipeline_run, instance, resume_from_failure=resume_from_failure
         )
@@ -181,7 +204,9 @@ def _run_in_subprocess(
     # https://amir.rachum.com/blog/2017/03/03/generator-cleanup/
     closed = False
     try:
-        for event in core_execute_run(recon_pipeline, pipeline_run, instance):
+        for event in core_execute_run(
+            recon_pipeline, pipeline_run, instance, inject_env_vars=False
+        ):
             run_event_handler(event)
     except GeneratorExit:
         closed = True
@@ -215,44 +240,33 @@ def start_run_in_subprocess(
 
 
 def get_external_pipeline_subset_result(
-    recon_pipeline: ReconstructablePipeline,
-    solid_selection: Optional[List[str]],
-    asset_selection: Optional[List[AssetKey]],
+    repo_def: RepositoryDefinition,
+    job_name: str,
+    solid_selection: Optional[Sequence[str]],
+    asset_selection: Optional[Sequence[AssetKey]],
 ):
-    check.inst_param(recon_pipeline, "recon_pipeline", ReconstructablePipeline)
-    check.opt_list_param(solid_selection, "solid_selection", str)
-    check.opt_list_param(asset_selection, "asset_selection", AssetKey)
-    if solid_selection or asset_selection:
-        try:
-            sub_pipeline = recon_pipeline.subset_for_execution(
-                solid_selection=solid_selection,
-                asset_selection=frozenset(asset_selection) if asset_selection else None,
-            )
-            definition = sub_pipeline.get_definition()
-        except Exception:
-            return ExternalPipelineSubsetResult(
-                success=False, error=serializable_error_info_from_exc_info(sys.exc_info())
-            )
-    else:
-        definition = recon_pipeline.get_definition()
+    try:
+        definition = repo_def.get_maybe_subset_job_def(
+            job_name,
+            op_selection=solid_selection,
+            asset_selection=frozenset(asset_selection) if asset_selection else None,
+        )
+    except Exception:
+        return ExternalPipelineSubsetResult(
+            success=False, error=serializable_error_info_from_exc_info(sys.exc_info())
+        )
     external_pipeline_data = external_pipeline_data_from_def(definition)
     return ExternalPipelineSubsetResult(success=True, external_pipeline_data=external_pipeline_data)
 
 
 def get_external_schedule_execution(
-    recon_repo,
-    instance_ref,
-    schedule_name,
-    scheduled_execution_timestamp,
-    scheduled_execution_timezone,
+    repo_def: RepositoryDefinition,
+    instance_ref: Optional[InstanceRef],
+    schedule_name: str,
+    scheduled_execution_timestamp: float,
+    scheduled_execution_timezone: str,
 ):
-    check.inst_param(
-        recon_repo,
-        "recon_repo",
-        ReconstructableRepository,
-    )
-    definition = recon_repo.get_definition()
-    schedule_def = definition.get_schedule_def(schedule_name)
+    schedule_def = repo_def.get_schedule_def(schedule_name)
     scheduled_execution_time = (
         pendulum.from_timestamp(
             scheduled_execution_timestamp,
@@ -277,40 +291,26 @@ def get_external_schedule_execution(
 
 
 def get_external_sensor_execution(
-    recon_repo, instance_ref, sensor_name, last_completion_timestamp, last_run_key, cursor
+    repo_def: RepositoryDefinition,
+    instance_ref: Optional[InstanceRef],
+    sensor_name: str,
+    last_completion_timestamp: Optional[float],
+    last_run_key: Optional[str],
+    cursor: Optional[str],
 ):
-    check.inst_param(
-        recon_repo,
-        "recon_repo",
-        ReconstructableRepository,
-    )
-
-    definition = recon_repo.get_definition()
-    sensor_def = definition.get_sensor_def(sensor_name)
+    sensor_def = repo_def.get_sensor_def(sensor_name)
 
     with ExitStack() as stack:
-
-        if isinstance(sensor_def, MultiAssetSensorDefinition):
-            sensor_context = stack.enter_context(
-                MultiAssetSensorEvaluationContext(
-                    instance_ref,
-                    last_completion_time=last_completion_timestamp,
-                    last_run_key=last_run_key,
-                    cursor=cursor,
-                    repository_name=recon_repo.get_definition().name,
-                    asset_keys=sensor_def.asset_keys,
-                )
-            )
-        else:
-            sensor_context = stack.enter_context(
-                SensorEvaluationContext(
-                    instance_ref,
-                    last_completion_time=last_completion_timestamp,
-                    last_run_key=last_run_key,
-                    cursor=cursor,
-                    repository_name=recon_repo.get_definition().name,
-                )
+        sensor_context = stack.enter_context(
+            SensorEvaluationContext(
+                instance_ref,
+                last_completion_time=last_completion_timestamp,
+                last_run_key=last_run_key,
+                cursor=cursor,
+                repository_name=repo_def.name,
+                repository_def=repo_def,
             )
+        )
 
         try:
             with user_code_error_boundary(
@@ -325,9 +325,13 @@ def get_external_sensor_execution(
             )
 
 
-def get_partition_config(recon_repo, partition_set_name, partition_name):
-    definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(partition_set_name)
+def get_partition_config(
+    repo_def: RepositoryDefinition,
+    partition_set_name: str,
+    partition_name: str,
+):
+
+    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
     partition = partition_set_def.get_partition(partition_name)
     try:
         with user_code_error_boundary(
@@ -352,9 +356,11 @@ def _get_target_for_partition_execution_error(partition_set_def):
         return f"partition set '{partition_set_def.name}'"
 
 
-def get_partition_names(recon_repo, partition_set_name):
-    definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(partition_set_name)
+def get_partition_names(
+    repo_def: RepositoryDefinition,
+    partition_set_name: str,
+):
+    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
     try:
         with user_code_error_boundary(
             PartitionExecutionError,
@@ -370,9 +376,12 @@ def get_partition_names(recon_repo, partition_set_name):
         )
 
 
-def get_partition_tags(recon_repo, partition_set_name, partition_name):
-    definition = recon_repo.get_definition()
-    partition_set_def = definition.get_partition_set_def(partition_set_name)
+def get_partition_tags(
+    repo_def: RepositoryDefinition,
+    partition_set_name: str,
+    partition_name: str,
+):
+    partition_set_def = repo_def.get_partition_set_def(partition_set_name)
     partition = partition_set_def.get_partition(partition_name)
     try:
         with user_code_error_boundary(
@@ -388,26 +397,27 @@ def get_partition_tags(recon_repo, partition_set_name, partition_name):
         )
 
 
-def get_external_execution_plan_snapshot(recon_pipeline, args):
-    check.inst_param(recon_pipeline, "recon_pipeline", ReconstructablePipeline)
-    check.inst_param(args, "args", ExecutionPlanSnapshotArgs)
-
+def get_external_execution_plan_snapshot(
+    repo_def: RepositoryDefinition,
+    job_name: str,
+    args: ExecutionPlanSnapshotArgs,
+):
     try:
-        pipeline = recon_pipeline
-
-        if args.solid_selection or args.asset_selection:
-            pipeline = pipeline.subset_for_execution(
-                solid_selection=args.solid_selection, asset_selection=args.asset_selection
-            )
+        job_def = repo_def.get_maybe_subset_job_def(
+            job_name,
+            op_selection=args.solid_selection,
+            asset_selection=args.asset_selection,
+        )
 
         return snapshot_from_execution_plan(
             create_execution_plan(
-                pipeline=pipeline,
+                job_def,
                 run_config=args.run_config,
                 mode=args.mode,
                 step_keys_to_execute=args.step_keys_to_execute,
                 known_state=args.known_state,
                 instance_ref=args.instance_ref,
+                repository_load_data=repo_def.repository_load_data,
             ),
             args.pipeline_snapshot_id,
         )
@@ -417,8 +427,11 @@ def get_external_execution_plan_snapshot(recon_pipeline, args):
         )
 
 
-def get_partition_set_execution_param_data(recon_repo, partition_set_name, partition_names):
-    repo_definition = recon_repo.get_definition()
+def get_partition_set_execution_param_data(
+    repo_definition: RepositoryDefinition,
+    partition_set_name: str,
+    partition_names: Sequence[str],
+) -> Union[ExternalPartitionSetExecutionParamData, ExternalPartitionExecutionErrorData]:
     partition_set_def = repo_definition.get_partition_set_def(partition_set_name)
     try:
         with user_code_error_boundary(
