@@ -1,9 +1,13 @@
 import datetime
+import logging
 import os
 import sys
+import threading
 import time
 from collections import defaultdict
-from typing import cast
+from concurrent.futures import Future, ThreadPoolExecutor
+from contextlib import ExitStack
+from typing import Dict, List, Optional, cast
 
 import pendulum
 
@@ -13,10 +17,13 @@ from dagster._core.definitions.schedule_definition import DefaultScheduleStatus
 from dagster._core.definitions.utils import validate_tags
 from dagster._core.errors import DagsterUserCodeUnreachableError
 from dagster._core.host_representation import ExternalSchedule, PipelineSelector
+from dagster._core.host_representation.external import ExternalPipeline
+from dagster._core.host_representation.repository_location import RepositoryLocation
 from dagster._core.instance import DagsterInstance
 from dagster._core.scheduler.instigation import (
     InstigatorState,
     InstigatorStatus,
+    InstigatorTick,
     InstigatorType,
     ScheduleInstigatorData,
     TickData,
@@ -26,7 +33,7 @@ from dagster._core.scheduler.scheduler import DEFAULT_MAX_CATCHUP_RUNS, DagsterS
 from dagster._core.storage.pipeline_run import PipelineRun, PipelineRunStatus, RunsFilter
 from dagster._core.storage.tags import RUN_KEY_TAG, SCHEDULED_EXECUTION_TIME_TAG
 from dagster._core.telemetry import SCHEDULED_RUN_CREATED, hash_name, log_action
-from dagster._core.workspace.context import IWorkspace
+from dagster._core.workspace.context import IWorkspaceProcessContext
 from dagster._seven.compat.pendulum import to_timezone
 from dagster._utils import merge_dicts
 from dagster._utils.error import serializable_error_info_from_exc_info
@@ -34,7 +41,14 @@ from dagster._utils.log import default_date_format_string
 
 
 class _ScheduleLaunchContext:
-    def __init__(self, external_schedule, tick, instance, logger, tick_retention_settings):
+    def __init__(
+        self,
+        external_schedule: ExternalSchedule,
+        tick: InstigatorTick,
+        instance: DagsterInstance,
+        logger: logging.Logger,
+        tick_retention_settings,
+    ):
         self._external_schedule = external_schedule
         self._instance = instance
         self._logger = logger
@@ -80,56 +94,83 @@ class _ScheduleLaunchContext:
 
 
 MIN_INTERVAL_LOOP_TIME = 5
-RELOAD_WORKSPACE = 60
+VERBOSE_LOGS_INTERVAL = 60
 
 
 def execute_scheduler_iteration_loop(
-    instance, workspace, logger, max_catchup_runs, max_tick_retries
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    max_catchup_runs: int,
+    max_tick_retries: int,
 ):
-    workspace_loaded_time = pendulum.now("UTC").timestamp()
+    schedule_state_lock = threading.Lock()
+    scheduler_run_futures: Dict[str, Future] = {}
 
-    workspace_iteration = 0
-    start_time = pendulum.now("UTC").timestamp()
-    while True:
-        start_time = pendulum.now("UTC").timestamp()
-        if start_time - workspace_loaded_time > RELOAD_WORKSPACE:
-            workspace.cleanup(cleanup_locations=True)
-            workspace_loaded_time = pendulum.now("UTC").timestamp()
-            workspace_iteration = 0
+    with ExitStack() as stack:
+        settings = workspace_process_context.instance.get_settings("schedules")
+        if settings.get("use_threads"):
+            threadpool_executor = stack.enter_context(
+                ThreadPoolExecutor(
+                    max_workers=settings.get("num_workers"),
+                    thread_name_prefix="schedule_daemon_worker",
+                )
+            )
+        else:
+            threadpool_executor = None
 
-        end_datetime_utc = pendulum.now("UTC")
-        yield from launch_scheduled_runs(
-            instance,
-            workspace,
-            logger,
-            end_datetime_utc=end_datetime_utc,
-            max_catchup_runs=max_catchup_runs,
-            max_tick_retries=max_tick_retries,
-            log_verbose_checks=(workspace_iteration == 0),
-        )
-        loop_duration = pendulum.now("UTC").timestamp() - start_time
-        sleep_time = max(0, MIN_INTERVAL_LOOP_TIME - loop_duration)
-        time.sleep(sleep_time)
-        yield
-        workspace_iteration += 1
+        last_verbose_time = None
+        while True:
+            start_time = pendulum.now("UTC").timestamp()
+            end_datetime_utc = pendulum.now("UTC")
+
+            # occasionally enable verbose logging (doing it always would be too much)
+            verbose_logs_iteration = (
+                last_verbose_time is None or start_time - last_verbose_time > VERBOSE_LOGS_INTERVAL
+            )
+            yield from launch_scheduled_runs(
+                workspace_process_context,
+                logger,
+                end_datetime_utc=end_datetime_utc,
+                threadpool_executor=threadpool_executor,
+                scheduler_run_futures=scheduler_run_futures,
+                schedule_state_lock=schedule_state_lock,
+                max_catchup_runs=max_catchup_runs,
+                max_tick_retries=max_tick_retries,
+                log_verbose_checks=verbose_logs_iteration,
+            )
+            end_time = pendulum.now("UTC").timestamp()
+
+            if verbose_logs_iteration:
+                last_verbose_time = end_time
+
+            loop_duration = end_time - start_time
+            sleep_time = max(0, MIN_INTERVAL_LOOP_TIME - loop_duration)
+            time.sleep(sleep_time)
+            yield
 
 
 def launch_scheduled_runs(
-    instance,
-    workspace,
-    logger,
-    end_datetime_utc,
-    max_catchup_runs=DEFAULT_MAX_CATCHUP_RUNS,
-    max_tick_retries=0,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    end_datetime_utc: datetime.datetime,
+    threadpool_executor: Optional[ThreadPoolExecutor] = None,
+    scheduler_run_futures: Optional[Dict[str, Future]] = None,
+    schedule_state_lock: Optional[threading.Lock] = None,
+    max_catchup_runs: int = DEFAULT_MAX_CATCHUP_RUNS,
+    max_tick_retries: int = 0,
     debug_crash_flags=None,
-    log_verbose_checks=True,
+    log_verbose_checks: bool = True,
 ):
-    check.inst_param(instance, "instance", DagsterInstance)
-    check.inst_param(workspace, "workspace", IWorkspace)
+    instance = workspace_process_context.instance
+
+    if not schedule_state_lock:
+        schedule_state_lock = threading.Lock()
 
     workspace_snapshot = {
         location_entry.origin.location_name: location_entry
-        for location_entry in workspace.get_workspace_snapshot().values()
+        for location_entry in workspace_process_context.create_request_context()
+        .get_workspace_snapshot()
+        .values()
     }
 
     all_schedule_states = {
@@ -139,7 +180,8 @@ def launch_scheduled_runs(
 
     tick_retention_settings = instance.get_tick_retention_settings(InstigatorType.SCHEDULE)
 
-    schedules = {}
+    schedules: Dict[str, ExternalSchedule] = {}
+    error_locations = set()
     for location_entry in workspace_snapshot.values():
         repo_location = location_entry.repository_location
         if repo_location:
@@ -150,10 +192,12 @@ def launch_scheduled_runs(
                         all_schedule_states.get(selector_id)
                     ).is_running:
                         schedules[selector_id] = schedule
-        elif location_entry.load_error and log_verbose_checks:
-            logger.warning(
-                f"Could not load location {location_entry.origin.location_name} to check for schedules due to the following error: {location_entry.load_error}"
-            )
+        elif location_entry.load_error:
+            if log_verbose_checks:
+                logger.warning(
+                    f"Could not load location {location_entry.origin.location_name} to check for schedules due to the following error: {location_entry.load_error}"
+                )
+            error_locations.add(location_entry.origin.location_name)
 
     # Remove any schedule states that were previously created with AUTOMATICALLY_RUNNING
     # and can no longer be found in the workspace (so that if they are later added
@@ -165,9 +209,16 @@ def launch_scheduled_runs(
         and schedule_state.status == InstigatorStatus.AUTOMATICALLY_RUNNING
     }
     for state in states_to_delete:
-        instance.schedule_storage.delete_instigator_state(
-            state.instigator_origin_id, state.selector_id
+        location_name = (
+            state.origin.external_repository_origin.repository_location_origin.location_name
         )
+        # don't clean up auto running state if its location is an error state
+        if location_name not in error_locations:
+            logger.info(
+                f"Removing state for automatically running schedule {state.instigator_name} "
+                f"that is no longer present in {location_name}."
+            )
+            instance.delete_instigator_state(state.instigator_origin_id, state.selector_id)
 
     if log_verbose_checks:
         unloadable_schedule_states = {
@@ -193,9 +244,9 @@ def launch_scheduled_runs(
                     f"{repo_location_name} that can no longer be found in the workspace. You can "
                     "turn off this schedule in the Dagit UI from the Status tab."
                 )
-            elif not workspace_snapshot[
-                repo_location_origin.location_name
-            ].repository_location.has_repository(repo_name):
+            elif not check.not_none(  # checked in case above
+                workspace_snapshot[repo_location_origin.location_name].repository_location
+            ).has_repository(repo_name):
                 logger.warning(
                     f"Could not find repository {repo_name} in location {repo_location_name} to "
                     + f"run schedule {schedule_name}. If this repository no longer exists, you can "
@@ -234,23 +285,56 @@ def launch_scheduled_runs(
                 )
                 instance.add_instigator_state(schedule_state)
 
-            yield from launch_scheduled_runs_for_schedule(
-                instance,
-                logger,
-                external_schedule,
-                schedule_state,
-                workspace,
-                end_datetime_utc,
-                max_catchup_runs,
-                max_tick_retries,
-                tick_retention_settings,
-                (
-                    debug_crash_flags.get(schedule_state.instigator_name)
-                    if debug_crash_flags
-                    else None
-                ),
-                log_verbose_checks=log_verbose_checks,
+            schedule_debug_crash_flags = (
+                debug_crash_flags.get(schedule_state.instigator_name) if debug_crash_flags else None
             )
+
+            if threadpool_executor:
+                if scheduler_run_futures is None:
+                    check.failed(
+                        "scheduler_run_futures dict must be passed with threadpool_executor"
+                    )
+
+                # only allow one tick per schedule to be in flight
+                if (
+                    external_schedule.selector_id in scheduler_run_futures
+                    and not scheduler_run_futures[external_schedule.selector_id].done()
+                ):
+                    continue
+
+                future = threadpool_executor.submit(
+                    launch_scheduled_runs_for_schedule,
+                    workspace_process_context,
+                    logger,
+                    external_schedule,
+                    schedule_state,
+                    schedule_state_lock,
+                    end_datetime_utc,
+                    max_catchup_runs,
+                    max_tick_retries,
+                    tick_retention_settings,
+                    schedule_debug_crash_flags,
+                    log_verbose_checks=log_verbose_checks,
+                )
+                scheduler_run_futures[external_schedule.selector_id] = future
+                yield
+
+            else:
+                # evaluate the sensors in a loop, synchronously, yielding to allow the sensor daemon to
+                # heartbeat
+                yield from launch_scheduled_runs_for_schedule_iterator(
+                    workspace_process_context,
+                    logger,
+                    external_schedule,
+                    schedule_state,
+                    schedule_state_lock,
+                    end_datetime_utc,
+                    max_catchup_runs,
+                    max_tick_retries,
+                    tick_retention_settings,
+                    schedule_debug_crash_flags,
+                    log_verbose_checks=log_verbose_checks,
+                )
         except Exception:
             error_info = serializable_error_info_from_exc_info(sys.exc_info())
             logger.error(
@@ -260,25 +344,59 @@ def launch_scheduled_runs(
 
 
 def launch_scheduled_runs_for_schedule(
-    instance,
-    logger,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
     external_schedule: ExternalSchedule,
     schedule_state: InstigatorState,
-    workspace,
+    schedule_state_lock: threading.Lock,
     end_datetime_utc: datetime.datetime,
-    max_catchup_runs,
-    max_tick_retries,
+    max_catchup_runs: int,
+    max_tick_retries: int,
     tick_retention_settings,
-    debug_crash_flags=None,
-    log_verbose_checks=True,
+    schedule_debug_crash_flags,
+    log_verbose_checks,
+):
+    # evaluate the tick immediately, but from within a thread.  The main thread should be able to
+    # heartbeat to keep the daemon alive
+    list(
+        launch_scheduled_runs_for_schedule_iterator(
+            workspace_process_context,
+            logger,
+            external_schedule,
+            schedule_state,
+            schedule_state_lock,
+            end_datetime_utc,
+            max_catchup_runs,
+            max_tick_retries,
+            tick_retention_settings,
+            schedule_debug_crash_flags,
+            log_verbose_checks,
+        )
+    )
+
+
+def launch_scheduled_runs_for_schedule_iterator(
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    external_schedule: ExternalSchedule,
+    schedule_state: InstigatorState,
+    schedule_state_lock: threading.Lock,
+    end_datetime_utc: datetime.datetime,
+    max_catchup_runs: int,
+    max_tick_retries: int,
+    tick_retention_settings,
+    schedule_debug_crash_flags,
+    log_verbose_checks,
 ):
-    instance = check.inst_param(instance, "instance", DagsterInstance)
     schedule_state = check.inst_param(schedule_state, "schedule_state", InstigatorState)
     end_datetime_utc = check.inst_param(end_datetime_utc, "end_datetime_utc", datetime.datetime)
+    instance = workspace_process_context.instance
 
-    instigator_origin_id = external_schedule.get_external_origin_id()
-    ticks = instance.get_ticks(instigator_origin_id, external_schedule.selector_id, limit=1)
-    latest_tick = ticks[0] if ticks else None
+    with schedule_state_lock:
+
+        instigator_origin_id = external_schedule.get_external_origin_id()
+        ticks = instance.get_ticks(instigator_origin_id, external_schedule.selector_id, limit=1)
+        latest_tick: Optional[InstigatorTick] = ticks[0] if ticks else None
 
     instigator_data = cast(ScheduleInstigatorData, schedule_state.instigator_data)
     start_timestamp_utc = instigator_data.start_timestamp if schedule_state else None
@@ -303,6 +421,8 @@ def launch_scheduled_runs_for_schedule(
     else:
         start_timestamp_utc = instigator_data.start_timestamp
 
+    start_timestamp_utc = check.not_none(start_timestamp_utc)
+
     schedule_name = external_schedule.name
 
     timezone_str = external_schedule.execution_timezone
@@ -314,7 +434,7 @@ def launch_scheduled_runs_for_schedule(
                 "an execution_timezone in its definition."
             )
 
-    tick_times = []
+    tick_times: List[datetime.datetime] = []
     for next_time in external_schedule.execution_time_iterator(start_timestamp_utc):
         if next_time.timestamp() > end_datetime_utc.timestamp():
             break
@@ -323,7 +443,7 @@ def launch_scheduled_runs_for_schedule(
 
     if not tick_times:
         if log_verbose_checks:
-            logger.info(f"No new runs for {schedule_name}")
+            logger.info(f"No new tick times to evaluate for {schedule_name}")
         return
 
     if not external_schedule.partition_set_name and len(tick_times) > 1:
@@ -363,22 +483,21 @@ def launch_scheduled_runs_for_schedule(
                 )
             )
 
-            _check_for_debug_crash(debug_crash_flags, "TICK_CREATED")
+            _check_for_debug_crash(schedule_debug_crash_flags, "TICK_CREATED")
 
         with _ScheduleLaunchContext(
             external_schedule, tick, instance, logger, tick_retention_settings
         ) as tick_context:
             try:
-                _check_for_debug_crash(debug_crash_flags, "TICK_HELD")
+                _check_for_debug_crash(schedule_debug_crash_flags, "TICK_HELD")
 
                 yield from _schedule_runs_at_time(
-                    instance,
+                    workspace_process_context,
                     logger,
-                    workspace,
                     external_schedule,
                     schedule_time,
                     tick_context,
-                    debug_crash_flags,
+                    schedule_debug_crash_flags,
                 )
             except Exception as e:
                 if isinstance(e, DagsterUserCodeUnreachableError):
@@ -396,7 +515,8 @@ def launch_scheduled_runs_for_schedule(
                             # or the schedule is turned off
                             failure_count=tick_context.failure_count,
                         )
-                        raise  # Raise the wrapped DagsterSchedulerError exception
+                        yield error_data
+                        return
 
                 else:
                     error_data = serializable_error_info_from_exc_info(sys.exc_info())
@@ -405,7 +525,8 @@ def launch_scheduled_runs_for_schedule(
                         error=error_data,
                         failure_count=tick_context.failure_count + 1,
                     )
-                    raise
+                    yield error_data
+                    return
 
 
 def _check_for_debug_crash(debug_crash_flags, key):
@@ -422,39 +543,29 @@ def _check_for_debug_crash(debug_crash_flags, key):
 
 
 def _schedule_runs_at_time(
-    instance,
-    logger,
-    workspace,
-    external_schedule,
-    schedule_time,
-    tick_context,
+    workspace_process_context: IWorkspaceProcessContext,
+    logger: logging.Logger,
+    external_schedule: ExternalSchedule,
+    schedule_time: datetime.datetime,
+    tick_context: _ScheduleLaunchContext,
     debug_crash_flags,
 ):
     schedule_name = external_schedule.name
-
+    instance = workspace_process_context.instance
     schedule_origin = external_schedule.get_external_origin()
     repository_handle = external_schedule.handle.repository_handle
 
-    pipeline_selector = PipelineSelector(
-        location_name=schedule_origin.external_repository_origin.repository_location_origin.location_name,
-        repository_name=schedule_origin.external_repository_origin.repository_name,
-        pipeline_name=external_schedule.pipeline_name,
-        solid_selection=external_schedule.solid_selection,
-    )
-
-    repo_location = workspace.get_repository_location(
+    repo_location = workspace_process_context.create_request_context().get_repository_location(
         schedule_origin.external_repository_origin.repository_location_origin.location_name
     )
 
-    external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
-
     schedule_execution_data = repo_location.get_external_schedule_execution_data(
         instance=instance,
         repository_handle=repository_handle,
         schedule_name=external_schedule.name,
         scheduled_execution_time=schedule_time,
     )
-    yield
+    yield None
 
     if not schedule_execution_data.run_requests:
         if schedule_execution_data.skip_message:
@@ -471,6 +582,15 @@ def _schedule_runs_at_time(
         return
 
     for run_request in schedule_execution_data.run_requests:
+        pipeline_selector = PipelineSelector(
+            location_name=schedule_origin.external_repository_origin.repository_location_origin.location_name,
+            repository_name=schedule_origin.external_repository_origin.repository_name,
+            pipeline_name=external_schedule.pipeline_name,
+            solid_selection=external_schedule.solid_selection,
+            asset_selection=run_request.asset_selection,
+        )
+        external_pipeline = repo_location.get_external_pipeline(pipeline_selector)
+
         run = _get_existing_run_for_request(instance, external_schedule, schedule_time, run_request)
         if run:
             if run.status != PipelineRunStatus.NOT_STARTED:
@@ -482,7 +602,7 @@ def _schedule_runs_at_time(
                     f"Run {run.run_id} already completed for this execution of {external_schedule.name}"
                 )
                 tick_context.add_run_info(run_id=run.run_id, run_key=run_request.run_key)
-                yield
+                yield None
                 continue
             else:
                 logger.info(
@@ -502,7 +622,7 @@ def _schedule_runs_at_time(
 
         if run.status != PipelineRunStatus.FAILURE:
             try:
-                instance.submit_run(run.run_id, workspace)
+                instance.submit_run(run.run_id, workspace_process_context.create_request_context())
                 logger.info(f"Completed scheduled launch of run {run.run_id} for {schedule_name}")
             except Exception:
                 error_info = serializable_error_info_from_exc_info(sys.exc_info())
@@ -557,12 +677,12 @@ def _get_existing_run_for_request(
 
 
 def _create_scheduler_run(
-    instance,
-    schedule_time,
-    repo_location,
-    external_schedule,
-    external_pipeline,
-    run_request,
+    instance: DagsterInstance,
+    schedule_time: datetime.datetime,
+    repo_location: RepositoryLocation,
+    external_schedule: ExternalSchedule,
+    external_pipeline: ExternalPipeline,
+    run_request: RunRequest,
 ):
     from dagster._daemon.daemon import get_telemetry_daemon_session_id
 
@@ -572,14 +692,16 @@ def _create_scheduler_run(
     external_execution_plan = repo_location.get_external_execution_plan(
         external_pipeline,
         run_config,
-        external_schedule.mode,
+        check.not_none(external_schedule.mode),
         step_keys_to_execute=None,
         known_state=None,
     )
     execution_plan_snapshot = external_execution_plan.execution_plan_snapshot
 
-    pipeline_tags = validate_tags(external_pipeline.tags, allow_reserved_tags=False) or {}
-    tags = merge_dicts(pipeline_tags, schedule_tags)
+    tags = merge_dicts(
+        validate_tags(external_pipeline.tags, allow_reserved_tags=False) or {},
+        schedule_tags,
+    )
 
     tags[SCHEDULED_EXECUTION_TIME_TAG] = to_timezone(schedule_time, "UTC").isoformat()
     if run_request.run_key:
@@ -613,4 +735,7 @@ def _create_scheduler_run(
         parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
         external_pipeline_origin=external_pipeline.get_external_origin(),
         pipeline_code_origin=external_pipeline.get_python_origin(),
+        asset_selection=frozenset(run_request.asset_selection)
+        if run_request.asset_selection
+        else None,
     )
