@@ -1,4 +1,4 @@
-from typing import List, Optional
+from typing import Optional, Sequence
 
 import dagster._check as check
 from dagster._core.host_representation import RepositoryLocation
@@ -10,7 +10,7 @@ from dagster._core.host_representation.selector import PipelineSelector
 def external_pipeline_from_location(
     repo_location: RepositoryLocation,
     external_pipeline_origin: ExternalPipelineOrigin,
-    solid_selection: Optional[List[str]],
+    solid_selection: Optional[Sequence[str]],
 ) -> ExternalPipeline:
     check.inst_param(repo_location, "repository_location", RepositoryLocation)
     check.inst_param(external_pipeline_origin, "external_pipeline_origin", ExternalPipelineOrigin)
